/**
 * 
 */
package newprocess.diagram.cust.annotations.dialogs;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import newprocess.diagram.cust.annotations.commands.DeleteCommand;
import newprocess.diagram.cust.annotations.commands.ReadCommand;
import newprocess.diagram.cust.annotations.commands.SortCommand;
import newprocess.diagram.cust.annotations.commands.WriteCommand;
import newprocess.diagram.cust.annotations.utils.URIConvertUtil;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.DecorationEditPolicy;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.EditPolicyRoles;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

/**
 * Class that plays the role of the domain model in the NewAnnotationDialog
 * @author sh
 */
public class AnnotationList {
	private Vector<Annotation> annotations = new Vector<Annotation>();
	private Set<IAnnotationViewer> changeListeners = new HashSet<IAnnotationViewer>();
	private EditPart selectedEditPart;
	private String projectName = null;
	public static final String STANDARD_TEXT = "(standard)";
	
	/**
	 * Constructor
	 */
	public AnnotationList(EditPart selectedEditPart, String projectName) {
		super();
		this.selectedEditPart = selectedEditPart;
		this.initData();
		this.projectName = projectName;
	}
	
	/**
	 * Return the collection of annotations
	 */
	public Vector<Annotation> getAnnotations() {
		return annotations;
	}
	
	/**
	 * initialize some default entries 
	 */
	private void initData() {
		// get the model data
		ReadCommand readCommand = new ReadCommand();
		EList urls = readCommand.getValues(selectedEditPart);

		if(urls == null || urls.isEmpty()) return;
		
		// create Annotation Objects
		Annotation exampleAnnotation = null;
		for(Iterator<Object> iter = urls.iterator(); iter.hasNext();) {
			Object element = (Object)iter.next();
			exampleAnnotation = new Annotation(element.toString());
			exampleAnnotation.setUri(element.toString());
			annotations.add(exampleAnnotation);
		}
	};
	
	/**
	 * Add a new annotation to the collection of annotations
	 */
	public void addAnnotation() {
		Annotation annotation = new Annotation(AnnotationList.STANDARD_TEXT);	
		annotations.add(annotations.size(), annotation);

		Iterator<IAnnotationViewer> iterator = changeListeners.iterator();
		while(iterator.hasNext()) {
			((IAnnotationViewer) iterator.next()).addAnnotation(annotation);
		}	
	}

	/**
	 * Removes an Annotation
	 * @param annotation
	 */
	public void removeAnnotation(Annotation annotation) {
		// check if a standard text entry is in the collection
		deleteDummies();
		
		// removes the annotation from the view
		annotations.remove(annotation);

		// remove the annotation from the model
		// get the annotation to be removed
		String removingAnno = annotation.getUri().toString();
		if(removingAnno == null) return;
		
		// fetch all stored Annotations from the model
		ReadCommand readCommand = new ReadCommand();
		EList urls = readCommand.getValues(selectedEditPart);
		if(urls.isEmpty() || urls == null) return;
		
		// remove the commited annotation
		DeleteCommand deleteCommand = new DeleteCommand();
		deleteCommand.executeRemoveCommand(selectedEditPart, removingAnno);
		
		// notify the decorator
		Object editPolicy = selectedEditPart.getEditPolicy(EditPolicyRoles.DECORATION_ROLE);
		if(editPolicy instanceof DecorationEditPolicy) {
			((DecorationEditPolicy)editPolicy).refresh();
		}
		
		Iterator<IAnnotationViewer> iterator = changeListeners.iterator();
		while(iterator.hasNext())
			((IAnnotationViewer) iterator.next()).removeAnnotation(annotation);
	}

	/**
	 * does the changes in the model
	 * @param annotation
	 */
	public void annotationChanged(Annotation annotation) {			
		// check if a standard text entry is in the collection
		deleteDummies();
		
		// first delete all entries in the model
		ReadCommand readCom = new ReadCommand();
		EList urls = readCom.getValues(selectedEditPart);
		DeleteCommand delCom = new DeleteCommand();
		delCom.executeRemoveCommand(selectedEditPart, urls);
		
		// write all current annotations
		WriteCommand writeCommand = new WriteCommand();
		for(Iterator<Annotation> iter = annotations.iterator(); iter.hasNext();) {
			Annotation element = (Annotation)iter.next();
			String url = element.getUri();
			if(url.equals(STANDARD_TEXT)) continue;
			writeCommand.executeWriteCommand(selectedEditPart, url);
		}
		
		// notify the decorator
		Object editPolicy = selectedEditPart.getEditPolicy(EditPolicyRoles.DECORATION_ROLE);
		if(editPolicy instanceof DecorationEditPolicy) {
			((DecorationEditPolicy)editPolicy).refresh();
		}
		
		// update the table view
		Iterator<IAnnotationViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IAnnotationViewer) iterator.next()).updateAnnotation(annotation);
	}
	
	/**
	 * moves the current annotation upwards 
	 */
	public void moveAnnotationUp(Annotation annotation) {	
		//check if it is the first element
		if(annotations.firstElement() == annotation) return;
		
		// check if a standard text entry is in the collection
		deleteDummies();
		
		EList list = new BasicEList();
		list.addAll(annotations);
		int annoListIndex = list.indexOf(annotation);
		list.move(annoListIndex-1, annotation);
		annotations.clear();
		annotations.addAll(list);
		
		//*********************** model part *******************************	
		// fetch annotations from the model
		ReadCommand readCom = new ReadCommand();
		EList items = readCom.getValues(selectedEditPart);
		// get the index of the selected annotation in the model
		int annoIndex = items.indexOf(annotation.getUri().toString());
		// if it is the first element, do nothing
		if(annoIndex<=0) return;
		// move one position upwards
		SortCommand sortCom = new SortCommand();
		sortCom.executeMoveCommand(selectedEditPart, annotation.getUri(), annoIndex-1);
		
		// notify the decorator
		Object editPolicy = selectedEditPart.getEditPolicy(EditPolicyRoles.DECORATION_ROLE);
		if(editPolicy instanceof DecorationEditPolicy) {
			((DecorationEditPolicy)editPolicy).refresh();
		}
		
		// update the table view
		Iterator<IAnnotationViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IAnnotationViewer) iterator.next()).updateAnnotation(annotation);
	}
	
	/**
	 * moves the current annotation downwards 
	 */
	public void moveAnnotationDown(Annotation annotation) {
		// check if it is the last element
		if(annotations.lastElement() == annotation) return;
		
		// check if a standard text entry is in the collection
		deleteDummies();
		
		EList list = new BasicEList();
		list.addAll(annotations);
		int annoListIndex = list.indexOf(annotation);
		list.move(annoListIndex+1, annotation);
		annotations.clear();
		annotations.addAll(list);
		
		//*********************** model part *******************************
		// fetch annotations from the model
		ReadCommand readCom = new ReadCommand();
		EList items = readCom.getValues(selectedEditPart);
		// get the index of the selected annotation in the model
		int annoIndex = items.indexOf(annotation.getUri().toString());	
		// move one position downwards
		SortCommand sortCom = new SortCommand();
		sortCom.executeMoveCommand(selectedEditPart, annotation.getUri(), annoIndex+1);
		
		// notify the decorator
		Object editPolicy = selectedEditPart.getEditPolicy(EditPolicyRoles.DECORATION_ROLE);
		if(editPolicy instanceof DecorationEditPolicy) {
			((DecorationEditPolicy)editPolicy).refresh();
		}
		
		// update the table view
		Iterator<IAnnotationViewer> iterator = changeListeners.iterator();
		while (iterator.hasNext())
			((IAnnotationViewer) iterator.next()).updateAnnotation(annotation);	
	}
	
	/**
	 * link to an existing Annotation
	 */
	public void linkToAnnotation() {
		// open the Workspace Selection Dialog
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		ElementChooserDialog chooserDia = new ElementChooserDialog(shell);
		chooserDia.open();
				
		//opens the selection dialog and fetch the project/file
		String project = chooserDia.getSelectedProject();
		IFile file = chooserDia.getSelectedFile();
		if((project == null) || (file == null) || (projectName == null)) return;
		
		// get the relative path for the created file
        URIConvertUtil uriUtil = new URIConvertUtil();
        String path = uriUtil.getPathForFile(file, projectName, project);    
        
        //Add a new annotation to the collection of annotations
		Annotation annotation = new Annotation(path.toString());	
		annotations.add(annotations.size(), annotation);
		
		// refreshing the table and the model
		annotationChanged(annotation);
	}
	
	/**
	 * delete the dummies in the TableViewer
	 */
	private void deleteDummies() {
		Vector<Annotation> v = new Vector<Annotation>();
		for(Iterator<Annotation> iter=annotations.iterator(); iter.hasNext();) {
			Annotation element = (Annotation)iter.next();
			if(element.getUri().equals(STANDARD_TEXT)) {
				v.add(element);
			}
		}
		annotations.removeAll(v);
	}
	
	/**
	 * @param viewer
	 */
	public void removeChangeListener(IAnnotationViewer viewer) {
		changeListeners.remove(viewer);
	}

	/**
	 * @param viewer
	 */
	public void addChangeListener(IAnnotationViewer viewer) {
		changeListeners.add(viewer);
	}

}
