/**
 * 
 */
package core;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.ThisExpression;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;


/**
 * This class is responsible for create a field
 * declaration and is used by the environment entry
 * refactoring.
 * 
 * @author sh
 */
public class CreateFieldDeclaration extends AbstractAST {
		
	// the Info Object containing all necessary refactor info
	private RefactorInfo info = null;
	
	/**
	 * This method creates a field declaration, getter
	 * and a setter method for an EnvEntry in the compilation 
	 * unit.
	 * 
	 * @param info the info object containing needed info for field creation
	 */
	public void createFieldDeclarations(RefactorInfo info) {		
		CompilationUnit u = parse(info.getUnit());
		this.info = info;
		new FieldCreator().process(u);
		
		// write changes
		ManipulateHelper.saveDirectlyModifiedUnit(u);
	}
		
	
	private class FieldCreator extends ASTVisitor {
		
		/**
		 * Looks for type declarations.
		 * For an occurence a new field declaration object
		 * will be created at the beginning of the type, followed
		 * by the getter setter method creation at the end.
		 * 
		 * @param node the node to visit
		 */	
		@Override
		public boolean visit(TypeDeclaration node) {								
			AST ast = node.getAST();
						
			// add the member variable
			node.bodyDeclarations().add(0,createMemberVariable(ast));
			
			// add the setter
			int size = node.bodyDeclarations().size();
			node.bodyDeclarations().add(size, createSetter(ast));
			
			// add the getter
			node.bodyDeclarations().add(size, createGetter(ast));
			return super.visit(node);
		}
		
		/**
		 * This helper method creates a member variable in
		 * the given ast.
		 * 
		 * @param ast the ast in which the member variable shoudl created
		 * @return field the created member variable
		 */
		private FieldDeclaration createMemberVariable(AST ast) {
			// create a variable declaration and set the name
			VariableDeclarationFragment varFrag = ast.newVariableDeclarationFragment();
			varFrag.setName(ast.newSimpleName(info.getNewVarName()));	
			FieldDeclaration field = ast.newFieldDeclaration(varFrag);
			
			// set the type
			ITypeBinding binding = ast.resolveWellKnownType(info.getNewName());
			if (binding != null) {
				// set the new type in the declaration
				PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
				PrimitiveType primType = ast.newPrimitiveType(code);
				field.setType(primType);
			}
			else {
				SimpleType newType = ast.newSimpleType(ast.newName(info.getNewName()));				
				field.setType(newType);
			}
			
			// set the modifier
			Modifier mf = ast.newModifier(Modifier.ModifierKeyword.PRIVATE_KEYWORD);
			field.modifiers().add(mf);
			
			return field;
		}
		
		/**
		 * This helper method creates a setter method for the
		 * member variable stored in the info object.
		 * 
		 * @param ast the ast in which the setter method should be created
		 * @return md the created method
		 */
		private MethodDeclaration createSetter(AST ast) {			
			// ************ create the method declaration **************************
			MethodDeclaration md = ast.newMethodDeclaration();
			md.setConstructor(false);
			Modifier mf = ast.newModifier(Modifier.ModifierKeyword.PUBLIC_KEYWORD);
			md.modifiers().add(mf);
			
			// set the method name
			String methodName = info.getNewVarName();
			methodName = "set" + methodName.substring(0,1).toUpperCase() + 
							methodName.substring(1, methodName.length()).toLowerCase() ;
			md.setName(ast.newSimpleName(methodName));
			
			//************ add the method parameter ********************************
			SingleVariableDeclaration varDecl = ast.newSingleVariableDeclaration();
			
			// set the parameter type
			ITypeBinding binding = ast.resolveWellKnownType(info.getNewName());
			if (binding != null) {
				PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
				PrimitiveType primType = ast.newPrimitiveType(code);
				varDecl.setType(primType);
			}
			else {
				SimpleType newType = ast.newSimpleType(ast.newName(info.getNewName()));				
				varDecl.setType(newType);
			}
			
			// set the parameter name
			varDecl.setName(ast.newSimpleName(info.getNewVarName()));
			md.parameters().add(varDecl);
			
			//************ add the method body *************************************
			Block methodBlock = ast.newBlock();
			md.setBody(methodBlock);
			
			// create the setter assignment statement
			Assignment a = ast.newAssignment();
			a.setOperator(Assignment.Operator.ASSIGN);
			ThisExpression thisExpr = ast.newThisExpression();
			FieldAccess fieldAccess = ast.newFieldAccess();
			fieldAccess.setName(ast.newSimpleName(info.getNewVarName()));
			fieldAccess.setExpression(thisExpr);
			a.setLeftHandSide(fieldAccess);
			a.setRightHandSide(ast.newSimpleName(info.getNewVarName()));
			methodBlock.statements().add(ast.newExpressionStatement(a)); 
			return md;
		}
		
		/**
		 * This helper method creates a getter method for the
		 * member variable stored in the info object.
		 * 
		 * @param ast the ast in which the gtter method should be created
		 * @return md the created method
		 */
		private MethodDeclaration createGetter(AST ast) {		
			// ************ create the method declaration **************************
			MethodDeclaration md = ast.newMethodDeclaration();
			md.setConstructor(false);
			Modifier mf = ast.newModifier(Modifier.ModifierKeyword.PUBLIC_KEYWORD);
			md.modifiers().add(mf);
			
			// set the method name
			String methodName = info.getNewVarName();
			methodName = "get" + methodName.substring(0,1).toUpperCase() + 
							methodName.substring(1, methodName.length()).toLowerCase() ;
			md.setName(ast.newSimpleName(methodName));
			
			// set the return type
			ITypeBinding binding = ast.resolveWellKnownType(info.getNewName());
			if (binding != null) {
				PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
				PrimitiveType primType = ast.newPrimitiveType(code);
				md.setReturnType2(primType);
			}
			else {
				SimpleType newType = ast.newSimpleType(ast.newName(info.getNewName()));				
				md.setReturnType2(newType);
			}
			
			//************ add the method body *************************************
			Block methodBlock = ast.newBlock();
			md.setBody(methodBlock);
			
			// create the return statement			
			ThisExpression thisExpr = ast.newThisExpression();
			//thisExpr.setQualifier(ast.newSimpleName(info.getNewVarName()));
			
			FieldAccess fieldAccess = ast.newFieldAccess();
			fieldAccess.setName(ast.newSimpleName(info.getNewVarName()));
			fieldAccess.setExpression(thisExpr);
			
			ReturnStatement ret = ast.newReturnStatement();
			ret.setExpression(fieldAccess);
			methodBlock.statements().add(ret);
			return md;
		}
		
		/**
		 * Starts the process.
		 * 
		 * @param unit the AST root node. Bindings have to been resolved.
		 */
		public void process(CompilationUnit unit) {
			unit.accept(this);
		}
	}
}
