package core;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.PrimitiveType;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SimpleType;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;


/**
 * @author sh
 *
 */
public class FieldDeclarationVisitor extends ASTVisitor {
	private String oldName = null;
	private String newName = null;
	private String oldVarName = null;
	private String newVarName = null;
	
	/**
	 * Looks for field declarations.
	 * For every occurence matching the renamed field
	 * the field type would be updated 
	 * e.g. private int a; --> private String a;
	 * If the use set a new variable name the new name
	 * will be set.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(FieldDeclaration node) {	
		// iterate over all variable declarations
		List<VariableDeclarationFragment> fragments = node.fragments();
		for (Iterator<VariableDeclarationFragment> iterator = fragments.iterator(); iterator.hasNext();) {
			VariableDeclarationFragment name = (VariableDeclarationFragment) iterator.next();
			String varName = name.getName().toString();
			
			// if the right variable type and name was found
			// do the refactoring
			Type type = node.getType();
			  if (varName.equals(this.oldVarName) &&
				type.toString().equals(this.oldName)) {
				
				// if the user set a primitive datatype
				AST ast = type.getAST();
				ITypeBinding binding = ast.resolveWellKnownType(this.newName);
				if (binding != null) {				
					// set the new type in the declaration
					PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
					PrimitiveType primType = ast.newPrimitiveType(code);
					node.setType(primType);					
				}
				// if the user set a complex datatype
				else {
					SimpleType newType = ast.newSimpleType(ast.newName(this.newName));				
					node.setType(newType);
					
					// resolve the package of the type for the import statement
				}
				
				// set the new variable name
				name.setName(ast.newSimpleName(this.newVarName));
			}
		}
		return super.visit(node);
	}
		
	/**
	 * Looks for method declarations.
	 * For each method declarations at first the return statement
	 * in the method body is searched. For an occurency it checks
	 * if the return value is the same type as the field declarations.
	 * If so, the methods return value, method name and method 
	 * parameter types will be adapted.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(MethodDeclaration node) {	
		// iterate over the whole method body
		List<Statement> statements = node.getBody().statements();
		for (Iterator<Statement> iterator = statements.iterator(); iterator.hasNext();) {
			Statement statement = (Statement) iterator.next();
			
			// handle the return value type
			// according to the variable declaration type, 
			// adapt the return value for a getter method.
			handleReturnValue(node, statement);
			
			// handle the parameters
			// according to the variable declaration type,
			// adapt the variable type in the parameter list for 
			// a setter method. This algorithm investigates the method body
			// and searches for the return statement. If the field to refactor
			// has been found, we know that we have to adapt the return value
			// and the variable declaration type in the method parameters.
			handleParameters(node, statement);
			
			// handles the getter methods name
			handleGetterMethodName(node, statement);
			
			// handles the setter methods name
			handleSetterMethodName(node, statement);
			
			// handle return statement
			// if the variable name should be renamed
			// update the return statement in a getter method
			handleReturnStatement(node, statement);
		}
		return super.visit(node);
	}
	
	/**
	 * A helper method to adapt the getter method name.
	 * 
	 * @param node the node to visit
	 * @param statement the current statement to investigate
	 */
	private void handleGetterMethodName(MethodDeclaration node, Statement statement) {
		// handle the return value type
		// according to the variable declaration type, 
		// adapt the method name for a getter method.
		if (statement instanceof ReturnStatement == false) return;
		
		Expression expr = ((ReturnStatement)statement).getExpression();
		if (expr == null || expr instanceof FieldAccess == false) return;
		String var = ((FieldAccess)expr).getName().toString();
		
		if (var.equals(this.oldVarName)) {
			// set the new name
			AST ast = node.getAST();
			StringBuilder sb = new StringBuilder(newVarName.substring(0,1).toUpperCase());
			sb.append(newVarName.substring(1,newVarName.length()).toLowerCase());
			String newMethodName = "get" + sb.toString();
			node.setName(ast.newSimpleName(newMethodName));
		}
	}
	
	/**
	 * A helper method to adapt the setter method name
	 * 
	 * @param node the node to visit
	 * @param statement the current statement to investigate
	 */
	private void handleSetterMethodName(MethodDeclaration node, Statement statement) {
		// handle the method name of the setter.
		// According to the variable declaration type,
		// adapt the method name a setter method. This algorithm investigates 
		// the method body and searches for the return statement. If the field 
		//to refactor has been found, we know that we have to adapt the method name

		if (statement instanceof ExpressionStatement == false) return;
		
		Expression expr = ((ExpressionStatement)statement).getExpression();
			
		// step over if it is not an assignment
		if (expr instanceof Assignment == false) return;
		Assignment assign = (Assignment)expr;
			
		// investigate the assignment
		Expression lhs = assign.getLeftHandSide();
			
		// step over if the left hand site of the expression is not a field access
		if (lhs instanceof FieldAccess == false) return;
			
		// investigate the field access, continue if 
		// the field to refactor isn't used
		FieldAccess lhsf = (FieldAccess)lhs;
		if (!lhsf.getName().toString().equals(this.oldVarName)) return;
				
		// get the the right hand site of the expression to know which 
		// variable declaration we have to adapt in a setter method
		Expression rhs = assign.getRightHandSide();
		String bodyVarName = ((SimpleName)rhs).getFullyQualifiedName().toString();
				
		// iterate over all method variable declarations 
		// to find the correct parameter
		List<SingleVariableDeclaration> parameters = node.parameters();
		for (Iterator<SingleVariableDeclaration> varIter = parameters.iterator(); varIter.hasNext();) {
			SingleVariableDeclaration varDecl = (SingleVariableDeclaration)varIter.next();
			String varDeclName = varDecl.getName().getFullyQualifiedName().toString();
				
			// check if it is the correct parameter
			if (!varDeclName.equals(bodyVarName)) continue;
			
			// adapt the parameter name
			AST ast = node.getAST();
			varDecl.setName(ast.newSimpleName(newVarName));
			
			// adapt the method name
			StringBuilder sb = new StringBuilder(newVarName.substring(0,1).toUpperCase());
			sb.append(newVarName.substring(1,newVarName.length()).toLowerCase());
			String newMethodName = "set" + sb.toString();
			node.setName(ast.newSimpleName(newMethodName));
			
			// adapt the body variable name
			assign.setRightHandSide(ast.newSimpleName(newVarName));
		}
	}
	
	/**
	 * A helper method to adapt the methods return type.
	 * 
	 * @param node the node to visit
	 * @param statement the current statement to investigate
	 */
	private void handleReturnValue(MethodDeclaration node, Statement statement) {
		// handle the return value type
		// according to the variable declaration type, 
		// adapt the return value for a getter method.
		if (statement instanceof ReturnStatement == false) return;
		
		Expression expr = ((ReturnStatement)statement).getExpression();
		if (expr == null || expr instanceof FieldAccess == false) return;
		String var = ((FieldAccess)expr).getName().toString();
		
		if (var.equals(this.oldVarName)) {
		   // resolve the type stored in the info object
		   AST ast = node.getAST();
		   ITypeBinding binding = ast.resolveWellKnownType(this.newName);
		   // if the user set a primitive datatype
		   if (binding != null) {				
			  PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
			  PrimitiveType primType = ast.newPrimitiveType(code);
			  node.setReturnType2(primType);					
		   }
		   // if the user set a complex datatype
		   else {
			  SimpleType newType = ast.newSimpleType(ast.newName(this.newName));				
			  node.setReturnType2(newType);
		   }
		}		
	}
	
	/**
	 * A helper method to adapt the methods parameter type.
	 * 
	 * @param node the node to visit
	 * @param statement the current statement to investigate
	 */
	private void handleParameters(MethodDeclaration node, Statement statement) {
		// handle the parameters
		// according to the variable declaration type,
		// adapt the variable type in the parameter list for 
		// a setter method. This algorithm investigates the method body
		// and searches for the return statement. If the field to refactor
		// has been found, we know that we have to adapt the return value
		// and the variable declaration type in the method parameters.
		if (statement instanceof ExpressionStatement == false) return;
		
		Expression expr = ((ExpressionStatement)statement).getExpression();
			
		// step over if it is not an assignment
		if (expr instanceof Assignment == false) return;
		Assignment assign = (Assignment)expr;
			
		// investigate the assignment
		Expression lhs = assign.getLeftHandSide();
			
		// step over if the left hand site of the expression is not a field access
		if (lhs instanceof FieldAccess == false) return;
			
		// investigate the field access, continue if 
		// the field to refactor isn't used
		FieldAccess lhsf = (FieldAccess)lhs;
		if (!lhsf.getName().toString().equals(this.oldVarName)) return;
				
		// get the the right hand site of the expression to know which 
		// variable declaration we have to adapt in a setter method
		Expression rhs = assign.getRightHandSide();
		String bodyVarName = ((SimpleName)rhs).getFullyQualifiedName().toString();
				
		// iterate over all method variable declarations 
		// to find the correct parameter
		List<SingleVariableDeclaration> parameters = node.parameters();
		for (Iterator<SingleVariableDeclaration> varIter = parameters.iterator(); varIter.hasNext();) {
			SingleVariableDeclaration varDecl = (SingleVariableDeclaration)varIter.next();
			// get the variable type
			Type type = varDecl.getType();
			String varDeclName = varDecl.getName().getFullyQualifiedName().toString();
				
			// check if it is the correct parameter
			if (!varDeclName.equals(bodyVarName)) continue;
			ITypeBinding binding = type.getAST().resolveWellKnownType(this.newName);
			// if the user set a primitive datatype
			if (binding != null) {			
				PrimitiveType.Code code = PrimitiveType.toCode(binding.getName());
				PrimitiveType primType = type.getAST().newPrimitiveType(code);
				varDecl.setType(primType);
			}
			// if the user set a complex datatype
			else {
				SimpleType newType = type.getAST().newSimpleType(type.getAST().newName(this.newName));				
				varDecl.setType(newType);
			}			
		}			
	}
	
	/**
	 * A helper method to adapt the return statement
	 * in a getter method body, if the variable name 
	 * should be renamed.
	 * 
	 * @param node
	 * @param statement
	 */
	private void handleReturnStatement(MethodDeclaration node, Statement statement) {
		if (statement instanceof ReturnStatement == false) return;
		
		ReturnStatement retStatement = (ReturnStatement)statement;
		Expression expr = retStatement.getExpression();
		if (expr == null) return;
		if (expr.toString().equals(this.oldVarName)) {
			AST ast = retStatement.getAST();
			retStatement.setExpression(ast.newSimpleName(this.newVarName));
		}
	}
	
	/**
	 * Looks for field acess.
	 * For every occurence matching the renamed field
	 * the field statement would be updated.
	 * 
	 * @param node the node to visit
	 */
	@Override
	public boolean visit(FieldAccess node) {
		String fqn = node.getName().getFullyQualifiedName();
		if (fqn.contains(this.oldVarName)) {
			AST ast = node.getAST();
			SimpleName name = ast.newSimpleName(fqn.replace(this.oldVarName, this.newVarName));
			node.setName(name);
		}
		return super.visit(node);
	}
	
	/**
	 * Starts the process.
	 * 
	 * @param unit the AST root node. Bindings have to been resolved.
	 */
	public void process(CompilationUnit unit, RefactorInfo info) {		
		this.oldName = info.getOldName();
		this.newName = info.getNewName();
		this.oldVarName = info.getOldVarName();
		this.newVarName = info.getNewVarName();
		unit.accept(this);
	}
}
