/**
 * 
 */
package core;

import org.eclipse.core.filebuffers.FileBuffers;
import org.eclipse.core.filebuffers.ITextFileBuffer;
import org.eclipse.core.filebuffers.ITextFileBufferManager;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.text.edits.MalformedTreeException;
import org.eclipse.text.edits.TextEdit;

/**
 * Helper class used to write the performed changes back to Java source.
 * 
 * @author sh
 */
public class ManipulateHelper {
	
	/**
	 * To avoid writing thow save methods, this interface hast been created.
	 * The implementing classes provide a TextEdit object.
	 * 
	 * @author sh
	 *
	 */
	public interface TextEditProvider {
		/**
		 * Provides a TextEdit document.
		 * 
		 * @param document the document the TextEdit object will be applied to
		 * @return the TextEdit instance
		 */
		TextEdit getTextEdit(IDocument document);
	}
	
	/**
	 * Convenience method that saves changes that have been made directly to an AST.
	 * 
	 * @param unit the unit that contains changes.
	 */
	public static void saveDirectlyModifiedUnit(final CompilationUnit unit) {
		save(unit, new TextEditProvider() {
			public TextEdit getTextEdit(IDocument document) {
				return unit.rewrite(document, null);
			}
		});
	}
 
	/**
	 * Writes changes to Java source. This method does not distinguish how the
	 * AST was changed.
	 * 
	 * @param unit the AST node of the compilation unit that has been changed
	 * @param textEditProvider provides the change information
	 * @param textEditProvider
	 */
	private static void save(CompilationUnit unit, TextEditProvider textEditProvider) {
		ITextFileBufferManager bufferManager = FileBuffers.getTextFileBufferManager();
		IPath path = unit.getJavaElement().getPath();
		try {
			// connect to path
			bufferManager.connect(path, null);
			
			ITextFileBuffer textFileBuffer = bufferManager.getTextFileBuffer(path);
			
			// retrieve the buffer
			IDocument document = textFileBuffer.getDocument();
			
			// ask the textEditProvider for the change information
			TextEdit edit = textEditProvider.getTextEdit(document);
			
			// apply the changes to the document
			edit.apply(document);
			
			// write the changes from the buffer to the file
			textFileBuffer.commit(null /* ProgressMonitor */, false /* Overwrite */);
		} catch (MalformedTreeException e) {
			e.printStackTrace();
		} catch (BadLocationException e) {
			e.printStackTrace();
		} catch (CoreException e) {
			e.printStackTrace();
		} finally {
			try {
				bufferManager.disconnect(path, null);
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
	}
}
