/**
 * 
 */
package core;

import java.util.Iterator;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

/**
 * This class is responsible for remove a field
 * declaration and is used by the environment entry
 * refactoring.
 * 
 * @author sh
 */
public class RemoveFieldDeclaration extends AbstractAST {
	
	// the Info Object containing all necessary refactor info
	private RefactorInfo info = null;
	
	/**
	 * This method removes a field declaration and the concerning
	 * getter and setter methods for an EnvEntry in the compilation 
	 * unit.
	 * 
	 * @param info the info object containing needed info for field creation
	 */
	public void removeFieldDeclarations(RefactorInfo info) {		
		CompilationUnit u = parse(info.getUnit());
		this.info = info;
		new FieldRemover().process(u);
		
		// write changes
		ManipulateHelper.saveDirectlyModifiedUnit(u);
	}
	
	private class FieldRemover extends ASTVisitor {
				
		/**
		 * Looks for field declarations.
		 * For an occurence we check if the type and the name of
		 * the member variable fits the environment entry. If so
		 * the field declaration will be removed.
		 * 
		 * @param node the node to visit
		 */
		@Override
		public boolean visit(FieldDeclaration node) {
			// iterate over all variable declarations
			List<VariableDeclarationFragment> fragments = node.fragments();
			for (Iterator<VariableDeclarationFragment> iterator = fragments.iterator(); iterator.hasNext();) {
				VariableDeclarationFragment name = (VariableDeclarationFragment) iterator.next();
				String varName = name.getName().toString();
				
				// if the right variable type and name was found
				// remove the field
				Type type = node.getType();
				if (varName.equals(info.getNewVarName()) &&
					type.toString().equals(info.getNewName())) {
					node.delete();
				}
			}
			return super.visit(node);
		}
		
		/**
		 * Looks for method declarations.
		 * For an occurence we check if the methods return
		 * value and the set assignment fit the member 
		 * variable. If so the method will be deleted.
		 */
		@Override
		public boolean visit(MethodDeclaration node) {						
			// we assume a setter and a getter method have
			// just one statement in the method body
			if (node.getBody().statements().size() != 1)
				return super.visit(node);
			Statement retStatement = (Statement)node.getBody().statements().get(0);
			
			// handle the setter method
			MethodDeclaration methodToDelete = findSetterMethod(node, retStatement);
			if (methodToDelete != null) {
				methodToDelete.delete();
			}
			
			// handle the getter method
			methodToDelete = findGetterMethod(node, retStatement);
			if (methodToDelete != null) {
				methodToDelete.delete();
			}
			return super.visit(node);
		}
		 
		/**
		 * Handle the Setter Method removing.
		 * This algorithm investigates the method body and searches for 
		 * the return statement. Then it checks if the method parameter 
		 * is used by the set assignment. If so the field to delete
		 * has been found, we know that we have right method to delete.
		 * 
		 * @param node the method declaration to check
		 * @param statement the statement used for the check
		 * @return node the method declaration to delete
		 */
		private MethodDeclaration findSetterMethod(MethodDeclaration node, Statement statement) {
			if (statement instanceof ExpressionStatement == false) return null;
			
			Expression expr = ((ExpressionStatement)statement).getExpression();
				
			// step over if it is not an assignment
			if (expr instanceof Assignment == false) return null;
			Assignment assign = (Assignment)expr;
				
			// investigate the assignment
			Expression lhs = assign.getLeftHandSide();
				
			// step over if the left hand site of the expression is not a field access
			if (lhs instanceof FieldAccess == false) return null;
				
			// investigate the field access, continue if 
			// the field to refactor isn't used
			FieldAccess lhsf = (FieldAccess)lhs;
			if (!lhsf.getName().toString().equals(info.getNewVarName())) return null;
					
			// get the the right hand site of the expression to know which 
			// variable declaration we have to adapt in a setter method
			Expression rhs = assign.getRightHandSide();
			String bodyVarName = ((SimpleName)rhs).getFullyQualifiedName().toString();
					
			// iterate over all method variable declarations 
			// to find the corrent parameter
			List<SingleVariableDeclaration> parameters = node.parameters();
			for (Iterator<SingleVariableDeclaration> varIter = parameters.iterator(); varIter.hasNext();) {
				SingleVariableDeclaration varDecl = (SingleVariableDeclaration)varIter.next();
				String varDeclName = varDecl.getName().getFullyQualifiedName().toString();
					
				// check if it is the correct parameter
				if (!varDeclName.equals(bodyVarName)) continue;
				
				// assume we found the right setter method
				return node;
			}	
			return null;
		}
		
		/**
		 * Handle the Getter Method removing.
		 * This algorithm investigates the method body and searches for 
		 * the assgin statement. If the assign statement uses the member
		 * variable to delete we assume that the getter method to delete
		 * has been found.
		 * 
		 * @param node the method declaration to check
		 * @param statement the statement used for the check
		 * @return node the method declaration to delete
		 */
		private MethodDeclaration findGetterMethod(MethodDeclaration node, Statement statement) {
			if (statement instanceof ReturnStatement == false) return null;
			
			ReturnStatement retStatement = (ReturnStatement)statement;
			Expression expr = retStatement.getExpression();
			if (expr == null || expr instanceof FieldAccess == false) return null;
			String name = ((FieldAccess)expr).getName().toString();
			if (name.equals(info.getNewVarName()))
				return node;
			return null;
		}
		
		/**
		 * Starts the process.
		 * 
		 * @param unit the AST root node. Bindings have to been resolved.
		 */
		public void process(CompilationUnit unit) {
			unit.accept(this);
		}
	}
}