/**
 * 
 */
package core;

import java.util.Iterator;
import java.util.List;

import model.ModelChangeCommand;
import model.ModelUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.gef.EditPart;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.CompositeChange;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.CheckConditionsContext;


/**
 * The Delegate Class which does the refactoring modifications 
 * in the model and onto the source. It also contains the undo
 * modifications.
 * 
 * @author sh
 *
 */
public class RenameImplementationDelegate {

	// the Info Object containing all nesessary refactor info
	private final ModelInfo info;
	
	private Log log = LogFactory.getLog(getClass());

	/**
	 * Constructor
	 * 
	 * @param info the refactoring info object
	 */
	public RenameImplementationDelegate(ModelInfo info) {
		super();
		this.info = info;
	}
	
	/**
	 * Check if the info object contains valid information.
	 * 
	 * @return result a Refactoring Status object
	 */
	public RefactoringStatus checkInitialConditions() {
	    RefactoringStatus result = new RefactoringStatus();
	     
	    if (info.getNewImplementation() == null || info.getNewImplementation().equals("")) {
	    	result.addFatalError("No new implementation could be determined.");
	    }
	    else if (info.getOldImplementation() == null || info.getOldImplementation().equals("")) {
	    	result.addFatalError("The old implementation could't be determined.");	
	    }
	    else if (info.getSelectedEditPart() == null) {
	    	result.addFatalError("The current selection could't be determined.");
	    }
	    return result;
	}
	
	/**
	 * Checks the final conditions before the refactoring will be executed.
	 * If an error will be returned no refactoring operation will be invoked.
	 * 
	 * @param pm a progress monitor
	 * @param ctxt
	 * @return result a Refactoring Status object
	 */
	public RefactoringStatus checkFinalConditions( final IProgressMonitor pm, 
            final CheckConditionsContext ctxt ) {
		RefactoringStatus result = new RefactoringStatus();
				
		// check if the model could be written
		if (!canExecute())
			result.addFatalError("implementation change couldn't be performed.");
			
		// check if corresponding source exist
		if (!sourceFound())
			result.addFatalError("no corresponding source code has been found.");
		
		return result;
	}
	
	/**
	 * The code refactoring and the model refactoring are added
	 * as one task to the Composite Changed object used by the processor.
	 * Each refactoring change object contains its undo handling.
	 * If an undo operation was executed, all tasks of the Composite
	 * Change object are considered.
	 * 
	 * @param pm a progress monitor
	 * @param rootChange the Composite Change object
	 */
	void createChange(final IProgressMonitor pm, final CompositeChange rootChange) {	
		try {
			pm.beginTask("Collecting changes", 100);
			rootChange.add(new ModelChange());
			pm.worked(50);
			rootChange.add(new TypeChange());
		}
		finally {
			pm.done();
		}
	}
	
	/**
	 * Helper method which checks if
	 * the model could be edited.
	 * 
	 * @return true if the model could be edited
	 */
	private boolean canExecute() {
		ModelChangeCommand changeCom = new ModelChangeCommand();
		return changeCom.canExecuteImplChange(info.getSelectedEditPart(), 
				info.getNewImplementation());	
	}
	
	/**
	 * Helper method which checks if
	 * the source refactoring could be
	 * applied and if corresponding
	 * class files exist.
	 * 
	 * @return true if corresponding source exist
	 */
	private boolean sourceFound() {
		EditPart editPart = info.getSelectedEditPart();
		String modelImplementation = ModelUtil.getModelImplProperty(editPart);
		
		boolean sourceFound = false;
		List<ICompilationUnit> units = Util.getICompilationUnits();
		for (Iterator<ICompilationUnit> iterator = units.iterator(); iterator.hasNext();) {
			ICompilationUnit u = (ICompilationUnit) iterator.next();
			String sourceImplementation = Util.getImplementation(u);			
			if (sourceImplementation.equals(modelImplementation)) 
				sourceFound = true;
		}	
		return sourceFound;
	}
	
	/**
	 * The change class responsible for source type refacturing
	 * and undo handling.
	 * 
	 * @author sh
	 *
	 */
	private class TypeChange extends Change {
		public Object getModifiedElement() { return null; }
		public String getName() { return "Class name change"; }
		public void initializeValidationData(IProgressMonitor pm) {}

		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			return new RefactoringStatus();
		}
		
		/**
		 * performs the type refactoring 
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			RenameImplementation renImpl = new RenameImplementation();
			renImpl.process(info);
			log.info("Source: class name change performed");
												
			// returns the undo change object
			return new Change() {				
				public Object getModifiedElement() { return null; }
				public String getName() { return "Undo class name change"; }
				public void initializeValidationData(IProgressMonitor pm) { }

				public RefactoringStatus isValid(IProgressMonitor pm)
						throws CoreException, OperationCanceledException {
					return new RefactoringStatus();
				}
				
				/**
				 * performs the undo operation
				 */
				@Override
				public Change perform(IProgressMonitor pm) throws CoreException {					
					RenameImplementation renImpl = new RenameImplementation();
					ModelInfo undoInfo = new ModelInfo();
					
					// invert values
					undoInfo.setSelectedEditPart(info.getSelectedEditPart());
					undoInfo.setNewImplementation(info.getOldImplementation());
					undoInfo.setOldImplementation(info.getNewImplementation());
					
					// start the undo operation
					renImpl.process(undoInfo);
					log.info("Source: class name change undone");
					return null;
				}				
			};
		}
	}
	
	/**
	 * The change class responsible for model type refacturing
	 * and undo handling.
	 * 
	 * @author sh
	 *
	 */
	private class ModelChange extends Change {
		public Object getModifiedElement() { return null; }
		public String getName() { return "Implementation change"; }
		public void initializeValidationData(IProgressMonitor pm) {}

		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			return new RefactoringStatus();
		}
		
		/**
		 * performs the type refactoring 
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			ModelChangeCommand changeCommand = new ModelChangeCommand();
			changeCommand.executeImplChange(info.getSelectedEditPart(), info.getNewImplementation());
			log.info("Model: Implementation change perfomed");
										
			// returns the undo change object
			return new Change() {
				public Object getModifiedElement() { return null; }
				public String getName() { return "Undo implementation change"; }
				public void initializeValidationData(IProgressMonitor pm) { }

				public RefactoringStatus isValid(IProgressMonitor pm)
						throws CoreException, OperationCanceledException {
					return new RefactoringStatus();
				}

				/**
				 * performs the undo operation
				 */
				@Override
				public Change perform(IProgressMonitor pm) throws CoreException {				
					// invert values
					ModelInfo undoInfo = new ModelInfo();
					undoInfo.setSelectedEditPart(info.getSelectedEditPart());
					undoInfo.setNewImplementation(info.getOldImplementation());
					
					// start the undo operation
					ModelChangeCommand changeCommand = new ModelChangeCommand();
					changeCommand.executeImplChange(undoInfo.getSelectedEditPart(), undoInfo.getNewImplementation());
					log.info("Model: Implementation change undone");
					return null;
				}				
			};
		}
	}
}
