package core;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.SourceField;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.CompositeChange;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.CheckConditionsContext;


/**
 * The Delegate Class which does the refactoring modifications 
 * in the source and onto the model. It also contains the undo
 * modifications.
 * 
 * @author sh
 *
 */
public class RenameTypeDelegate {
	
	// the Info Object containing all nesessary refactor info
	private final RefactorInfo info;
	
	private Log log = LogFactory.getLog(getClass());

	/**
	 * Constructor
	 * 
	 * @param info the refactoring info object
	 */
	public RenameTypeDelegate(RefactorInfo info) {
		super();
		this.info = info;
	}
	
	/**
	 * Check if the info object contains valid information.
	 * 
	 * @return result a Refactoring Status object
	 */
	public RefactoringStatus checkInitialConditions() {
	    RefactoringStatus result = new RefactoringStatus();
	        
	    if (info.getNewName() == null || info.getNewName().equals("")) {
	    	result.addFatalError("No new type name could be determined.");
	    }
	    else if (info.getOldName() == null || info.getOldName().equals("")) {
	    	result.addFatalError("The old name could't be determined.");	
	    }
	    
	    else if (((SourceField)info.getElement()).getElementName() == null ||
	    		((SourceField)info.getElement()).getElementName().equals("")) {
	    	result.addFatalError("The variable name could't be determined.");
	    }
	    
	    boolean isConsitent = false;
	    try {
			isConsitent = info.getUnit().isConsistent();
		} catch (JavaModelException e) {
			result.addFatalError("No source file could be determined.");
		}
	    
		if (!isConsitent) result.addFatalError("No source file could be determined.");
		else if (info.getUnit() == null || info.getUnit().isReadOnly()) {
	    	result.addFatalError("No source file could be determined.");
	    }
	    return result;
	}
	
	/**
	 * Checks the final conditions before the refactoring will be executed.
	 * If an error will be returned no refactoring operation will be invoked.
	 * 
	 * @param pm a progress monitor
	 * @param ctxt
	 * @return result a Refactoring Status object
	 */
	public RefactoringStatus checkFinalConditions( final IProgressMonitor pm, 
            final CheckConditionsContext ctxt ) {
		RefactoringStatus result = new RefactoringStatus();
				
		// TODO use the CheckConditionContext and check the model
		
		return result;
	}
	
	/**
	 * The code refactoring and the model refactoring are added
	 * as one task to the Composite Changed object used by the processor.
	 * Each refactoring change object contains its undo handling.
	 * If an undo operation was executed, all tasks of the Composite
	 * Change object are considered.
	 * 
	 * @param pm a progress monitor
	 * @param rootChange the Composite Change object
	 */
	void createChange(final IProgressMonitor pm, final CompositeChange rootChange) {	
		try {
			pm.beginTask("Collecting changes", 100);
			rootChange.add(new TypeChange());
			pm.worked(50);
			rootChange.add(new ModelChange());
		}
		finally {
			pm.done();
		}
	}

	/**
	 * The change class responsible for source type refacturing
	 * and undo handling.
	 * 
	 * @author sh
	 *
	 */
	private class TypeChange extends Change {
		public Object getModifiedElement() { return null; }
		public String getName() { return "Environment Entry source code change"; }
		public void initializeValidationData(IProgressMonitor pm) {}

		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			return new RefactoringStatus();
		}
		
		/**
		 * performs the type refactoring 
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			RenameFieldDeclaration renField = new RenameFieldDeclaration();
			renField.process(info);
			log.info("Source Code: Environment Entry change perfomed");
									
			// returns the undo change object
			return new Change() {				
				public Object getModifiedElement() { return null; }
				public String getName() { return "Undo Environment Entry source code change";  }
				public void initializeValidationData(IProgressMonitor pm) { }

				public RefactoringStatus isValid(IProgressMonitor pm)
						throws CoreException, OperationCanceledException {
					return new RefactoringStatus();
				}
				
				/**
				 * performs the undo operation
				 */
				@Override
				public Change perform(IProgressMonitor pm) throws CoreException {
					RenameFieldDeclaration renField = new RenameFieldDeclaration();
					RefactorInfo undoInfo = new RefactorInfo();
					
					// invert values
					undoInfo.setElement(info.getElement());
					undoInfo.setUnit(info.getUnit());
					undoInfo.setNewName(info.getOldName());
					undoInfo.setOldName(info.getNewName());
					undoInfo.setNewVarName(info.getOldVarName());
					undoInfo.setOldVarName(info.getNewVarName());
									
					// start the undo operation
					renField.process(undoInfo);
					log.info("Source Code: Environment Entry change undone");
					return null;
				}				
			};
		}
	}
	
	/**
	 * The change class responsible for model type refacturing
	 * and undo handling.
	 * 
	 * @author sh
	 *
	 */
	private class ModelChange extends Change {
		public Object getModifiedElement() { return null; }
		public String getName() { return "Environment Entry model change"; }
		public void initializeValidationData(IProgressMonitor pm) {}

		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			return new RefactoringStatus();
		}
		
		/**
		 * performs the type refactoring 
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			RenameModelField renField = new RenameModelField();
			renField.process(info);
			log.info("Model: Environment Entry change performed");
							
			// returns the undo change object
			return new Change() {
				public Object getModifiedElement() { return null; }
				public String getName() { return "Undo Environment Entry model change"; }
				public void initializeValidationData(IProgressMonitor pm) { }

				public RefactoringStatus isValid(IProgressMonitor pm)
						throws CoreException, OperationCanceledException {
					return new RefactoringStatus();
				}

				/**
				 * performs the undo operation
				 */
				@Override
				public Change perform(IProgressMonitor pm) throws CoreException {
					RenameModelField renField = new RenameModelField();
					RefactorInfo undoInfo = new RefactorInfo();
										
					// invert values
					undoInfo.setElement(info.getElement());
					undoInfo.setUnit(info.getUnit());
					undoInfo.setNewName(info.getOldName());
					undoInfo.setOldName(info.getNewName());
					undoInfo.setNewVarName(info.getOldVarName());
					undoInfo.setOldVarName(info.getNewVarName());
									
					// start the undo operation
					renField.process(undoInfo);
					log.info("Model: Environment Entry change undone");
					return null;
				}				
			};
		}
	}
}