/**
 * 
 */
package core;

import gui.actions.ActionUtil;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import logging.LoggingPlugin;
import model.ModelUtil;
import model.PropertyUtil;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.internal.core.JarPackageFragmentRoot;
import org.eclipse.jdt.internal.core.PackageFragmentRoot;
import org.eclipse.jface.dialogs.MessageDialog;

import diagram.section.EnvEntry;


/**
 * @author sh
 *
 */
public class Util {
			
	/**
	 * Converts the path of a Class File to the full qualified Java Class File Notation.
	 * For Example (src/org/osbl/process/sh.java) to (org.osbl.process.sh).
	 * 
	 * @param path the path to the Class File
	 * @param className the Name of the Class File
	 * @return
	 */
	public static String implementation(IPath path, String className) {
		path = path.removeLastSegments(1);
		path = path.append(className);
		path = path.removeFirstSegments(1);
		String implementation = path.toString().replace('/', '.');
		int ih = implementation.lastIndexOf('.');
		if(ih > -1) 
		   return implementation.substring(0,ih);
		return null;
	}
		
	/**
	 * get all Compilation Units in the workspace.
	 * Just opened projects are considered.
	 * 
	 * @return cuList a list containing all compilation units of the workspace
	 */
	public static List<ICompilationUnit> getICompilationUnits() {	
		List<IFile> javaFiles = ModelUtil.scanWorkspace("java");
		
		List<ICompilationUnit> cuList = new ArrayList<ICompilationUnit>();
		for (Iterator iterator = javaFiles.iterator(); iterator.hasNext();) {
			IFile file = (IFile) iterator.next();
			IProject project = file.getProject();
			
			IJavaProject javaProject = JavaCore.create(project);
			try {
				IPackageFragmentRoot roots[] = javaProject.getPackageFragmentRoots();
				for (int i = 0; i < roots.length; i++) {
					IPackageFragmentRoot packageFragmentRoot = roots[i];
					if (packageFragmentRoot instanceof JarPackageFragmentRoot) continue;
					
					PackageFragmentRoot root = (PackageFragmentRoot)packageFragmentRoot;
					IJavaElement childs[] = root.getChildren();
					for (int j = 0; j < childs.length; j++) {
						IJavaElement javaElement = childs[j];
						if (javaElement instanceof IPackageFragment) {
							IPackageFragment frag = (IPackageFragment)javaElement;
							ICompilationUnit[] units = frag.getCompilationUnits();
							for (int k = 0; k < units.length; k++) {
								ICompilationUnit compilationUnit = units[k];
								cuList.add(compilationUnit);
							}
						}
						else if (javaElement instanceof ICompilationUnit) {
							cuList.add((ICompilationUnit)javaElement);
						}
					}	
				}
			} catch (JavaModelException e) {
				String message = e.getMessage(); 
				ILog logger = LoggingPlugin.getDefault().getLog();
				String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
				logger.log(new Status(IStatus.ERROR,symName,0,message,e));
				throw new RuntimeException(e);
			}
		}
		return cuList;
	}
					
	/**
	 * helper method to get the implementation of the 
	 * given compilation unit. 
	 * 
	 * @param unit the compilation unit 
	 * @return the implementation string of the compilation unit
	 */
	public static String getImplementation(ICompilationUnit unit) {		
		String implementation = null;
		IJavaElement parent = unit.getParent();
		if (parent instanceof IPackageFragment) {
			IPackageFragment packageFragment = (IPackageFragment)parent;
			String packageName = packageFragment.getElementName();
			
			implementation = packagename(unit.getElementName());
			if (packageName != null && !packageName.equals("")) {
				implementation = packageName + "." + packagename(unit.getElementName());	
			}
		}
		else if(parent instanceof IPackageFragmentRoot) {
			IPackageFragmentRoot packageFragmentRoot = (IPackageFragmentRoot)parent;
			String packageName = packageFragmentRoot.getElementName();
			
			implementation = packagename(unit.getElementName());
			if (packageName != null && !packageName.equals("")) {
				implementation = packageName + "." + packagename(unit.getElementName());	
			}
		}
		return implementation;
	}
	
	/**
	 * Helper method to remove the source code for given environment entry.
	 * The source code means the member variable, setter and getter methods.
	 * 
	 * @param editPart the EditPart of the model containing the 
	 * 					environment entry
	 * @param envEntry the environment entry whichs source code should be deleted
	 */
	public static void removeEnvEntrySource(EditPart editPart, EnvEntry envEntry) {
		// get the implementation value
		String implementation = PropertyUtil.getImplementation(editPart);
		if (implementation == null || implementation.equals("")) return;
			
		// get the compilation unit
		ICompilationUnit target = null;
		List<ICompilationUnit> units = getICompilationUnits();
		for (Iterator<ICompilationUnit> iterator = units.iterator(); iterator.hasNext();) {
			ICompilationUnit unit = (ICompilationUnit) iterator.next();
			if (getImplementation(unit).equals(implementation)) {
				// the compilation unit has been found
				// check if the member variable is available
				FieldDeclarationParser fieldParser = new FieldDeclarationParser();
				FieldDeclaration fDecl = fieldParser.parseFieldDeclarations(unit, envEntry);
				
				// if the member variable has been found, we set it as target
				if (fDecl != null)
					target = unit;
			}
		}
		
		// return if no source is available
		if (target == null) return;
		
		// ask if source code should be deleted
		boolean answer = MessageDialog.openQuestion(ActionUtil.getShell(),
	             "Delete source code", "also delete source code ?");
	    
		// if the user has accepted and a underlying java source file is available
		// delete the environment entry source
		if (answer) {
			RefactorInfo info = new RefactorInfo();
			info.setUnit(target);
			info.setNewVarName(envEntry.getName());
			info.setNewName(envEntry.getType());
			
			RemoveFieldDeclaration removeField = new RemoveFieldDeclaration();
			removeField.removeFieldDeclarations(info);
	    }
	}
	
	/**
	 * Helper method to extract the packagename
	 * of the given full qualified class name.
	 * 
	 * @param className the full qualified class name
	 * @return the package of the full qualified class name
	 */
	public static String packagename(String className) {
		int i = className.lastIndexOf('.');
        if (i > -1)
            return className.substring(0, i);
        else
            return null; 
	}
	
	/**
	 * Helper method to extract the class name
	 * of the given full qualified class name.
	 * 
	 * @param className the full qualified class name
	 * @return the class name of the full qualified name
	 */
	public static String classname(String className) {
		int i = className.lastIndexOf('.');
		if (i > -1)
			return className.substring(i + 1);
		else
			return className;
	}
	
	/**
	 * Helper method to get the class name with the 
	 * java extension.
	 * 
	 * @param className the class name
	 * @return the class file name
	 */
	public static String classFileName(String className) {
		return classname(className) + ".java";
	}
		
	/**
	 * Helper method to get the class path with the '/' notation.
	 * 
	 * @param packageName the full qualified class name with '.' notation.
	 * @return
	 */
	public static String packageStructure(String packageName) {
		return packagename(packageName).replace('.', '/') + '/';
	}
}
