/**
 * 
 */
package gui.actions;

import gui.section.IEnvEntryViewer;
import gui.wizards.RenameTypeWizard;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import logging.LoggingPlugin;
import model.ModelUtil;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ltk.core.refactoring.participants.RefactoringProcessor;
import org.eclipse.ltk.ui.refactoring.RefactoringWizardOpenOperation;

import core.EnvEntryProcessor;
import core.FieldDeclarationParser;
import core.RefactorInfo;
import core.RenameProcessorHolder;
import core.Util;
import diagram.section.EnvEntry;

/**
 * This action will be called from the Environment Entry
 * property sheet page. It startes the refactor type 
 * process, which refactors a variable declaration type. 
 * It openes a refactoring wizard and adapts the changes 
 * to the source and the model. This currently only workes 
 * when the file in the workspace is selected.
 * 
 * @author sh
 */
public class RefactorEnvEntryAction extends Action {
	
	// the selected EditPart
	private EditPart selectedEditPart = null;
	
	// the selected environment entry
	private EnvEntry envEntry = null;
	
	// the Info Object to refacter
	private RefactorInfo info = new RefactorInfo();
	
	private Log log = LogFactory.getLog(getClass());
	
	// FIXME
	// the viewer set is needed to execute the table model update
	// after an environment entry has been changed. It will be
	// transmitted from EnvEntryList, to RefactorEnvEntryAction,
	// to EnvEntryProcessor and finally to EnvEntryDelegate where the
	// update will be performed.
	private Set<IEnvEntryViewer> changeListeners = null;
	
	/**
	 * Constructor
	 * 
	 * @param selectedEditPart the selected EditPart
	 * @param envEntry the selected environment entry
	 */
	public RefactorEnvEntryAction(EditPart selectedEditPart, EnvEntry envEntry, 
			Set<IEnvEntryViewer> changeListeners) {
		super();
		this.selectedEditPart = selectedEditPart;
		this.envEntry = envEntry;
		this.changeListeners = changeListeners;
	}

	/* 
	 * Starts the action. Before the refactorig wizard appears we check
	 * if a java class exists for the selected model element.
	 * If so we check if the selected environment entry exists.
	 * If so the source code refactoring will be performed, otherwise
	 * the environment entry source variable will be created. 
	 * 
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	@Override
	public void run() {
		super.run();
		
		// check if the context is ok
		if (selectedEditPart == null || envEntry == null) {
			refuse();
		}
		else {
			// apply the values of the current environment entry
			applySelection();
			
			// check if source code is available
			ICompilationUnit unit = getCompilationUnit();
			if (unit == null) {
				noSourceCodeMessage();
				return;
			}
			else {
				// add the source code file to the info object
				info.setUnit(unit);
			}
			
			// check if the environment entry already exist
			FieldDeclaration fDecl = new FieldDeclarationParser().parseFieldDeclarations(unit, envEntry);
			info.setElement(fDecl);
			
			if (ActionUtil.saveAll()) {
			    openWizard();
			}
		}    
	}
	
	/**
	 * helper method which is called if the current
	 * selection cannot be used for that refactoring.
	 */
	private void refuse() {
		String title = "Environment Entry Refactoring";
		String message = "This refactoring is only available on Java field properties.";
		MessageDialog.openInformation(ActionUtil.getShell(), title, message);
	}
	
	/**
	 * helper method wich is called if no underlying java source
	 * file has been found for the model element.
	 */
	private void noSourceCodeMessage() {
		String title = "Environment Entry Refactoring";
		String message = "No concerning Java source file available.";
		MessageDialog.openInformation(ActionUtil.getShell(), title, message);
	}
	
	/**
	 * helper method which applies the selection 
	 * and stores it in the info object.
	 */
	private void applySelection() {
		info.setNewName(envEntry.getType());
		info.setOldName(envEntry.getType());
		info.setNewVarName(envEntry.getName());
		info.setOldVarName(envEntry.getName());
	}
	
	/**
	 * helper method which searchs the compilation
	 * unit for the selected EditPart.
	 * 
	 * @return unit the compilation unit
	 */
	private ICompilationUnit getCompilationUnit() {
		// get the implementation statement of the EditPart
		String epImpl = ModelUtil.getModelImplProperty(selectedEditPart);
		if (epImpl == null) return null;
		
		// get all compilation units
		List<ICompilationUnit> units = Util.getICompilationUnits();
		for (Iterator<ICompilationUnit> iterator = units.iterator(); iterator.hasNext();) {
			ICompilationUnit unit = (ICompilationUnit) iterator.next();
			String sourceImpl = Util.getImplementation(unit);
			
			// check the concerning cu has been found
			if (epImpl.equals(sourceImpl)) return unit;
		}
		return null;
	}
	
	/**
	 * helper method for opening the type refactoring wizard.
	 */
	private void openWizard() {		 
		RefactoringProcessor processor = new EnvEntryProcessor(info, envEntry, changeListeners);
		RenameProcessorHolder ref = new RenameProcessorHolder(processor);
		RenameTypeWizard wizard = new RenameTypeWizard(ref, info);
		RefactoringWizardOpenOperation op = new RefactoringWizardOpenOperation(wizard);
		try {
		    String titleForFailedChecks = ""; 
		    op.run(ActionUtil.getShell(), titleForFailedChecks);
		} 
		catch(final InterruptedException irex) {
			String message = "operation was canceled";
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,irex));
			throw new RuntimeException(irex + message);
		}
	}
}