/**
 * 
 */
package gui.participants;

import logging.LoggingPlugin;
import model.ModelModifier;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.internal.core.PackageFragment;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.RenameArguments;

import core.Util;

/**
 * This change class is responsible for updating the implementation
 * property in the process models. It checks which model
 * element has the same implementation statement. If so
 * the new string will be set. It also returns the 
 * undo operation for this modification.
 *
 * @author sh
 */
public class CUModelChange extends Change {
	// the selected element to refacter
	private Object selection = null;
	
	// the input of the refactoring dialog
	private RenameArguments arguments = null;
	
	private Log log = LogFactory.getLog(getClass());
	
	/**
	 * Constructor
	 * 
	 * @param selection the element to refacter
	 * @param arguments the input data of the refactoring dialog
	 */	
	public CUModelChange(Object selection, RenameArguments arguments) {
		super();
		this.selection = selection;
		this.arguments = arguments;
	}

	/**
	 * Returns the element modified by this <code>Change</code>. The method may return 
	 * <code>null</code> if the change isn't related to an element.
	 * 
	 * @return the element modified by this change
	 */
	@Override
	public Object getModifiedElement() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#getName()
	 */
	@Override
	public String getName() {
		return new String("Implementation change");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#initializeValidationData(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void initializeValidationData(IProgressMonitor pm) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#isValid(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public RefactoringStatus isValid(IProgressMonitor pm) throws CoreException,
			OperationCanceledException {
		
		// if an fatal error will return, the refactoring will stoped
		RefactoringStatus result = new RefactoringStatus();
		
		// TODO: check if some model could be read
		
		return result;
	}

	/**
	 * Performs this change after the method isValid has been called. If isValid
	 * returns an fatal error or the change is disabled this change will not be performed. 
	 * Changes should in general not respond to {@link IProgressMonitor#isCanceled()} 
	 * since canceling a change tree in the middle of its execution leaves the workspace 
	 * in a half changed state.   
	 * 
	 * @param pm a progress monitor
	 * @return the undo change for this change object or <code>null</code> if no
	 *  undo is provided
	 * @throws CoreException if an error occurred during change execution
	 */
	@Override
	public Change perform(IProgressMonitor pm) throws CoreException {
		// performs the model modification
				
		// get new implementation 
		String newImplementation = getNewImplementation();
		
		// format old implementation
		String oldImplementation = getOldImplementation();
				
		// get the new class name
		//String newClassName = Util.packagename(this.arguments.getNewName());
		
		// check if values are valid
		if (!valuesValid(newImplementation, oldImplementation)) {
			String message = "Model Update could't be done";						 
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,null));
			throw new RuntimeException(message);
		}
		
		// start the model update
		new ModelModifier().doImplementationModification(oldImplementation, newImplementation);
		log.info("Model: Implementation change performed");
		
		// for the transaction handling the undo change element is returned
		// which reverts the implementation property change to the original state
		String undoOldImpl = newImplementation;
		String undoNewImpl = oldImplementation;
		return new CUUndoModelChange(undoOldImpl,undoNewImpl);
	}
	
	/**
	 * check if the given values are valid, means they are initialized and
	 * the string is not empty.
	 * 
	 * @param newImpl the full qualified name of a compilation unit
	 * @param oldImpl the full qualified name of a compilation unit
	 * 
	 * @return true if all values pass the check
	 */
	private boolean valuesValid(String newImpl, String oldImpl) {
		boolean valid = false;
		
		if (newImpl != null) 
			if (!newImpl.equals("")) valid = true; 
			
		if (oldImpl != null) {
			if (!oldImpl.equals("")) valid = true; 
		}
				
		return valid;
	}
	
	/**
	 * helper method to get the new full qualified
	 * name of the compilation unit.
	 * 
	 * @return the full qualified name of the 
	 * 		compilation unit or null of no was found 
	 */
	private String getNewImplementation() {
		// get new implementation 
		String newUnitName = this.arguments.getNewName();	
		// it is possible to assume the selection is a CompilationUnit
		ICompilationUnit u = (ICompilationUnit)this.selection;
				
		// get the package
		PackageFragment packageFrag = ((PackageFragment)u.getParent());
		String packageName = null;
		if (packageFrag != null && !packageFrag.equals("")) { 
		    packageName = packageFrag.getElementName();
		}   
		   
		// get new implementation 
		newUnitName = this.arguments.getNewName();	
		String classname = Util.packagename(newUnitName);
		
		if (!packageName.equals(""))
			packageName += ".";
			
		// return the new implementation
		return packageName + classname;
	}
	
	/**
	 * helper method to get the old full qualified
	 * name of the compilation unit.
	 * 
	 * @return the old full qualified name of the 
	 * 		compilation unit or null of no was found
	 */
	private String getOldImplementation() {
		String oldUnitName = ((ICompilationUnit)this.selection).getElementName(); 
		String oldClassName = Util.packagename(oldUnitName);
		String newClassName = Util.packagename(this.arguments.getNewName());
		return getNewImplementation().replace(newClassName, oldClassName);
	}
	
		
	/**
	 * The undo change class which is used by the CUModelChange participant.
	 * If a model modification will be undone, an instance of that class
	 * will be returned to set the original state.
	 * 
	 * @author sh
	 *
	 */
	private class CUUndoModelChange extends Change {
		// the old implementation value to change
		private String oldImplementation = null;
		// the new implementation value to set
		private String newImplementation = null;
		
		public Object getModifiedElement() { return null; }
		public String getName() { return "Undo implementation change"; }
		public void initializeValidationData(IProgressMonitor pm) {}
		
		public CUUndoModelChange(String oldImplementation,
				String newImplementation) {
			super();
			this.oldImplementation = oldImplementation;
			this.newImplementation = newImplementation;
		}

		/**
		 * check if properties are set.
		 */
		@Override
		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			
			if (this.oldImplementation == null ||
				this.newImplementation == null) {
				String message = "unable to undo the model implementation modification";
				ILog logger = LoggingPlugin.getDefault().getLog();
				String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
				logger.log(new Status(IStatus.ERROR,symName,0,message,null));
				throw new RuntimeException(message);
			}	
			return new RefactoringStatus();
		}

		/**
		 * perform the undo operation
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			new ModelModifier().doImplementationModification(oldImplementation, newImplementation);
			log.info("Model: Implementation change undone");
			return null;
		}
	}
}