/**
 * 
 */
package gui.participants;

import logging.LoggingPlugin;
import model.ModelModifier;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.internal.core.SourceField;
import org.eclipse.jdt.internal.core.SourceType;
import org.eclipse.ltk.core.refactoring.Change;
import org.eclipse.ltk.core.refactoring.RefactoringStatus;
import org.eclipse.ltk.core.refactoring.participants.RenameArguments;


/**
 * This change class is responsible for updating the environment
 * entries in the model. It checks if the type containing
 * this field is an element in the model. If so it checks
 * if the element contains this field as environment entry.
 * It also returns the undo operations for this modification.
 * 
 * @author sh
 *
 */
public class FieldModelChange extends Change {
	// the selected element to refacter
	private Object selection = null;
	
	// the input of the refactoring dialog
	private RenameArguments arguments = null;
	
	private Log log = LogFactory.getLog(getClass());
	
	/**
	 * Constructor
	 * 
	 * @param selection the element to refacter
	 * @param arguments the input data of the refactoring dialog
	 */
	public FieldModelChange(Object selection, RenameArguments arguments) {
		super();
		this.selection = selection;
		this.arguments = arguments;
	}

	/**
	 * Returns the element modified by this <code>Change</code>. The method may return 
	 * <code>null</code> if the change isn't related to an element.
	 * 
	 * @return the element modified by this change
	 */
	@Override
	public Object getModifiedElement() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#getName()
	 */
	@Override
	public String getName() {
		return new String("Environment Entry change");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#initializeValidationData(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void initializeValidationData(IProgressMonitor pm) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ltk.core.refactoring.Change#isValid(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public RefactoringStatus isValid(IProgressMonitor pm) throws CoreException,
			OperationCanceledException {
		
		// if an fatal error will return, the refactoring will stoped
		RefactoringStatus result = new RefactoringStatus();
		
		// TODO: check if some model could be read
		
		return result;
	}

	/**
	 * Performs this change after the method isValid has been called. If isValid
	 * returns an fatal error or the change is disabled this change will not be performed. 
	 * Changes should in general not respond to {@link IProgressMonitor#isCanceled()} 
	 * since canceling a change tree in the middle of its execution leaves the workspace 
	 * in a half changed state.   
	 * 
	 * @param pm a progress monitor
	 * @return the undo change for this change object or <code>null</code> if no
	 *  undo is provided
	 * @throws CoreException if an error occurred during change execution
	 */
	@Override
	public Change perform(IProgressMonitor pm) throws CoreException {
		Object sel = this.selection;
		RenameArguments args = this.arguments;
		
		// get the fqn of the type
		SourceField field = (SourceField)this.selection;
		SourceType jElement = (SourceType)field.getParent();
		String implementation = jElement.getFullyQualifiedName();

		// if field is in subclass e.g. testpackage.p8.Anybody$foo
		// replace the '$'
		implementation = implementation.replace('$', '.');
		
		// get the old field name
		String oldEntryName = field.getElementName();
		
		// get the new field name
		String newEntryName = ((RenameArguments)this.arguments).getNewName();
		
		// check if values are valid
		if (!valuesValid(implementation, oldEntryName, newEntryName)) {
			String message = "Model Update could't be done !";
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,null));
			throw new RuntimeException(message);
		}
		
		// start the model update
		new ModelModifier().doEnvEntryNameModification(implementation, oldEntryName, newEntryName);
		log.info("Model: Environment Entry name change performed");
		
		// for the transaction handling the undo change element is returned
		// which reverts the environment entry name property change to the original state
		String undoOldImpl = newEntryName;
		String undoNewImpl = oldEntryName;
		return new FieldUndoModelChange(implementation,undoOldImpl,undoNewImpl);
	}
	
	/**
	 * check if the given values are valid, means they are initialized and
	 * the string is not empty.
	 * 
	 * @param implementation the full qualified name of a compilation unit
	 * @param oldEntryName an environment entry name
	 * @param newEntryName an environment entry name
	 * 
	 * @return true if all values pass the check
	 */
	private boolean valuesValid(String implementation, String oldEntryName, String newEntryName) {
		boolean valid = false;
		
		if (implementation != null) 
			if (!implementation.equals("")) valid = true; 
			
		if (oldEntryName != null) {
			if (!oldEntryName.equals("")) valid = true; 
		}
		
		if (newEntryName != null) {
			if (!newEntryName.equals("")) valid = true; 
		}
		
		return valid;
	}
	
	/**
	 * The undo change class which is used by the FieldModelChange participant.
	 * If a model modification will be undone, an instance of that class
	 * will be returned to set the original state.
	 * 
	 * @author sh
	 *
	 */
	private class FieldUndoModelChange extends Change {
		// the implementation of the model element containing the environment entry
		private String implementation = null;
		// the old environment entry name
		private String oldEntryName = null;
		// the new environment entry name
		private String newEntryName = null;
		
		public Object getModifiedElement() { return null; }
		public String getName() { return "Undo Environment Entry change"; }
		public void initializeValidationData(IProgressMonitor pm) {}
		
		public FieldUndoModelChange(String implementation,
				String oldEntryName, String newEntryName) {
			super();
			this.implementation = implementation;
			this.oldEntryName = oldEntryName;
			this.newEntryName = newEntryName;
		}
	
		/**
		 * check if properties are set.
		 */
		@Override
		public RefactoringStatus isValid(IProgressMonitor pm)
				throws CoreException, OperationCanceledException {
			
			if (this.implementation == null ||
				this.oldEntryName == null ||
				this.newEntryName == null) {
				String message = "unable to undo the model environment entry name modification";
				ILog logger = LoggingPlugin.getDefault().getLog();
				String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
				logger.log(new Status(IStatus.ERROR,symName,0,message,null));
				throw new RuntimeException(message);
			}
			return new RefactoringStatus();
		}

		/**
		 * perform the undo operation
		 */
		@Override
		public Change perform(IProgressMonitor pm) throws CoreException {
			new ModelModifier().doEnvEntryNameModification(implementation, oldEntryName, newEntryName);
			log.info("Model: Environment Entry name change undone");
			return null;
		}
	}
}
