

import java.lang.reflect.InvocationTargetException;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.TextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;

import refactor.RenameRefactoring;
import gui.actions.ActionUtil;

/**
 * Our rename action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class RenameAction implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	
	// the Info Object to refacter
	private Object selection = null;
	private String title;
	private String initialValue;
	
	/**
	 * The constructor.
	 */
	public RenameAction() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @throws InterruptedException 
	 * @throws InvocationTargetException 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	public void run(IAction action) {	
		// create the input dialog and set the input values
		InputDialog inputDia = createInputDialog();
		
		// if no input data were specified, open an info dialog
		if (inputDia == null) {
			showInfoDialog();
			return;
		}
			
		// open the dialog
		if(inputDia.open() == Window.OK) {
		   String newName = inputDia.getValue();
		   
		   // start the rename refactoring
		   new RenameRefactoring().startRenameRefactoring(selection, newName);
		}
	}
		
	/**
	 * creates and initializes an Input Dialog
	 * @return a new InputDialog
	 */
	private InputDialog createInputDialog() {
		if(selection == null) return null;
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		setGUIValues();
		return new InputDialog(shell, this.title, 
				   "Rename the selected Resource:", this.initialValue, new InputValidator(this.selection));
	}

	/**
	 * sets the values such as initialValue, titel for the input dialog
	 */
	private void setGUIValues() {
		// if the selection is a IPackageFragment (Java Package)
		if(selection instanceof IPackageFragment) {
		   this.title = "Rename Package";
		   this.initialValue = ((IPackageFragment)selection).getElementName();
		}
		
		// if the selection is a ICompilationUnit (Java Class File)
		if(selection instanceof ICompilationUnit) {
		   this.title = "Rename Compilation Unit";
		   this.initialValue = ((ICompilationUnit)selection).getElementName();
		}
		   
		// if the selection is a IType (Java Casse)
		if(selection instanceof IType) {
		   this.title = "Rename Type";
		   this.initialValue = ((IType)selection).getElementName();
		}
		   
		// if the selection is a IField (Java Variable)
		if(selection instanceof IField) {
		   this.title = "Rename Field";
		   this.initialValue = ((IField)selection).getElementName();
		}
		   
		// if the selection is a IMethod (Java Method)
		if(selection instanceof IMethod) {
		   this.title = "Rename Method";
		   this.initialValue = ((IMethod)selection).getElementName();
		}
		
		// if the selection is a FieldDeclaration,
		if(selection instanceof FieldDeclaration) {
		   this.title = "Rename Type";
		   this.initialValue = ((FieldDeclaration)selection).getType().toString();
		}
	}
	
	private void showInfoDialog() {
		Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		MessageDialog messageDia = new MessageDialog(
				shell,
				"Rename",
				null,
				"Operation unavailable on the current selection",
				MessageDialog.INFORMATION,
				new String[] {"OK"}, 1);
		
	    messageDia.open();
		
	}
	
	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {				
		// if a Resource in the Explorer has been selected
		if(selection instanceof TreeSelection) 
		   this.selection = ((TreeSelection)selection).getFirstElement();   
		  
		// if a text in an editor has been selected
		else if(selection instanceof TextSelection) {
			// check if the selection exists somewhere in the compilation units
			// if a field declaration was found for the selection a map containing
			// the field declaration and its compilation unit will be returned and
			// assigned to the selection object.
			this.selection = ActionUtil.scanForSimpleType((TextSelection)selection);	
		}
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
}