/**
 * 
 */
package model;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import logging.LoggingPlugin;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.openarchitectureware.emf.XmiReader;
import org.openarchitectureware.workflow.WorkflowContext;
import org.openarchitectureware.workflow.WorkflowContextDefaultImpl;
import org.openarchitectureware.workflow.issues.Issues;
import org.openarchitectureware.workflow.issues.IssuesImpl;
import org.openarchitectureware.workflow.monitor.NullProgressMonitor;


/**
 * @author sh
 *
 */
public class ModelEditor {
		
	// singleton
	public static final ModelEditor INSTANCE = new ModelEditor();
	
	private Issues issues = null;
		
	/**
	 * Read the Process Metamodel by a XMI Reader
	 * 
	 * @return model the Ecore model
	 * @throws Exception
	 */
	public List<Map<WorkflowContext,String>> loadModels() {
		// initialize the issues object which stores errors and warnings
		// of the load process
		this.issues = new IssuesImpl();
		
		// 1. search for the Meta Model File
		List<String> metaModelFile = getMetaModelFiles();
						
		// 2. search for the model File(s)
		List<String> modelFiles = getModelFiles();
					
		// 3. check if model files and meta model files were found
		if (metaModelFile == null || modelFiles == null) return null;
		if (metaModelFile.isEmpty() || modelFiles.isEmpty()) return null;
		
		// 4. read the object trees from the models
		return readModels(metaModelFile, modelFiles);
	}
	
	/**
	 * Searches for meta model files in the workspace.
	 * For the process editor these files ends with ".ecore".
	 * Only opened projects are considered.
	 * 
	 * @return metaModelFile a list containing all found
	 * 			meta model files
	 */
	private List<String> getMetaModelFiles() {
		List<String> metaModelFile = ModelUtil.getMetaModelFiles();
		if(metaModelFile == null || metaModelFile.isEmpty()) {
		   String message = "No Meta Model File (*.ecore) has been found !";
		   issues.addError(message);
		   ILog logger = LoggingPlugin.getDefault().getLog();
		   String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
		   logger.log(new Status(IStatus.ERROR,symName,0,message,null));
		   throw new RuntimeException(message);	
		}
		return metaModelFile;
	}
	
	/**
	 * Searches for model files in the workspace.
	 * For the process editor these files ends with "concept".
	 * Only opened projects are considered.
	 *  
	 * @return modelFiles a list containing all found
	 * 			model files
	 */
	private List<String> getModelFiles() {
		List<String> modelFiles = ModelUtil.getModelFiles();
		if(modelFiles == null || modelFiles.isEmpty()) {
		   String message = "No Process Model Files to refactor have been found !";
		   issues.addError(message);
		   ILog logger = LoggingPlugin.getDefault().getLog();
		   String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
		   logger.log(new Status(IStatus.ERROR,symName,0,message,null));
		   throw new RuntimeException(message);		
		}
		return modelFiles;
	}
	
	/**
	 * This helper method reads the given model files an creates 
	 * the object tree for that model. The meta model file is needed
	 * to instanciate the object tree for the given models.
	 *   
	 * @param metaModelFile a list containing all meta models
	 * @param modelFiles al list containing all model files to load
	 * @return models a map containing a list consisting of object tree 
	 * 				model file pairs
	 */
	private List<Map<WorkflowContext,String>> readModels(List<String> metaModelFile, List<String> modelFiles) {
		XmiReader reader = new XmiReader();
		reader.setMetaModelFile(metaModelFile.get(0).toString());
		Map<WorkflowContext,String> contextFileMap = null;
		WorkflowContext ctx = null;
		List<Map<WorkflowContext,String>> models = new ArrayList<Map<WorkflowContext,String>>();
		for (Iterator<String> iterator = modelFiles.iterator(); iterator.hasNext();) {
			String modelFile = (String)iterator.next();
			reader.setModelFile(modelFile);
			try {
				reader.setMetaModelPackage("newprocess.NewprocessPackage");
			} catch (RuntimeException e) {
				//throw new RuntimeException(e);
				// TODO: has to be fixed because a ClassCast Exception occures
			}
			reader.setFirstElementOnly(true);
			ctx = new WorkflowContextDefaultImpl();
			reader.invoke(ctx, new NullProgressMonitor(), this.issues);
			
			// add the Workflow Context and the concerning ModelFile Path to the Collection
			contextFileMap = new HashMap<WorkflowContext,String>();
			contextFileMap.put(ctx, modelFile);
			models.add(contextFileMap);
		}
		return models;
	}
	

	/**
	 * Write the edited Process Metamodel by a XMI Writer
	 * 
	 * @param ctx the Workflow Context containing the model
	 */
	public void writeModel(List<Map<WorkflowContext,String>> binding) {
		for (Iterator<Map<WorkflowContext,String>> iterator = binding.iterator(); iterator.hasNext();) {
			Map<WorkflowContext,String> ctxModelMap = iterator.next();
						
			for(Iterator<WorkflowContext> contextIter = ctxModelMap.keySet().iterator(); contextIter.hasNext();) {
				WorkflowContext ctx = (WorkflowContext) contextIter.next();
				String path = ctxModelMap.get(ctx);
							
				IPath filePath = new Path(path);
				IContainer[] root = ResourcesPlugin.getWorkspace().getRoot().findContainersForLocation(filePath);
				for (int i = 0; i < root.length; i++) {
					IContainer container = root[i];
					filePath = container.getFullPath();
				}	
				
				Writer writer = new Writer();
				writer.setModelSlot("default");
				URI url = URI.createPlatformResourceURI(filePath.toString(),true);
				writer.setUri(url.toString());
				ResourceSet rs = new ResourceSetImpl();
				writer.setResourceSet(rs);
				new StandaloneSetup().setPlatformUri(new File("..").getAbsolutePath());
				this.issues = new IssuesImpl();
				writer.invoke(ctx, new NullProgressMonitor(), this.issues);
			}
		}
	}
	
	/**
	 * getter for the issues property.
	 * 
	 * @return issues
	 */
	public Issues getIssues() {
		return this.issues;
	}
}
