/**
 * 
 */
package model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import logging.LoggingPlugin;
import newprocess.Element;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.Node;

/**
 * @author sh
 *
 */
public class ModelUtil {
		
	/**
	 * Helper method to return the locations of the given files.
	 * 
	 * @param files a list of files
	 * @return modelList a list containing the location of all files
	 */
	private static List<String> getModelFilePaths(List<IFile> files) {
		List<String> modelList = new ArrayList<String>();
		for (Iterator<IFile> iterator = files.iterator(); iterator.hasNext();) {
			IFile file = (IFile) iterator.next();
			String path = file.getLocation().toString();
			modelList.add(path);
		}
		return modelList;
	}
	
	/**
	 * get all Meta Model Files (*.ecore) in the workspace.
	 * Just opened projects are considered.
	 * 
	 * @return modelList a list containing all Meta Model Files of the workspace 
	 */
	public static List<String> getMetaModelFiles() {
		List<IFile> modelFiles = scanWorkspace("ecore");
		return getModelFilePaths(modelFiles);
	}
	
	/**
	 * Scans the whole workspace for files with the given file extension.
	 * Just opened projects are considered.
	 * 
	 * @param fileExtension the file ending of the wanted files
	 * @return matches a list containing all found files
	 */
	public static List<IFile> scanWorkspace(String fileExtension) {
		try { 
			List<IFile> matches = new ArrayList<IFile>();
			
			// scan the whole workspace for opened projects 
			IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
			for (IProject project : projects) {
				
				// just search if if the current project is open
				if(!project.isAccessible()) continue;
				
				// get the content of the current project and scan for the wanted files
				IResource[] members = project.members();
				for(IResource resource : members) {
					
				    if(resource instanceof IFile) {
					   if(((IFile)resource).getFileExtension() != null &&
						  ((IFile)resource).getFileExtension().equals(fileExtension)) {   
						  matches.add((IFile)resource);
					   }
					}
				    
				    // scan recursivly all subfolders
					else if(resource instanceof IFolder)
						matches.addAll(search(((IFolder)resource), fileExtension));
				}
			}
			
			return matches;
		} catch (IllegalStateException e1) {
			String message = " You are not running under Eclipse but " +
							"instead under just a standard java application";
			ILog logger = LoggingPlugin.getDefault().getLog();
			String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
			logger.log(new Status(IStatus.ERROR,symName,0,message,e1));
			throw new RuntimeException(message, e1);
			// TODO: finding the Meta Model without running under Eclipse.
		} catch (CoreException e) {
		    return null;
		} 
	}
	
	/**
	 * Searches for files with the given file extension starting at
	 * the given folder.
	 * 
	 * @param folder the folder to start
	 * @param ending the file extension of the file to search
	 * @return matches a list containing all found files
	 */
	public static List<IFile> search(IFolder folder, String ending) {
		List<IFile> matches = new ArrayList<IFile>();
		if(folder != null) {
			IResource[] members;
			try {
				members = folder.members();
			} catch (CoreException e) {
				String message = e.getMessage(); 
				ILog logger = LoggingPlugin.getDefault().getLog();
				String symName = LoggingPlugin.getDefault().getBundle().getSymbolicName();
				logger.log(new Status(IStatus.ERROR,symName,0,message,e));
				throw new RuntimeException(e);
			}
			
			for (int i = 0; i < members.length; i++) {
				IResource resource = members[i];
				if(resource instanceof IFile) {
					if (((IFile)resource).getFileExtension() != null &&
						((IFile)resource).getFileExtension().equals(ending)) {
						matches.add((IFile)resource);
					}
				}
				else if(resource instanceof IFolder) {
					matches.addAll(search((IFolder)resource, ending));
				}
			}
		}
		return matches;
	}
	
	/**
	 * get all Model Files (*.concept) in the workspace.
	 * Just opened projects are considered.
	 * 
	 * @return modelList a list containing all Model Files of the workspace
	 */
	public static List<String> getModelFiles() {
		List<IFile> modelFiles = scanWorkspace("concept");
		return getModelFilePaths(modelFiles);
	}
	
	/**
	 * get the implementation property of a model element.
	 * 
	 * @param editPart the EditPart containing the 
	 * 				implementation property
	 * @return implementation the implementation property
	 */
	public static String getModelImplProperty(EditPart editPart) {
		String implementation = null;
		if (editPart instanceof DiagramEditPart) 
			implementation = ((newprocess.Process)((Diagram)editPart.getModel()).getElement()).getImplementation();
		else if (editPart instanceof ShapeNodeEditPart) 
			implementation = ((Element)((Node)editPart.getModel()).getElement()).getImplementation();
		return implementation;
	}
}
