/*
 * $Id: BSFCondition.java,v 1.5 2006/03/10 22:47:14 hengels Exp $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.library.bsf;

import org.apache.bsf.BSFException;
import org.apache.bsf.BSFManager;
import org.concern.controller.AbstractCondition;
import org.concern.controller.ConditionEvaluationException;

import java.util.*;

/**
 * Allows any BSF script to be used as a con:cern condition.  Two environment
 * variables must be set in order for this to concern: language and script.
 * "language" should contain the BSF name for the scripting language of the
 * script, and "script" should contain the script itself.  As a convenience,
 * you may set the "default-script-language" environment variable at the
 * process level if you plan on using the same scripting language throughout
 * your process.
 * <p/>
 * This implementation assumes that you have already configured BSFManager
 * with any additional languages you need.  This implementation will declare
 * the following additional beans before executing the script:
 * <ul>
 * <li>subject = the subject instance, as passed into the condition</li>
 * <li>condition = the condition instance</li>
 * <li>environment = the java.util.Map representing the environment for
 * this condition.</li>
 * <li>controller = the controller instance</li>
 * </ul>
 * For most scripting languages, these beans will be made available for
 * direct use.  For example (in groovy):
 * <pre><code>  subject.approval == subject.ACCEPTED</code></pre>
 * <p/>
 * Note: for those using the Groovy scripting language, testing has shown
 * that Groovy's current implementation of the BSF scripting engine
 * (org.codehaus.groovy.bsf.GroovyEngine) is <i>extremely slow</i>,
 * probably because it is parsing the script for every invocation (by slow,
 * we mean that we have observed 500-1100 ms execution time <i>per
 * invocation</i> of a condition).  We've found that a good workaround is to
 * use Groovy's caching BSF scripting engine
 * (org.codehaus.groovy.bsf.CachingGroovyEngine).  Obviously, since
 * this engine caches based on the script String, it will consume more
 * memory.  However, Con:cern processes tend to use a fairly static set of
 * small script Strings, which fits well with the caching strategy, bringing
 * memory consumption in line with typical expectations.  Here is an example
 * configuration:
 * <pre><code>
 * BSFManager.registerScriptingEngine("groovy",
 *                                    "org.codehaus.groovy.bsf.CachingGroovyEngine",
 *                                    new String[] { "groovy", "gy" });
 * </code></pre>
 *
 * @author Andy DePue
 */
public class BSFCondition
    extends AbstractCondition
{
    public static final String ENV_KEY_LANGUAGE = "language";
    public static final String ENV_KEY_SCRIPT = "script";
    public static final String PROCESS_ENV_KEY_LANGUAGE = "default-script-language";

    private BSFManager bsfManager;
    private String sourceName;
    private String language;
    private String script;

    public BSFManager getBsfManager() {
        if (this.bsfManager == null) {
            setBsfManager(createBsfManager());
        }
        return this.bsfManager;
    }

    public void setBsfManager(final BSFManager bsfManager) {
        this.bsfManager = bsfManager;
    }

    protected BSFManager createBsfManager() {
        return new BSFManager();
    }

    public String getLanguage() {
        if (this.language == null) {
            if ((this.language = (String)getEnvironment().get(ENV_KEY_LANGUAGE)) == null) {
                this.language = (String)controller.getProcess().getEnvironmentEntry(PROCESS_ENV_KEY_LANGUAGE);
            }
        }
        return this.language;
    }

    public void setLanguage(String language) {
        this.language = language;
    }

    public Object getScript() {
        if (script == null)
            script = (String)getEnvironment().get(ENV_KEY_SCRIPT);
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    public String getSourceName() {
        if (this.sourceName == null) {
            this.sourceName = getProcessName().trim().replace(' ', '_').replace('.', '_').replace(',', '_');
        }
        return this.sourceName;
    }


    /**
     * Subclasses can override this method to further prepare the BSFManager
     * before script execution takes place.
     *
     * @param bsfm
     * @param subject
     * @throws ConditionEvaluationException
     * @throws BSFException
     */
    protected void declareAdditionalBeans(final BSFManager bsfm, final Object subject)
        throws ConditionEvaluationException, BSFException {
    }

    //
    // METHODS FROM ABSTRACT CLASS AbstractCondition
    //

    public boolean eval(final Object subject) throws ConditionEvaluationException {
        final BSFManager bsfm = getBsfManager();
        try {
            bsfm.declareBean("subject", subject, subject.getClass());
            bsfm.declareBean("environment", getEnvironment(), Map.class);
            bsfm.declareBean("controller", this.controller, this.controller.getClass());
            bsfm.declareBean("condition", this, BSFCondition.class);
            declareAdditionalBeans(bsfm, subject);

            final Object ret = bsfm.eval(getLanguage(), getSourceName(), 0, 0, getScript());

            if (!(ret instanceof Boolean)) {
                throw new ConditionEvaluationException(getLanguage() + " script must evaluate to a boolean result (result=" + ret + ")");
            }
            return ((Boolean)ret).booleanValue();
        }
        catch (BSFException e) {
            if (e.getTargetException() != null) {
                e.initCause(e.getTargetException());
            }
            throw new ConditionEvaluationException(e);
        }
    }
}
