/*
 * $Id: Controller.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern;

import java.util.List;
import java.util.Map;

/**
 * The controller interface is the runtime client interface of con:cern. An implementation is responsible for
 * controlling the process flow and invoking the activities. It provides access to the log and the current state
 * information of the subjects in process.
 * <p/>
 * Methods, that import the transactional context:
 *  o createSubject
 *  o lockSubject
 *  o destroySubject
 *  o getSubjects(..)
 *  o complete
 *  o notify
 *  o getLog
 *  o getArchive
 *  o matchPrecondition
 *  o matchPostcondition
 *
 * @author hengels[at]mercatis[dot]de
 * @version $Revision: 881 $
 */
public interface Controller
{
    public static final int STATE_RUNNING = 0;
    public static final int STATE_STALLED = 1;
    public static final int STATE_SUSPENDED = 2;
    public static final int STATE_DESTROYED = 3;

    /**
     * A process is identified by a unique name. A separate controller instance is required for every process.
     * @return the name of the process that is controlled by this controller
     */
    String getProcessName();

    /**
     * Return the process description.
     * @return the process description
     */
    org.concern.model.Process getProcess();

    /**
     * Create a new subject in the process. The userValue is used by both, the activities and the client to identify
     * the subject. Candidates are a primary key, a distinguished name, a filename, whatever .. depending on the
     * datastore, that holds the subject's business information.
     *
     * @param userValue identifies the subject
     * @return an internal id
     * @throws org.concern.SubjectCreationException
     */
    Integer createSubject(String userValue) throws org.concern.SubjectCreationException, ControllerException;

    /**
     * Determines if {@link #createSubject(String)} has already been called
     * for the specified <code>userValue</code>.
     *
     * @param userValue identifies the subject
     * @return <code>true</code> if <code>createSubject(...)</code> has already
     *         been called for the specified <code>userValue</code>.
     * @throws ControllerException
     */ 
    boolean isKnownSubject(String userValue) throws ControllerException;  
    
    /**
     * Retrieve a list of all subjects, that are currently in the process.
     * @return a list with the userValues of all subjects
     * @param state one of STATE_RUNNING, STATE_IDLE, STATE_DESTROYED
     */
    List<String> getSubjects(int state) throws ControllerException;

    /**
     * Tell the controller, that a subjects has changed. This method has to be called in order to tell the controller,
     * that the particular subject should be examined for further execution.
     * @param userValue identifies the subject
     */
    void announceSubject(String userValue) throws UnknownSubjectException, ControllerException;

    /**
     *
     * @param userValue identifies the subject
     * @param duration in seconds
     */
    void scheduleAnnouncement(String userValue, long duration) throws UnknownSubjectException, ControllerException;

    /**
     * Append a message to the protocol of the specified subject.
     * @param log the record to be logged
     */
    void log(Log log) throws UnknownSubjectException, ControllerException;

    /**
     * Return a list of names of the asynchronous activities, for which the subject is enlisted
     * @param userValue identifies the subject
     * @return the enlistments
     * @throws org.concern.UnknownSubjectException
     */
    List<String> getEnlistments(String userValue) throws org.concern.UnknownSubjectException, ControllerException;

    /**
     * Return a list of names of the none optional asynchronous activities, for which the subject is enlisted
     * @param userValue identifies the subject
     * @return the tasks
     * @throws org.concern.UnknownSubjectException
     */
    List<String> getTasks(String userValue) throws org.concern.UnknownSubjectException, ControllerException;

    /**
     * Return a list of names of the optional asynchronous activities, for which the subject is enlisted
     * @param userValue identifies the subject
     * @return the tasks
     * @throws org.concern.UnknownSubjectException
     */
    List<String> getOptions(String userValue) throws org.concern.UnknownSubjectException, ControllerException;

    /**
     * Get the ids of the subjects, that are enlisted for the specified asynchronous actvity.
     * @param activity the activity name
     * @return a list of subject ids
     */
    List<String> getSubjects(String activity) throws ControllerException;

    /**
     * The controller logs every successful or unsuccessful execution of an activity as well as the creation, destroyal
     * and stall of subjects.
     * @param userValue identifies the subject
     * @return a list of org.concern.Log records
     * @throws org.concern.UnknownSubjectException
     */
    List<Log> getLog(String userValue) throws org.concern.UnknownSubjectException, ControllerException;

    /**
     * Subjects, that have reached been destroyed are removed from the process. Their log is archived.
     * @param userValue identifies the subject
     * @return a list of org.concern.Log records
     * @throws org.concern.ControllerException
     */
    List<Log> getArchive(String userValue) throws ControllerException, UnknownSubjectException;

    /**
     * Complete an asynchronous activtiy. The controller checks the postcondition an logs the successful completion
     * of the activity.
     * @param userValue identifies the subject
     * @param activityName the activity
     * @return true if the postcondition is matching, false otherwise
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    boolean complete(String userValue, String activityName) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Complete an asynchronous activtiy. The controller checks the postcondition and logs the successful completion
     * of the activity.  If the <code>logAnnotations</code> property is not
     * <code>null</code>, then those annotations will be stored in the log
     * entry for the completed activity.
     *
     * @param userValue identifies the subject
     * @param activityName the activity
     * @param logAnnotations may contain client specific information to be stored with the log entry that will be created for this activity.  This parameter may also be <code>null</code>.
     * @return true if the postcondition is matching, false otherwise
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    boolean complete(String userValue, String activityName, Map<String, String> logAnnotations) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Check if the subject matches the specified condition.
     * @param userValue identifies the subject
     * @param conditionName the activity
     * @return match
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    Boolean matchCondition(String userValue, String conditionName) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Check if the subject matches the precondition of the specified activity.
     * @param userValue identifies the subject
     * @param activityName the activity
     * @return match
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    boolean matchPrecondition(String userValue, String activityName) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Check if the subject matches the postcondition of the specified activity.
     * @param userValue identifies the subject
     * @param activityName the activity
     * @return match
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    boolean matchPostcondition(String userValue, String activityName) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Process the specified subject immediately. When this method returns, all actions are performed.
     * @param userValue identifies the subject
     * @throws org.concern.UnknownSubjectException
     * @throws org.concern.ControllerException
     */
    void process(String userValue) throws org.concern.UnknownSubjectException, org.concern.ControllerException;

    /**
     * Mark the subject destroyed. When it is processed next, it will be moved to the archive.
     * @param userValue identifies the subject
     * @throws UnknownSubjectException
     * @throws ControllerException
     */
    void destroySubject(String userValue) throws UnknownSubjectException, ControllerException;

    /**
     * Restore the subject from the archive into the pool of running instances.
     * @param userValue identifies the subject
     * @throws UnknownSubjectException
     * @throws ControllerException
     */
    void reviveSubject(String userValue) throws UnknownSubjectException, ControllerException;

    /**
     * Notify the controller that an event has occured.
     * @param userValue
     * @param eventName
     * @return
     * @throws UnknownSubjectException
     */
    boolean notify(String userValue, String eventName) throws UnknownSubjectException, ControllerException;

    /**
     * Notify the controller that an event has occured.
     * If the <code>logAnnotations</code> property is not
     * <code>null</code>, then those annotations will be stored in the log
     * entry for the notification.
     * 
     * @param userValue
     * @param eventName
     * @param logAnnotations may contain client specific information to be stored with the log entry that will be created for this notification.  This parameter may also be <code>null</code>.
     * @return
     * @throws UnknownSubjectException
     */
    boolean notify(String userValue, String eventName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException;

    /**
     * Write-lock the specified subject for the duration of the current transaction.
     * @param userValue
     * @throws UnknownSubjectException
     * @throws ControllerException
     */
    void lockSubject(String userValue) throws UnknownSubjectException, ControllerException;

    boolean isForwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException;

    void forward(String userValue, String activityName) throws UnknownSubjectException, ControllerException;

    void forward(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException;

    boolean isBackwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException;

    void backward(String userValue, String activityName) throws UnknownSubjectException, ControllerException;

    void backward(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException;

    void reset(String userValue, String activityName) throws UnknownSubjectException, ControllerException;

    void suspendSubject(String userValue) throws UnknownSubjectException, ControllerException;

    void resumeSubject(String userValue) throws UnknownSubjectException, ControllerException;

    /**
     * Start the controller.
     */
    void start();

    /**
     * Free all resources.
     * @throws ControllerException
     */
    void stop() throws ControllerException;

    /**
     * Reread the parameters from the parameter resolver
     */
    void reload();
}
