package org.concern.controller;

import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.ImprovedNamingStrategy;
import org.concern.ControllerException;
import org.concern.model.Activity;
import org.concern.model.Condition;
import org.concern.model.*;
import org.concern.model.Actor;
import org.concern.model.Event;
import org.concern.model.Listener;
import org.concern.model.Process;
import org.concern.model.cpd.Dispatcher;
import org.concern.model.cpd.ProcessHandler;
import org.apache.commons.logging.LogFactory;

import javax.transaction.TransactionManager;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.IOException;
import java.io.InputStream;
import java.beans.*;
import java.util.*;

/**
 * Set the URL or the process and classLoader. Optionally set the hibernateConfiguration.
 * Then call buildController().
 */
public class Configuration
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Configuration.class);

    private Process process;
    private ClassLoader classLoader;
    private TransactionManager transactionManager;
    private SessionFactory sessionFactory;
    private ResourceLocator resourceLocator;
    private ParameterResolver parameterResolver;
    private long timerInterval = 10000;

    public void setClassLoader(ClassLoader classLoader) {
        //this.classLoader = classLoader;
    }

    public void setProcess(Process process) {
        this.process = process;
    }

    public void setResourceLocator(ResourceLocator resourceLocator) {
        this.resourceLocator = resourceLocator;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public ParameterResolver getParameterResolver() {
        return parameterResolver;
    }

    public void setParameterResolver(ParameterResolver parameterResolver) {
        this.parameterResolver = parameterResolver;
    }

    protected SessionFactory getSessionFactory() throws HibernateException {
        return sessionFactory;
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    protected TransactionManager getTransactionManager() throws HibernateException {
        return transactionManager;
    }

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public Process getProcess() {
        return process;
    }

    public ResourceLocator getResourceLocator() {
        return resourceLocator;
    }

    public static Process loadProcess(InputStream config) {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try {
            SAXParser saxParser = factory.newSAXParser();
            ProcessHandler processHandler = new ProcessHandler();
            Dispatcher dispatcher = new Dispatcher(processHandler);
            saxParser.parse(config, dispatcher);
            return processHandler.getProcess();
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
        finally {
            try {
                if (config != null) config.close();
            }
            catch (IOException ignore) {}
        }
    }

    public Controller buildController() {
        try {
            Controller controller = new Controller();
            controller.setResourceLocator(resourceLocator);
            controller.setTimerInterval(timerInterval);
            controller.setSessionFactory(getSessionFactory());
            controller.setConfiguration(this);
            TransactionManager transactionManager = getTransactionManager();
            if (transactionManager != null)
                controller.setTransactionManager(transactionManager);

            initializeController(controller);
            return controller;
        }
        catch (Exception e) {
            e.printStackTrace();
            throw new ControllerException(e);
        }
    }

    public void initializeController(Controller controller) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
        controller.setProcessDescription(process);
        LOG.info("Process: " + process.getName());

        {
            Map environment = processEnvironment(process);
            controller.setEnvironment(environment);
        }

        if (process.getLoader() != null) {
            org.concern.model.Loader loaderModel = process.getLoader();
            Class loaderClass = getClassLoader().loadClass(loaderModel.getImplementation());
            org.concern.controller.Loader loader = (org.concern.controller.Loader)loaderClass.newInstance();
            loader.setController(controller);

            Map environment = loaderEnvironment(loaderModel);
            loader.setEnvironment(environment);
            configure(loader, environment);
            controller.setLoader(loader);
            LOG.info("Loader: " + loader.getName());
        }

        List actors = new LinkedList();
        for (Iterator iterator = process.getActors().iterator(); iterator.hasNext();) {
            org.concern.model.Actor actorModel = (Actor)iterator.next();
            Class actorClass = getClassLoader().loadClass(actorModel.getImplementation());
            org.concern.controller.Actor actor = (org.concern.controller.Actor)actorClass.newInstance();
            actor.setController(controller);

            Map environment = actorEnvironment(actorModel);
            actor.setEnvironment(environment);
            configure(actor, environment);
            actors.add(actor);
            LOG.info("Actor: " + actor.getName());
        }
        controller.setActors(actors);

        List conditions = new LinkedList();
        for (Iterator iterator = process.getConditions().iterator(); iterator.hasNext();) {
            org.concern.model.Condition conditionModel = (Condition)iterator.next();
            Class conditionClass = getClassLoader().loadClass(conditionModel.getImplementation());
            org.concern.controller.Condition condition = (org.concern.controller.Condition)conditionClass.newInstance();
            condition.setController(controller);

            Map environment = conditionEnvironment(conditionModel);
            condition.setEnvironment(environment);
            configure(condition, environment);
            conditions.add(condition);
            LOG.info("Condition: " + condition.getName());
        }
        controller.setConditions(conditions);

        List activities = new LinkedList();
        for (Iterator iterator = process.getActivities().iterator(); iterator.hasNext();) {
            Activity activityModel = (Activity)iterator.next();
            Class activityClass = getClassLoader().loadClass(activityModel.getImplementation());
            org.concern.controller.Activity activity = (org.concern.controller.Activity)activityClass.newInstance();
            activity.setController(controller);

            Map environment = activityEnvironment(activityModel);
            activity.setEnvironment(environment);
            configure(activity, environment);
            activities.add(activity);
            LOG.info("Activity: " + activity.getName());
        }
        controller.setActivities(activities);

        List events = new LinkedList();
        for (Iterator iterator = process.getEvents().iterator(); iterator.hasNext();) {
            Event eventModel = (Event)iterator.next();
            Class eventClass = getClassLoader().loadClass(eventModel.getImplementation());
            org.concern.controller.Event event = (org.concern.controller.Event)eventClass.newInstance();
            event.setController(controller);

            Map environment = eventEnvironment(eventModel);
            event.setEnvironment(environment);
            configure(event, environment);
            events.add(event);
            LOG.info("Event: " + event.getName());
        }
        controller.setEvents(events);

        List listeners = new LinkedList();
        for (Iterator iterator = process.getListeners().iterator(); iterator.hasNext();) {
            Listener listenerModel = (Listener)iterator.next();
            Class listenerClass = getClassLoader().loadClass(listenerModel.getImplementation());
            org.concern.controller.Listener listener = (org.concern.controller.Listener)listenerClass.newInstance();
            listener.setController(controller);

            Map environment = listenerEnvironment(listenerModel);
            listener.setEnvironment(environment);
            configure(listener, environment);
            listeners.add(listener);
            LOG.info("Listener: " + listener.getName());
        }
        controller.setListeners(listeners);
    }

    protected void configure(Node node, Map environment) {
        PropertyDescriptor[] descriptors = new PropertyDescriptor[0];
        try {
            descriptors = Introspector.getBeanInfo(node.getClass()).getPropertyDescriptors();
            for (int i = 0; i < descriptors.length; i++) {
                PropertyDescriptor descriptor = descriptors[i];
                if (descriptor.getWriteMethod() == null)
                    continue;

                Object value = environment.get(descriptor.getName());
                if (value == null)
                    continue;

                descriptor.getWriteMethod().invoke(node, new Object[] { value });
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    private Map actorEnvironment(Actor actorModel) {
        Map environment = environment(actorModel.getEnvironment());
        environment.put("actor.name", actorModel.getName());
        return environment;
    }

    private Map processEnvironment(org.concern.model.Process processModel) {
        Map environment = environment(processModel.getEnvironment());
        environment.put("process.name", processModel.getName());
        return environment;
    }

    private Map loaderEnvironment(org.concern.model.Loader loaderModel) {
        Map environment = environment(loaderModel.getEnvironment());
        environment.put("loader.name", loaderModel.getName());
        return environment;
    }

    private Map conditionEnvironment(org.concern.model.Condition conditionModel) {
        Map environment = environment(conditionModel.getEnvironment());
        environment.put("condition.name", conditionModel.getName());
        return environment;
    }

    private Map activityEnvironment(org.concern.model.Activity activityModel) {
        Map environment = environment(activityModel.getEnvironment());
        environment.put("activity.name", activityModel.getName());
        environment.put("activity.precondition", activityModel.getPrecondition());
        environment.put("activity.postcondition", activityModel.getPostcondition());
        environment.put("activity.reentrant", Boolean.valueOf(activityModel.isReentrant()));
        if( activityModel instanceof org.concern.model.AsynchronousActivity ) {
            environment.put("activity.actor", ((org.concern.model.AsynchronousActivity)activityModel).getActor());
            environment.put("activity.timeout", new Integer(((org.concern.model.AsynchronousActivity)activityModel).getTimeout()));
            environment.put("activity.optional", ((org.concern.model.AsynchronousActivity)activityModel).isOptional() ? Boolean.TRUE : Boolean.FALSE);
            environment.put("activity.user", ((org.concern.model.AsynchronousActivity)activityModel).isUser() ? Boolean.TRUE : Boolean.FALSE);
        } else if( activityModel instanceof org.concern.model.SynchronousActivity ) {
            environment.put("activity.retryDelay", new Integer(((org.concern.model.SynchronousActivity)activityModel).getRetryDelay()));
            environment.put("activity.trials", new Integer(((org.concern.model.SynchronousActivity)activityModel).getTrials()));
        } else {
            System.out.println( "dji ******************* class: " + activityModel.getClass().getName() );
            throw new ClassCastException( "No matching activity found." );
        }

        return environment;
    }

    private Map eventEnvironment(org.concern.model.Event eventModel) {
        Map environment = environment(eventModel.getEnvironment());
        environment.put("event.name", eventModel.getName());
        environment.put("event.postcondition", eventModel.getPostcondition());
        return environment;
    }

    private Map listenerEnvironment(org.concern.model.Listener listenerModel) {
        Map environment = environment(listenerModel.getEnvironment());
        environment.put("listener.name", listenerModel.getName());
        environment.put("listener.precondition", listenerModel.getPrecondition());
        environment.put("listener.retryDelay", new Integer(listenerModel.getRetryDelay()));
        environment.put("listener.trials", new Integer(listenerModel.getTrials()));
        return environment;
    }

    private Map environment(List list) {
        Map environment = new HashMap();
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            EnvironmentEntry entry = (EnvironmentEntry)iterator.next();
            Object value;
            if (entry.getValue().startsWith("resource:"))
                value = resourceLocator.lookup(entry.getValue().substring("resource:".length()));
            else if (entry.getValue().startsWith("parameter:"))
                value = parameterResolver.resolve(process.getName(), entry.getValue().substring("parameter:".length()), entry.getType());
            else
                value = EnvironmentEntry.createInstance(entry.getType(), entry.getValue());
            environment.put(entry.getName(), value);
        }
        return environment;
    }

    public ClassLoader getClassLoader() {
        return classLoader != null ? classLoader : Thread.currentThread().getContextClassLoader();
    }

    public static class PrefixNamingStrategy extends ImprovedNamingStrategy {
        private final String processName;

        public PrefixNamingStrategy(String processName) {
            this.processName = processName.replace(' ', '_');
        }

        public String propertyToTableName(String className, String propertyName) {
            return processName + "_" + propertyName;
        }

        public String classToTableName(String string) {
            return processName + "_" + super.classToTableName(string);
        }

        public String tableName(String string) {
            return processName + "_" + super.tableName(string);
        }
    }
}
