package org.concern.controller;

import org.hibernate.*;
import org.apache.commons.logging.LogFactory;

import javax.transaction.*;
import javax.transaction.Transaction;
import javax.transaction.xa.XAResource;
import java.util.List;
import java.util.Iterator;
import java.util.LinkedList;

public class NoTransactionManager
    implements TransactionManager
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(NoTransactionManager.class);

    private Sessions sessions;
    private ThreadLocal transactions = new ThreadLocal();

    public NoTransactionManager() {
    }

    public NoTransactionManager(Sessions sessions) {
        setSessions(sessions);
    }

    public void setSessions(Sessions sessions) {
        this.sessions = sessions;
        sessions.setTransactionManager(this);
    }

    public void setRollbackOnly() throws IllegalStateException, SystemException {
        getTransaction().setRollbackOnly();
    }

    public void rollback() throws IllegalStateException, SecurityException, SystemException {
        getTransaction().rollback();
    }

    public int getStatus() throws SystemException {
        Transaction transaction = getTransaction();
        int status;
        if (transaction != null)
            status = transaction.getStatus();
        else
            status = Status.STATUS_NO_TRANSACTION;
        return status;
    }

    public Transaction getTransaction() throws SystemException {
        return (Transaction)transactions.get();
    }

    public void resume(Transaction transaction) throws InvalidTransactionException, IllegalStateException, SystemException {
    }

    public void setTransactionTimeout(int i) {
    }

    public Transaction suspend() throws SystemException {
        return null;
    }

    public void commit() throws SecurityException, IllegalStateException, SystemException {
        try {
            getTransaction().commit();
        }
        catch (SystemException e) {
            throw e;
        }
        catch (Exception e) {
            e.printStackTrace();
            throw new SystemException(e.getMessage());
        }
    }

    public void begin() throws SystemException {
        new NoTransaction();
    }

    private class NoTransaction
        implements Transaction
    {
        private int status = Status.STATUS_UNKNOWN;
        private org.hibernate.Transaction transaction;
        List synchronizations = new LinkedList();

        public NoTransaction() throws SystemException {
            transactions.set(this);
            try {
                status = Status.STATUS_ACTIVE;
                transaction = sessions.getSession().beginTransaction();
            }
            catch (HibernateException e) {
                LOG.error(status, e);
                throw new SystemException(e.getMessage());
            }
        }

        public void commit() throws RollbackException, HeuristicMixedException, HeuristicRollbackException, SecurityException, IllegalStateException, SystemException {
            try {
                if (status != Status.STATUS_ACTIVE)
                    throw new IllegalStateException("status " + status);

                beforeCompletion();
                transaction.commit();
                status = Status.STATUS_COMMITTED;
            }
            catch (HibernateException e) {
                LOG.error(status, e);
                throw new SystemException(e.getMessage());
            }
            finally {
                afterCompletion();
            }
        }

        public boolean delistResource(XAResource xaResource, int i) throws IllegalStateException, SystemException {
            return true;
        }

        public boolean enlistResource(XAResource xaResource) throws RollbackException, IllegalStateException, SystemException {
            return true;
        }

        public int getStatus() throws SystemException {
            return status;
        }

        public void registerSynchronization(Synchronization synchronization) throws RollbackException, IllegalStateException, SystemException {
            synchronizations.add(synchronization);
        }

        public void rollback() throws IllegalStateException, SystemException {
            try {
                transaction.rollback();
                status = Status.STATUS_ROLLEDBACK;
            }
            catch (HibernateException e) {
                LOG.error(status, e);
                throw new SystemException(e.getMessage());
            }
            finally {
                afterCompletion();
            }
        }

        private void beforeCompletion() {
            for (Iterator iterator = synchronizations.iterator(); iterator.hasNext();) {
                Synchronization synchronization = (Synchronization)iterator.next();
                synchronization.beforeCompletion();
            }
        }

        private void afterCompletion() {
            for (Iterator iterator = synchronizations.iterator(); iterator.hasNext();) {
                Synchronization synchronization = (Synchronization)iterator.next();
                synchronization.afterCompletion(status);
            }
        }

        public void setRollbackOnly() throws IllegalStateException, SystemException {
            status = Status.STATUS_MARKED_ROLLBACK;
        }
    }
}
