package org.concern.controller;


import org.concern.ControllerException;
import org.apache.commons.logging.LogFactory;

import javax.transaction.*;
import java.util.Stack;

public class Transactions
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Transactions.class);

    public static final int REQUIRED = 1;
    public static final int MANDATORY = 2;
    public static final int NEVER = 3;

    private ThreadLocal currentStatus = new ThreadLocal();
    private int countRequired = 0;
    private int countMandatory = 0;
    private int countNever = 0;
    private int count = 0;
    private TransactionManager transactionManager;

    public Transactions() {
    }

    public Transactions(TransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public void setRollbackOnly() {
        try {
            transactionManager.setRollbackOnly();
        }
        catch (SystemException e) {
            LOG.error("Transaction Propagation Nesting: " + count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    private Stack getStatusStack() {
        Stack stack = (Stack)currentStatus.get();
        if (stack == null)
            currentStatus.set(stack = new Stack());
        return stack;
    }

    public void beginRequired() throws ControllerException {
        count++;
        countRequired++;
        try {
            int status = transactionManager.getStatus();
            getStatusStack().push(new Integer(status));
            if (status != Status.STATUS_ACTIVE) {
                LOG.trace("TRANSACTION BEGIN " + Thread.currentThread().getName());
                transactionManager.begin();
            }
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public void endRequired() throws ControllerException {
        countRequired--;
        try {
            int status = ((Integer)getStatusStack().pop()).intValue();
            if (status != Status.STATUS_ACTIVE)
                end();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public void beginMandatory() throws ControllerException {
        count++;
        countMandatory++;
        try {
            int status = transactionManager.getStatus();
            if (status != Status.STATUS_ACTIVE)
                throw new ControllerException("transaction required");
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public void endMandatory() {
        count--;
        countMandatory--;
    }

    public void beginNever() throws ControllerException {
        count++;
        countNever++;
        try {
            int status = transactionManager.getStatus();
            if (status == Status.STATUS_ACTIVE)
                throw new ControllerException("transaction not allowed");

            transactionManager.begin();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public void endNever() {
        countNever--;
        try {
            end();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    private void end() throws SystemException, HeuristicMixedException, HeuristicRollbackException, RollbackException {
        count--;
        if (transactionManager.getStatus() == Status.STATUS_MARKED_ROLLBACK) {
            LOG.trace("TRANSACTION ROLLBACK " + Thread.currentThread().getName());
            transactionManager.rollback();
        }
        else {
            LOG.trace("TRANSACTION COMMIT " + Thread.currentThread().getName());
            transactionManager.commit();
        }
    }

    public void begin() {
        count++;
        try {
            LOG.trace("TRANSACTION BEGIN " + Thread.currentThread().getName());
            transactionManager.begin();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public void commit() {
        count--;
        try {
            LOG.trace("TRANSACTION COMMIT " + Thread.currentThread().getName());
            transactionManager.commit();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }
    public void rollback() {
        count--;
        try {
            LOG.trace("TRANSACTION ROLLBACK " + Thread.currentThread().getName());
            transactionManager.rollback();
        }
        catch (Exception e) {
            LOG.error(count, e);
            throw new ControllerException(e.getMessage());
        }
    }

    public int getPropagationNesting() {
        return count;
    }
}
