/*
 * $Id: ControllerService.java 881 2007-02-12 07:54:31Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller.jmx;

import org.hibernate.SessionFactory;
import org.concern.*;
import org.concern.Controller;
import org.concern.Log;
import org.concern.controller.*;

import javax.naming.*;
import javax.transaction.TransactionManager;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.*;


/**
 * @see ControllerServiceMBean
 * @author hengels
 */
public class ControllerService
    implements ControllerServiceMBean
{
    private static final Map<String, Controller> controllers = new HashMap<String, Controller>();

    private String resourceLocatorClassName = JndiResourceLocator.class.getName();
    private String parameterResolverClassName = NoParameterResolver.class.getName();
    private String jndiName;
    private String transactionManagerJndiName;
    private String sessionFactoryJndiName;
    private String processDescription;

    private long timerInterval = 10000;
    protected Configuration configuration;

    static {
        System.setProperty("concern.controller.lookup", JndiControllerLookup.class.getName());
    }

    public String getJndiName() {
        return jndiName;
    }

    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    public String getTransactionManagerJndiName() {
        return transactionManagerJndiName;
    }

    public void setTransactionManagerJndiName(String transactionManagerJndiName) {
        this.transactionManagerJndiName = transactionManagerJndiName;
    }

    public String getSessionFactoryJndiName() {
        return sessionFactoryJndiName;
    }

    public void setSessionFactoryJndiName(String sessionFactoryJndiName) {
        this.sessionFactoryJndiName = sessionFactoryJndiName;
    }

    public long getTimerInterval() {
        return timerInterval;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public void setResourceLocatorClassName(String name) {
        this.resourceLocatorClassName = name;
    }

    public String getResourceLocatorClassName() {
        return resourceLocatorClassName;
    }

    public String getParameterResolverClassName() {
        return parameterResolverClassName;
    }

    public void setParameterResolverClassName(String parameterResolverClassName) {
        this.parameterResolverClassName = parameterResolverClassName;
    }

    public String getProcessDescription() {
        return processDescription;
    }

    public void setProcessDescription(String processDescription) {
        this.processDescription = processDescription;
    }

    private InputStream getProcessDescriptionAsStream() {
        if (processDescription != null)
            try {
                return new ByteArrayInputStream(processDescription.getBytes("UTF-8"));
            }
            catch (UnsupportedEncodingException e) {
                throw new RuntimeException(e);
            }
        else
            return Thread.currentThread().getContextClassLoader().getResourceAsStream("META-INF/concern-process.cpd");
    }

    public void start() throws ControllerException {
        try {
            Controller controller = buildController();
            if (jndiName == null)
                jndiName = "java:controller/" + controller.getProcessName();

            ControllerProxy controllerProxy = new ControllerProxy(jndiName);
            Object proxy = Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
                new Class[] { Controller.class }, controllerProxy );
            rebind(new InitialContext(), jndiName, proxy);
            controllers.put(jndiName, controller);
            controller.start();
        }
        catch (NamingException e) {
            e.printStackTrace();
        }
    }

    public void stop() {
        try {
            Controller controller = controllers.remove(jndiName);
            controller.stop();
            unbind(new InitialContext(), jndiName);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    Controller buildController() throws ControllerException {
        try {
            configuration = new Configuration();
            configuration.setClassLoader(Thread.currentThread().getContextClassLoader());
            configuration.setProcess(Configuration.loadProcess(getProcessDescriptionAsStream()));
            configuration.setTransactionManager((TransactionManager)new InitialContext().lookup(transactionManagerJndiName));
            configuration.setSessionFactory((SessionFactory)new InitialContext().lookup(sessionFactoryJndiName));
            configuration.setResourceLocator((ResourceLocator)Class.forName(resourceLocatorClassName).newInstance());
            configuration.setParameterResolver((ParameterResolver)Class.forName(parameterResolverClassName).newInstance());
            configuration.setTimerInterval(timerInterval);

            return configuration.buildController();
        }
        catch (Exception e) {
            throw new ControllerException(e);
        }
    }

    static void rebind(Context context, String string, Object value)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        int size = name.size();
        String atom = name.get(size-1);
        Context parentContext = createChildContext(context, name.getPrefix(size - 1));
        parentContext.rebind(atom, value);
    }

    static Context createChildContext(Context context, Name name)
        throws NamingException
    {
        Context childContext = context;
        for (int pos = 0; pos < name.size(); pos++)
        {
            String string = name.get(pos);
            try {
                childContext = (Context) context.lookup(string);
            }
            catch (NameNotFoundException e) {
                childContext = context.createSubcontext(string);
            }
            context = childContext;
        }
        return childContext;
    }

    static void unbind(Context context, String string)
        throws NamingException
    {
        Name name = context.getNameParser("").parse(string);
        context.unbind(name);
        /*
        int size = name.size();
        while (--size > 0) {
            Name prefixName = name.getPrefix(size);
            if (context.listBindings(prefixName).hasMore())
                break;
            else
                context.unbind(prefixName);
        }
        */
    }

    static Controller getController(String name) {
        return controllers.get(name);
    }

    Controller getController() {
        return controllers.get(jndiName);
    }

    public String getProcessName() {
        return getController().getProcessName();
    }

    public Integer createSubject(String userValue) throws SubjectCreationException {
        return getController().createSubject(userValue);
    }

    public boolean isKnownSubject(String userValue) throws ControllerException {
        return getController().isKnownSubject(userValue);
    }

    public List<String> getSubjects(int state) {
        return getController().getSubjects(state);
    }

    public void announceSubject(String userValue) throws UnknownSubjectException {
        getController().announceSubject(userValue);
    }

    public void scheduleAnnouncement(String userValue, long duration) throws UnknownSubjectException {
        getController().scheduleAnnouncement(userValue, duration);
    }

    public void log(Log log) throws UnknownSubjectException {
        getController().log(log);
    }

    public void lockSubject(String userValue) throws UnknownSubjectException, ControllerException {
        getController().lockSubject(userValue);
    }

    public boolean isForwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        return getController().isForwardPossible(userValue, activityName);
    }

    public void forward(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        getController().forward(userValue, activityName);
    }

    public void forward(String userValue, String activityName, Map<String,String> logAnnotations) throws UnknownSubjectException, ControllerException {
        getController().forward(userValue, activityName, logAnnotations);
    }

    public boolean isBackwardPossible(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        return getController().isBackwardPossible(userValue, activityName);
    }

    public void backward(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        getController().backward(userValue, activityName);
    }

    public void backward(String userValue, String activityName, Map<String,String> logAnnotations) throws UnknownSubjectException, ControllerException {
        getController().backward(userValue, activityName, logAnnotations);
    }

    public void reset(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        getController().reset(userValue, activityName);
    }

    public void suspendSubject(String userValue) throws UnknownSubjectException, ControllerException {
        getController().suspendSubject(userValue);
    }

    public void resumeSubject(String userValue) throws UnknownSubjectException, ControllerException {
        getController().resumeSubject(userValue);
    }

    public org.concern.model.Process getProcess() {
        return getController().getProcess();
    }

    public List<String> getEnlistments(String userValue) throws UnknownSubjectException, ControllerException {
        return getController().getEnlistments(userValue);
    }

    public List<String> getTasks(String userValue) throws UnknownSubjectException, ControllerException {
        return getController().getTasks(userValue);
    }

    public List<String> getOptions(String userValue) throws UnknownSubjectException, ControllerException {
        return getController().getOptions(userValue);
    }

    public List<String> getSubjects(String activity) {
        return getController().getSubjects(activity);
    }

    public List<Log> getLog(String userValue) throws UnknownSubjectException {
        return getController().getLog(userValue);
    }

    public List<Log> getArchive(String userValue) throws ControllerException, UnknownSubjectException {
        return getController().getArchive(userValue);
    }

    public boolean complete(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        return getController().complete(userValue, activityName);
    }

    public boolean complete(String userValue, String activityName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
        return getController().complete(userValue, activityName, logAnnotations);
    }

    public Boolean matchCondition(String userValue, String conditionName) throws UnknownSubjectException, ControllerException {
        return getController().matchCondition(userValue, conditionName);
    }

    public boolean matchPrecondition(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        return getController().matchPrecondition(userValue, activityName);
    }

    public boolean matchPostcondition(String userValue, String activityName) throws UnknownSubjectException, ControllerException {
        return getController().matchPostcondition(userValue, activityName);
    }

    public void process(String userValue) throws UnknownSubjectException, ControllerException {
        getController().process(userValue);
    }

    public void destroySubject(String userValue) throws UnknownSubjectException {
        getController().destroySubject(userValue);
    }

    public void reviveSubject(String userValue) throws UnknownSubjectException, ControllerException {
        getController().reviveSubject(userValue);
    }

    public boolean notify(String userValue, String eventName) throws UnknownSubjectException {
        return getController().notify(userValue, eventName);
    }

    public boolean notify(String userValue, String eventName, Map<String, String> logAnnotations) throws UnknownSubjectException, ControllerException {
        return getController().notify(userValue, eventName, logAnnotations);
    }

    public void reload() {
        getController().reload();
    }
}
