package org.concern.controller.spring;

import org.concern.controller.*;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.core.io.Resource;
import org.springframework.web.context.support.ServletContextResource;
import org.apache.commons.logging.LogFactory;

import javax.transaction.TransactionManager;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;

public class ControllerFactoryBean
    implements FactoryBean, InitializingBean, DisposableBean, BeanFactoryAware
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ControllerFactoryBean.class);

    protected Controller controller;
    protected Resource processLocation;
    protected TransactionManager transactionManager;
    protected SessionFactory sessionFactory;
    protected Map resources;
    protected Map parameters;
    private long timerInterval = 10000;
    private boolean autowireByType;
    private String processDescription;
    private BeanFactory beanFactory;

    /**
     * @param processLocation
     */
    public void setProcessLocation(Resource processLocation) {
        this.processLocation = processLocation;
    }

    /**
     * Set reference to the JTA TransactionManager (optional). Alternatively setup a spring
     * HibernateTransactionManager with the session factory.
     * @param transactionManager the JTA TransactionManager
     */
    public void setTransactionManager(TransactionManager transactionManager) {
        if (transactionManager instanceof NoTransactionManager)
            return;
        this.transactionManager = transactionManager;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public void setResources(Map resources) {
        this.resources = resources;
    }

    public void setBeanFactory(BeanFactory beanFactory) {
        this.beanFactory = beanFactory;
    }

    public void setTimerInterval(long timerInterval) {
        this.timerInterval = timerInterval;
    }

    public void setAutowireByType(boolean autowireByType) {
        this.autowireByType = autowireByType;
    }

    public String getProcessDescription() {
        return processDescription;
    }

    public void setProcessDescription(String processDescription) {
        this.processDescription = processDescription;
    }

    private InputStream getProcessDescriptionAsStream() {
        if (processDescription != null)
            try {
                return new ByteArrayInputStream(processDescription.getBytes("UTF-8"));
            }
            catch (UnsupportedEncodingException e) {
                throw new RuntimeException(e);
            }
        else {
            try {
                URL url = processLocation.getURL();

                // workaround for tomcat bug
                if (processLocation instanceof ServletContextResource) {
                    ServletContextResource servletContextResource = (ServletContextResource)processLocation;
                    try {
                        LOG.info("Workaround for tomcat bug.");
                        url = servletContextResource.getFile().toURL();
                    }
                    catch (IOException e) {
                        LOG.error("Workaround failed", e);
                    }
                }
                return url.openStream();
            }
            catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public Object getObject() throws Exception {
        return controller;
    }

    public Class getObjectType() {
        return (controller != null) ? controller.getClass() : org.concern.Controller.class;
    }

    public boolean isSingleton() {
        return true;
    }

    public void afterPropertiesSet() throws Exception {
        Configuration configuration = new Configuration();
        configuration.setProcess(Configuration.loadProcess(getProcessDescriptionAsStream()));
        configuration.setTransactionManager(transactionManager);
        configuration.setSessionFactory(sessionFactory);
        configuration.setResourceLocator(resources != null ? (ResourceLocator)new LocalResourceLocator(resources) : (ResourceLocator)new BeanFactoryResourceLocator(beanFactory));
        configuration.setParameterResolver(parameters != null ? (ParameterResolver)new LocalParameterResolver(parameters) : (ParameterResolver)new NoParameterResolver());
        configuration.setTimerInterval(timerInterval);

        controller = configuration.buildController();
        if (autowireByType)
            autowireByType();
        controller.start();

        System.setProperty("concern.controller.lookup", LocalControllerLookup.class.getName());
        ((LocalControllerLookup)LocalControllerLookup.getInstance()).add(controller);
    }

    private void autowireByType() throws IllegalAccessException, InvocationTargetException {
        if (beanFactory instanceof ListableBeanFactory) {
            ListableBeanFactory listableBeanFactory = (ListableBeanFactory)beanFactory;
            List elements = new LinkedList();
            elements.addAll(controller.getConditions());
            elements.addAll(controller.getEvents());
            elements.addAll(controller.getActivities());
            elements.addAll(controller.getListeners());
            elements.addAll(controller.getActors());
            elements.add(controller.getLoader());
            for (Iterator iterator = elements.iterator(); iterator.hasNext();) {
                Object element = iterator.next();
                PropertyDescriptor[] descriptors = BeanUtils.getPropertyDescriptors(element.getClass());
                for (int i = 0; i < descriptors.length; i++) {
                    PropertyDescriptor descriptor = descriptors[i];
                    if (descriptor.getWriteMethod() == null)
                        continue;
                    if (!descriptor.getWriteMethod().getDeclaringClass().equals(element.getClass()))
                        continue;
                    try {
                        Object value = BeanFactoryUtils.beanOfTypeIncludingAncestors(listableBeanFactory, descriptor.getPropertyType());
                        descriptor.getWriteMethod().invoke(element, new Object[] { value });
                    }
                    catch (BeansException e) {
                    }
                }
            }
        }
    }

    public void destroy() throws Exception {
        ((LocalControllerLookup)LocalControllerLookup.getInstance()).remove(controller.getProcessName());
        controller.stop();
    }
}
