/*
 * $Id: Process.java 561 2005-10-24 16:27:03Z nebulosu $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.sf.net).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model;

import javax.swing.event.EventListenerList;
import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

/**
 * @version $Revision: 561 $
 */
public class Process
    extends Element implements PropertyChangeListener {
    private String subject;
    private Loader loader;
    private SortedSet actors = new TreeSet();
    private SortedSet activities = new TreeSet();
    private SortedSet conditions = new TreeSet();
    private SortedSet events = new TreeSet();
    private SortedSet listeners = new TreeSet();
    private SortedSet collaborators = new TreeSet();

    EventListenerList listenerList = new EventListenerList();

    public Process(String name) {
        super(name);
    }

    public String getSubject() {
        return subject;
    }

    public void setSubject(String subject) {
        Object old = this.subject;
        this.subject = subject;
        changes.firePropertyChange("subject", old, subject);
    }

    public Loader getLoader() {
        return loader;
    }

    public void setLoader(Loader loader) {
        Object old = this.loader;
        this.loader = loader;
        changes.firePropertyChange("loader", old, loader);
    }

    public void addActor(Actor actor) {
        if (actors.add(actor)) {
            actor.addPropertyChangeListener("name", this);
            fireNodeAdded(actor);
        }
    }

    public void removeActor(Actor actor) {
        if (actors.remove(actor)) {
            fireNodeRemoved(actor);
            actor.removePropertyChangeListener("name", this);
        }
    }

    public Actor getActor(String name) {
        return (Actor)getElement(actors, name);
    }

    public Collection getActors() {
        return Collections.unmodifiableCollection(actors);
    }

    public void addActivity(Activity activity) {
        if (activities.add(activity)) {
            activity.addPropertyChangeListener("name", this);
            fireNodeAdded(activity);
        }
    }

    public void removeActivity(Activity activity) {
        if (activities.remove(activity)) {
            fireNodeRemoved(activity);
            activity.removePropertyChangeListener("name", this);
        }
    }

    public Activity getActivity(String name) {
        return (Activity)getElement(activities, name);
    }

    public Collection getActivities() {
        return Collections.unmodifiableCollection(activities);
    }

    public void addCondition(Condition condition) {
        if (conditions.add(condition)) {
            condition.addPropertyChangeListener("name", this);
            fireNodeAdded(condition);
        }
    }

    public void removeCondition(Condition condition) {
        if (conditions.remove(condition)) {
            fireNodeRemoved(condition);
            condition.removePropertyChangeListener("name", this);
        }
    }

    public Condition getCondition(String name) {
        return (Condition)getElement(conditions, name);
    }

    public Collection getConditions() {
        return Collections.unmodifiableCollection(conditions);
    }

    public void addEvent(Event event) {
        if (events.add(event)) {
            event.addPropertyChangeListener("name", this);
            fireNodeAdded(event);
        }
    }

    public void removeEvent(Event event) {
        if (events.remove(event)) {
            fireNodeRemoved(event);
            event.removePropertyChangeListener("name", this);
        }
    }

    public Event getEvent(String name) {
        return (Event)getElement(events, name);
    }

    public Collection getEvents() {
        return Collections.unmodifiableCollection(events);
    }

    public void addListener(Listener listener) {
        if (listeners.add(listener)) {
            listener.addPropertyChangeListener("name", this);
            fireNodeAdded(listener);
        }
    }

    public void removeListener(Listener listener) {
        if (listeners.remove(listener)) {
            fireNodeRemoved(listener);
            listener.removePropertyChangeListener("name", this);
        }
    }

    public Listener getListener(String name) {
        return (Listener)getElement(listeners, name);
    }

    public Collection getListeners() {
        return Collections.unmodifiableCollection(listeners);
    }

    public void addCollaborator(Collaborator collaborator) {
        if (collaborators.add(collaborator)) {
            collaborator.addPropertyChangeListener("name", this);
            fireNodeAdded(collaborator);
        }
    }

    public void removeCollaborator(Collaborator collaborator) {
        if (collaborators.remove(collaborator)) {
            fireNodeRemoved(collaborator);
            collaborator.removePropertyChangeListener("name", this);
        }
    }

    public Collaborator getCollaborator(String name) {
        return (Collaborator)getElement(collaborators, name);
    }

    public Collection getCollaborators() {
        return Collections.unmodifiableCollection(collaborators);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        fireNodeChanged((Element)evt.getSource());
    }

    protected void fireNodeAdded(Element element) {
        Object[] listeners = listenerList.getListenerList();
        ProcessChangeEvent e = new ProcessChangeEvent(this, element, ProcessChangeEvent.ADDED);

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ProcessChangeListener.class) {
                ((ProcessChangeListener)listeners[i+1]).nodeAdded(e);
            }
        }
    }
    protected void fireNodeRemoved(Element element) {
        Object[] listeners = listenerList.getListenerList();
        ProcessChangeEvent e = new ProcessChangeEvent(this, element, ProcessChangeEvent.REMOVED);

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ProcessChangeListener.class) {
                ((ProcessChangeListener)listeners[i+1]).nodeRemoved(e);
            }
        }
    }

    protected void fireNodeChanged(Element element) {
        Object[] listeners = listenerList.getListenerList();
        ProcessChangeEvent e = new ProcessChangeEvent(this, element, ProcessChangeEvent.CHANGED);

        for (int i = listeners.length - 2; i >= 0; i -= 2) {
            if (listeners[i] == ProcessChangeListener.class) {
                ((ProcessChangeListener)listeners[i+1]).nodeChanged(e);
            }
        }
    }

    public void addProcessChangeListener(ProcessChangeListener listener) {
        listenerList.add(ProcessChangeListener.class, listener);
    }

    public void removeProcessChangeListener(ProcessChangeListener listener) {
        listenerList.remove(ProcessChangeListener.class, listener);
    }

    public void addElement(Element element) {
        if (element instanceof Collaborator) {
            Collaborator collaborator = (Collaborator)element;
            element.setName(unifyName(getCollaborators(), element.getName()));
            addCollaborator(collaborator);
        }
        else if (element instanceof Actor) {
            Actor actor = (Actor)element;
            element.setName(unifyName(getActors(), element.getName()));
            addActor(actor);
        }
        else if (element instanceof Condition) {
            Condition condition = (Condition)element;
            element.setName(unifyName(getConditions(), element.getName()));
            addCondition(condition);
        }
        else if (element instanceof Event) {
            Event event = (Event)element;
            element.setName(unifyName(getEvents(), element.getName()));
            addEvent(event);
        }
        else if (element instanceof Activity) {
        	Activity activity = (Activity)element;
            element.setName(unifyName(getActivities(), element.getName()));
            addActivity(activity);
        }
        else if (element instanceof Listener) {
            Listener listener = (Listener)element;
            element.setName(unifyName(getListeners(), element.getName()));
            addListener(listener);
        }
    }

    private String unifyName(Collection elements, String name) {
        if (getElement(elements, name) == null)
            return name;

        int i = 2;
        while(getElement(elements, name + " " + i) != null)
            i++;

        return name + " " + i;
    }

    private Element getElement(Collection elements, String name) {
        for (Iterator iterator = elements.iterator(); name != null && iterator.hasNext();) {
            Element element = (Element)iterator.next();
            if (name.equals(element.getName()))
                return element;
        }
        return null;
    }


    public void removeNode(Element element) {
        boolean removed = false;
        removed |= conditions.remove(element);
        removed |= activities.remove(element);
        removed |= events.remove(element);
        removed |= listeners.remove(element);
        removed |= actors.remove(element);
        removed |= collaborators.remove(element);
        if (removed)
            fireNodeRemoved(element);
    }
}
