/*
 * $Id: ProcessWriter.java 606 2005-12-12 12:49:05Z nebulosu $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.model.cpd;

import org.concern.model.*;
import org.concern.model.Process;

import java.io.*;
import java.util.*;
import java.nio.charset.Charset;

/**
 * <activity name="...">
 *   <interaction direction="out" message="start" target="Positions">
 * </activity>
 */
public class ProcessWriter
{
    private static final String[] CARDINALITIES = { "one to one", "one to many", "many to one", "many to many" };
    private static final String[] DIRECTIONS = { null, "in", "out", "both" };

    public static void save(Process process, File file) throws FileNotFoundException {

        OutputStream outputStream = new FileOutputStream(file);
        PrintWriter writer = new PrintWriter(new OutputStreamWriter(outputStream, Charset.forName("UTF-8")));

        writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        writer.print("    <process name=\"");
        writer.print(process.getName());
        writer.print("\" subject=\"");
        writer.print(process.getSubject());
        writer.println("\">");

        environment(process, writer);
        attributes(process, writer);

        if (process.getLoader() != null) {
            Loader loader = process.getLoader();
            writer.print("        <loader name=\"");
            writer.print(loader.getName());
            writer.print("\" class=\"");
            writer.print(loader.getImplementation());
            writer.println("\">");

            environment(loader, writer);
            writer.println("        </loader>");
        }

        List actors = new ArrayList(process.getActors());
        Collections.sort(actors);
        for (Iterator iterator2 = actors.iterator(); iterator2.hasNext();) {
            Actor actor = (Actor)iterator2.next();
            writer.print("        <actor name=\"");
            writer.print(actor.getName());
            writer.print("\" class=\"");
            writer.print(actor.getImplementation());
            writer.println("\">");

            if (actor.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(actor.getDescription());
                writer.println("]]></description>");
            }

            environment(actor, writer);
            writer.println("        </actor>");
        }
        writer.println();

        List collaborations = new ArrayList(process.getCollaborators());
        Collections.sort(collaborations);
        for (Iterator iterator2 = collaborations.iterator(); iterator2.hasNext();) {
            Collaborator collaborator = (Collaborator)iterator2.next();
            writer.print("        <collaboration name=\"");
            writer.print(collaborator.getName());
            writer.print("\" cardinality=\"");
            writer.print(CARDINALITIES[collaborator.getCardinality()]);
            writer.println("\">");

            if (collaborator.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(collaborator.getDescription());
                writer.println("]]></description>");
            }

            environment(collaborator, writer);
            writer.println("        </collaboration>");
        }
        writer.println();

        List conditions = new ArrayList(process.getConditions());
        Collections.sort(conditions);
        for (Iterator iterator2 = conditions.iterator(); iterator2.hasNext();) {
            Condition condition = (Condition)iterator2.next();
            writer.print("        <condition name=\"");
            writer.print(condition.getName());
            writer.print("\" class=\"");
            writer.print(condition.getImplementation());
            writer.print("\" temporal=\"");
            writer.print(condition.isTemporal() ? "true" : "false");
            writer.println("\">");

            if (condition.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(condition.getDescription());
                writer.println("]]></description>");
            }

            environment(condition, writer);
            writer.println("        </condition>");
        }
        writer.println();

        List activities = new ArrayList(process.getActivities());
        Collections.sort(activities);
        for (Iterator iterator2 = activities.iterator(); iterator2.hasNext();) {
            Activity activity = (Activity)iterator2.next();
            if( activity instanceof SynchronousActivity ) {
            	writer.print("        <synchronous-activity name=\"");
            } else {
            	writer.print("        <asynchronous-activity name=\"");
			}
            writer.print(activity.getName());
            writer.print("\" class=\"");
            writer.print(activity.getImplementation());
            if( activity instanceof AsynchronousActivity ) {
	            if( ((AsynchronousActivity)activity).getActor() != null) {
	                writer.print("\" actor=\"");
	                writer.print(((AsynchronousActivity)activity).getActor());
	            }
	            writer.print("\" user=\"");
	            writer.print(((AsynchronousActivity)activity).isUser() ? "true" : "false");
	            writer.print("\" optional=\"");
	            writer.print(((AsynchronousActivity)activity).isOptional() ? "true" : "false");
                writer.print("\" timeout=\"");
                writer.print("" + ((AsynchronousActivity)activity).getTimeout());
            } else {
                writer.print("\" retry-delay=\"");
                writer.print("" + ((SynchronousActivity)activity).getRetryDelay());
                writer.print("\" trials=\"");
                writer.print("" + ((SynchronousActivity)activity).getTrials());
            }
            writer.print("\" reentrant=\"");
            writer.print(activity.isReentrant() ? "true" : "false");
            writer.println("\">");

            writer.print("            <precondition><![CDATA[");
            writer.print(activity.getPrecondition() != null ? activity.getPrecondition() : "");
            writer.println("]]></precondition>");
            writer.print("            <postcondition><![CDATA[");
            writer.print(activity.getPostcondition() != null ? activity.getPostcondition() : "");
            writer.println("]]></postcondition>");

            if (activity.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(activity.getDescription());
                writer.println("]]></description>");
            }

            interaction(activity.getInteraction(), writer);
            environment(activity, writer);
            attributes(activity, writer);

            if( activity instanceof SynchronousActivity ) {
            	writer.println("        </synchronous-activity>");
            } else {
            	writer.println("        </asynchronous-activity>");
			}
        }

        List events = new ArrayList(process.getEvents());
        Collections.sort(events);
        for (Iterator iterator2 = events.iterator(); iterator2.hasNext();) {
            Event event = (Event)iterator2.next();
            writer.print("        <event name=\"");
            writer.print(event.getName());
            writer.print("\" class=\"");
            writer.print(event.getImplementation());
            writer.print("\" user=\"");
            writer.print(event.isUser() ? "true" : "false");
            writer.println("\">");

            writer.print("            <postcondition><![CDATA[");
            writer.print(event.getPostcondition() != null ? event.getPostcondition() : "");
            writer.println("]]></postcondition>");

            if (event.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(event.getDescription());
                writer.println("]]></description>");
            }

            interaction(event.getInteraction(), writer);
            environment(event, writer);
            attributes(event, writer);

            writer.println("        </event>");
        }

        List listeners = new ArrayList(process.getListeners());
        Collections.sort(listeners);
        for (Iterator iterator2 = listeners.iterator(); iterator2.hasNext();) {
            Listener listener = (Listener)iterator2.next();
            writer.print("        <listener name=\"");
            writer.print(listener.getName());
            writer.print("\" class=\"");
            writer.print(listener.getImplementation());
            writer.println("\">");

            writer.print("            <precondition><![CDATA[");
            writer.print(listener.getPrecondition() != null ? listener.getPrecondition() : "");
            writer.println("]]></precondition>");

            if (listener.getDescription() != null) {
                writer.print("            <description><![CDATA[");
                writer.print(listener.getDescription());
                writer.println("]]></description>");
            }

            interaction(listener.getInteraction(), writer);
            environment(listener, writer);
            attributes(listener, writer);

            writer.println("        </listener>");
        }

        writer.println("    </process>");
        writer.close();
    }

    private static void interaction(Interaction interaction, PrintWriter writer) {
        if (interaction != null) {
            writer.print("            <interaction direction=\"");
            writer.print(DIRECTIONS[interaction.getDirection()]);
            writer.print("\" message=\"");
            writer.print(interaction.getMessage() != null ? interaction.getMessage() : "");
            writer.print("\" collaborator=\"");
            writer.print(interaction.getCollaborator());
            writer.println("\"/>");
        }
    }

    private static void environment(Element element, PrintWriter writer) {
        if (element.getEnvironment() == null)
            return;

        for (Iterator iterator = element.getEnvironment().iterator(); iterator.hasNext();) {
            EnvironmentEntry entry = (EnvironmentEntry)iterator.next();
            writer.println("            <env-entry>");
            if (entry.getDescription() != null) {
                writer.print("                <description>");
                writer.print(entry.getDescription());
                writer.println("</description>");
            }
            writer.print("                <env-entry-name>");
            writer.print(entry.getName());
            writer.println("</env-entry-name>");
            writer.print("                <env-entry-type>");
            writer.print(entry.getType());
            writer.println("</env-entry-type>");
            writer.print("                <env-entry-value><![CDATA[");
            writer.print(entry.getValue());
            writer.println("]]></env-entry-value>");
            writer.println("            </env-entry>");
        }
    }

    private static void attributes(DynaBean dynaBean, PrintWriter writer) {
        for (Iterator iterator = dynaBean.getDynaPropertyNames().iterator(); iterator.hasNext();) {
            String name = (String)iterator.next();
            Object value = dynaBean.get(name);
            if (value == null)
                continue;

            writer.println("            <attribute>");
            writer.print("                <attribute-name>");
            writer.print(name);
            writer.println("</attribute-name>");
            writer.print("                <attribute-type>");
            writer.print(value.getClass().getName());
            writer.println("</attribute-type>");
            writer.print("                <attribute-value><![CDATA[");
            writer.print(value.toString());
            writer.println("]]></attribute-value>");
            writer.println("            </attribute>");
        }
    }
}
