package org.concern.controller;

import junit.framework.TestCase;
import org.hibernate.cfg.Configuration;
import org.concern.*;
import org.concern.Log;
import org.hibernate.*;

import java.util.*;


/** @noinspection ALL*/
// NOTE: @noinspection ALL is to prevent IntelliJ IDEA from displaying
//       (many!) warnings if you happen to be in a project with 1.5 syntax
//       enabled.
public class ControllerTest extends TestCase
{
    Controller controller;
    SessionFactory sessionFactory;
    private boolean throwException;

    public ControllerTest(String name) {
      super(name);
    }

    protected void setUp()
        throws Exception
    {
        initializeController();

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        ScrollableResults results = session.createQuery("FROM Subject").scroll();
        while (results.next()) {
            Subject subject = (Subject)results.get()[0];
            session.delete(subject);
        }
        results = session.createQuery("FROM ArchiveLog").scroll();
        while (results.next()) {
            ArchiveLog archive = (ArchiveLog)results.get()[0];
            session.delete(archive);
        }
        transaction.commit();
        session.close();
    }

    protected void initializeController() {
        Configuration configuration = new Configuration();
        configuration.configure("controller-hibernate.cfg.xml");
        configuration.addResource("Controller.hbm.xml");

        sessionFactory = configuration.buildSessionFactory();
        controller = new Controller("ControllerTest");
        controller.setLoader(new DummyLoader());
        controller.setSessionFactory(sessionFactory);
    }

    protected void tearDown() throws Exception {
        for (Iterator iterator = controller.getSubjects(Controller.STATE_RUNNING).iterator(); iterator.hasNext();) {
            String userValue = (String)iterator.next();
            controller.destroySubject(userValue);
        }
        controller.timeout();
    }

    public void testCreateAndRemoveSubject()
        throws Exception
    {
        String userValue = "createAndRemoveSubject" + Thread.currentThread().getName();
        controller.createSubject(userValue);
        Subject subject = getSubject(userValue);
        assertNotNull(subject);
        assertEquals(userValue, subject.getUserValue());
        controller.destroySubject(userValue);
        controller.timeout();
        subject = getSubject(userValue);
        assertNull(subject);
    }

    private Subject getSubject(String userValue) throws Exception {
        controller.transactionManager.begin();
        List list = controller.sessions.getSession().createQuery("FROM " + Subject.class.getName() + " AS s WHERE s.userValue = '" + userValue + "'").list();
        Subject subject = list.size() == 1 ? (Subject)list.get(0) : null;
        if (subject != null) {
            subject.getLogs().size();
            subject.getEnlistments().size();
        }
        controller.transactionManager.commit();
        return subject;
    }

    public void testGetSubjects()
        throws Exception
    {
        controller.createSubject("getSubjects_1" + Thread.currentThread().getName());
        controller.createSubject("getSubjects_2" + Thread.currentThread().getName());
        List subjects = controller.getSubjects(Controller.STATE_RUNNING);
        int size = subjects.size();
        subjects.removeAll(Arrays.asList(new String[] { "getSubjects_1" + Thread.currentThread().getName(), "getSubjects_2" + Thread.currentThread().getName() }));
        assertEquals(2, size - subjects.size());
        controller.destroySubject("getSubjects_1" + Thread.currentThread().getName());
        controller.destroySubject("getSubjects_2" + Thread.currentThread().getName());
        controller.timeout();
    }

    public void testScheduleAnnouncement()
        throws Exception
    {
        String userValue = "scheduleAnnouncement" + Thread.currentThread().getName();
        controller.createSubject(userValue);
        controller.scheduleAnnouncement(userValue, 10);
        Subject subject = getSubject(userValue);
        Enlistment enlistment = (Enlistment)subject.getEnlistments().first();
        assertTrue(enlistment.getTimeout() <= System.currentTimeMillis() + 10000);
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testEnlistments()
        throws Exception
    {
        String userValue = "testEnlistment" + Thread.currentThread().getName();
        List activities = new LinkedList();
        activities.add(new DummyAsynchronousActivity("async_a"));
        activities.add(new DummyAsynchronousActivity("async_b"));
        activities.add(new DummySynchronousActivity("sync_c"));
        controller.setActivities(activities);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("async_a"));
        conditions.add(new DummyCondition("async_b"));
        conditions.add(new DummyCondition("sync_c"));
        controller.setConditions(conditions);

        controller.createSubject(userValue);
        controller.process(userValue);

        List enlistments = controller.getEnlistments(userValue);
        assertEquals("" + enlistments, 2, enlistments.size());

        enlistments = controller.getSubjects("async_a");
        assertEquals("" + enlistments, 1, enlistments.size());
        assertEquals(userValue, enlistments.get(0));
        controller.complete(userValue, "async_a");
        enlistments = controller.getSubjects("async_a");
        assertEquals(0, enlistments.size());

        enlistments = controller.getSubjects("async_b");
        assertEquals(1, enlistments.size());
        assertEquals(userValue, enlistments.get(0));
        controller.complete(userValue, "async_b");
        enlistments = controller.getSubjects("async_b");
        assertEquals(0, enlistments.size());

        enlistments = controller.getSubjects("sync_c");
        assertEquals(0, enlistments.size());
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testLogAndArchive()
        throws Exception
    {
        String userValue = "testLogAndArchive" + Thread.currentThread().getName();
        List activities = new LinkedList();
        activities.add(new DummyAsynchronousActivity("async_a"));
        activities.add(new DummyAsynchronousActivity("async_b"));
        activities.add(new DummySynchronousActivity("sync_c"));
        controller.setActivities(activities);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("async_a"));
        conditions.add(new DummyCondition("async_b"));
        conditions.add(new DummyCondition("sync_c"));
        controller.setConditions(conditions);

        controller.createSubject(userValue);
        controller.process(userValue);
        controller.complete(userValue, "async_a");
        controller.complete(userValue, "async_b");
        List logs = controller.getLog(userValue);
        controller.destroySubject(userValue);
        controller.timeout();
        List archives = controller.getArchive(userValue);

        assertEquals("" + logs, 4, logs.size());
        assertEquals("" + archives, 5, archives.size());
        for (int i=0; i < 3; i++) {
            Log log = (Log)logs.get(i);
            Log archive = (Log)archives.get(i);
            assertEquals(log.getUserValue(), archive.getUserValue());
            assertEquals(log.getMessage(), archive.getMessage());
            assertEquals(log.getActivity(), archive.getActivity());
            assertEquals(log.getTimestamp().getTime(), archive.getTimestamp().getTime());
            assertEquals(log.isSuccess(), archive.isSuccess());
        }
        System.out.println("archives = " + archives);
        Log archive = (Log)archives.get(archives.size() - 1);
        assertEquals(userValue, archive.getUserValue());
        assertEquals("DESTROYED", archive.getActivity());
        assertEquals(true, archive.isSuccess());
    }

    public void testExecuteSynchronous()
        throws Exception
    {
        String userValue = "testExecuteSynchronous" + Thread.currentThread().getName();
        List activities = new LinkedList();
        activities.add(new DummySynchronousActivity("sync_success"));
        activities.add(new DummySynchronousActivity("sync_exception") {
            public void execute(Object subject) throws ActivityExecutionException {
                throw new ActivityExecutionException("sync_exception");
            }

            public void escalate(Object subject) throws ActivityExecutionException {
                environment.put("activity.precondition", "!x");
            }

            public int getTrials() {
                return 2;
            }

            public int getRetryDelay() {
                return 0;
            }
        });
        activities.add(new DummySynchronousActivity("sync_postcondition") {
            public void setController(Controller controller) {
                super.setController(controller);
                environment.put("activity.postcondition", "!x");
            }

            public void escalate(Object subject) throws ActivityExecutionException {
                environment.put("activity.precondition", "!x");
            }

            public int getTrials() {
                return 2;
            }

            public int getRetryDelay() {
                return 3600;
            }
        });
        controller.setActivities(activities);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("sync_success"));
        conditions.add(new DummyCondition("sync_exception"));
        conditions.add(new DummyCondition("sync_postcondition"));
        conditions.add(new DummyCondition("x"));
        controller.setConditions(conditions);

        controller.createSubject(userValue);
        controller.process(userValue);
        List logs = controller.getLog(userValue);
        System.out.println("logs = " + logs);
        for (Iterator iterator = logs.iterator(); iterator.hasNext();) {
            Log log = (Log)iterator.next();
            if ("sync_success".equals((log.getActivity()))) {
                assertEquals(true, log.isSuccess());
                assertEquals("EXECUTED", log.getMessage());
            }
            else if ("sync_exception".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                if ("FAILED".equals(log.getMessage()))
                    assertEquals("sync_exception", log.getDetails());
                else
                    assertEquals("ESCALATED", log.getMessage());
            }
            else if ("sync_postcondition".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                if ("FAILED".equals(log.getMessage()))
                    assertTrue(log.getDetails().startsWith("postcondition - "));
                else
                    assertEquals("ESCALATED", log.getMessage());
            }
            else if (!"CREATED".equals(log.getActivity()) && !"STALLED".equals(log.getActivity()))
                fail(log.getActivity());
        }

        Subject subject = getSubject(userValue);
        SortedSet enlistments = subject.getEnlistments();
        for (Iterator iterator = enlistments.iterator(); iterator.hasNext();) {
            Enlistment enlistment = (Enlistment)iterator.next();
            if ("RETRY".equals(enlistment.getCommand()) && "sync_postcondition".equals(enlistment.getActivity())) {
                assertTrue(System.currentTimeMillis() + 3600000 > enlistment.getTimeout());
                assertTrue(System.currentTimeMillis() + 3500000 < enlistment.getTimeout());
            }
        }
        System.out.println("subject.getLogs() = " + subject.getLogs());
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testExecuteAsynchronous()
        throws Exception
    {
        String userValue = "testExecuteAsynchronous" + Thread.currentThread().getName();
        List activities = new LinkedList();
        activities.add(new DummyAsynchronousActivity("async_success"));
        activities.add(new DummyAsynchronousActivity("async_timeout") {
            {
                getEnvironment().put("activity.timeout", new Integer(0));
            }

            public void escalate(Object subject) throws ActivityExecutionException {
                environment.put("activity.precondition", "!x");
            }
        });
        activities.add(new DummyAsynchronousActivity("async_postcondition") {
            public void setController(Controller controller) {
                super.setController(controller);
                environment.put("activity.postcondition", "!x");
            }

            public void escalate(String subject) throws ActivityExecutionException {
                environment.put("activity.precondition", "!x");
            }
        });
        controller.setActivities(activities);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("async_success"));
        conditions.add(new DummyCondition("async_timeout"));
        conditions.add(new DummyCondition("async_postcondition"));
        conditions.add(new DummyCondition("x"));
        controller.setConditions(conditions);

        controller.createSubject(userValue);
        controller.process(userValue);
        boolean success = controller.complete(userValue, "async_success");
        assertTrue(success);
        success = controller.complete(userValue, "async_postcondition");
        assertFalse(success);
        controller.process(userValue);

        List logs = controller.getLog(userValue);
        for (Iterator iterator = logs.iterator(); iterator.hasNext();) {
            Log log = (Log)iterator.next();
            if ("async_success".equals((log.getActivity()))) {
                assertEquals(true, log.isSuccess());
                assertEquals("EXECUTED", log.getMessage());
            }
            else if ("async_timeout".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                assertEquals("ESCALATED", log.getMessage());
            }
            else if ("async_postcondition".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                assertEquals("FAILED", log.getMessage());
                assertEquals("NOT COMPLETE", log.getDetails());
            }
            else if (!"CREATED".equals(log.getActivity()))
                fail(log.getActivity());
        }
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testExecuteEvent()
        throws Exception
    {
        String userValue = "testExecuteEvent" + Thread.currentThread().getName();
        List events = new LinkedList();
        events.add(new DummyEvent("event_occured"));
        events.add(new DummyEvent("event_not_occured") {
            public void setController(Controller controller) {
                super.setController(controller);
                environment.put("event.postcondition", "!x");
            }
        });
        controller.setEvents(events);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("event_occured"));
        conditions.add(new DummyCondition("event_not_occured"));
        conditions.add(new DummyCondition("x"));
        controller.setConditions(conditions);

        controller.createSubject(userValue);
        boolean success = controller.notify(userValue, "event_occured");
        assertTrue(success);
        success = controller.notify(userValue, "event_not_occured");
        assertFalse(success);

        List logs = controller.getLog(userValue);
        boolean occured = false;
        for (Iterator iterator = logs.iterator(); iterator.hasNext();) {
            Log log = (Log)iterator.next();
            if ("event_occured".equals((log.getActivity()))) {
                assertEquals(true, log.isSuccess());
                assertEquals("EXECUTED", log.getMessage());
                occured = true;
            }
            else if ("event_not_occured".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                assertEquals("FAILED", log.getMessage());
                occured = true;
            }
            else if (!"CREATED".equals(log.getActivity()))
                fail(log.getActivity());
        }
        assertTrue(occured);
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testExecuteListener()
        throws Exception
    {
        String userValue = "testExecuteListener" + Thread.currentThread().getName();
        List listeners = new LinkedList();
        listeners.add(new DummyListener("listener_success"));
        listeners.add(new DummyListener("listener_exception") {
            public void notify(Object tsubject) throws ActivityExecutionException {
                throw new ActivityExecutionException("listener_exception");
            }

            public void escalate(Object subject) throws ActivityExecutionException {
                environment.put("listener.precondition", "!x");
            }
        });
        controller.setListeners(listeners);
        List conditions = new LinkedList();
        conditions.add(new DummyCondition("listener_success"));
        conditions.add(new DummyCondition("listener_exception"));
        conditions.add(new DummyCondition("x"));
        controller.setConditions(conditions);
        controller.setEvents(Collections.EMPTY_LIST);
        controller.setActivities(Collections.EMPTY_LIST);

        controller.createSubject(userValue);
        controller.process(userValue);

        List logs = controller.getLog(userValue);
        boolean notified = false;
        boolean failed = false;
        for (Iterator iterator = logs.iterator(); iterator.hasNext();) {
            Log log = (Log)iterator.next();
            if ("listener_success".equals((log.getActivity()))) {
                assertEquals(true, log.isSuccess());
                assertEquals("EXECUTED", log.getMessage());
                notified = true;
            }
            else if ("listener_exception".equals((log.getActivity()))) {
                assertEquals(false, log.isSuccess());
                if ("FAILED".equals(log.getMessage()))
                    assertEquals("listener_exception", log.getDetails());
                else
                    assertEquals("ESCALATED", log.getMessage());
                failed = true;
            }
            else if (!"CREATED".equals(log.getActivity()) && !"STALLED".equals(log.getActivity()))
                fail(log.getActivity());
        }
        assertTrue(notified);
        assertTrue(failed);
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testTemporalConditions()
        throws Exception
    {
        String userValue = "temporalConditions" + Thread.currentThread().getName();
        long millis = System.currentTimeMillis() + 1000;
        List conditions = new LinkedList();
        conditions.add(new DummyTemporalCondition("temporalCondition", false, millis));
        controller.setConditions(conditions);
        controller.setLoader(new DummyLoader());
        controller.setEvents(Collections.EMPTY_LIST);
        controller.setActivities(Collections.EMPTY_LIST);
        controller.setListeners(Collections.EMPTY_LIST);

        controller.createSubject(userValue);
        controller.process(userValue);

        Subject subject = getSubject(userValue);
        Enlistment enlistment = (Enlistment)subject.getEnlistments().first();
        assertEquals(millis, enlistment.getTimeout());

        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testTimerTask()
        throws Exception
    {
        controller.setTimerInterval(1000);
        controller.start();

        String userValue = "testTimerTask" + Thread.currentThread().getName();
        controller.createSubject(userValue);
        controller.announceSubject(userValue);

        Subject subject = getSubject(userValue);
        assertEquals(1, subject.getEnlistments().size());
        Thread.sleep(2000);
        subject = getSubject(userValue);
        assertEquals("" + subject.getEnlistments(), 0, subject.getEnlistments().size());
        controller.stop();
        controller.destroySubject(userValue);
        controller.timeout();
    }

    public void testSubjectNotFound()
        throws Exception
    {
        String userValue = "testSubjectNotFound" + Thread.currentThread().getName();
        throwException = false;
        controller.setLoader(new DummyLoader() {
            public Object load(String id) throws SubjectNotFoundException {
                if (throwException)
                    throw new SubjectNotFoundException(id);
                return id;
            }

            public String idOf(Object subject) throws SubjectNotFoundException {
                if (throwException)
                    throw new SubjectNotFoundException("" + subject);
                return subject.toString();
            }
        });
        List activities = new LinkedList();
        activities.add(new DummyAsynchronousActivity("async"));
        activities.add(new DummySynchronousActivity("sync"));
        controller.setActivities(activities);
        List events = new LinkedList();
        events.add(new DummyEvent("event"));
        controller.setEvents(events);
        List listeners = new LinkedList();
        listeners.add(new DummyListener("event"));
        controller.setListeners(listeners);

        controller.createSubject(userValue);
        throwException = true;
        try {
            controller.complete(userValue, "async");
            fail("expected UnknownSubjectException");
        }
        catch (ControllerException e) {
            assertEquals(userValue, e.getMessage());
        }

        try {
            controller.notify(userValue, "event");
            fail("expected UnknownSubjectException");
        }
        catch (ControllerException e) {
            assertEquals(userValue, e.getMessage());
        }

        controller.process(userValue);
        controller.destroySubject(userValue);
        controller.process(userValue);
        List archive = controller.getArchive(userValue);

        boolean load_failed = false;
        boolean suspended = false;
        boolean destroyed = false;
        for (Iterator iterator = archive.iterator(); iterator.hasNext();) {
            Log log = (Log)iterator.next();
            if ("LOAD".equals((log.getActivity())))
                load_failed = true;
            else if ("SUSPENDED".equals((log.getActivity())))
                suspended = true;
            else if ("DESTROYED".equals((log.getActivity())))
                destroyed = true;
        }
        assertTrue(load_failed);
        assertTrue(suspended);
        assertTrue(destroyed);
    }


  /**
   * Test bug: if the subject is dirtied (by mutating one of its collections)
   * during a transaction, and then that transaction is rolled back (as is the
   * case when a postcondition fails), then an exception was being thrown
   * when the transaction was started again and the subject was reassociated
   * with the Hibernate Session.
   * <p/>
   * Note: this is really a regression test.
   * 
   * @throws Exception
   */
  public void testDirtySubjectAfterRollback() throws Exception
  {
    System.out.println("-------------------------- testDirtySubjectAfterRollback() begin ---");
    final String userValue = "testDirtySubjectAfterRollback" + Thread.currentThread().getName();
    List activities = new LinkedList();

    final String activityName = "sync_failpostcondition";
    activities.add(new DummySynchronousActivity(activityName) {
      public void setController(Controller controller) {
        super.setController(controller);
        environment.put("activity.precondition", "x");
        environment.put("activity.postcondition", "!x");
      }
      public void execute(Object subject) {
        // Dirty the subject by adding an enlistment.

        try {
          controller.scheduleAnnouncement(userValue, 3600);
        } catch (Exception e) {
          e.printStackTrace();
          fail("Unexpected exception: " + e);
        }
      }
      public int getTrials() {
        return 2;
      }
      public int getRetryDelay() {
        return 3600;
      }
    });

    controller.setActivities(activities);
    List conditions = new LinkedList();
    conditions.add(new DummyCondition(activityName));
    conditions.add(new DummyCondition("x"));

    controller.setConditions(conditions);

    controller.createSubject(userValue);
    
    try {
      // The controller will attempt to execute the activity within a
      // transaction, our activity will dirty the subject, the postcondition
      // will fail, the Controller will rollback the transaction, start a
      // new transaction, and attempt to reassociate the Subject instance it
      // holds with the new transaction.  At that point is when an exception
      // was being thrown, and if it is now thrown, it shows the bug has
      // crept back into the system.
      controller.process(userValue);
    } catch(Throwable t) {
      t.printStackTrace();
      fail("controller.process should not have thrown exception: " + t);
    }

    Subject subject = getSubject(userValue);
    SortedSet enlistments = subject.getEnlistments();
    System.out.println("subject.getLogs() = " + subject.getLogs());
    System.out.println("subject.getEnlistments = " + enlistments);
    
    // We should have two enlistments: one from the scheduleAnnouncement
    // made by our excute() method in our synchronous activity and one for the
    // RETRY because our postcondition failed.
    assertEquals( 1, enlistments.size());
    boolean foundRetry = false;
    boolean foundEvaluate = false;
    for (Iterator iterator = enlistments.iterator(); iterator.hasNext();) {
        Enlistment enlistment = (Enlistment)iterator.next();
        if ("RETRY".equals(enlistment.getCommand()) && activityName.equals(enlistment.getActivity())) {
            foundRetry = true;
        } else if("EVALUATE".equals(enlistment.getCommand())) {
            foundEvaluate = true;
        } else {
          fail("Unexpected enlistment found for subject: " + enlistment);
        }

        assertTrue(System.currentTimeMillis() + 3600000 >= enlistment.getTimeout());
        assertTrue(System.currentTimeMillis() + 3500000 < enlistment.getTimeout());
    }

    assertTrue(foundRetry);
    assertFalse(foundEvaluate);

    controller.destroySubject(userValue);
    controller.timeout();
    System.out.println("-------------------------- testRollbackBecauseOfFailedPostcondition() end ---");
  }

    // TODO: test for actors
}
