/*
 * $Id: MappingTest.java 819 2007-01-18 07:59:50Z hengels $
 * (c) Copyright 2004 con:cern development team.
 *
 * This file is part of con:cern (http://concern.org).
 *
 * con:cern is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * Please see COPYING for the complete licence.
 */
package org.concern.controller;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;

import java.sql.Timestamp;
import java.util.*;


public class MappingTest extends TestCase
{
    private SessionFactory sessionFactory;
    private Subject subject;
    private ArchiveSubject archiveSubject;
    private long millis = System.currentTimeMillis();

    public MappingTest(String name) {
      super(name);
    }

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.setNamingStrategy(new org.concern.controller.Configuration.PrefixNamingStrategy("MT"));
        configuration.configure("/controller-hibernate.cfg.xml");
        configuration.addResource("Controller.hbm.xml");
        sessionFactory = configuration.buildSessionFactory();

        subject = new Subject("subject");
        subject.setProcess("ControllerTest");
        subject.setSubjectLines(new HashSet(Arrays.asList(new SubjectLine[] {
            new SubjectLine("de", "hallo"),
            new SubjectLine("en", "hello"),
        })));
        subject.setLogs(Arrays.asList(new Log[] {
            new Log("activity1", new Timestamp(millis + 3600000)),
            new Log("activity2", new Timestamp(millis - 1000)),
            new Log("activity3", new Timestamp(millis - 2000)),
        }));
        Enlistment firstEnlistment;
        subject.setEnlistments(new TreeSet(Arrays.asList(new Enlistment[] {
            firstEnlistment = new Enlistment("TIMEOUT", "activity1", millis + 3600000),
            new Enlistment("EXECUTE", "activity2", millis - 1000),
            new Enlistment("EXECUTE", "activity3", millis - 2000),
        })));
        firstEnlistment.setAssignments(new HashSet(Arrays.asList(new Assignment[] {
            new Assignment("you"),
            new Assignment("me"),
        })));

        archiveSubject = new ArchiveSubject("archiveSubject");
        archiveSubject.setProcess("ControllerTest");
        archiveSubject.setSubjectLines(new HashSet(Arrays.asList(new ArchiveSubjectLine[] {
            new ArchiveSubjectLine("de", "hallo"),
            new ArchiveSubjectLine("en", "hello"),
        })));
        archiveSubject.setLogs(Arrays.asList(new ArchiveLog[] {
            new ArchiveLog("activity1", new Timestamp(millis + 3600000)),
            new ArchiveLog("activity2", new Timestamp(millis - 1000)),
            new ArchiveLog("activity3", new Timestamp(millis - 2000)),
        }));

        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("FROM org.concern.controller.Subject subject");
        List list = query.list();
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            Subject subject = (Subject)iterator.next();
            System.out.println("deleting subject = " + subject);
            session.delete(subject);
        }
        session.save(subject);
        session.save(archiveSubject);
        transaction.commit();
        session.close();
    }

    protected void tearDown() throws Exception {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("FROM org.concern.controller.Subject subject");
        List list = query.list();
        for (Iterator iterator = list.iterator(); iterator.hasNext();) {
            Subject subject = (Subject)iterator.next();
            System.out.println("deleting subject = " + subject);
            session.delete(subject);
        }
        transaction.commit();
        session.close();

        sessionFactory.close();
    }

    public void testMapping()
        throws Exception
    {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Subject loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(2, loaded.getSubjectLines().size());
        assertEquals(3, loaded.getLogs().size());
        assertEquals(3, loaded.getEnlistments().size());
        assertEquals(2, loaded.getTimeoutEnlistment("activity1").getAssignments().size());
        Iterator iterator = loaded.getEnlistments().iterator();

        assertEquals(millis - 2000, ((Enlistment)iterator.next()).getTimeout());
        assertEquals(millis - 1000, ((Enlistment)iterator.next()).getTimeout());
        assertEquals(millis + 3600000, ((Enlistment)iterator.next()).getTimeout());
        loaded.getEnlistments().add(new Enlistment("EXECUTE", "activity1", millis + 3600000));
        //loaded.getAssignments().add(new Assignment("watch", "others", new Timestamp(millis + 3600000)));
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(3, loaded.getLogs().size());
        assertEquals(4, loaded.getEnlistments().size());
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        ArchiveSubject archiveLoaded = (ArchiveSubject)session.load(ArchiveSubject.class, archiveSubject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(2, archiveLoaded.getSubjectLines().size());
        assertEquals(3, archiveLoaded.getLogs().size());
        transaction.commit();
        session.close();
    }

    public void testTimeoutQuery()
        throws Exception
    {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("select subject.userValue from org.concern.controller.Subject subject" +
            " join subject.enlistments enlistment" +
            " where enlistment.timeout < :now");
        query.setLong("now", millis);
        Set userValues = new HashSet();
        for (Iterator iterator = query.iterate(); iterator.hasNext();)
            userValues.add(iterator.next());

        assertEquals(1, userValues.size());

        transaction.commit();
        session.close();
    }

    public void testEnlistmentQuery()
        throws Exception
    {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("select subject.userValue from org.concern.controller.Subject subject" +
            " join subject.enlistments enlistment" +
            " where enlistment.activity = :activity");
        query.setString("activity", "activity1");
        assertEquals(1, query.list().size());

        transaction.commit();
        session.close();
    }

    public void testEnlistment()
        throws Exception
    {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Subject loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        for (Iterator iterator = loaded.getEnlistments().iterator(); iterator.hasNext();) {
            Enlistment enlistment = (Enlistment)iterator.next();
            if ("activity2".equals(enlistment.getActivity()))
                iterator.remove();
        }
        session.update(loaded);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(2, loaded.getEnlistments().size());
        Enlistment enlistment = new Enlistment("activity2", millis);
        loaded.getEnlistments().add(enlistment);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(3, loaded.getEnlistments().size());
        transaction.commit();
        session.close();
    }

    public void testAssignmentQuery()
        throws Exception
    {
        Session session = sessionFactory.openSession();
        Transaction transaction = session.beginTransaction();
        Query query = session.createQuery("select subject.userValue from org.concern.controller.Subject subject" +
            " inner join subject.enlistments as enlistment" +
            " inner join enlistment.assignments as assignment" +
            " where assignment.assignee = :assignee");
        query.setString("assignee", "me");
        assertEquals(1, query.list().size());
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        Subject loaded = (Subject)session.load(Subject.class, subject.getId(), LockMode.UPGRADE_NOWAIT);
        assertEquals(2, loaded.getTimeoutEnlistment("activity1").getAssignments().size());
        Assignment assignment = new Assignment("others");
        loaded.getTimeoutEnlistment("activity1").getAssignments().add(assignment);
        transaction.commit();
        session.close();

        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        query = session.createQuery("select subject.userValue from org.concern.controller.Subject subject" +
            " inner join subject.enlistments as enlistment" +
            " inner join enlistment.assignments as assignment" +
            " where assignment.assignee = :assignee");
        query.setString("assignee", "others");
        assertEquals(1, query.list().size());
        transaction.commit();
        session.close();
    }
}
