package org.concern.model.cpd;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.concern.model.Activity;
import org.concern.model.Actor;
import org.concern.model.AsynchronousActivity;
import org.concern.model.Collaborator;
import org.concern.model.Condition;
import org.concern.model.Event;
import org.concern.model.Interaction;
import org.concern.model.Listener;
import org.concern.model.Loader;
import org.concern.model.Process;
import org.concern.model.SynchronousActivity;

public class CPDFilePersistenceTest extends TestCase
{
    private Process process;
    private Actor actor;
    private Collaborator collaborator;
    private Condition condition_a;
    private Condition condition_b;
    private Activity activity_A;
    private Activity activity_B;
    private Activity activity_C;
    private Event event_E;
    private Event event_F;
    private Listener listener;
    private Loader loader;

    public CPDFilePersistenceTest(String name) {
      super(name);
   }

   protected void setUp()
       throws Exception
   {
       actor = new Actor("W");
       actor.setImplementation("test.Actor");
       collaborator = new Collaborator("C");
       collaborator.setCardinality(Collaborator.MANY_TO_ONE);

       condition_a = new Condition("a");
       condition_a.setImplementation("test.Condition$A");
       condition_b = new Condition("b");
       condition_b.setImplementation("test.Condition$B");
       condition_b.setTemporal(true);

       activity_A = new AsynchronousActivity("A");
       activity_A.setImplementation("test.Activity$A");
       activity_A.setPrecondition("!a");
       activity_A.setPostcondition("a");
       ((AsynchronousActivity)activity_A).setActor("W");
       ((AsynchronousActivity)activity_A).setTimeout(3600);
       activity_A.setInteraction(new Interaction(Interaction.IN, "stop", "C"));

       activity_B = new AsynchronousActivity("B");
       activity_B.setImplementation("test.Activity$B");
       activity_B.setPrecondition("!a");
       activity_B.setPostcondition("a");
       ((AsynchronousActivity)activity_B).setOptional(true);
       activity_B.setInteraction(new Interaction(Interaction.BOTH, "communicate", "C"));

       activity_C = new SynchronousActivity("C");
       activity_C.setImplementation("test.Activity$C");
       activity_C.setPrecondition("a && !b");
       activity_C.setPostcondition("b");
       ((SynchronousActivity)activity_C).setRetryDelay(3600);
       ((SynchronousActivity)activity_C).setTrials(3);
       activity_C.setInteraction(new Interaction(Interaction.OUT, "start", "C"));

       event_E = new Event("E");
       event_E.setImplementation("test.Event$E");
       event_E.setPostcondition("!a");
       event_E.setInteraction(new Interaction(Interaction.IN, "listen", "C"));

       event_F = new Event("E");
       event_E.setImplementation("test.Event$F");
       event_F.setPostcondition("!a");
       event_F.setUser(true);

       listener = new Listener("L");
       listener.setImplementation("test.Listener");
       listener.setPrecondition("b");
       listener.setInteraction(new Interaction(Interaction.OUT, "notify", "C"));

       loader = new Loader("S");

       process = new Process("Test");
       process.setSubject("test.Subject");
       process.setLoader(loader);
       process.addActor(actor);
       process.addCollaborator(collaborator);
       process.addCondition(condition_a);
       process.addCondition(condition_b);
       process.addActivity(activity_A);
       process.addActivity(activity_B);
       process.addActivity(activity_C);
       process.addEvent(event_E);
       process.addEvent(event_F);
       process.addListener(listener);

   }

    public void testPersistence()
        throws Exception
    {
        File file = File.createTempFile("dji-concern", ".cpd");
        ProcessWriter.save(process, file);

        Process reloaded = ProcessLoader.load(file);

        /*
        assertEquals(loader, reloaded.getInteraction());
        assertEquals(actor.getName(), reloaded.getActor("W").getName());
        assertEquals(collaboration, reloaded.getActor("C"));
        */

        compareBeans("process", process, reloaded);
    }

    public void compareBeans(String name, Object bean, Object with) throws Exception {
        String className = (bean != null) ? bean.getClass().getName() : (with != null) ? with.getClass().getName() : "";
        System.out.println(name + "[" + className + "]: " + bean + " " + with);

        if (bean instanceof Collection) {
            Collection beanCollection = (Collection)bean;
            Collection withCollection = (Collection)with;
            assertEquals(name, beanCollection.size(), withCollection.size());

            Iterator beanIterator = beanCollection.iterator();
            Iterator withIterator = withCollection.iterator();
            int i = 0;
            while (beanIterator.hasNext()) {
                Object beanMember = beanIterator.next();
                Object withMember = withIterator.next();
                compareBeans(name + "[" + i++ + "]", beanMember, withMember);
            }
        }
        else if (bean == null) {
            assertNull(name, with);
        }
        else {
            PropertyDescriptor[] descriptors = Introspector.getBeanInfo(bean.getClass(), Object.class).getPropertyDescriptors();
            if (bean.getClass().getName().startsWith("java.lang") || descriptors.length == 0) {
                assertEquals(name, bean, with);
            }
            else {
                for (int i = 0; i < descriptors.length; i++) {
                    PropertyDescriptor descriptor = descriptors[i];
                    if (descriptor.getReadMethod() == null || descriptor.getPropertyType().getName().startsWith("["))
                        continue;
                    
                    Object beanValue = descriptor.getReadMethod().invoke(bean, null);
                    try {
                    	Object withValue = descriptor.getReadMethod().invoke(with, null);
                    	compareBeans(name + "." + descriptor.getName(), beanValue, withValue);
                    } catch( IllegalArgumentException ex ) {
                        System.out.println();
                        System.out.println( "dji ******************** " + name + "[" + className + "]: " + bean + " " + with);
                        System.out.println( "dji ******************** bean class: " + bean.getClass().getName() );
                        System.out.println( "dji ******************** bean descriptor: " + descriptor.getName() );
                        System.out.println( "dji ******************** bean value: " + beanValue );
                        System.out.println( "dji ******************** with class: " + with.getClass().getName() );
                        System.out.println();
                        throw ex;
                    }
                }
            }
        }
    }
}
