/* $Id: AbstractBeanData.java 812 2007-01-15 20:48:12Z hengels $ */
package org.conform;

import java.util.*;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

/**
 * @version $Revision: 812 $
 */
public abstract class AbstractBeanData
    extends AbstractData
    implements BeanData
{
    BeanMeta beanMeta;
    Map<PropertyMeta, PropertyData> propertyDatas = new HashMap<PropertyMeta, PropertyData>();

    PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
    ValidationListener forwarder = new ValidationListener() {
        public void addIssues(ValidationEvent event) {
            fireAddIssue(event);
        }

        public void removeIssues(ValidationEvent event) {
            fireRemoveIssue(event);
        }

        public void clearIssues(Meta meta) {
        }
    };

    protected AbstractBeanData(BeanMeta beanMeta) {
        this.beanMeta = beanMeta;

        for (PropertyMeta propertyMeta : this.beanMeta.getProperties()) {
            PropertyData propertyData = doGetPropertyData(propertyMeta);

            propertyDatas.put(propertyMeta, propertyData);
            propertyData.addValidationListener(forwarder);
            propertyData.setIssuePublishingActive(true);

            PropertyChangeListener listener = (PropertyChangeListener)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_CHANGE_LISTENER);
            if (listener != null)
                addPropertyChangeListener(propertyMeta.getName(), listener);
        }
    }

    public PropertyData getPropertyData(PropertyMeta propertyMeta) {
        return propertyDatas.get(propertyMeta);
    }

    protected abstract PropertyData doGetPropertyData(PropertyMeta propertyMeta);

    public BeanMeta getBeanMeta() {
        return beanMeta;
    }

    public Meta getMeta() {
        return beanMeta;
    }

    public Object getValue() {
        return doGetValue();
    }

    protected abstract Object doGetValue();

    public void setValue(Object bean) {
        Object oldBean = doGetValue();
        if (oldBean != null) {
            for (PropertyData propertyData : propertyDatas.values()) {
                if (propertyData.getPropertyMeta().isApplicable())
                    changeSupport.firePropertyChange(propertyData.getPropertyMeta().getName(), propertyData.getValue(), null);
            }
        }

        doSetValue(validate(bean));

        if (bean != null) {
            for (PropertyData propertyData : propertyDatas.values()) {
                PropertyMeta propertyMeta = propertyData.getPropertyMeta();
                if (propertyMeta.isApplicable() && propertyMeta.isReadable() && propertyMeta.isWritable())
                    propertyData.revalidate();

                propertyData.unsetInvalidValue();

                if (propertyMeta.isApplicable() && propertyMeta.isReadable())
                    changeSupport.firePropertyChange(propertyMeta.getName(), null, propertyData.getValue());
            }
        }
    }

    protected abstract void doSetValue(Object bean);

    public void revalidate() {
        try {
            Object value = doGetValue();
            validate(value);
        }
        catch (ValidationException e) {}
    }

    public void revalidateAll() {
        bufferedIssues.clear();
        if (isIssuePublishingActive()) {
            setIssuePublishingActive(false);
            setIssuePublishingActive(true);
        }
        revalidate();
        for (PropertyData propertyData : propertyDatas.values()) {
            PropertyMeta propertyMeta = propertyData.getPropertyMeta();
            if (propertyMeta.isApplicable() && propertyMeta.isReadable() && propertyMeta.isWritable())
                propertyData.revalidate();
        }
    }

    public Object validate(Object value) throws ValidationException {
        if (value == null)
            return null;

        ValidationEvent addEvent = new ValidationEvent(this);
        ValidationEvent removeEvent = new ValidationEvent(this);
        List<ValidationException.Message> messages = new LinkedList<ValidationException.Message>();

        Collection validators = beanMeta.getValidators();
        if (validators != null) {
            Iterator iter = validators.iterator();
            while (iter.hasNext()) {
                BeanValidator validator = (BeanValidator)iter.next();
                try {
                    if (arePropertiesValid(validator.getAffectedPropertyNames()))
                        value = validator.validate(value);

                    removeEvent.addIssue(new Issue(System.identityHashCode(validator), properties(validator.getAffectedPropertyNames())));
                }
                catch (ValidationException e) {
                    messages.addAll(e.getMessages());
                    for (ValidationException.Message message : e.getMessages()) {
                        addEvent.addIssue(new Issue(System.identityHashCode(validator), properties(validator.getAffectedPropertyNames()), message.getCode(), message.getArguments()));
                    }
                }
            }
        }

        // fireRemoveIssue sends the issue to be removed
        if (removeEvent.getIssues().size() != 0) {
            fireRemoveIssue(removeEvent);
        }

        // fireAddIssue sends an update of the issue
        if (addEvent.getIssues().size() != 0) {
            fireAddIssue(addEvent);
            throw new ValidationException(messages);
        }

        return value;
    }

    private boolean arePropertiesValid(String[] propertyNames) {
        for (String name : propertyNames) {
            PropertyMeta propertyMeta = beanMeta.getProperty(name);
            PropertyData propertyData = propertyDatas.get(propertyMeta);
            if (propertyData.isInvalid())
                return false;
        }
        return true;
    }

    private PropertyMeta[] properties(String[] affectedPropertyNames) {
        List<PropertyMeta> list = new ArrayList<PropertyMeta>(affectedPropertyNames.length);
        for (String name : affectedPropertyNames) {
            list.add(beanMeta.getProperty(name));
        }
        return list.toArray(new PropertyMeta[affectedPropertyNames.length]);
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.addPropertyChangeListener(listener);
    }
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        changeSupport.removePropertyChangeListener(listener);
    }
    public PropertyChangeListener[] getPropertyChangeListeners() {
        return changeSupport.getPropertyChangeListeners();
    }
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        changeSupport.addPropertyChangeListener(propertyName, listener);
    }
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        changeSupport.removePropertyChangeListener(propertyName, listener);
    }
    public PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        return changeSupport.getPropertyChangeListeners(propertyName);
    }

    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        changeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }

    public void initialize() {
        for (PropertyData propertyData : propertyDatas.values()) {
            if (propertyData.getPropertyMeta().isApplicable()) {
                propertyData.initialize();
            }
        }
    }

    public void update() {
        for (PropertyData propertyData : propertyDatas.values()) {
            if (propertyData.getPropertyMeta().isApplicable()) {
                propertyData.update();
            }
        }
    }

    public void flush() {
        for (PropertyData propertyData : propertyDatas.values()) {
            if (propertyData.getPropertyMeta().isApplicable()) {
                propertyData.flush();
            }
        }
    }

    public void clear() {
        for (PropertyData propertyData : propertyDatas.values()) {
            if (propertyData.getPropertyMeta().isApplicable()) {
                propertyData.clear();
            }
        }
    }

    public String toString() {
        return getMeta().getName();
    }
}
