/* $Id: AbstractPropertyData.java 812 2007-01-15 20:48:12Z hengels $ */
package org.conform;

import java.util.*;

/**
 * @version $Revision: 812 $
 */
public abstract class AbstractPropertyData
    extends AbstractData
    implements PropertyData
{
    PropertyMeta propertyMeta;
    AbstractBeanData beanData;
    Object invalidValue;
    boolean invalid = true;
    private final Issue requiredIssue;

    protected AbstractPropertyData(AbstractBeanData beanData, PropertyMeta propertyMeta) {
        this.beanData = beanData;
        this.propertyMeta = propertyMeta;
        requiredIssue = new Issue(0, this.propertyMeta, "validation.required");
    }

    public PropertyMeta getPropertyMeta() {
        return propertyMeta;
    }
    
    public Meta getMeta() {
        return propertyMeta;
    }

    public BeanData getBeanData() {
        return beanData;
    }

    public void setValue(Object value) {
        Object oldValue = getValue();
        if (!isDifferent(oldValue, value)) {
            return;
        }
        
        try {
            if (propertyMeta.isWritable())
                value = validate(value);

            doSetValue(value);
            unsetInvalidValue();
        }
        catch (ValidationException e) {
            setInvalidValue(value);
        }
        beanData.changeSupport.firePropertyChange(propertyMeta.getName(), oldValue, value);
    }

    private boolean isDifferent(Object oldObject, Object newObject) {
        if (oldObject == newObject)
            return false;

        if (oldObject == null)
            return true;

        return !oldObject.equals(newObject);
    }

    public Object getValue() {
        return isInvalid() ? getInvalidValue() : doGetValue();
    }

    public Object getInvalidValue() {
        return invalidValue;
    }

    public void setInvalidValue(Object invalidValue) {
        this.invalidValue = invalidValue;
        invalid = true;
    }

    public void unsetInvalidValue() {
        invalidValue = null;
        invalid = false;
    }

    public boolean isInvalid() {
        return invalid;
    }

    public void revalidate() {
        try {
            Object value = doGetValue();
            doSetValue(validate(value));
        }
        catch (ValidationException e) {}
    }

    public Object validate(Object value) throws ValidationException {
        ValidationEvent addEvent = new ValidationEvent(this);
        ValidationEvent removeEvent = new ValidationEvent(this);
        List<ValidationException.Message> messages = new LinkedList<ValidationException.Message>();

        if (propertyMeta.isMandatory() && propertyMeta.isWritable()) {
            if (value == null) {
                addEvent.addIssue(requiredIssue);
                messages.add(new ValidationException.Message("validation.required"));
            }
            else {
                removeEvent.addIssue(requiredIssue);
            }
        }

        Collection validators = propertyMeta.getValidators();
        if (validators != null) {
            Iterator iter = validators.iterator();
            while (iter.hasNext()) {
                Validator validator = (Validator) iter.next();
                try {
                    value = validator.validate(value);
                    removeEvent.addIssue(new Issue(System.identityHashCode(validator), propertyMeta));
                }
                catch (ValidationException e) {
                    messages.addAll(e.getMessages());
                    for (ValidationException.Message message : e.getMessages()) {
                        addEvent.addIssue(new Issue(System.identityHashCode(validator), propertyMeta, message.getCode(), message.getArguments()));
                    }
                }
            }
        }

        // fireRemoveIssue sends the issue to be removed
        if (removeEvent.getIssues().size() != 0) {
            fireRemoveIssue(removeEvent);
        }

        // fireAddIssue sends an update of the issue
        if (addEvent.getIssues().size() != 0) {
            fireAddIssue(addEvent);
            throw new ValidationException(messages);
        }

        return value;
    }

    public void initialize() {
        Initializer initializer = propertyMeta.getInitializer();
        if (initializer != null) {
            Object oldValue = getValue();
            Object value = initializer.getValue();
            doSetValue(value);
            unsetInvalidValue();
            beanData.changeSupport.firePropertyChange(propertyMeta.getName(), oldValue, value);
        }
        unsetInvalidValue();
    }

    public void update() {
        unsetInvalidValue();
        doSetValue(doGetValue());
    }

    public void clear() {
    }

    public void flush() {
    }

    protected abstract void doSetValue(Object value);
    protected abstract Object doGetValue();
}
