package org.conform;

import java.util.*;
import java.beans.*;
import java.lang.reflect.Method;

import org.conform.format.Format;

/**
 * Holds meta information of a bean class.
 */
public class BeanMeta
    implements Meta
{
    // the name of the property (field name)
    protected String name;
    // the type of the property (e.g. String, Long...)
    protected Class type;
    // specifies if the bean is applicable
    protected boolean applicable = true;
    // any additional attributes, if any
    protected Map attributes;
    // a collection of validators for the field (e.g. max length of a string 20)
    protected DomainProvider domainProvider;
    protected Collection validators;
    // the format which must be suported by the field
    protected Format format;

    // the localized label for this field (localization takes place in a modifier)
    private String label;
    // the localized micro helper for this field (localization takes place in a modifier)
    private String microHelp;

    private PropertyMeta[] properties;
    private transient Map propertyByName;

    public BeanMeta(Class type) {
        this.type = type;
        this.name = type.getName();

        try {
            BeanInfo info = Introspector.getBeanInfo(type, Object.class);
            PropertyDescriptor[] descriptors = info.getPropertyDescriptors();
            List propertyList = new LinkedList();
            for (int i = 0; i < descriptors.length; i++) {
                PropertyDescriptor descriptor = descriptors[i];
                Method getter = descriptor.getReadMethod();
                Method setter = descriptor.getWriteMethod();

                if (getter == null)
                    continue;

                PropertyMeta property = new PropertyMeta(this, descriptor.getName(), descriptor.getPropertyType());
                property.setWritable(setter != null);
                propertyList.add(property);
            }
            setProperties((PropertyMeta[])propertyList.toArray(new PropertyMeta[propertyList.size()]));
        }
        catch (IntrospectionException e) {
            throw new RuntimeException(e);
        }
    }

    public String getName() { return name; }

    public void setName(String name) {
        this.name = name;
    }

    public Class getType() {
        return type;
    }

    public void setType(Class type) {
        this.type = type;
    }

    public boolean isApplicable() {
        return applicable;
    }

    public void setApplicable(boolean applicable) {
        this.applicable = applicable;
    }

    public DomainProvider getDomainProvider() {
        return domainProvider;
    }

    public void setDomainProvider(DomainProvider domainProvider) {
        this.domainProvider = domainProvider;
    }

    public Collection getValidators() {
        return validators;
    }

    public void setValidators(Collection validators) {
        this.validators = validators;
    }

    public void addValidator(BeanValidator validator) {
        if (validators == null) {
            validators = new ArrayList();
        }
        validators.add(validator);
    }

    public Format getFormat() {
        return format;
    }

    public void setFormat(Format format) {
        this.format = format;
    }

    public void setAttribute(String name, Object value) {
        if (attributes == null)
            attributes = new HashMap();

        attributes.put(name, value);
    }

    public Object getAttribute(String name) {
        if (attributes == null)
            return null;

        return attributes.get(name);
    }

    public Map getAttributes() {
        return attributes;
    }

    public void setAttributes(Map attributes) {
        this.attributes = attributes;
    }

    public PropertyMeta getProperty(String name) {
        if (name == null)
            return null;

        if (propertyByName == null) {
            propertyByName = new HashMap();
            for (int i = 0; i < properties.length; i++) {
                PropertyMeta property = properties[i];
                propertyByName.put(property.getName(), property);
            }
        }
        return (PropertyMeta)propertyByName.get(name);
    }

    public PropertyMeta[] getProperties() {
        return properties;
    }

    public void setProperties(PropertyMeta[] properties) {
        this.properties = properties;
        this.propertyByName = null;
    }

    public String getMicroHelp() {
        return microHelp;
    }

    public void setMicroHelp(String microHelp) {
        this.microHelp = microHelp;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String parameterString() {
        StringBuffer buffer = new StringBuffer(name.length() + 10);
        buffer.append(name);
        buffer.append(": ");
        buffer.append(format);
        buffer.append(' ');
        buffer.append('\n');
        for (Iterator iter = validators.iterator(); iter.hasNext();) {
            Validator validator = (Validator) iter.next();
            buffer.append(" ");
            buffer.append(validator);
        }
        buffer.append('\n');
        for (int i = 0; i < properties.length; i++) {
            PropertyMeta propertyMeta = properties[i];
            buffer.append("  ");
            buffer.append(propertyMeta.parameterString());
            buffer.append('\n');
        }
        return buffer.toString();
    }

    public Object clone() {
        try {
            BeanMeta beanMeta = (BeanMeta)super.clone();
            if (attributes != null)
                beanMeta.attributes = new HashMap(attributes);

            if (format != null)
                beanMeta.format = (Format)format.clone();

            beanMeta.properties = new PropertyMeta[properties.length];
            beanMeta.propertyByName = new HashMap();
            for (int i = 0; i < beanMeta.properties.length; i++) {
                PropertyMeta propertyMeta = (PropertyMeta)properties[i].clone();
                beanMeta.properties[i] = propertyMeta;
                propertyMeta.beanMeta = beanMeta;
                beanMeta.propertyByName.put(propertyMeta.name, propertyMeta);
            }

            return beanMeta;
        }
        catch (CloneNotSupportedException e) {
            throw new RuntimeException(e);
        }
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final BeanMeta beanMeta = (BeanMeta)o;

        if (!name.equals(beanMeta.name)) return false;
        if (!type.equals(beanMeta.type)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = name.hashCode();
        result = 29 * result + type.hashCode();
        return result;
    }

    public String toString() {
        return name;
    }
}
