package org.conform;

import java.util.*;

import org.conform.format.Format;

/**
 * Holds meta information of a property.
 */
public class PropertyMeta
    implements Meta
{
    public static final int NO_RELATION = 0;
    public static final int ONE_TO_ONE_RELATION = 1;
    public static final int ONE_TO_MANY_RELATION = 2;
    public static final int MANY_TO_MANY_RELATION = 3;
    public static final int MANY_TO_ONE_RELATION = 4;
    public static final int DERIVATE = 5;
    public static final int VALUE = 6;

    public static final String ATTRIBUTE_LENGTH = "length";
    public static final String ATTRIBUTE_PRECISION = "precision";
    public static final String ATTRIBUTE_SCALE = "scale";
    public static final String ATTRIBUTE_ALIGNMENT = "alignment";
    public static final String ATTRIBUTE_IMPORTANT = "important";
    public static final String ATTRIBUTE_IDENTIFIER = "identifier";
    public static final String ATTRIBUTE_PATTERN = "pattern";
    public static final String ATTRIBUTE_PATTERN_KEY = "patternKey";
    public static final String ATTRIBUTE_CHANGE_LISTENER = "changeListener";

    // the BeanMeta, this PropertyMeta belongs to
    protected BeanMeta beanMeta;
    // the name of the property (field name)
    protected String name;
    // the type of the property (e.g. String, Long...)
    protected Class type;
    // specifies if the property is mandatory
    protected boolean mandatory;
    // specifies if the property is readonly
    protected boolean writable = true;
    // specifies if the property is hidden
    protected boolean readable = true;
    // specifies if the property is applicable
    protected boolean applicable = true;
    // initializer
    protected Initializer initializer;
    // any additional attributes, if any
    protected Map attributes;

    // a collection of validators for the field (e.g. max length of a string 20)
    protected Collection validators;
    // the format which must be suported by the field
    protected Format format;
    // the type of relation of this property meta with other property meta
    private int relationType = NO_RELATION;
    // the beanMeta to which this property meta has a relation to 
    private BeanMeta relationBean;
    private PropertyMeta relationProperty;
    // used to eventually sort the bean meta fields in the frontend based on priority 
    private int priority = 0;
    protected DomainProvider domainProvider;
    // the localized label for this field (localization takes place in a modifier)
    private String label;
    // the localized micro helper for this field (localization takes place in a modifier)
    private String microHelp;

    private static Map wrapperTypeMap = new HashMap();

    static {
        wrapperTypeMap.put(boolean.class, Boolean.class);
    }

    PropertyMeta(String name, Class type) {
        this(null, name, type);
    }

    public PropertyMeta(BeanMeta beanMeta, String name, Class type) {
        this.beanMeta = beanMeta;
        this.name = name;
        this.type = type;
        this.mandatory = type.isPrimitive();
        
        if (name == null)
            throw new IllegalArgumentException("null not allowed for name");
        if (type == null)
            throw new IllegalArgumentException("null not allowed for type");
    }

    public BeanMeta getBeanMeta() {
        return beanMeta;
    }

    public String getName() { return name; }

    public void setName(String name) {
        this.name = name;
    }

    public Class getType() {
        return type;
    }

    public void setType(Class type) {
        this.type = type;
    }

    public boolean isMandatory() {
        return mandatory;
    }

    public void setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
    }

    public boolean isWritable() {
        return writable;
    }

    public void setWritable(boolean writable) {
        this.writable = writable;
    }

    public boolean isReadable() {
        return readable;
    }

    public void setReadable(boolean readable) {
        this.readable = readable;
    }

    public boolean isApplicable() {
        return applicable;
    }

    public void setApplicable(boolean applicable) {
        this.applicable = applicable;
    }

    public Initializer getInitializer() {
        return initializer;
    }

    public void setInitializer(Initializer initializer) {
        this.initializer = initializer;
    }

    public Collection getValidators() {
        return validators;
    }

    public void setValidators(Collection validators) {
        this.validators = validators;
    }

    public void addValidator(Validator validator) {
        if (validator == null)
            throw new IllegalArgumentException("null not allowed");

        if (validators == null) {
            validators = new ArrayList();
        }
        validators.add(validator);
    }

    public Format getFormat() {
        return format;
    }

    public void setFormat(Format format) {
        this.format = format;
    }

    public DomainProvider getDomainProvider() {
        return domainProvider;
    }

    public void setDomainProvider(DomainProvider domainProvider) {
        this.domainProvider = domainProvider;
    }

    public int getRelationType() {
        return relationType;
    }

    public void setRelationType(int relationType) {
        this.relationType = relationType;
    }

    public BeanMeta getRelationBean() {
        return relationBean;
    }

    public void setRelationBean(BeanMeta relationBean) {
        this.relationBean = relationBean;
    }

    public PropertyMeta getRelationProperty() {
        return relationProperty;
    }

    public void setRelationProperty(PropertyMeta relationProperty) {
        this.relationProperty = relationProperty;
    }

    public int getPriority() {
        return priority;
    }

    public void setPriority(int priority) {
        this.priority = priority;
    }

    public void setAttribute(String name, Object value) {
        if (attributes == null)
            attributes = new HashMap();

        attributes.put(name, value);
    }

    public Object getAttribute(String name) {
        if (attributes == null)
            return null;

        return attributes.get(name);
    }

    public Map getAttributes() {
        return attributes;
    }

    public void setAttributes(Map attributes) {
        this.attributes = attributes;
    }

    public String getMicroHelp() {
        return microHelp;
    }

    public void setMicroHelp(String microHelp) {
        this.microHelp = microHelp;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String parameterString() {
        StringBuffer buffer = new StringBuffer(name.length() + 10);
        buffer.append(name);
        buffer.append(": ");
        buffer.append(readable ? 'r' : ' ');
        buffer.append(writable ? 'w' : ' ');
        buffer.append(mandatory ? 'm' : ' ');
        buffer.append(' ');
        buffer.append(format);
        buffer.append(' ');
        buffer.append('\n');
        for (Iterator iter = validators.iterator(); iter.hasNext();) {
            Validator validator = (Validator) iter.next();
            buffer.append(" ");
            buffer.append(validator);
        }
        return buffer.toString();
    }

    public String toString() {
        return name;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final PropertyMeta that = (PropertyMeta)o;

        if (!beanMeta.equals(that.beanMeta)) return false;
        if (!name.equals(that.name)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = beanMeta.hashCode();
        result = 29 * result + name.hashCode();
        return result;
    }

    public Object clone() {
        try {
            PropertyMeta property = (PropertyMeta)super.clone();
            if (attributes != null)
                property.attributes = new HashMap(attributes);

            if (format != null)
                property.format = (Format)format.clone();
            return property;
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
