package org.conform.hibernate;

import org.conform.*;
import org.hibernate.HibernateException;
import org.hibernate.SessionFactory;
import org.hibernate.engine.SessionFactoryImplementor;
import org.hibernate.metadata.ClassMetadata;
import org.hibernate.metadata.CollectionMetadata;
import org.hibernate.type.*;
import org.apache.commons.logging.LogFactory;

import java.util.*;

public class HibernateModifier
    implements Modifier
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(Meta.class);

    SessionFactory sessionFactory;
    BeanMetaProvider beanMetaProvider;
    private Map allClassMetadata;

    public HibernateModifier() {
    }

    public HibernateModifier(SessionFactory sessionFactory, BeanMetaProvider beanMetaProvider) {
        this.sessionFactory = sessionFactory;
        this.beanMetaProvider = beanMetaProvider;
    }

    public SessionFactory getSessionFactory() {
        return sessionFactory;
    }

    public void setSessionFactory(SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }

    public BeanMetaProvider getBeanMetaProvider() {
        return beanMetaProvider;
    }

    public void setBeanMetaProvider(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void modify(BeanMeta beanMeta) {
        firstRun(beanMeta);
        secondRun(beanMeta);
        thirdRun(beanMeta);
    }

    private void firstRun(BeanMeta beanMeta) {
        try {
            ClassMetadata metadata = sessionFactory.getClassMetadata(beanMeta.getType());
            if (metadata == null)
                return;

            LOG.debug("metadata = " + metadata + " property = " + beanMeta);

            String identifierPropertyName = metadata.getIdentifierPropertyName();
            beanMeta.getProperty(identifierPropertyName).setAttribute(PropertyMeta.ATTRIBUTE_IDENTIFIER, Boolean.TRUE);

            String[] propertyNames = metadata.getPropertyNames();
            Type[] propertyTypes = metadata.getPropertyTypes();
            boolean[] propertyNullabilities = metadata.getPropertyNullability();
            for (int i = 0; i < propertyTypes.length; i++) {
                String name = propertyNames[i];
                Type type = propertyTypes[i];
                boolean nullability = propertyNullabilities[i];

                PropertyMeta propertyMeta = beanMeta.getProperty(name);
                if (propertyMeta == null)
                    continue;
                
                propertyMeta.setMandatory(!nullability);

                if (type.isCollectionType()) {
                    CollectionType collectionType = (CollectionType) type;
                    String role = collectionType.getRole();
                    CollectionMetadata meta = sessionFactory.getCollectionMetadata(role);
                    LOG.trace("collection type = " + meta.getElementType().getReturnedClass());

                    BeanMeta relationBean = beanMetaProvider.getBeanMeta(meta.getElementType().getReturnedClass());
                    propertyMeta.setRelationType(PropertyMeta.ONE_TO_MANY_RELATION);
                    propertyMeta.setRelationBean(relationBean);

                    LOG.trace("MAYBE one to many : " + beanMeta + " to " + relationBean);
                }
                else if (type.isEntityType()) {
                    EntityType entityType = (EntityType)type;
                    LOG.trace("entity type = " + entityType.getReturnedClass());

                    BeanMeta relationBean = beanMetaProvider.getBeanMeta(entityType.getReturnedClass());
                    propertyMeta.setRelationBean(relationBean);
                    if (entityType.isOneToOne()) {
                        propertyMeta.setRelationType(PropertyMeta.ONE_TO_ONE_RELATION);
                        LOG.trace("DEFINATELY one to one : " + beanMeta + " to " + relationBean);
                    }
                    else {
                        propertyMeta.setRelationType(PropertyMeta.MANY_TO_ONE_RELATION);
                        LOG.trace("DEFINATELY many to one : " + beanMeta + " to " + relationBean);
                    }
                }
                else if (type.isComponentType()) {
                    ComponentType componentType = (ComponentType)type;
                    LOG.trace("component type = " + componentType.getReturnedClass());

                    if (Map.class.isAssignableFrom(componentType.getReturnedClass())) {
                        System.out.println("dynamic component: " + Arrays.asList(componentType.getPropertyNames()));
                    }
                    else {
                        BeanMeta relationBean = beanMetaProvider.getBeanMeta(componentType.getReturnedClass());
                        if (relationBean != null) {
                            propertyMeta.setRelationType(PropertyMeta.ONE_TO_ONE_RELATION);
                            propertyMeta.setRelationBean(relationBean);
                            relationBean.setAttribute("value", Boolean.TRUE);

                            LOG.trace("DEFINATELY one to one : " + beanMeta + " to " + relationBean);
                        }
                    }
                }
                else if (type.isAssociationType()) {
                    AssociationType associationType = (AssociationType)type;
                    //associationType.getAssociatedEntityName();
                }

                // TODO: consider the following
                /*
                if (!(type instanceof PrimitiveType))
                    propertyMeta.setMandatory(true);
                */

                if (type instanceof StringType) {
                    StringType stringType = (StringType)type;
                    //propertyMeta.setAttribute(StringEditor.ATTRIBUTE_LENGTH, stringType.)
                }
                if (type instanceof CustomType) {
                    CustomType customType = (CustomType)type;
                    propertyMeta.setAttribute("hibernate.user.type", customType.getReturnedClass());
                }
            }
        }
        catch (HibernateException e) {
            e.printStackTrace();
        }
    }

    private void secondRun(BeanMeta beanMeta) {
        for (int i = 0; i < beanMeta.getProperties().length; i++) {
            PropertyMeta property = beanMeta.getProperties()[i];

            if (property.getRelationType() == PropertyMeta.ONE_TO_ONE_RELATION)
                mapXXXToOne(beanMeta, property);
            else if (property.getRelationType() == PropertyMeta.ONE_TO_MANY_RELATION)
                mapXXXToMany(beanMeta, property);
        }
    }

    private void mapXXXToMany(BeanMeta bean, PropertyMeta property) {
        BeanMeta relationBean = property.getRelationBean();

        for (int i = 0; i < relationBean.getProperties().length; i++) {
            PropertyMeta relationProperty = relationBean.getProperties()[i];
            if (relationProperty.getRelationBean() == bean) {
                if (relationProperty.getRelationType() == PropertyMeta.ONE_TO_MANY_RELATION) {
                    mapManyToMany(property, relationProperty);
                    LOG.trace("DEFINATELY many to many : " + bean + " to " + relationBean);
                    break;
                }
                else if (relationProperty.getRelationType() == PropertyMeta.ONE_TO_ONE_RELATION) {
                    mapOneToMany(property, relationProperty);
                    LOG.trace("DEFINATELY one to many : " + bean + " to " + relationBean);
                    break;
                }
            }
        }
    }

    private void mapManyToMany(PropertyMeta property, PropertyMeta relationProperty) {
        property.setRelationType(PropertyMeta.MANY_TO_MANY_RELATION);
        property.setRelationProperty(relationProperty);
        relationProperty.setRelationType(PropertyMeta.MANY_TO_MANY_RELATION);
        relationProperty.setRelationProperty(property);
    }

    private void mapOneToMany(PropertyMeta property, PropertyMeta relationProperty) {
        property.setRelationType(PropertyMeta.ONE_TO_MANY_RELATION);
        property.setRelationProperty(relationProperty);
        relationProperty.setRelationType(PropertyMeta.MANY_TO_ONE_RELATION);
        relationProperty.setRelationProperty(property);
    }

    private void mapXXXToOne(BeanMeta bean, PropertyMeta property) {
        BeanMeta relationBean = property.getRelationBean();

        for (int i = 0; i < relationBean.getProperties().length; i++) {
            PropertyMeta relationProperty = relationBean.getProperties()[i];
            if (relationProperty.getRelationBean() == bean) {
                if (relationProperty.getRelationType() == PropertyMeta.ONE_TO_MANY_RELATION) {
                    mapManyToOne(property, relationProperty);
                    LOG.trace("DEFINATELY many to one : " + bean + " to " + relationBean);
                    break;
                }
                else if (relationProperty.getRelationType() == PropertyMeta.ONE_TO_ONE_RELATION) {
                    mapOneToOne(property, relationProperty);
                    LOG.trace("DEFINATELY one to one : " + bean + " to " + relationBean);
                    break;
                }
            }
        }
    }

    private void mapManyToOne(PropertyMeta property, PropertyMeta relationProperty) {
        property.setRelationType(PropertyMeta.MANY_TO_ONE_RELATION);
        property.setRelationProperty(relationProperty);
        relationProperty.setRelationType(PropertyMeta.ONE_TO_MANY_RELATION);
        relationProperty.setRelationProperty(property);
    }

    private void mapOneToOne(PropertyMeta property, PropertyMeta relationProperty) {
        property.setRelationType(PropertyMeta.ONE_TO_ONE_RELATION);
        property.setRelationProperty(relationProperty);
        relationProperty.setRelationType(PropertyMeta.ONE_TO_ONE_RELATION);
        relationProperty.setRelationProperty(property);
    }


    // TODO: optimization: build a map: who ref's whom
    private void thirdRun(BeanMeta beanMeta) {
        String entityName = beanMeta.getClass().getName();

        Set<String> references = new HashSet<String>();

        for (Iterator<ClassMetadata> iterator = getAllClassMetadata().values().iterator(); iterator.hasNext();) {
            ClassMetadata metadata = iterator.next();
            for (Type type : metadata.getPropertyTypes()) {
                if (!type.isEntityType())
                    continue;

                if (type instanceof ManyToOneType) {
                    ManyToOneType manyToOneType = (ManyToOneType)type;
                    if (manyToOneType.getAssociatedEntityName().equals(entityName))
                        references.add(metadata.getEntityName());
                }
                else if (type instanceof OneToOneType) {
                    OneToOneType oneToOneType = (OneToOneType)type;
                    if (oneToOneType.getAssociatedEntityName().equals(entityName))
                        references.add(metadata.getEntityName());
                }
                else if (type instanceof CollectionType) {
                    CollectionType collectionType = (CollectionType)type;
                    if (collectionType.getAssociatedEntityName((SessionFactoryImplementor)sessionFactory).equals(entityName))
                        references.add(metadata.getEntityName());
                }
            }
        }

        if (references.size() > 0)
            beanMeta.setAttribute("references", references);
    }

    private Map getAllClassMetadata() {
        if (allClassMetadata == null)
            allClassMetadata = sessionFactory.getAllClassMetadata();
        return allClassMetadata;
    }
}
