package org.conform.mdl;

import org.xml.sax.*;

import java.util.*;
import java.beans.ExceptionListener;

class ObjectHandler
    extends ScopeHandler
{
    private Hashtable environment = new Hashtable();
    private Vector expStack = new Vector();
    private StringBuffer chars = new StringBuffer();
    private int itemsRead = 0;

    public ObjectHandler(ExceptionListener exceptionListener) {
        super(exceptionListener);
    }

    public void reset() {
        environment.clear();
        expStack.clear();
        chars.setLength(0);
        itemsRead = 0;
    }

    private Object getValue(Expression exp) {
        try {
            return exp.getValue();
        }
        catch (Exception e) {
            exceptionListener.exceptionThrown(e);
            return null;
        }
    }

    private void addArg(Object arg) {
        // logger.finest("addArg: " + instanceName(arg));
        lastExp().addArg(arg);
    }

    private Object pop(Vector v) {
        int last = v.size() - 1;
        Object result = v.get(last);
        v.remove(last);
        return result;
    }

    private Object eval() {
        return getValue(lastExp());
    }

    private MutableExpression lastExp() {
        return (MutableExpression)expStack.lastElement();
    }

    Object dequeueResult() {
        // logger.finest("dequeueResult: " + expStack);
        Object[] results = lastExp().getArguments();
        reset();
        return results[itemsRead++];
    }

    private boolean isPrimitive(String name) {
        return name != "void" && Statement.typeNameToClass(name) != null;
    }

    private void simulateException(String message) {
        Exception e = new Exception(message);
        e.fillInStackTrace();
        exceptionListener.exceptionThrown(e);
    }

    private Class classForName(String name) {
        try {
            return Statement.classForName(name);
        }
        catch (ClassNotFoundException e) {
            exceptionListener.exceptionThrown(e);
        }
        return null;
    }

    private HashMap getAttributes(AttributeList attrs) {
        HashMap attributes = new HashMap();
        if (attrs != null && attrs.getLength() > 0) {
            for (int i = 0; i < attrs.getLength(); i++) {
                attributes.put(attrs.getName(i), attrs.getValue(i));
            }
        }
        return attributes;
    }

    public void startElement(String name, AttributeList attrs) throws SAXException {
        // logger.finest("startElement" + name);
        name = name.intern(); // Xerces parser does not supply unique tag names.
        chars.setLength(0);
        if (name == "null" ||
            name == "string" ||
            name == "class" ||
            isPrimitive(name)) {
            return;
        }
        HashMap attributes = getAttributes(attrs);

        MutableExpression e = new MutableExpression();

        // Target
        String className = (String)attributes.get("class");
        if (className != null) {
            e.setTarget(classForName(className));
        }

        // Property
        Object property = attributes.get("property");
        String index = (String)attributes.get("index");
        if (index != null) {
            property = new Integer(index);
            e.addArg(property);
        }
        e.setProperty(property);

        // Method
        String methodName = (String)attributes.get("method");
        if (methodName == null && property == null) {
            methodName = "new";
        }
        e.setMethodName(methodName);

        // Tags
        if (name == "void") {
            if (e.getTarget() == null) { // this check is for "void class="foo" method= ..."
                e.setTarget(eval());
            }
        }
        else if (name == "array") {
            // The class attribute means sub-type for arrays.
            String subtypeName = (String)attributes.get("class");
            Class subtype = (subtypeName == null) ? Object.class : classForName(subtypeName);
            String length = (String)attributes.get("length");
            if (length != null) {
                e.setTarget(java.lang.reflect.Array.class);
                e.addArg(subtype);
                e.addArg(new Integer(length));
            }
            else {
                Class arrayClass = java.lang.reflect.Array.newInstance(subtype, 0).getClass();
                e.setTarget(arrayClass);
            }
        }
        else if (name == "java") {
            // TODO: getScope() ???
            e.setValue(null); // The outermost scope is null.
        }
        else if (name == "object") {
        }
        else {
            simulateException("Unrecognized opening tag: " + name + " " + attrsToString(attrs));
            return;
        }

        // ids
        String idName = (String)attributes.get("id");
        if (idName != null) {
            environment.put(idName, e);
        }

        // idrefs
        String idrefName = (String)attributes.get("idref");
        if (idrefName != null) {
            e.setValue(lookup(idrefName));
        }

        // fields
        String fieldName = (String)attributes.get("property");
        if (fieldName != null) {
            e.setValue(getFieldValue(e.getTarget(), fieldName));
        }
        expStack.add(e);
    }

    private Object getFieldValue(Object target, String fieldName) {
        try {
            Class type = target.getClass();
            if (type == Class.class) {
                type = (Class)target;
            }
            java.lang.reflect.Field f = type.getField(fieldName);
            return f.get(target);
        }
        catch (Exception e) {
            exceptionListener.exceptionThrown(e);
            return null;
        }
    }

    private String attrsToString(AttributeList attrs) {
        StringBuffer b = new StringBuffer();
        for (int i = 0; i < attrs.getLength(); i++) {
            b.append(attrs.getName(i) + "=\"" + attrs.getValue(i) + "\" ");
        }
        return b.toString();
    }

    public void characters(char buf [], int offset, int len) throws SAXException {
        chars.append(new String(buf, offset, len));
    }

    private Object lookup(String s) {
        Expression e = (Expression)environment.get(s);
        if (e == null) {
            simulateException("Unbound variable: " + s);
        }
        return getValue(e);
    }

    public void endElement(String name) throws SAXException {
        // logger.finest("endElement: " + expStack);
        name = name.intern(); // Xerces parser does not supply unique tag names.
        if (name == "null") {
            addArg(null);
            return;
        }
        if (name == "java") {
            ObjectScope scope = (ObjectScope)getScope();
            scope.setObject(dequeueResult());
            return;
        }
        if (name == "string") {
            addArg(chars.toString());
            return;
        }
        if (name == "class") {
            addArg(classForName(chars.toString()));
            return;
        }
        if (isPrimitive(name)) {
            Class wrapper = Expression.typeNameToClass(name);
            Expression e = new Expression(wrapper, "new", new Object[]{chars.toString()});
            addArg(getValue(e));
            return;
        }
        if (name == "object" || name == "array" || name == "void") {
            Expression e = (Expression)pop(expStack);
            Object value = getValue(e);
            if (name == "object" || name == "array") {
                addArg(value);
            }
        }
        else {
            simulateException("Unrecognized closing tag: " + name);
        }
    }
}
