package org.conform.modifier;

import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import java.lang.annotation.Annotation;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.conform.format.*;
import org.conform.*;
import org.conform.validator.ValidatorImplementation;

public class AnnotationModifier
    implements Modifier
{
    private BeanMetaProvider beanMetaProvider;
    private Log logger = LogFactory.getLog(getClass());

    public AnnotationModifier() {
    }

    public AnnotationModifier(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void setBeanMetaProvider(BeanMetaProvider beanMetaProvider) {
        this.beanMetaProvider = beanMetaProvider;
    }

    public void modify(BeanMeta beanMeta) throws ModifierException {
        Class type = beanMeta.getType();
        org.conform.Bean beanAnnotation = (org.conform.Bean)type.getAnnotation(org.conform.Bean.class);
        if (beanAnnotation != null) {
            String pattern = beanAnnotation.pattern();
            if (pattern != null)
                beanMeta.setAttribute(PropertyMeta.ATTRIBUTE_PATTERN, pattern);
            String patternKey = beanAnnotation.patternKey();
            if (patternKey != null)
                beanMeta.setAttribute(PropertyMeta.ATTRIBUTE_PATTERN_KEY, patternKey);
        }

        FormatType formatType = beanAnnotation != null ? beanAnnotation.format() : null;
        String formatClassName = beanAnnotation != null ? beanAnnotation.formatClass() : null;
        beanMeta.setFormat(configureFormat(beanMeta, formatType, formatClassName));

        formatDeclaredFields(type, beanMeta);
    }

    private void formatDeclaredFields(Class type, BeanMeta beanMeta) {
        /*
         Class.getFields() returns all accessible (public) fields (also fields from super classes, but only the public ones)
         Class.getDeclaredFields() returns all (also non public ones) fields from the class (does not inspect the supper classes)
        */
        for (Field field : type.getDeclaredFields()) {
            PropertyMeta propertyMeta = beanMeta.getProperty(field.getName());
            if (propertyMeta == null)
                continue;

            // copy meta from referenced property
            PropertyReference propertyReferenceAnnotation = field.getAnnotation(PropertyReference.class);
            if (propertyReferenceAnnotation != null) {
                try {
                    Class inheritClazz = Class.forName(propertyReferenceAnnotation.className());
                    BeanMeta inheritBeanMeta = beanMetaProvider.getBeanMeta(inheritClazz);
                    PropertyMeta inheritPropertyMeta = inheritBeanMeta.getProperty(propertyReferenceAnnotation.propertyName());
                    if( inheritPropertyMeta == null)
                        throw new IllegalArgumentException("Referenced property '"+ propertyReferenceAnnotation.propertyName() + "' not found in '" + propertyReferenceAnnotation.className() + "'");

                    propertyMeta.setDomainProvider(inheritPropertyMeta.getDomainProvider());
                    propertyMeta.setFormat(inheritPropertyMeta.getFormat());
                    propertyMeta.setMandatory(inheritPropertyMeta.isMandatory());
                    propertyMeta.setPriority(inheritPropertyMeta.getPriority());
                    propertyMeta.setReadable(inheritPropertyMeta.isReadable());
                    propertyMeta.setValidators(inheritPropertyMeta.getValidators());
                    propertyMeta.setWritable(inheritPropertyMeta.isWritable());
                    propertyMeta.setApplicable(inheritPropertyMeta.isApplicable());
                    if( propertyMeta.getAttributes() == null ) {
                        propertyMeta.setAttributes(new HashMap());
                    }
                    if( inheritPropertyMeta.getAttributes() == null ) {
                        logger.warn("No attribute found for property: " + inheritPropertyMeta.getName() + " of class: " + inheritBeanMeta.getName());
                    } else {
                        propertyMeta.getAttributes().putAll(inheritPropertyMeta.getAttributes());
                    }
                    propertyMeta.setAttribute("inheritBean", inheritBeanMeta.getType());
                    propertyMeta.setAttribute("inheritProperty", inheritPropertyMeta.getName());
                }
                catch (Exception e) {
                    e.printStackTrace();
                }
            }

            // modify own meta
            org.conform.Property propertyAnnotation = field.getAnnotation(org.conform.Property.class);
            if (propertyAnnotation != null) {
                String readable = propertyAnnotation.readable();
                if (readable != null && readable.length() > 0)
                    propertyMeta.setReadable(Boolean.valueOf(readable));
                String writable = propertyAnnotation.writable();
                if (writable != null && writable.length() > 0)
                    propertyMeta.setWritable(Boolean.valueOf(writable));
                String applicable = propertyAnnotation.applicable();
                if (applicable != null && applicable.length() > 0)
                    propertyMeta.setApplicable(Boolean.valueOf(applicable));
                String mandatory = propertyAnnotation.mandatory();
                if (mandatory != null && mandatory.length() > 0)
                    propertyMeta.setMandatory(Boolean.valueOf(mandatory));
                String pattern = propertyAnnotation.pattern();
                if (pattern != null && pattern.length() > 0)
                    propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_PATTERN, pattern);
                String patternKey = propertyAnnotation.patternKey();
                if (patternKey != null && patternKey.length() > 0)
                    propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_PATTERN_KEY, patternKey);
                int scale = propertyAnnotation.scale();
                if (scale != -1)
                    propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_SCALE, scale);
                int precision = propertyAnnotation.precision();
                if (precision != -1)
                    propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_PRECISION, precision);
                int length = propertyAnnotation.length();
                if (length != -1)
                    propertyMeta.setAttribute(PropertyMeta.ATTRIBUTE_LENGTH, length);
                int priority = propertyAnnotation.priority();
                if (priority != -1)
                    propertyMeta.setPriority(priority);
            }

            FormatType formatType = propertyAnnotation != null ? propertyAnnotation.format() : null;
            String formatClassName = propertyAnnotation != null ? propertyAnnotation.formatClass() : null;
            propertyMeta.setFormat(configureFormat(propertyMeta, formatType, formatClassName));
            configureValidators(propertyMeta, field);
        }

        if( type.getSuperclass()!= null && !type.getSuperclass().getName().startsWith("java") ) {
            formatDeclaredFields(type.getSuperclass(), beanMeta);
        }
    }

    private void configureValidators(PropertyMeta propertyMeta, Field field) {
        for (Annotation annotation : field.getAnnotations()) {
            try {
                ValidatorImplementation implementation = annotation.annotationType().getAnnotation(ValidatorImplementation.class);
                if (implementation != null) {
                    Class<? extends Validator> validatorClass = implementation.CLASS();
                    Constructor<? extends Validator> constructor = validatorClass.getConstructor(Annotation.class);
                    Validator validator = constructor.newInstance(annotation);
                    propertyMeta.addValidator(validator);
                }
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private Format configureFormat(Meta meta, FormatType formatType, String formatClassName) {
        if (formatType == null)
            formatType = FormatType.NONE;

        switch (formatType) {
            case TEMPLATE:
                return new TemplateFormat();
            case COLLECTION_SIZE:
                return new CollectionSizeFormat();
            case CUSTOM:
                if (formatClassName != null) {
                    try {
                        Class formatClass = getClassLoader().loadClass(formatClassName);
                        return (Format)formatClass.newInstance();
                    }
                    catch (Exception e) {
                        logger.warn(meta.getName(), e);
                    }
                }
            default:
                if (meta instanceof PropertyMeta) {
                    meta.setAttribute("formatType", formatType);
                    return FormatFactory.getInstance().getFormat((PropertyMeta)meta);
                }
                else
                    return null;
        }
    }

    public ClassLoader getClassLoader() {
        return Thread.currentThread().getContextClassLoader();
    }
}
