package org.conform.wings;

import java.util.*;
import java.text.MessageFormat;

import org.conform.*;
import org.conform.format.ResourceProvider;
import org.conform.wings.editor.DefaultEditorStyles;
import org.wings.*;
import org.wings.util.SessionLocal;
import org.wings.event.SParentFrameListener;
import org.wings.event.SParentFrameEvent;
import org.wings.table.*;
import org.wingx.table.XTableColumn;
import org.apache.commons.logging.LogFactory;

/**
 * // TODO: make this a DefaultComponentFactory implementing ComponentFactory
 * @author gabriel pantazi
 */
public class ComponentFactory
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(ComponentFactory.class);

    public static final ComponentFactory INSTANCE = new ComponentFactory();

    protected EditorFactory editorFactory;
    private ResourceProvider resourceProvider;

    public void setEditorFactory(EditorFactory editorFactory) {
        this.editorFactory = editorFactory;
    }

    /**
     * Creates a graphical component based on the meta information and on data information for 
     * the specified property name.
     * 
     * @param beanMeta the whole meta information for a certain class used to initialize the meta
     * @param beanData the whole data information
     * @param propertyName property name used to generate the component for
     * @return the SComponent generated for the specified property.
     */
    public SComponent createComponent(BeanMeta beanMeta, BeanData beanData, String propertyName) {
        try {
            PropertyMeta propertyMeta = beanMeta.getProperty(propertyName);
            if (propertyMeta == null)
                throw new IllegalArgumentException(beanMeta.getName() + " has no propertyMeta named " + propertyName);

            Editor editor = getEditor(propertyMeta);
            SComponent component = editor.createComponent(propertyMeta);
            component.setVerticalAlignment(SConstants.TOP_ALIGN);
            component.setHorizontalAlignment(SConstants.LEFT_ALIGN);

            PropertyData propertyData = beanData.getPropertyData(propertyMeta);
            if (propertyData != null)
                editor.setPropertyData(component, propertyData);

            beanData.addValidationListener(new InplaceValidationListener(editor, component, propertyMeta));

            component.addParentFrameListener(new StableNaming(component, beanMeta, propertyName));
            return component;
        }
        catch (RuntimeException e) {
            LOG.error("Exception creating Component for " + beanMeta.getName() + "." + propertyName);
            throw e;
        }
    }

    public SComponent createReadOnlyComponent(BeanMeta beanMeta, BeanData beanData, String name) {
        try {
            PropertyMeta propertyMeta = beanMeta.getProperty(name);
            if (propertyMeta == null)
                throw new IllegalArgumentException(beanMeta.getName() + " has no propertyMeta named " + name);

            Editor editor = getReadOnlyEditor(propertyMeta);
            SComponent component = editor.createComponent(propertyMeta);

            PropertyData propertyData = beanData.getPropertyData(propertyMeta);
            if (propertyData != null)
                editor.setPropertyData(component, propertyData);

            return component;
        }
        catch (RuntimeException e) {
            LOG.error("Exception creating readonly Component for " + beanMeta.getName() + "." + name);
            throw e;
        }
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean errorneus) {
        Editor editor = getEditor(propertyMeta);
        editor.configureComponent(propertyMeta, component, false);
    }

    /**
     * Creates the apropriate label for the specified property name. If the label is not set (null) 
     * a label with the property name will be returned. 
     * 
     * @param beanMeta the meta information with localized label and micro help
     * @param name property name used to generate the label for.
     * @return the SLabel generated for the specified property.
     */
    public SLabel createLabel(BeanMeta beanMeta, String name) {
        PropertyMeta propertyMeta = beanMeta.getProperty(name);
        if (propertyMeta == null)
            throw new IllegalArgumentException(beanMeta.getName() + " has no propertyMeta named " + name);

        SLabel label = new SLabel();
        label.setVerticalAlignment(SConstants.TOP_ALIGN);
        DefaultEditorStyles.applyLabelState(propertyMeta, label);

        String text = propertyMeta.getLabel();
        label.setText(text != null ? text : propertyMeta.getName());
        label.setToolTipText(propertyMeta.getMicroHelp());
        return label;
    }

    public void configureLabel(PropertyMeta propertyMeta, SLabel label, boolean errorneus) {
        DefaultEditorStyles.applyLabelState(propertyMeta, label);
    }

    /**
     * Creates a STableColumn for the specified property name.
     *
     * @param beanMeta the whole meta information
     * @param name the property name for which the STableColumn is generated.
     * @param modelIndex Sets the cmp2 index for this column. The cmp2 index is the
     * 					 index of the column in the cmp2 that will be displayed by this
     * 					 <code>STableColumn</code>. As the <code>STableColumn</code>
     * 					 is moved around in the view the cmp2 index remains constant.
     * @return the STableColumn properly initialized based on the given property name and meta information.
     */
    public XTableColumn createTableColumn(BeanMeta beanMeta, String name, int modelIndex) {
        try {
            PropertyMeta property = beanMeta.getProperty(name);
            if (property == null)
                throw new IllegalArgumentException(beanMeta.getName() + " has no property named " + name);

            Editor editor = getEditor(property);
            Editor readOnlyEditor = getReadOnlyEditor(property);

            STableCellEditor cellEditor = null;
            if (property.isWritable())
                cellEditor = new PropertyCellEditor(editor, property);

            STableCellRenderer cellRenderer = new PropertyCellRenderer(readOnlyEditor, property);
            XTableColumn column = new XTableColumn(modelIndex, null, cellRenderer, cellEditor);
            column.setHidden(!property.isReadable());
            column.setIdentifier(property.getName());
            String label = property.getLabel();
            column.setHeaderValue(label != null ? label : property.getName());
            column.setWidth(getWidth(property));
            return column;
        }
        catch (RuntimeException e) {
            LOG.error("Exception creating Column for " + beanMeta.getName() + "." + name);
            throw e;
        }
    }

    /**
     * Creates a List of STableColum(s) based on the provided BeanMeta. For each property meta from bean meta
     * a new STableColumn will be added to the list.
     *
     * @param beanMeta the whole meta information
     * @return a List of STableColumn(s)
     */
    public List createTableColumns(BeanMeta beanMeta) {
        List columns = new ArrayList();
        for (int i = 0; i < beanMeta.getProperties().length; i++) {
            PropertyMeta propertyMeta = beanMeta.getProperties()[i];
            XTableColumn tableColumn = createTableColumn(beanMeta, propertyMeta.getName(), columns.size());
            columns.add(tableColumn);
        }
        return columns;
    }

    public Editor getEditor(PropertyMeta property) {
        if (editorFactory == null)
            editorFactory = new DefaultEditorFactory();
        return editorFactory.getEditor(property);
    }

    public Editor getReadOnlyEditor(PropertyMeta property) {
        if (editorFactory == null)
            editorFactory = new DefaultEditorFactory();
        return editorFactory.getReadOnlyEditor(property);
    }

    protected String getWidth(PropertyMeta propertyMeta) {
        if (Date.class.isAssignableFrom(propertyMeta.getType()))
            return "80";
        else if (boolean.class.equals(propertyMeta.getType())
            || Boolean.class.equals(propertyMeta.getType()))
            return "60";
        else if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_PRECISION) != null) {
            int cols =  1 + (Integer)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_PRECISION);
            if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_SCALE) != null)
                cols += 1;
            return cols * 10 + "";
        }
        else if (propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_LENGTH) != null) {
            int cols =  (Integer)propertyMeta.getAttribute(PropertyMeta.ATTRIBUTE_LENGTH);
            return cols > 32 ? "*" : cols * 10 + "";
        }
        else
            return null;
    }

    public void setResourceProvider(ResourceProvider resourceProvider) {
        this.resourceProvider = resourceProvider;
    }

    private class InplaceValidationListener
        implements ValidationListener
    {
        Editor edior;
        SComponent component;
        PropertyMeta propertyMeta;

        Set<Issue> issues = new HashSet<Issue>();
        StringBuilder detail = new StringBuilder();

        public InplaceValidationListener(Editor editor, SComponent component, PropertyMeta propertyMeta) {
            this.edior = editor;
            this.component = component;
            this.propertyMeta = propertyMeta;
        }

        public void addIssues(ValidationEvent event) {
            boolean change = false;
            for (Issue issue : event.getIssues()) {
                if (Arrays.asList(issue.getMetas()).contains(propertyMeta)) {
                    issues.remove(issue);
                    issues.add(issue);
                    change = true;
                }
            }

            if (change)
                update((Data)event.getSource());
        }

        public void removeIssues(ValidationEvent event) {
            boolean change = false;
            for (Issue issue : event.getIssues()) {
                if (Arrays.asList(issue.getMetas()).contains(propertyMeta)) {
                    change = issues.remove(issue) || change;
                }
            }

            if (change)
                update((Data)event.getSource());
        }

        void update(Data data) {
            if (data instanceof PropertyData)
                data = ((PropertyData)data).getBeanData();

            update(data.isIssuePublishingActive());
        }

        private void update(boolean issuePublishingActive) {
            if (issuePublishingActive && issues.size() > 0) {
                component.setToolTipText(composeToolTipText());
                edior.configureComponent(propertyMeta, component, true);
            }
            else {
                component.setToolTipText(null);
                edior.configureComponent(propertyMeta, component, false);
            }
        }

        public String composeToolTipText() {
            detail.setLength(0);
            for (Iterator<Issue> iterator = issues.iterator(); iterator.hasNext();) {
                Issue issue = iterator.next();
                PropertyMeta[] metas = issue.getMetas();

                if (metas.length == 1) {
                    label(metas[0]);
                    detail.append(":\n");
                }
                else {
                    for (int i = 0; i < metas.length; i++) {
                        if (i > 0)
                            detail.append(", ");
                        label(metas[i]);
                    }
                    detail.append(":\n");
                }

                String code = issue.getCode();
                Object[] arguments = issue.getArguments();

                if (code == null)
                    code = "validation.incorrect";

                code = resourceProvider.getMessage(component.getSession().getLocale(), code);

                if ( arguments == null )
                    detail.append("  - ").append(code);
                else
                    detail.append("  - ").append(MessageFormat.format(code, arguments));

                detail.append('\n');
            }
            return detail.toString();
        }

        private void label(Meta meta) {
            String label = meta.getLabel();
            detail.append(label != null ? label : meta.getName());
        }

        public void clearIssues(Meta meta) {
            issues.clear();
            update(false);
        }
    }

    /**
     * a set of per session unique names
     */
    private static final SessionLocal usedComponentNames = new SessionLocal() {
        @Override
        protected Set<String> initialValue() {
            return new HashSet<String>();
        }
    };

    private static class StableNaming
        implements SParentFrameListener
    {
        private final SComponent component;
        private final BeanMeta beanMeta;
        private final String propertyName;

        public StableNaming(SComponent component, BeanMeta beanMeta, String propertyName) {
            this.component = component;
            this.beanMeta = beanMeta;
            this.propertyName = propertyName;
        }

        public void parentFrameAdded(SParentFrameEvent e) {

            // unregister with the old name
            if (component.getSession().getDispatcher() != null && component instanceof LowLevelEventListener) {
                component.getSession().getDispatcher().unregister((LowLevelEventListener) component);
            }

            String name = createComponentName(beanMeta.getType().getSimpleName(), propertyName);
            Set<String> existingNames = (Set<String>)usedComponentNames.get();
            if (!existingNames.add(name)) {
                for (int i = 1 ; ; i++) {
                    String tryName = name + NAME_SEPARATOR + i;
                    if (existingNames.add(tryName)) {
                        name = tryName;
                        break;
                    }
                }
            }

            // set the new name
            component.setName(name);

            // unregister with the old name
            if (component.getSession().getDispatcher() != null && component instanceof LowLevelEventListener) {
                component.getSession().getDispatcher().register((LowLevelEventListener) component);
            }
        }

        private final static String NAME_SEPARATOR = "";

        public void parentFrameRemoved(SParentFrameEvent e) {
            ((Set)usedComponentNames.get()).remove(component.getName());
        }

        /**
         * create a name from the class name and the property name
         */
        private String createComponentName(String className, String propertyName) {
            StringBuffer buffer = new StringBuffer(className);
            buffer.append(NAME_SEPARATOR);
            buffer.append(propertyName);
            return buffer.toString();
        }
    }
}
