package org.conform;

import junit.framework.TestCase;

import java.util.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import org.conform.validator.StringLengthValidator;

public class DataTest
    extends TestCase
{
    Beans beans = new Beans();

    protected void setUp() throws Exception {
        beans.addBeanMeta(new BeanMeta(Person.class));
        beans.addBeanMeta(new BeanMeta(Address.class));
    }

    public void testInitialization() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        personBeanMeta.getProperty("addresses").setInitializer(new AddressesInitializer());

        DefaultBeanData personBeanData = new DefaultBeanData(personBeanMeta);

        personBeanData.setValue(new Person());
        assertNull(((Person)personBeanData.getValue()).getAddresses());

        personBeanData.initialize();
        assertNotNull(((Person)personBeanData.getValue()).getAddresses());
    }

    public void testPropertyChangeNotification() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        personBeanMeta.getProperty("addresses").setInitializer(new AddressesInitializer());
        BeanMeta addressBeanMeta = beans.getBeanMeta(Address.class);
        addressBeanMeta.getProperty("postcode").addValidator(new StringLengthValidator(5));

        DefaultBeanData personBeanData = new DefaultBeanData(personBeanMeta);
        DefaultBeanData addressBeanData = new DefaultBeanData(addressBeanMeta);

        TestPropertyChangeListener personBeanChangeListener = new TestPropertyChangeListener();
        personBeanData.addPropertyChangeListener(personBeanChangeListener);

        TestPropertyChangeListener addressBeanChangeListener = new TestPropertyChangeListener();
        addressBeanData.addPropertyChangeListener(addressBeanChangeListener);

        TestPropertyChangeListener postcodePropertyChangeListener = new TestPropertyChangeListener();
        addressBeanData.addPropertyChangeListener("postcode", postcodePropertyChangeListener);

        personBeanData.setValue(new Person());
        personBeanData.initialize();

        assertNotNull(personBeanChangeListener.lastEvent);
        assertEquals("addresses", personBeanChangeListener.lastEvent.getPropertyName());
        assertEquals(null, personBeanChangeListener.lastEvent.getOldValue());
        assertEquals(LinkedList.class, personBeanChangeListener.lastEvent.getNewValue().getClass());

        personBeanChangeListener.lastEvent = null;
        Address address = new Address();
        addressBeanData.setValue(address);
        assertNotNull(postcodePropertyChangeListener.lastEvent);
        assertNull(postcodePropertyChangeListener.lastEvent.getNewValue());
        assertNotNull(addressBeanChangeListener.lastEvent);
        assertNull(addressBeanChangeListener.lastEvent.getNewValue());

        postcodePropertyChangeListener.lastEvent = null;
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("123456");
        assertNull(postcodePropertyChangeListener.lastEvent);

        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("12345");
        assertNotNull(postcodePropertyChangeListener.lastEvent);
        assertEquals("postcode", postcodePropertyChangeListener.lastEvent.getPropertyName());
        assertEquals(null, postcodePropertyChangeListener.lastEvent.getOldValue());
        assertEquals("12345", postcodePropertyChangeListener.lastEvent.getNewValue());

        addressBeanData.getPropertyData(addressBeanMeta.getProperty("city")).setValue("Neverland");
        assertNotNull(addressBeanChangeListener.lastEvent);
        assertEquals("city", addressBeanChangeListener.lastEvent.getPropertyName());
        assertEquals(null, addressBeanChangeListener.lastEvent.getOldValue());
        assertEquals("Neverland", addressBeanChangeListener.lastEvent.getNewValue());
    }

    public void testBinding() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        BeanMeta addressBeanMeta = beans.getBeanMeta(Address.class);
        addressBeanMeta.getProperty("postcode").addValidator(new StringLengthValidator(5));

        DefaultBeanData personBeanData = new DefaultBeanData(personBeanMeta);
        DefaultBeanData addressBeanData = new DefaultBeanData(addressBeanMeta);

        Person person = new Person();
        personBeanData.setValue(person);
        Address address = new Address();
        addressBeanData.setValue(address);

        personBeanData.getPropertyData(personBeanMeta.getProperty("firstName")).setValue("Holger");
        personBeanData.getPropertyData(personBeanMeta.getProperty("lastName")).setValue("Engels");
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("street")).setValue("Road to Nowhere");
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("123456");
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("city")).setValue("Neverland");

        assertEquals("Holger", personBeanData.getPropertyData(personBeanMeta.getProperty("firstName")).getValue());
        assertEquals("Engels", personBeanData.getPropertyData(personBeanMeta.getProperty("lastName")).getValue());
        assertEquals("Road to Nowhere", addressBeanData.getPropertyData(addressBeanMeta.getProperty("street")).getValue());
        assertEquals("123456", addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).getValue());
        assertEquals("Neverland", addressBeanData.getPropertyData(addressBeanMeta.getProperty("city")).getValue());

        assertEquals("Holger", person.getFirstName());
        assertEquals("Engels", person.getLastName());
        assertEquals("Road to Nowhere", address.getStreet());
        assertEquals("123456", addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).getInvalidValue());
        assertNull(address.getPostcode());
        assertEquals("Neverland", address.getCity());
    }

    public void testValidation() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        personBeanMeta.getProperty("firstName").setMandatory(true);
        personBeanMeta.getProperty("lastName").setMandatory(true);

        BeanMeta addressBeanMeta = beans.getBeanMeta(Address.class);
        addressBeanMeta.getProperty("postcode").addValidator(new StringLengthValidator(5));
        addressBeanMeta.addValidator(new BeanValidator() {
            public Object validate(Object value) throws ValidationException {
                Address address = (Address)value;
                if ((address.getPostcode() != null) != (address.getCity() != null))
                    throw new ValidationException(new ValidationException.Message("Specify postcode and city or nothing at all"));
                else
                    return address;
            }

            public String[] getAffectedPropertyNames() {
                return new String[] { "postcode", "city" };
            }
        });

        Set<Issue> issues = new HashSet<Issue>();
        ValidationListener validations = new TestValidationListener(issues);

        DefaultBeanData personBeanData = new DefaultBeanData(personBeanMeta);
        DefaultBeanData addressBeanData = new DefaultBeanData(addressBeanMeta);
        personBeanData.addValidationListener(validations);
        addressBeanData.addValidationListener(validations);
        assertEquals(0, issues.size());

        personBeanData.setValue(new Person());
        addressBeanData.setValue(new Address());
        assertEquals(0, issues.size());

        personBeanData.setIssuePublishingActive(true);
        addressBeanData.setIssuePublishingActive(true);
        assertEquals(2, issues.size());

        personBeanData.getPropertyData(personBeanMeta.getProperty("firstName")).setValue("Holger");
        personBeanData.getPropertyData(personBeanMeta.getProperty("lastName")).setValue("Engels");
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("123456");
        validate(personBeanData, addressBeanData, false);
        System.out.println("issues = " + issues);
        assertEquals(1, issues.size());

        personBeanData.setIssuePublishingActive(false);
        addressBeanData.setIssuePublishingActive(false);
        assertEquals(0, issues.size());

        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("12345");
        validate(personBeanData, addressBeanData, true);

        personBeanData.setIssuePublishingActive(true);
        addressBeanData.setIssuePublishingActive(true);
        assertEquals("" + issues, 1, issues.size());

        addressBeanData.getPropertyData(addressBeanMeta.getProperty("city")).setValue("Ulm");
        validate(personBeanData, addressBeanData, false);
        assertEquals(0, issues.size());

        personBeanData.getPropertyData(personBeanMeta.getProperty("firstName")).setValue(null);
        personBeanData.getPropertyData(personBeanMeta.getProperty("lastName")).setValue(null);
        assertEquals(2, issues.size());
    }

    public void testValidationChanges() throws Exception {
        BeanMeta personBeanMeta = beans.getBeanMeta(Person.class);
        personBeanMeta.getProperty("firstName").setMandatory(true);
        personBeanMeta.getProperty("lastName").setMandatory(true);

        BeanMeta addressBeanMeta = beans.getBeanMeta(Address.class);
        addressBeanMeta.getProperty("postcode").addValidator(new StringLengthValidator(5));
        BeanValidator addressValidator = new BeanValidator() {
            public Object validate(Object value) throws ValidationException {
                Address address = (Address)value;
                if ((address.getPostcode() != null) != (address.getCity() != null))
                    throw new ValidationException(new ValidationException.Message("Specify postcode and city or nothing at all"));
                else
                    return address;
            }

            public String[] getAffectedPropertyNames() {
                return new String[] { "postcode", "city" };
            }
        };
        addressBeanMeta.addValidator(addressValidator);

        Set<Issue> issues = new HashSet<Issue>();
        ValidationListener validations = new TestValidationListener(issues);

        DefaultBeanData personBeanData = new DefaultBeanData(personBeanMeta);
        DefaultBeanData addressBeanData = new DefaultBeanData(addressBeanMeta);
        personBeanData.addValidationListener(validations);
        addressBeanData.addValidationListener(validations);

        personBeanData.setValue(new Person());
        addressBeanData.setValue(new Address());
        addressBeanData.getPropertyData(addressBeanMeta.getProperty("postcode")).setValue("12345");

        personBeanData.setIssuePublishingActive(true);
        addressBeanData.setIssuePublishingActive(true);
        validate(personBeanData, addressBeanData, true);
        assertEquals(3, issues.size()); // firstName, lastName, Address(postcode, city)

        personBeanMeta.getProperty("firstName").setMandatory(false);
        personBeanMeta.getProperty("lastName").setMandatory(false);
        personBeanData.revalidateAll();
        System.out.println("issues = " + issues);
        assertEquals(1, issues.size()); // Address(postcode, city)

        addressBeanData.getPropertyData(addressBeanMeta.getProperty("city")).setValue("Ulm");
        addressBeanData.revalidateAll();
        assertEquals(0, issues.size());
    }

    private void validate(DefaultBeanData personBeanData, DefaultBeanData addressBeanData, boolean expectException) {
        boolean exception = false;
        try {
            personBeanData.validate(personBeanData.getValue());
        }
        catch (ValidationException e) {
            exception = true;
        }
        try {
            addressBeanData.validate(addressBeanData.getValue());
        }
        catch (ValidationException e) {
            exception = true;
        }
        if (exception != expectException)
            fail();
    }

    private static class TestValidationListener
        implements ValidationListener
    {
        private final Set<Issue> issues;

        public TestValidationListener(Set<Issue> issues) {
            this.issues = issues;
        }

        public void addIssues(ValidationEvent event) {
            issues.removeAll(event.getIssues());
            issues.addAll(event.getIssues());
        }

        public void removeIssues(ValidationEvent event) {
            issues.removeAll(event.getIssues());
        }

        public void clearIssues(Meta meta) {
            for (Iterator<Issue> iterator = issues.iterator(); iterator.hasNext();) {
                Issue issue = iterator.next();
                PropertyMeta[] issueMetas = issue.getMetas();
                for (PropertyMeta issueMeta : issueMetas) {
                    if (issueMeta.getBeanMeta() == meta) {
                        iterator.remove();
                        break;
                    }
                }
            }
        }
    }

    private static class TestPropertyChangeListener
        implements PropertyChangeListener
    {
        public PropertyChangeEvent lastEvent;

        public void propertyChange(PropertyChangeEvent event) {
            lastEvent = event;
        }
    }

    private static class AddressesInitializer
        implements Initializer
    {
        public Object getValue() {
            return new LinkedList<Address>();
        }
    }
}
