package org.osbl.persistence.hibernate;

import junit.framework.TestCase;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.osbl.persistence.*;

import java.util.ArrayList;
import java.util.List;

public class HibernateCommandTest extends TestCase {

    private Session session;
    private Transaction transaction;
    private HibernatePersistence commandFactory;

    protected void setUp()
        throws Exception
    {
        Configuration configuration = new Configuration();
        configuration.configure("test-hibernate.cfg.xml");
        configuration.addResource("Identity.hbm.xml");

        SessionFactory sessionFactory = configuration.buildSessionFactory();
        session = sessionFactory.openSession();
        transaction = session.beginTransaction();
        commandFactory = new HibernatePersistence();
        commandFactory.setSessionFactory(new PseudoSessionFactory((org.hibernate.classic.Session)session));
        commandFactory.registerCommand("load", HibernateLoadCommand.class);
        commandFactory.registerCommand("query", HibernateQueryCommand.class);
        commandFactory.registerCommand("simpleQuery", HibernateSimpleQueryCommand.class);
        commandFactory.registerCommand("joinedQuery", HibernateJoinedQueryCommand.class);
    }

    public void testCRUD()
    {
        Identity identity = new Identity();
        identity.setKey("HE");

        User user = new User(identity);
        user.setAccount("hengels");
        user.setPassword("hengels".toCharArray());

        Person person = new Person(identity);
        person.setFirstName("Holger");
        person.setLastName("Engels");

        Address address = new Address();
        address.setStreet("Nowhere");
        address.setPostcode("777");
        address.setCity("California");

        List<Address> addresses = new ArrayList<Address>();
        addresses.add(address);
        person.setAddresses(addresses);

        System.out.println("identity = " + identity);
        System.out.println("user = " + user);
        System.out.println("person = " + person);

        session.save(identity);
        session.save(user);
        session.save(person);

        session.flush();
        session.clear();

        LoadCommand<Person> loadCommand = (LoadCommand<Person>)commandFactory.createCommand("load");
        loadCommand.setType(Person.class);
        loadCommand.setId(person.getId());
        Person loadedPerson = loadCommand.execute();
        assertNotNull(loadedPerson);

        QueryCommand<Person> queryCommand = (QueryCommand<Person>)commandFactory.createCommand("query");
        queryCommand.setType(Person.class);
        List<Person> persons = queryCommand.execute();
        assertEquals(1, persons.size());

        SimpleQueryCommand<Person> simpleQueryCommand = (SimpleQueryCommand<Person>)commandFactory.createCommand("simpleQuery");
        simpleQueryCommand.setType(Person.class);
        simpleQueryCommand.addFilter("firstName", Operator.EQUAL, "Holger");
        simpleQueryCommand.addSort("lastName", Sort.ASCENDING);
        persons = simpleQueryCommand.execute();
        assertEquals(1, persons.size());

        JoinedQueryCommand<Person> joinedQueryCommand = (JoinedQueryCommand<Person>)commandFactory.createCommand("joinedQuery");
        joinedQueryCommand.setType(Person.class);
        joinedQueryCommand.addJoin("address", "a");
        joinedQueryCommand.addFilter("a.street", Operator.EQUAL, "Nowhere");
        persons = joinedQueryCommand.execute();
        assertEquals(1, persons.size());
    }

    protected void tearDown() throws Exception {
        transaction.rollback();
        session.close();
    }
}
