package org.osbl;

import org.conform.*;
import org.conform.format.DefaultFormatFactory;
import org.conform.modifier.AnnotationModifier;

import java.util.*;

public class ValidationHelper
{
    private Set<Issue> issues = new HashSet<Issue>();

    private ValidationListener forwarder = new ValidationListener() {
        public void addIssues(ValidationEvent event) {
            issues.addAll(event.getIssues());
        }

        public void removeIssues(ValidationEvent event) {
        }

        public void clearIssues(Meta meta) {
        }
    };

    private BeanMetaProvider getBeanMetaProvider() {
        return (BeanMetaProvider)ServiceProvider.getInstance().getService("StaticBeanMetaProvider");
    }

    public void staticValidation(Object object) {
        if (object == null)
            throw new NullPointerException("Object must not be null!");

        BeanMeta beanMeta = getBeanMetaProvider().getBeanMeta(object.getClass());
        DefaultBeanData beanData = new DefaultBeanData(beanMeta);
        beanData.setValue(object);

        beanData.addValidationListener(forwarder);
        beanData.setIssuePublishingActive(true);
        beanData.revalidateAll();
    }

    public void validation(Object object, Validator validator, String... affectedPropertyNames) {
        if (object == null)
            throw new NullPointerException("Object must not be null!");
        if (affectedPropertyNames == null || affectedPropertyNames.length == 0)
            throw new NullPointerException("Affected property names are required!");

        try {
            validator.validate(object);
        }
        catch (ValidationException e) {
            PropertyMeta[] affectedPropertyMetas = propertyMetas(object.getClass(), affectedPropertyNames);

            for (ValidationException.Message message : e.getMessages()) {
                issues.add(new Issue(System.identityHashCode(validator), affectedPropertyMetas, message.getCode(), message.getArguments()));
            }
        }
    }

    public void validation(Object object, BeanValidator validator) {
        if (object == null)
            throw new NullPointerException("Object must not be null!");

        try {
            validator.validate(object);
        }
        catch (ValidationException e) {
            PropertyMeta[] affectedPropertyMetas = propertyMetas(object.getClass(), validator.getAffectedPropertyNames());

            for (ValidationException.Message message : e.getMessages()) {
                issues.add(new Issue(System.identityHashCode(validator), affectedPropertyMetas, message.getCode(), message.getArguments()));
            }
        }
    }

    public void addIssue(Class type, String[] propertyNames, String code, Object... arguments) {
        issues.add(new Issue(-issues.size(), propertyMetas(type, propertyNames), code, arguments));
    }

    public void addIssue(Class type, String propertyName, String code, Object... arguments) {
        issues.add(new Issue(-issues.size(), propertyMeta(type, propertyName), code, arguments));
    }

    public void throwIfIssues() throws ValidationIssueException {
        if (issues.size() > 0)
            throw new ValidationIssueException(issues);
    }

    private PropertyMeta[] propertyMetas(Class type, String[] affectedPropertyNames) {
        BeanMeta beanMeta = getBeanMetaProvider().getBeanMeta(type);
        PropertyMeta[] affectedPropertyMetas = new PropertyMeta[affectedPropertyNames.length];
        for (int i = 0; i < affectedPropertyNames.length; i++) {
            String affectedPropertyName = affectedPropertyNames[i];
            affectedPropertyMetas[i] = beanMeta.getProperty(affectedPropertyName);
        }
        return affectedPropertyMetas;
    }

    private PropertyMeta propertyMeta(Class type, String affectedPropertyName) {
        BeanMeta beanMeta = getBeanMetaProvider().getBeanMeta(type);
        return beanMeta.getProperty(affectedPropertyName);
    }

    public static void main(String[] args) {
        new DefaultFormatFactory();
        ServiceProvider.INSTANCE = new ServiceProvider() {
            public List<String> getServiceNames() {
                return null;
            }

            public Object getService(String name) {
                VariationBeanMetaProvider metaProvider = new VariationBeanMetaProvider(new Beans());
                metaProvider.addModifier(new AnnotationModifier(metaProvider));
                return metaProvider;
            }

            public <T> T getService(Class<T> type) {
                return null;
            }

            public void autowireServices(Object bean) {
            }
        };
        Address address = new Address();

        ValidationHelper validationHelper = new ValidationHelper();
        validationHelper.staticValidation(address);

        // callback validator
        validationHelper.validation(address, new Validator() {
            public Object validate(Object value) throws ValidationException {
                throw new ValidationException(new ValidationException.Message("blub", new Object[] { value }));
            }
        }, "city", "postcode");

        // callback bean validator
        validationHelper.validation(address, new BeanValidator() {
            public Object validate(Object value) throws ValidationException {
                throw new ValidationException(new ValidationException.Message("bla", new Object[] { value }));
            }

            public String[] getAffectedPropertyNames() {
                return new String[] { "street" };
            }
        });

        // inline
        if (true)
            validationHelper.addIssue(Address.class, new String[] { "street", "postcode", "city" }, "fasel", 7);

        System.out.println("Issues: " + validationHelper.issues);
    }

    public static class Address
    {
        Long id;

        String street;
        String postcode;
        String city;

        public Long getId() {
            return id;
        }

        public void setId(Long id) {
            this.id = id;
        }

        public String getStreet() {
            return street;
        }

        public void setStreet(String street) {
            this.street = street;
        }

        public String getPostcode() {
            return postcode;
        }

        public void setPostcode(String postcode) {
            this.postcode = postcode;
        }

        public String getCity() {
            return city;
        }

        public void setCity(String city) {
            this.city = city;
        }
    }
}
