package org.osbl.agent.gui;

import org.osbl.agent.model.Action;


/**
 * Knows how to return its corresponding action and how to set up the
 * UI-controls to reflect an action given.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class ActionController extends OperationController {

	// TODO: this is not very portable or elegant. Avoid hardcoded and hidden values like this.
	/** The package location in which ActionControllers can be found. */
	protected static final String ACTION_CONTROLLERS_PACKAGE = "org.osbl.agent.gui.action";
	
	/**
	 * Given an Action instance, it tries to retrieve a corresponding ActionController object.
	 * That object reflects the state of the given Action.
	 * 
	 * @param action an Action instance.
	 * 
	 * @return the corresponding instantiated ActionController.
	 * 
	 * @throws InstantiationException the instantiation exception
	 * @throws IllegalAccessException the illegal access exception
	 * @throws ClassNotFoundException the class not found exception
	 */
	public static ActionController newInstance(Action action)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException {

		// Create the ConditionController corresponding to the given condition. 
		ActionController result = (ActionController) newInstance(action,
				ACTION_CONTROLLERS_PACKAGE);

		// Make the ActionController set itself up according to the
		// parameters in the condition.
		result.setAction(action);

		// Return the recently created and set-up Controller
		return result;
	}

	/**
	 * Makes the ActionController re-configure itself based on the passed Action.
	 * (So, this is no normal setter method; the Action is not retained).
	 * 
	 * @param action the Action this ActionController should reflect.
	 */
	public abstract void setAction(Action action);

	/**
	 * Generates a (new) Action instance according to the state of the UI controls.
	 * Subclasses should not override this method but {@link #populateAction(Action)} instead.
	 * 
	 * @return a new Action instance.
	 */
	public final Action getAction() {

		// Instantiate the corresponding Action (note that this will always
		// instantiate a subclass of Action, depending in which ActionController
		// this gets called).
		// For example, if this is called inside a SetValueActionController then
		// a SetValueAction will be instantiated.
		Action result = (Action) newOperationInstance();

		// The Action instance just created is lacks relevant information, so...
		
		// ... this fills the Action with all relevant information corrsponding
		// to the current status of the ActionController. 
		
		// Note that each class overrides this method and sets the information 
		// corresponding to the Action it represents.
		populateAction(result);

		// Return the populated Action.
		return result;

	}

	/**
	 * Populates and configures a given Action object to reflect the state of this Controller.
	 * (especially the UI components). 
	 * 
	 * @param action the Action to be populated.
	 */
	protected abstract void populateAction(Action action);

}
