package org.osbl.agent.gui;

import org.osbl.agent.model.Action;
import org.osbl.agent.model.Condition;


/**
 * Knows how to return its corresponding Condition and how to set up the
 * UI-controls to reflect a Condition given.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class ConditionController extends OperationController {

	/** The Constant CONDITION_CONTROLLERS_PACKAGE. */
	private static final String CONDITION_CONTROLLERS_PACKAGE = "org.osbl.agent.gui.condition";

	/**
	 * Given a Condition instance, it tries to retrieve a corresponding ConditionController object.
	 * That object reflects the state of the given Condition.
	 * 
	 * @param condition a Condition instance.
	 * 
	 * @return the corresponding instantiated ConditionController.
	 * 
	 * @throws InstantiationException the instantiation exception
	 * @throws IllegalAccessException the illegal access exception
	 * @throws ClassNotFoundException the class not found exception
	 */
	public static ConditionController newInstance(Condition condition)
			throws InstantiationException, IllegalAccessException,
			ClassNotFoundException {

		// Create the ConditionController corresponding to the given condition. 
		ConditionController result = (ConditionController) newInstance(
				condition, CONDITION_CONTROLLERS_PACKAGE);

		// Make the ConditionController set itself up according to the
		// parameters in the condition.
		result.setCondition(condition);

		// Return the recently created and set-up Controller
		return result;
	}

	/**
	 * Makes the ConditionController re-configure itself based on the passed Condition.
	 * (So, this is no normal setter method; the Condition is not retained).
	 * 
	 * @param condition the Condition this ConditionController should reflect.
	 */
	public abstract void setCondition(Condition condition);

	/**
	 * Generates a (new) Condition instance according to the state of the UI controls.
	 * Subclasses should not override this method but {@link #populateCondition(Condition)} instead.
	 * 
	 * @return a new Condition instance.
	 */
	public final Condition getCondition() {

		// Create the corresponding Condition instance.
		// (for example, creates a StringCondition if we are a StringConditionController).
		Condition result = (Condition) newOperationInstance();

		// Ask our subclass to populate it according to its current settings.
		populateCondition(result);

		// Now the Condition reflects the state of the Controller.
		// Return the populated Condition.
		return result;

	}

	/**
	 * Populates and configures a given Condition object to reflect the state of this Controller.
	 * (especially the UI components). 
	 * 
	 * @param condition the Condition to be populated.
	 */
	protected abstract void populateCondition(Condition condition);

}
