package org.osbl.agent.gui;

import java.util.ArrayList;
import java.util.List;

import org.osbl.agent.logic.AgentSystem;
import org.osbl.agent.logic.RuntimeContext;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;
import org.osbl.client.wings.form.*;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.wings.session.SessionManager;


/**
 * The DesignContext is one of the most important classes in the implementation of the Agent System.
 * Many classes present context-dependent interface and/or behaviour. Instead of having to subclass
 * these classes for each Context, this presentation and/or behaviour is this class.
 * Classes that delegate to DesignContext are:
 * <ul>
 * <li>{@link SidePanel}, which implements the Side Panel</li>
 * <li>{@link RuleEditorPanel}, which implements the Rule Editor</li>
 * <li>{@link PreviewController}, which implements the Preview Mode</li>
 * </ul>
 * 
 * @author Sebastian Nozzi.
 */
public abstract class DesignContext {

	/** The object editor. */
	protected ObjectEditor objectEditor;

	/** The current rules. */
	protected List<Rule> currentRules;
	
	/** The preview controller. */
	protected PreviewController previewController;

	/** The agent system. */
	protected AgentSystem agentSystem;

	/** The rule editor panel. */
	protected RuleEditorPanel ruleEditorPanel;

	/** The side panel. */
	protected SidePanel sidePanel;
	
	/**
	 * Instantiates a new design context.
	 * It has to be passed an ObjectEditor, used to display modified objects and persist those
	 * changes (primarily used in the Preview Mode).
	 * 
	 * @param objectEditor an OSBL object editor instance
	 */
	public DesignContext(ObjectEditor objectEditor){
		
		this.objectEditor = objectEditor;
		
		sidePanel = null;	
		ruleEditorPanel = null;
		previewController = null;

		currentRules = new ArrayList<Rule>();

		// We need THE AgentSystem instance for accessing the Rules repository
		agentSystem = AgentSystem.getSingletonInstance();
	}
	
	/**
	 * Returns the user-id of the currently logged-in user.
	 * 
	 * @return the current user's id.
	 */
	static public String getCurrentUser(){
		
		String user = SessionManager.getSession().getServletRequest().getUserPrincipal().getName();

		return user;
	}	
	
	/**
	 * Convenience method to retrieve a localized version of a String. The String might contain
	 * format indicators, in which case the data corresponding to those are provided as
	 * additional arguments.
	 * 
	 * @param code the code of the localizable String.
	 * @param args additional arguments applicable to the (format) String.
	 * 
	 * @return the localized String
	 */
	static protected String getMsg(String code, Object ... args){
		
		Client client = Client.getInstance();
		
		if(client!=null)
			return client.getResourceProvider().getMessage("org.osbl.agents.gui."+code, args);
		else
			return "<<"+code+">>";

	}
	
	/**
	 * Convenience method to retrieve a localized version of a String. The String might contain
	 * format indicators, in which case the data corresponding to those are provided as
	 * additional arguments.
	 * 
	 * @param code the code of the localizable String.
	 * @param args additional arguments applicable to the (format) String.
	 * 
	 * @return the localized String
	 */
	public String msg(String code, Object ... args){
		return DesignContext.getMsg(code,args);
	}

	/**
	 * Obtains a context-dependent PreviewController instance. The instance will be
	 * created only once. Successive calls to this method will return the same instance.
	 * 
	 * @return the context-specific PreviewController instance.
	 */
	public PreviewController getPreviewController() {
		
		if(previewController==null){
			previewController = getPreviewControllerInstance();
			previewController.setRuntimeContext(getRuntimeContext());
		}

		return previewController;
	}

	/**
	 * Obtains the context-specific RuleEditorPanel instance. Successive calls to this
	 * method will yield the same instance.
	 * 
	 * @return the context-specific RuleEditorPanel instance.
	 */
	public RuleEditorPanel getRuleEditorPanel() {
		
		if(ruleEditorPanel==null)
			ruleEditorPanel = new RuleEditorPanel(this);
		
		return ruleEditorPanel;
	}

	/**
	 * Obtains the context-specific SidePanel instance. Successive calls to this methods
	 * yields the same instance.
	 * 
	 * @return the context-specific SidePanel instance.
	 */
	public SidePanel getAgentSidePanel() {
		
		if(sidePanel==null)
			sidePanel = new SidePanel(this);
		
		return sidePanel;
	}

	/**
	 * Populates a SidePanel with context-specific filters. Called by the SidePanel when
	 * it is time to include the filters during its UI construction.
	 * 
	 * @param targetPanel the target panel where the filter components are to be added.
	 */
	public abstract void addSideFilters(SidePanel targetPanel);

	/**
	 * Answers whether a given Rule belongs to this DesignContext.
	 * 
	 * @param rule the Rule to be analyzed.
	 * 
	 * @return true, if it does belong to this context.
	 */
	public boolean belongsToDesignContext(Rule rule) {
		return rule.getMetaInformation()
				.get(RuleContext.RUNTIME_OBJECT_CLASS).equals(
						getRuntimeObjectClass().toString());
	}
	
	/**
	 * Gets the parent window. Used for opening new panels in the central window.
	 * 
	 * @return the parent window
	 */
	protected Window getParentWindow() {
		return Window.getWindow(objectEditor.getEnvironment());
	}
	
	/**
	 * Returns the context-specific OSBL ObjectEditor passed upon construction.
	 * Requested principally by the PreviewMode to show the modified object,
	 * allow further edition to it, and persist it.
	 * 
	 * @return the ObjectEditor.
	 */
	public ObjectEditor getObjectEditor() {
		return objectEditor;
	}

	/**
	 * Recompute the current Rules to be shown in the UI, based on the logged user-id,
	 * the filters applied (if any) and the public status of the other Rules.
	 * 
	 * Called whenever the filters have changed.
	 * 
	 * Subclasses implement a context-specific way of recomputing the Rules, usually using
	 * the filter settings.
	 */
	public abstract void recomputeCurrentRules();

	/**
	 * Retrieves the list of the current Rules. This call does not recompute the Rules.
	 * 
	 * @return the current Rules, as a List.
	 */
	public List<Rule> getCurrentRules() {
		return currentRules;
	}

	
	/**
	 * Adds the Specifier components to a RuleEditorPanel. Called by the latter when building
	 * its UI. The DesignContext can put context-specific components in it.
	 * 
	 * @param targetPanel the target RuleEditorPanel instance.
	 */
	public abstract void addSpecifierComponents(RuleEditorPanel targetPanel);
	
	/**
	 * Enable/disable Specifier UI-components. Permits/avoids edition or change of the parameters.
	 * Used when the user is not editable.
	 * 
	 * @param enabled whether to enable the components or not.
	 */
	public abstract void enableSpecifierComponents(boolean enabled);

	/**
	 * Gets the ActionControllers available on this context.
	 * Requested by the RuleEditorPanel.
	 * 
	 * @return the ActionControllers for this context.
	 */
	public abstract List<OperationController> getActionControllers(); 

	/**
	 * Gets the ConditionControllers available on this context.
	 * Requested by the RuleEditorPanel.
	 * 
	 * @return the ConditionControllers for this context.
	 */
	public abstract List<OperationController> getConditionControllers();

	/**
	 * Populate rule.
	 * Requested by the RuleEditorPanel.
	 * 
	 * @param editedRule the edited rule
	 */
	public void populateRule(Rule editedRule){
		editedRule.getMetaInformation().put(RuleContext.RUNTIME_OBJECT_CLASS,
				getRuntimeObjectClass().toString());
	}

	/**
	 * Change the Specifier Parameters on-screen to reflect the ones set in the Rule.
	 * 
	 * @param rule the Rule which contains Specifier Parameters whose state should be reflected in the UI.
	 */
	abstract public void changeSpecifiers(Rule rule);

	/**
	 * Gets the class of the runtime target object.
	 * 
	 * @return the class of the runtime target object.
	 */
	abstract public Class getRuntimeObjectClass();

	/**
	 * Gets the RuntimeContext that corresponds to this (Design)Context.
	 * Usually invoked by the PreviewController.
	 * 
	 * @return the context-specific RuntimeContext.
	 */
	abstract public RuntimeContext getRuntimeContext();
	
	/**
	 * Factory method that instantiates and returns a context-specific PreviewController.
	 * Usually invoked by the SidePanel.
	 * 
	 * @return the context-specific PreviewController instance.
	 */
	abstract protected PreviewController getPreviewControllerInstance();

}
