package org.osbl.agent.gui;

import java.util.ArrayList;
import java.util.List;

import org.concern.Controller;
import org.concern.ControllerLookup;
import org.concern.Subject;
import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.osbl.agent.gui.action.BeanShellActionController;
import org.osbl.agent.gui.action.SetCurrentTimeActionController;
import org.osbl.agent.gui.action.SetEnumActionController;
import org.osbl.agent.gui.action.SetPropertyActionController;
import org.osbl.agent.gui.action.SetValueActionController;
import org.osbl.agent.gui.condition.BeanShellConditionController;
import org.osbl.agent.gui.condition.EnumConditionController;
import org.osbl.agent.gui.condition.FieldConditionController;
import org.osbl.agent.gui.condition.NumberConditionController;
import org.osbl.agent.gui.condition.OgnlConditionController;
import org.osbl.agent.gui.condition.PropertyConditionController;
import org.osbl.agent.gui.condition.StringConditionController;
import org.osbl.agent.logic.ObjectRuntimeContext;
import org.osbl.agent.logic.RuntimeContext;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.action.SetCurrentTimeAction;
import org.osbl.client.wings.form.GenericObjectList;
import org.osbl.client.wings.form.ObjectEditor;
import org.osbl.client.wings.form.ObjectForm;
import org.osbl.client.wings.shell.Client;


/**
 * The Class ObjectDesignContext.
 * 
 * @author Sebastian Nozzi.
 */
public class ObjectDesignContext extends DesignContext {

	/** The bean meta. */
	protected BeanMeta beanMeta;

	/**
	 * Instantiates a new object design context.
	 * 
	 * @param objectEditor the object editor
	 */
	public ObjectDesignContext(ObjectEditor objectEditor) {
		super(objectEditor);
        beanMeta = Client.getInstance().getBeanMetaProvider().getBeanMeta(getRuntimeObjectClass());
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#addSideFilters(org.osbl.agent.gui.SidePanel)
	 */
	public void addSideFilters(SidePanel targetPanel) {
		// No filters
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#addSpecifierComponents(org.osbl.agent.gui.RuleEditorPanel)
	 */
	public void addSpecifierComponents(RuleEditorPanel targetPanel) {
		// No specifiers
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#changeSpecifiers(org.osbl.agent.model.Rule)
	 */
	public void changeSpecifiers(Rule rule) {
		// Since no specifiers, nothing to do
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#enableSpecifierComponents(boolean)
	 */
	public void enableSpecifierComponents(boolean enabled) {
		// Since no specifiers, nothing to do
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getActionControllers()
	 */
	public List<OperationController> getActionControllers() {

		// We'll be returning a List of OperationControllers (actually
		// ActionControllers)
		List<OperationController> result = new ArrayList<OperationController>();

		// Now we'll process all properties of the Subject we are working with
		// and add a suitable ActionController for each of them so that the user
		// can set some field values.

		// We get the properties from the beanMeta, which contains (meta)
		// information
		// about the subject type we are dealing with. This information can be
		// very
		// easily retrieved (easier than using java-reflection).
		PropertyMeta[] properties = beanMeta.getProperties();

		// Iterate over all properties of the subject class
		for (PropertyMeta prop : properties) {

			SetPropertyActionController newController;

			if (prop.getType().isEnum()) {
				newController = new SetEnumActionController();
			// If the type is one of the supported time classes (see class
			// documentation)
			}else if (SetCurrentTimeAction.isSupportedTimeClass(prop.getType()))
				// The controller will be an automatic time-setter...
				newController = new SetCurrentTimeActionController();
			else
				// ...otherwise it will be a normal value setter (via input
				// field).
				newController = new SetValueActionController();

			// Setting the propertyMeta is vital when later the ActionController
			// is
			// asked to generate the corresponding Action
			newController.setPropertyMeta(prop);
			
			// Add our new controller for the property in turn to the list of
			// choices
			result.add(newController);
		}

		// Add one (and only one) BeanShellActionController to the choices
		BeanShellActionController beanShellActionController = new BeanShellActionController(); 
		result.add(beanShellActionController);

		// Return our list of ActionControllers
		return result;
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getConditionControllers()
	 */
	public List<OperationController> getConditionControllers() {

		// We'll be returning a List of OperationControllers (actually
		// ConditionControllers)
		List<OperationController> result = new ArrayList<OperationController>();

		// These Controllers let you test a property of the Subject against some
		// user
		// provided value.

		// Get the properties of the subject and for each of them...
		for (PropertyMeta propertyMeta : beanMeta.getProperties()) {

			PropertyConditionController newController = null;

			// ... according to the type of the property find and instantiate
			// the corresponding PropertyConditionController...

			// ... either a special version of it
			if (propertyMeta.getType().isEnum()) {
				newController = new EnumConditionController();
			} else if (Number.class.isAssignableFrom(propertyMeta.getType())) {
				newController = new NumberConditionController();
			} else if (String.class.isAssignableFrom(propertyMeta.getType())) {
				newController = new StringConditionController();
			} else {
				// ... or a generic form...
				newController = new FieldConditionController();
			}

			// Setting the propertyMeta is vital when later the ConditionController
			// is asked to generate the corresponding Condition
			newController.setPropertyMeta(propertyMeta);

			// Add our new controller for the property in turn to the list of
			// choices
			result.add(newController);
		}

		// Add an OgnlConditionController (only one)
		OgnlConditionController ognlConditionController = new OgnlConditionController();
		result.add(ognlConditionController);

		// Add an BeanShellConditionController (only one)
		BeanShellConditionController beanShellConditionController = new BeanShellConditionController();
		result.add(beanShellConditionController);

		return result;
	}

    /* (non-Javadoc)
     * @see org.osbl.agent.gui.DesignContext#recomputeCurrentRules()
     */
    public void recomputeCurrentRules() {

        currentRules.clear();

        currentRules.addAll(agentSystem.getRulesForUser(getCurrentUser()));

        currentRules.addAll(agentSystem.getPublicRulesOfOthers(getCurrentUser()));

        List<Rule> deleteThese = new ArrayList<Rule>();

        for(Rule rule : currentRules){
            if(belongsToDesignContext(rule)==false)
                deleteThese.add(rule);
        }

        currentRules.removeAll(deleteThese);
    }

    /* (non-Javadoc)
     * @see org.osbl.agent.gui.DesignContext#getRuntimeObjectClass()
     */
    public Class getRuntimeObjectClass() {
        return getObjectEditor().getType();
    }

    /* (non-Javadoc)
     * @see org.osbl.agent.gui.DesignContext#getRuntimeContext()
     */
    public RuntimeContext getRuntimeContext() {
        return new ObjectRuntimeContext();
    }

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getPreviewControllerInstance()
	 */
	public PreviewController getPreviewControllerInstance() {

		PreviewController result = new PreviewController(this) {

            ObjectForm form;

			public ObjectForm getObjectForm() {
                if (form == null) {
                    try {
                        Class type = designContext.getObjectEditor().getForm().getClass();
                        form = (ObjectForm)type.newInstance();
                    }
                    catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }
                return form;
			}

			public Object getListObject(int rowIdx) {
                return ((GenericObjectList)designContext.getObjectEditor().getList()).getRow(rowIdx);
			}

			public void populateForm(Object targetObject) {
                form.getEnvironment().setActive(true);
                form.setObject(targetObject);
			}

			protected void postUpdateActions(Object targetObject) {
			}

			protected Controller getController(Object targetObject) {
				Subject subject = (Subject) targetObject;
				return ControllerLookup.getInstance().getController(
						subject.getProcess());
			}

			public Object getSubjectInstance(Object targetObject) {
				return ((Subject) targetObject).getSubject();
			}

		};

		return result;
	}
}
