package org.osbl.agent.gui;

import java.util.ArrayList;
import java.util.List;

import org.wings.SComponent;


/**
 * An OperationController provides the UI components to manipulate Conditions or Actions.
 * 
 * It also knows if another given OperationController can pose as a replacement
 * to itself.
 * 
 * This class provides a factory method to instantiate the corresponding
 * Condition or Action for a given Condition- or ActionController.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class OperationController {

	
	/**
	 * For a given Condition or Action instance, it instantiates the corresponding
	 * Condition- or ActionController.
	 * 
	 * @param conditionOrAction a Condition or Action instance.
	 * @param packagePrefix the prefix of the package where the correspoding OperationController can be found.
	 * 
	 * @return the corresponding OperationController instance.
	 * 
	 * @throws InstantiationException the instantiation exception
	 * @throws IllegalAccessException the illegal access exception
	 * @throws ClassNotFoundException the class not found exception
	 */

	static protected OperationController newInstance(Object conditionOrAction,
			String packagePrefix) throws InstantiationException,
			IllegalAccessException, ClassNotFoundException {

		// Just take the simple name (e.g. just "NumberCondition" instead of
		// "org.osbl.agent.model.condition.NumberCondition")
		String operationName = conditionOrAction.getClass().getSimpleName();

		// Wrap it around the package prefix and "Controller" at the end.
		// (in our example: "org.osbl.agent.gui.condition.NumberConditionController")
		String controllerClassName = packagePrefix + "." + operationName
				+ "Controller";

		// Note that for this to work you should only call this method for
		// Conditions/Actions for which a Controller exists and has the 
		// exact class name + "Controller".
		
		// Create an instance of the class...
		OperationController result = (OperationController) Class.forName(
				controllerClassName).newInstance();

		// ...and return it.
		return result;

	}

	/**
	 * Convenience method to retrieve localized Strings.
	 * 
	 * @param code the String-code.
	 * @param args optional additional arguments.
	 * 
	 * @return the localized String
	 */
	protected String msg(String code, Object ... args){
		return DesignContext.getMsg(code, args);
	}
	
	/**
	 * Returns the UI components that represent the corresponding model class.
	 * They let the user manipulate the Condition- or ActionController in a visual way.
	 * 
	 * Note that subclasses should not override this method but {@link #populateComponentList(List)}.
	 * 
	 * @return a list of UI components.
	 */
	public final List<SComponent> getComponents() {
		
		// Create an empty list of SComponents...
		List<SComponent> result = new ArrayList<SComponent>();

		// ...let the subclass populate it...
		populateComponentList(result);
		
		// ...and return it.
		return result;
	}

	/**
	 * Populates the component list with the components corresponding to the current
	 * Condition or ActionController.
	 * 
	 * @param componentList the component list to be populated.
	 */
	protected abstract void populateComponentList(List<SComponent> componentList);

	/**
	 * Returns true if the candidateController can replace this
	 * controller.
	 * 
	 * OperationController can replace one another if they perform the same function
	 * regardless of its internal state.
	 * 
	 * @param candidateController the candidate controller that might, or not, serve as a 
	 * replacement for this one.
	 * 
	 * @return true, if can be replaced by the candidate controller.
	 */
	abstract public boolean canBeReplacedBy(
			OperationController candidateController);

	/**
	 * Tries to retrieve an instance of the model class that corresponds to this
	 * OperationController. So, for example, if called on a ConditionController 
	 * it tries to instantiate the corresponding Condition. It performs the opposite
	 * operation that {@link #newInstance(Object, String)} does.
	 * 
	 * @return the corresponding Condition or Action instance, or null if not successfull.
	 */
	protected Object newOperationInstance() {

		// Get complete name of our class (which is not OperationController but
		// a non abstract subclass of this one).
		String operationName = this.getClass().getCanonicalName();

		// Remove the trailing "Controller" from the class name.
		operationName = operationName.substring(0, operationName
				.lastIndexOf("Controller"));
		
		// Replace ".gui." for ".model", so for example
		// "org.osbl.agent.gui.condition.NumberConditionController" becomes
		// "org.osbl.agent.model.condition.NumberCondition".
		operationName = operationName.replaceFirst("\\.gui\\.", ".model.");

		try {
			// Instantiate the class and return it.
			return Class.forName(operationName).newInstance();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

}
