package org.osbl.agent.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import java.util.List;

import org.wings.*;


/**
 * Represents a line (slot) in the {@link RuleEditorPanel} with controls to visually manipulate
 * Conditions or Actions. 
 * 
 * @author Sebastian Nozzi.
 */
abstract public class OperationPanel extends SPanel {

	/** The component count. */
	private int componentCount;

	/** The first field, the combo with all OperationControllers. */
	protected SComboBox controllerCombo;

    private final GridBagConstraints layoutConstraints = new GridBagConstraints(-1, -1, 1, 1, 0, 0, SConstants.RIGHT, GridBagConstraints.BOTH, new Insets(0,0,0,0), 0, 0);

	/** The Rule Editor. */
	protected RuleEditorPanel ruleEditorPanel;

	/**
	 * Instantiates a new OperationPanel for the given editor.
	 * 
	 * @param ruleEditorPanel the Rule Editor object.
	 */
	public OperationPanel(RuleEditorPanel ruleEditorPanel) {
		this(ruleEditorPanel, null);
	}

	/**
	 * Instantiates a new OperationPanel, with a pre-existing OperationController as a candidate.
	 * This candidate controller will be shown as the selected element. Eventually this means
	 * that one of the existing controllers in the combo will get replaced with the candidate
	 * controller. For this, the replacement mechanism of OperationController is used.
	 * 
	 * 
	 * @param ruleEditorPanel the rule controller
	 * @param candidateController the candidate controller
	 * @see OperationController#canBeReplacedBy(OperationController)
	 */
	public OperationPanel(RuleEditorPanel ruleEditorPanel,
			OperationController candidateController) {
		super(new SGridBagLayout());

		this.ruleEditorPanel = ruleEditorPanel;
		componentCount = 0;

		controllerCombo = new SComboBox();
		controllerCombo.setHorizontalAlignment(SConstants.LEFT_ALIGN);

		// Let the panel be as wide as possible
		setPreferredSize(SDimension.FULLWIDTH);

		int initialSelectedIndex = 0;
		int comboItemIndex = 0;

		// Get a list of Controllers (overriden by subclasses) and iterate over them...
		for (OperationController opc : getControllers()) {

			// If we have been given an already existing Controller (in case we want
			// to re-create the UI for an existing Rule) ...
			if (candidateController != null) {
				// ... we should prefer using that controller than the default one
				// since the candidateController correctly reflects the state of the
				// Operation that is part of the Rule we want to re-create.
				
				// If a match is found...
				if (opc.canBeReplacedBy(candidateController)) {
					// make sure we use the candidateController instead of the one in list...
					opc = candidateController;
					// and remember in which position it was on the list (used below)
					initialSelectedIndex = comboItemIndex;
				}
			}
			
			// Add the Controller to the combo-box.
			controllerCombo.addItem(opc);
			comboItemIndex++;
		}

		// Add the combo-box of Controllers to ourself (SPanel)
        layoutConstraints.weightx = .3;
        add(controllerCombo, layoutConstraints);

		// In case candidateController found a match, 
		// make the combo-box point to it
		controllerCombo.setSelectedIndex(initialSelectedIndex);

		// IMPORTANT: we add the action-listener AFTER changing the selected of 
		// index of the combo-box. Otherwise we would automatically trigger an
		// unwanted ActionEvent...
		controllerCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// This will update the other components depending on the currently
				// selected Controller.
				updateParameters();
			}
		});

		// Update parameters according to the currently selected Controller.
		updateParameters();
	}

	/**
	 * Gets the all the OperationControllers supported by this OperationPanel in this Context.
	 * 
	 * @return the OperationControllers for this panel in this context.
	 */
	protected abstract List<OperationController> getControllers();

	/**
	 * Populates this panel with the controls provided by the selected OperationController.
	 * These controls are put right to the OperationController combo-box (first field).
	 */
	private void populateParameters() {

		// Get currently selected Controller
		OperationController currentController = (OperationController) controllerCombo
				.getSelectedItem();
		
		List<SComponent> components = currentController.getComponents();
		
        layoutConstraints.weightx = .7 / components.size();

		// Ask the Controller for parameter components (used to further refine and
		// set-up the Action/Condition it represents) and
		// iterate over them...
		for (SComponent comp : components) {
            // Remember how many components we added, so that we know
            // how many to remove, should we need to do so.
            componentCount++;

			// Make every of them full width
			comp.setPreferredSize(SDimension.FULLWIDTH);
			// Add it to ourselves (SPanel)
            add(comp, layoutConstraints);
		}
	}

	/**
	 * Removes the controls (that were once provided by the selected OperationController), 
	 * from this panel.
	 */
	private void removeParameters() {
		// For every extra component we added...
		while (componentCount > 0) {
			// ... remove the component next to the controllerCombo
			// (which never gets, nor should it be, deleted)
			remove(1);
			// Update our component count accordingly.
			componentCount--;
		}
	}

	/**
	 * Update parameters, to reflect the selected OperationController.
	 */
	public void updateParameters() {

		// Remove all parameter components...
		removeParameters();

		// ... and re-populate them using the selected Controller.
		populateParameters();
	}

	/** 
	 * Enables or disables this panel, effectively allowing or preventing edition.
	 * 
	 * @see org.wings.SComponent#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {
		// Call superclass implementation for the SPanel, but...
		super.setEnabled(enabled);

		// ...also invalidate all our child components so that, for
		// example, no editing is possible.
		for (SComponent comp : getComponents()) {
			comp.setEnabled(enabled);
		}
	}

}
