package org.osbl.agent.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import org.concern.Controller;
import org.osbl.agent.logic.RuntimeContext;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.form.GenericObjectEditor;
import org.osbl.client.wings.form.GenericObjectList;
import org.osbl.client.wings.form.ObjectForm;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.wings.SButton;
import org.wings.SComponent;


/**
 * This class drives the operation of the Preview Mode.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class PreviewController {

	/** The runtime context. */
	private RuntimeContext runtimeContext;
	
	/** The index of the target object being shown. */
	private int targetIndex = 0;
	
	/** The list item objects. */
	private List<Object> listItemTargets;
	
	/** The window where the preview mode is shown. */
	private Window previewWindow;
	
	/** Whether the window is visible or not. */
	private boolean showingWindow = false;
	
	/** The previous button. */
	private SButton previousButton;
	
	/** The apply button. */
	private SButton applyButton;
	
	/** The skip button. */
	private SButton skipButton;
	
	/** The finish button. */
	private SButton finishButton;
	
	/** The design context. */
	protected DesignContext designContext;
	
	/** The Rule being run. */
	protected Rule currentRule;

	/**
	 * Instantiates a new PreviewController with a given DesignContext.
	 * 
	 * @param designContext the design context
	 */
	public PreviewController(DesignContext designContext){
	
		this.designContext = designContext;
		
	}

	/**
	 * Convenience method to get localized versions of strings/labels.
	 * 
	 * @param code the String-code
	 * @param args optional arguments
	 * 
	 * @return the localized String
	 */
	private String msg(String code, Object ... args){
		return designContext.msg(code, args);
	}

	/**
	 * Initiates the Preview Mode using the given Rule.
	 * 
	 * @param rule the Rule for which the Preview Mode will be started.
	 */
	public void runRule(Rule rule) {
		
		// Get the object-editor and object-list, used to retrieve and manipulate the objects
		GenericObjectEditor objectEditor = (GenericObjectEditor)designContext.getObjectEditor();
        GenericObjectList objectList = (GenericObjectList)objectEditor.getList();

		listItemTargets = new ArrayList<Object>();

		// Retrieve all applicable list-items
        for(int rowIdx=0;rowIdx< objectList.getRowCount();rowIdx++){
			
			Object listItemObject = getListObject(rowIdx);
			
			if(runtimeContext.ruleIsApplicable(rule, listItemObject)){
				listItemTargets.add(listItemObject);
			}
		}

        // If items were found...
		if(listItemTargets.size()>0){
			
			targetIndex = 0;

			// ...setup the window if not already done so
			if(previewWindow==null)
				setupWindow();

			currentRule = rule;
			
			// ...and begin showing the items in the UI.
			showListItemTarget();
		}
	}	
	
	/**
	 * Setups the window.
	 */
	private void setupWindow() {

		previewWindow = new Window();
		
		List<SComponent> controls = new ArrayList<SComponent>();		
		
		previousButton = new XButton("Previous");
		applyButton = new XButton("Apply &amp; Proceed");
		skipButton = new XButton("Skip");
		finishButton = new XButton("Finish");

		controls.add(previousButton);
		controls.add(applyButton);
		controls.add(skipButton);
		controls.add(finishButton);
		
		previewWindow.setTitle(msg("previewWindowTitle"));

		getObjectForm().getEnvironment().setControls(controls);

		previousButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				previous();
			}
		});
		
		applyButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				apply();
			}
		});
		
		skipButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				skip();
			}
		});
		
		finishButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				finish();
			}
		});
		
	}
	
	/**
	 * Runs the Rule and then shows the current list-item in the UI.
	 */
	private void showListItemTarget(){
		
		Object listItemTarget = listItemTargets.get(targetIndex);

		// This makes use of the persistence functionality of the editor, letting the
		// editor retrieve the object for us.
        ((GenericObjectEditor)designContext.getObjectEditor()).revertObject(listItemTarget);

        // Once retrieved, we show it in the UI.
        populateForm(listItemTarget);

		// We create and populate a RuleContext.
        RuleContext ruleContext = new RuleContext();
		runtimeContext.populateRuleContext(ruleContext, listItemTarget);
		
		// Conditions apply?
		if(currentRule.evaluateConditions(ruleContext)){
			
			// Then Execute the Actions...
			currentRule.executeActions(ruleContext);
			
			// ...and show the results on screen...
			getObjectForm().getEnvironment().setDelegate(previewWindow);

			if(showingWindow==false){
				Client.getInstance().pushWindow(designContext.getParentWindow(), previewWindow);
				showingWindow = true;
			}

			// "Previous" not possible if at beginning
			previousButton.setEnabled(canGoBack());
			// It's always possible to advance (apply or skip), but when
			// you reach the end, we close the window automatically.
			
			
		}else{
			// On to the next target!
			skip();
		}
		
	}

	/**
	 * Closes the window.
	 */
	private void closeWindow(){
		if(showingWindow){
			Client.getInstance().popWindow(previewWindow);
			showingWindow = false;
		}
	}

	/**
	 * Finish.
	 */
	private void finish() {
        designContext.getObjectEditor().getList().refresh();
        closeWindow();
	}

	/**
	 * Skip.
	 */
	private void skip() {
		if(canAdvance()){
			targetIndex++;
			showListItemTarget();
		}else
			finish();
	}
	
	/**
	 * Previous.
	 */
	private void previous() {
		if(canGoBack()){
			targetIndex--;
			showListItemTarget();
		}
	}
	
	/**
	 * Can advance?
	 * 
	 * @return true, if it is possible to advance to the next item.
	 */
	private boolean canAdvance(){
		return targetIndex+1<listItemTargets.size();
	}

	/**
	 * Can go back?
	 * 
	 * @return true, if it is possible to go back to the previous item.
	 */
	private boolean canGoBack(){
		return targetIndex-1>=0;
	}
	
	/**
	 * Applies the changes done by the Rule and/or user to the target object. 
	 */
	private void apply() {

		Object targetToPersist = listItemTargets.get(targetIndex);
		
    	ObjectForm objectForm = getObjectForm();
    	
    	// Use the object-form to check for validation issues (already part of OSBL)
        if (!objectForm.hasValidationIssues()) {
            saveObject(targetToPersist);
        }
		
        // Move to next item
		skip();
	}

	/**
	 * Gets the con:cern controller, used to retrieve con:cern related objects.
	 * 
	 * @param targetObject the target object for which a con:cern controller will be sought.
	 * 
	 * @return the con:cern controller found.
	 */
	protected abstract Controller getController(Object targetObject);

	/**
	 * Persists the object.
	 * 
	 * @param targetObject the target object to be persisted.
	 */
	private void saveObject(Object targetObject){
		// Makes use of the editor's persistence functionality to store the object.
		((GenericObjectEditor)designContext.getObjectEditor()).saveObject(targetObject);
		// Call possible post-update actions.
        postUpdateActions(targetObject);
	}
	
	/**
	 * Post update actions.
	 * 
	 * @param targetObject the target object for which post-update actions will be done.
	 */
	protected abstract void postUpdateActions(Object targetObject); 

	/**
	 * Sets the runtime context.
	 * 
	 * @param runtimeContext the new RuntimeContext
	 */
	public void setRuntimeContext(RuntimeContext runtimeContext) {
		this.runtimeContext = runtimeContext;		
	}
	
	/**
	 * Returns the real subject instance from a (possibly) con:cern object (like Work/Subject)
	 * 
	 * @param targetObject the target object
	 * 
	 * @return the subject instance, in con:cern terms.
	 */
	protected abstract Object getSubjectInstance(Object targetObject);

	/**
	 * Gets the object form.
	 * 
	 * @return the object form
	 */
	protected abstract ObjectForm getObjectForm();
	
	/**
	 * Populates the on-screen form (which is part of the editor).
	 * 
	 * @param targetObject the target object used to populate the form.
	 */
	protected abstract void populateForm(Object targetObject);

	/**
	 * Retrieves an item from the list at the given index.
	 * 
	 * @param rowIdx the row index.
	 * 
	 * @return the list item.
	 */
	protected abstract Object getListObject(int rowIdx);
	
}
