package org.osbl.agent.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import org.osbl.agent.model.Action;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.Rule;
import org.osbl.client.wings.XButton;
import org.wings.*;
import org.wingx.XDivision;

import javax.swing.*;


/**
 * Provides the Editor panel and controls its operation.
 * 
 * @author Sebastian Nozzi.
 */
public class RuleEditorPanel extends SPanel {

    SPanel contentPane = new SPanel(new SGridBagLayout());

    /** The current user id. */
	private String currentUser;

	/** The Rule being edited. */
	private Rule rule;

	/** The SidePanel. */
	private SidePanel sidePanel;

	/** The name field. */
	private STextField nameField;

	/** The "add condition" button. */
	private SButton addConditionButton;

	/** The "add action" button. */
	private SButton addActionButton;

	/** The conditions division, all ConditionPanels go here. */
	private XDivision conditionsDivision;

	/** The actions division, all ActionPanels go here. */
	private XDivision actionsDivision;

	/** The action panel count. */
	private int actionPanelCount;

	/** The condition panel count. */
	private int conditionPanelCount;

	/** The design context. */
	private DesignContext designContext;

	/** The close button. */
	protected SButton closeButton;
	
	/** The revert button. */
	protected SButton revertButton;
	
	/** The apply button. */
	protected SButton applyButton;

	/**
	 * Instantiates a new rule controller for the given DesignContext.
	 * 
	 * @param designContext the design context
	 */
	public RuleEditorPanel(DesignContext designContext) {
		super(new SBorderLayout());
        contentPane.setVerticalAlignment(SConstants.TOP_ALIGN);
        ((SGridBagLayout)contentPane.getLayout()).setHgap(8);
        ((SGridBagLayout)contentPane.getLayout()).setVgap(8);
        add(contentPane, SBorderLayout.CENTER);

        setDesignContext(designContext);

		// Panel starts up empty (no action/condtion panels).
		actionPanelCount = 0;
		conditionPanelCount = 0;

		// Build up the visual elements of our panel.
		populateRulePanel();
		
		closeButton = new XButton(msg("close"));
		revertButton = new XButton(msg("revert"));
		applyButton = new XButton(msg("applyChanges"));

		// Initialize some visual aspects.
		setPreferredSize(SDimension.FULLAREA);
        setStyle("XForm");
        
        changeEditingMode();
	}

	/**
	 * Gets the SidePanel.
	 * 
	 * @return the SidePanel object.
	 */
	public SidePanel getAgentSidebarPanel() {
		return sidePanel;
	}

	/**
	 * Sets the SidePanel.
	 * 
	 * @param sidePanel the SidePanel object.
	 */
	public void setAgentSidebarPanel(SidePanel sidePanel) {
		this.sidePanel = sidePanel;
	}

	/**
	 * Gets the design context.
	 * 
	 * @return the design context
	 */
	public DesignContext getDesignContext() {
		return designContext;
	}

	/**
	 * Sets the design context.
	 * 
	 * @param designContext the design context
	 */
	public void setDesignContext(DesignContext designContext) {
		this.designContext = designContext;
	}

	/**
	 * Convenience method to get localized versions of strings/labels
	 * 
	 * @param code the String-code
	 * @param args the String-args
	 * 
	 * @return the localized string
	 */
	protected String msg(String code, Object ... args){
		return designContext.msg(code, args);
	}

	/**
	 * Checks if this panel is editable.
	 * The panel is not editable for non-editable Rules (for which the user
	 * lacks the necessary permissions).
	 * 
	 * @return true, if it is editable
	 */
	protected boolean isEditable(){
		
		// Some elements are only enabled (as in SComponent's setEnabled) only
		// if a Rule and a Process are set, and the user owns the Rule being 
		// viewed (and potentially edited).
		return (rule != null && rule.getCreatorUser().equals(currentUser));
	}
	
	/**
	 * Change editing mode, according to the current Rule.
	 */
	protected void changeEditingMode(){
	
		setEnabled(isEditable());
	}
	
	
	/** 
	 * Enables/disables all editable components in the editor.
	 * This either turns this panel to the normal editor or just a Rule viewer.
	 * 
	 * @see org.wings.SComponent#setEnabled(boolean)
	 */
	public void setEnabled(boolean enabled) {

		super.setEnabled(enabled);
		
		// Handle buttons and fields
		addActionButton.setEnabled(enabled);
		addConditionButton.setEnabled(enabled);
		nameField.setEnabled(enabled);
		applyButton.setEnabled(enabled);
		revertButton.setEnabled(enabled);

		// ...make all condition panels non-editable... 
		for (SComponent comp : conditionsDivision.getComponents()) {
			comp.setEnabled(enabled);
		}

		// ...and so for the action panels.
		for (SComponent comp : actionsDivision.getComponents()) {
			comp.setEnabled(enabled);
		}

		// Handle specifier parameters
		designContext.enableSpecifierComponents(enabled);
	}

	/**
	 * By setting the Rule, the editor re-configures itself for the given Rule.
	 * 
	 * @param newRule the Rule that will be shown/edited in this panel.
	 */
	public void setRule(Rule newRule) {

		// Return immediately if the newRule is the one we already know.
		if (newRule == this.rule) {
			return;
		}

		// From this point on, newRule is really NEW.
		
		removeAllConditionPanels();

		removeAllActionPanels();

		// Acquire newRule.
		this.rule = newRule;
		
		// If the newRule is not null...
		if (rule != null) {
			// Show the Rule's name in the UI...
			nameField.setText(rule.getName());
			
			designContext.changeSpecifiers(rule);
			
			// Auto-generate condition/action panels according to Rule's contents.
			populateOperationPanels();
		} else {
			// If it was null, do not populate and also clear up the nameField.
			nameField.setText("");
		}

		// Update editability of components for the new rule.
		changeEditingMode();
		
		// If it is a freshly created, editable, Rule then it that lacks any meta-information.
		// We'll provide the corresponding meta-information, since if we don't the
		// comparison at isDirty() will fail... (at the point of comparing the meta-informations
		// of the two Rules... see Rule#equals() ).
		if(isEditable() && rule.getMetaInformation().size()==0){
			designContext.populateRule(rule);
		}

	}
	
	/**
	 * Removes all action panels.
	 */
	public void removeAllActionPanels(){
		// Remove all action-panels...
		while (actionPanelCount > 0) {
			actionsDivision.remove(0); // remove the panel itself
			actionsDivision.remove(0); // remove the "Remove" button
			actionPanelCount--;
		}
	}

	/**
	 * Removes all condition panels.
	 */
	public void removeAllConditionPanels(){
		// Remove all condition-panels...
		while (conditionPanelCount > 0) {
			conditionsDivision.remove(0); // remove the panel itself
			conditionsDivision.remove(0); // remove the "Remove" button
			conditionPanelCount--;
		}
	}

	/**
	 * Gets the action panel count.
	 * 
	 * @return the action panel count
	 */
	public int getActionPanelCount() {
		return actionPanelCount;
	}

	/**
	 * Gets the condition panel count.
	 * 
	 * @return the condition panel count
	 */
	public int getConditionPanelCount() {
		return conditionPanelCount;
	}

	/**
	 * Gets the current Rule.
	 * 
	 * @return the current Rule
	 */
	public Rule getRule() {
		return rule;
	}

	/**
	 * Gets the current user-id.
	 * 
	 * @return the current user-id
	 */
	public String getCurrentUser() {
		return currentUser;
	}

	/**
	 * Sets the current user-id.
	 * 
	 * @param currentUser the new current user
	 */
	public void setCurrentUser(String currentUser) {
		this.currentUser = currentUser;
	}

	/**
	 * Gets the name field.
	 * 
	 * @return the name field
	 */
	public STextField getNameField() {
		return nameField;
	}

	/**
	 * Populates this panel with the basic UI components.
	 * This method is only invoked once, upon initial construction.
	 * For setting up the editor for the given Rule see {@link #populateOperationPanels()}.
	 *  
	 */
	protected void populateRulePanel() {

		// Instantiate the components that will make up our panel...
		nameField = new STextField("");
		addConditionButton = new SButton(msg("addCondition"));
		addActionButton = new SButton(msg("addAction"));
		conditionsDivision = new XDivision();
		actionsDivision = new XDivision();

		designContext.addSpecifierComponents(this);
		
        GridBagConstraints constraints = new GridBagConstraints();
		// Add the label for the Rule's name...
		constraints.gridwidth = 1;
		constraints.weightx = 0.2;
		contentPane.add(new SLabel("Name:"));
		
		// Add the input-field for the Rule's name.
		constraints.weightx = 0.8;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		nameField.setHorizontalAlignment(SConstants.LEFT_ALIGN);
		nameField.setAttribute("width", "80%");
		contentPane.add(nameField, constraints);

		// Add the conditions/actions divisions...
		// Each of these will contain the condition/action panels.
		
		constraints.weightx = 1;
		constraints.fill = GridBagConstraints.VERTICAL;

		conditionsDivision.setTitle(msg("conditions"));
		conditionsDivision.setAttribute("width", "100%");

		actionsDivision.setTitle(msg("actions"));
		actionsDivision.setAttribute("width", "100%");

		contentPane.add(conditionsDivision, constraints);
		contentPane.add(actionsDivision, constraints);

		// Add the "Add Condition/Action" buttons to the corresponding division...
		
		conditionsDivision.setLayout(new SGridLayout(0, 2));
		actionsDivision.setLayout(new SGridLayout(0, 2));

		addConditionButton.setHorizontalAlignment(SConstants.LEFT_ALIGN);
		addConditionButton.setVerticalAlignment(SConstants.BOTTOM_ALIGN);

		addActionButton.setHorizontalAlignment(SConstants.LEFT_ALIGN);
		addActionButton.setVerticalAlignment(SConstants.BOTTOM_ALIGN);

		conditionsDivision.add(addConditionButton);
		actionsDivision.add(addActionButton);

		// Now, add all the ActionListeners...
		
		addConditionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addConditionClicked();
			}
		});

		addActionButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addActionClicked();
			}
		});

	}

	/**
	 * Populate the Condition- and ActionDivisions with the Condition- and ActionPanels
	 * that correspond to the current Rule.
	 */
	private void populateOperationPanels() {

		// For each condition in the current Rule...
		for (Condition condition : rule.getConditions()) {
			ConditionController controller;
			try {
				// ... ask for the corresponding ConditionController...
				controller = ConditionController.newInstance(condition);
				// ... and add it to the panel.
				addConditionPanel(new ConditionPanel(RuleEditorPanel.this,
						controller));
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		// Same principle applies for actions in the Rule...
		
		for (Action action : rule.getActions()) {
			ActionController controller;
			try {
				controller = ActionController.newInstance(action);
				addActionPanel(new ActionPanel(RuleEditorPanel.this, controller));
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}
	
	/**
	 * Called when "new action" was clicked.
	 */
	protected void addActionClicked() {
		
		// Create a new ActionPanel attached to us (RuleEditorPanel) and...
		final ActionPanel actionPanel = new ActionPanel(RuleEditorPanel.this);

		// ...add it to the UI.
		addActionPanel(actionPanel);
	}

	/**
	 * Adds an action panel to the ActionDivision-
	 * 
	 * @param panel the panel to be added.
	 */
	protected void addActionPanel(final ActionPanel panel) {

		// Instantiate an ActionListener that will remove this ActionPanel when 
		// pressed on its corresponding "Remove" button...
		ActionListener removeButtonActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				actionsDivision.remove(panel);
				actionsDivision.remove((SComponent) e.getSource());
				actionPanelCount--;
			}
		};

		// Add this ActionPanel to the actionsDivision.
		addOperationPanel(actionsDivision, panel, removeButtonActionListener);
		
		// Update this; important when later removing the panels (see setRule()).
		actionPanelCount++;
	}

	/**
	 * Called when "new condition" is clicked.
	 */
	protected void addConditionClicked() {

		// Create a new ConditionPanel attached to us (RuleEditorPanel) and...
		final ConditionPanel conditionPanel = new ConditionPanel(
				RuleEditorPanel.this);

		// ...add it to the UI.
		addConditionPanel(conditionPanel);
	}

	/**
	 * Adds a ConditionPanel to the ConditionDivision.
	 * 
	 * @param panel the panel to be added.
	 */
	protected void addConditionPanel(final ConditionPanel panel) {

		// Instantiate an ActionListener that will remove this ConditionPanel when 
		// pressed on its corresponding "Remove" button...
		ActionListener removeButtonActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				conditionsDivision.remove(panel);
				conditionsDivision.remove((SComponent) e.getSource());
				conditionPanelCount--;
			}
		};

		// Add this ConditionPanel to the conditionsDivision.
		addOperationPanel(conditionsDivision, panel, removeButtonActionListener);
		// Update this; important when later removing the panels (see setRule()).
		conditionPanelCount++;
	}


	/**
	 * Adds an OperationPanel to a XDivision.
	 * 
	 * @param operationsDivision the XDivision for the panel
	 * @param operationPanel the operation panel
	 * @param removeButtonActionListener the "remove button" handler
	 */
	protected void addOperationPanel(XDivision operationsDivision,
			OperationPanel operationPanel, ActionListener removeButtonActionListener) {

		// Create a "Remove" button for this OperationPanel.
		XButton removeButton = new XButton(new RemoveAction(removeButtonActionListener));

        operationPanel.setAttribute("width", "100%");

		// Add the operationPanel at the end, just before the 
		// "Add XXX" (condition/action) button...
		operationsDivision.add(operationPanel, operationsDivision
				.getComponentCount() - 1);
		// ... followed by its corresponding "Remove" button...
		operationsDivision.add(removeButton, operationsDivision
				.getComponentCount() - 1);
		// ... which is attached the given ActionListener.
		removeButton.addActionListener(removeButtonActionListener);
	}

    class RemoveAction
        extends AbstractAction
    {
        private ActionListener forward;

        public RemoveAction(ActionListener removeButtonActionListener) {
            this.forward = removeButtonActionListener;
            putValue(SMALL_ICON, AgentIcons.REMOVE);
            putValue(SHORT_DESCRIPTION, msg("remove"));
        }

        public void actionPerformed(ActionEvent e) {
            forward.actionPerformed(e);
        }
    }

    /**
	 * Applies changes to the current Rule.
	 */
	public void applyChanges() {

		// Generate a new Rule from the current UI state of the panel.
		Rule generatedRule = generateRuleFromPanel();

		// Transfer the contents of the generated rule to our "working" Rule.
		rule.setActions(generatedRule.getActions());
		rule.setConditions(generatedRule.getConditions());
		rule.setCreatorUser(generatedRule.getCreatorUser());
		rule.setName(generatedRule.getName());
		rule.setMetaInformation(generatedRule.getMetaInformation());
	}

	/**
	 * From the whole visual state of the editor, it generates a new Rule instance.
	 * 
	 * @return the generated Rule object.
	 */
	protected Rule generateRuleFromPanel() {

		Rule newRule = new Rule();

		// Set the creator to the currentUser.
		newRule.setCreatorUser(currentUser);
		// Set the public state to the working rule's state.
		newRule.setPublic(rule.isPublic());
		// Get the Rule's name from the UI.
		newRule.setName(nameField.getText());

		// Get the placeholder lists.
		List<Condition> conditions = newRule.getConditions();
		List<Action> actions = newRule.getActions();

		// Ask each Action/ConditionController to return an Action/Condition that
		// reflects their current UI contents, and add that Action/Condition to
		// the newRule.
		
		for (SComponent comp : conditionsDivision.getComponents()) {
			if (comp instanceof ConditionPanel) {
				conditions.add(((ConditionPanel) comp).getCondition());
			}
		}

		for (SComponent comp : actionsDivision.getComponents()) {
			if (comp instanceof ActionPanel) {
				actions.add(((ActionPanel) comp).getAction());
			}
		}

		designContext.populateRule(newRule);

		// Return the "populated" newRule.
		return newRule;

	}

	/**
	 * Reverts the changes made in the UI, making it show the original (unmodified) Rule.
	 */
	public void revertChanges() {

		// This is because setRule() also assigns to this.rule and
		// the reference would be lost below (in setRule(null)).
		Rule tmpRule = rule;

		// This sets the editor-panel to an empty state.
		setRule(null);

		// And this populates it with the contents of our
		// original Rule.
		setRule(tmpRule);

	}

	/**
	 * Checks if the editor is "dirty". It is dirty if the visual state differs from the
	 * model (the Rule object being edited).
	 * 
	 * @return true, if editor is dirty
	 */
	public boolean isDirty() {
		
		if(isEditable()){
		
			Rule generatedRule = generateRuleFromPanel();
		
			return (rule.equals(generatedRule)) == false;
		}else
			// Non-editable Rules are per-definition non-dirty...
			return false;
	}

}
