package org.osbl.agent.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osbl.agent.logic.AgentSystem;
import org.osbl.agent.model.Rule;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.shell.Window;
import org.osbl.client.wings.XButton;
import org.wings.LowLevelEventListener;
import org.wings.SButton;
import org.wings.SCheckBox;
import org.wings.SComponent;
import org.wings.SDimension;
import org.wings.SFileChooser;
import org.wings.SFont;
import org.wings.SGridBagLayout;
import org.wings.SLabel;
import org.wings.SOptionPane;
import org.wings.SPanel;
import org.wings.STable;
import org.wings.border.SLineBorder;
import org.wings.externalizer.ExternalizeManager;
import org.wings.resource.FileResource;
import org.wings.script.JavaScriptListener;
import org.wings.script.ScriptListener;
import org.wings.session.SessionManager;
import org.wings.table.STableCellRenderer;
import org.wingx.XTable;
import org.wingx.table.EditableTableCellRenderer;
import org.wingx.table.XTableClickListener;
import org.wingx.table.XTableColumn;

import javax.swing.*;


/**
 * Provides the sidepanel that lets you browse existing Rules and perform some
 * operations on them (like add/remove/run/edit/publish...)
 * 
 * When selecting a Rule for editing, it will open itself in the Editor panel.
 * 
 * @author Sebastian Nozzi.
 */
public class SidePanel extends SPanel {

	/** The agent system. */
	private AgentSystem agentSystem;

	/** The rule editor panel. */
	private RuleEditorPanel ruleEditorPanel;

	/** The rules table. */
	private XTable rulesTable;

	/** The SideBar table model. */
	private SidePanelTableModel sidePanelTableModel;

	/** The window in which the ruleEditorPanel resides. */
	private Window ruleEditorWindow;

	/** The design context. */
	private DesignContext designContext;

	/**
	 * Instantiates a new SidePanel in the given DesignContext.
	 * 
	 * @param designContext the design context
	 */
	public SidePanel(DesignContext designContext) {
		super();

		setDesignContext(designContext);

		// We need THE AgentSystem instance for accessing the Rules repository
		agentSystem = AgentSystem.getSingletonInstance();

		// This defers instantiation of the editor panel and window until
		// it is really needed.
		ruleEditorWindow = null;
		ruleEditorPanel = null;

		populatePanel();
	}

	/**
	 * Gets the design context.
	 * 
	 * @return the design context
	 */
	public DesignContext getDesignContext() {
		return designContext;
	}

	/**
	 * Sets the design context.
	 * 
	 * @param designContext the new design context
	 */
	public void setDesignContext(DesignContext designContext) {
		this.designContext = designContext;
	}

	/**
	 * Convenience method to get localized versions of strings/labels
	 * 
	 * @param code the String-code
	 * @param args additional, optional String-args
	 * 
	 * @return the localized String
	 */
	protected String msg(String code, Object ... args){
		return designContext.msg(code, args);
	}

	/**
	 * Initially populates the SidePanel with the basic components.
	 * Only called upon construction.
	 * 
	 */
	protected void populatePanel() {

		// We'll be using SGridBagLayout for our panel...
		SGridBagLayout layout = new SGridBagLayout();
		// ... and for it we need the constraints used when adding components.
		GridBagConstraints constraints = new GridBagConstraints();

		// Set up visual properties of our panel...
		layout.setHgap(5);
		layout.setVgap(5);

		setLayout(layout);
		setPreferredSize(new SDimension("250px", null));

		// The RuleTableModel knows for example how to order Rules by
		// public-state, etc.
		sidePanelTableModel = new SidePanelTableModel() {
			List<Rule> getCurrentRules() {
				// The model needs "live" access to the current list of Rules
				// (which is different as soon as something in the
				// process/activity combos changes)
				return SidePanel.this.getCurrentRules();
			}

			// Whenever the Rules are updated in the Model...
			void rulesUpdated() {
				// ... force the AgentSystem to persist the new changes.
				agentSystem.updateRules();
			}
		};
		// The model also needs to know the current user, to make Rules editable
		// or not.
		sidePanelTableModel.setCurrentUser(DesignContext.getCurrentUser());

		// Create a new XTable using our custom table model.
		// We use an XTable in order to be able to use a ClickListener (see
		// below).
		rulesTable = new XTable(sidePanelTableModel);
        rulesTable.setBorder(new SLineBorder(new Color(0x99, 0x99, 0x99)));

        // Make the columns sortable.
		((XTableColumn) rulesTable.getColumnModel().getColumn(0))
				.setSortable(true);
		((XTableColumn) rulesTable.getColumnModel().getColumn(1))
				.setSortable(true);

		((XTableColumn) rulesTable.getColumnModel().getColumn(1))
				.setWidth("30px");

		// We wanted the name of the Rule being edited to be differently
		// rendered
		// (for example in bold letters, and the rest in plain text)
		rulesTable.getColumnModel().getColumn(0).setCellRenderer(
		// For this we need to specify a custom renderer...
				new STableCellRenderer() {
					// ...which will return a SLabel.
					public SComponent getTableCellRendererComponent(
							STable table, Object value, boolean isSelected,
							int row, int column) {
						// The SLabel consists of the name of the Rule (gotten
						// from the model)
						SLabel result = new SLabel(((String) value));

						List<Rule> rules = getCurrentRules();

						// If there are rules to begin with...
						if (rules.size() > 0) {
							Rule ruleToRender = rules.get(row);
							Rule editedRule = null;

							if (ruleEditorPanel != null)
								editedRule = ruleEditorPanel.getRule();

							// ...and if the rule we have to render is the same
							// that is
							// being editer in the editor panel...
							if (editedRule != null
									&& editedRule.equals(ruleToRender))
								// ...make the SLabel in BOLD letters
								result.setFont(new SFont(SFont.BOLD));
						} else
							// ...otherwise make it normal.
							result.setFont(new SFont(SFont.PLAIN));

						return result;
					}
				});
		// The BooleanEditableCellR.. is used to make your Rules public or not.
		rulesTable.getColumnModel().getColumn(1).setCellRenderer(
				new BooleanEditableCellRenderer());

		designContext.addSideFilters(this);

		// Accordingly, recompute current Rules (table)
		recomputeCurrentRules();
		refreshTable();
				
		// It's recommended to take a look at the SGridBagLayout documentation
		// to understand
		// the settings in the contraints.

		// Add the rule's table (previously configured, see above)
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = GridBagConstraints.REMAINDER;
		this.add(rulesTable, constraints);

		// Main buttons
		XButton addRuleButton = new XButton(new AddRuleAction());
        XButton removeRuleButton = new XButton(new RemoveRuleAction());
		XButton runButton = new XButton(new RunRuleAction());
		XButton importButton = new XButton(new ImportAction());
		XButton exportButton = new XButton(new ExportAction());

		// Add the buttons
		constraints.gridx = GridBagConstraints.RELATIVE;
		constraints.gridy = 3;
		constraints.gridwidth = 1;

		this.add(new SLabel(msg("agents") + ":"), constraints);
		this.add(addRuleButton, constraints);
		this.add(removeRuleButton, constraints);
		this.add(runButton, constraints);
		this.add(importButton, constraints);
        constraints.gridwidth = GridBagConstraints.REMAINDER;
		this.add(exportButton, constraints);


        // This is used to intercept clicks done on the name of a Rule.
		rulesTable.addClickListener(0, new XTableClickListener() {
			public void clickOccured(int row, int column) {
				Rule clickedRule = getCurrentRules().get(row);

				// This not only shows the editor window but instantiates the
				// window/panel if not already done so. Because of this,
				// this should be called before trying to access
				// ruleEditorPanel...
				showEditorWindow();

				ruleEditorPanel.setRule(clickedRule);
				// Force a table-refresh so that the Rule appears in BOLD.
				refreshTable();
			}
		});

	}

    class AddRuleAction
        extends AbstractAction
    {
        public AddRuleAction() {
            putValue(SMALL_ICON, AgentIcons.AGENT_NEW);
            putValue(SHORT_DESCRIPTION, msg("add"));
        }

        public void actionPerformed(ActionEvent e) {
            addRule();
            recomputeCurrentRules();
            refreshTable();
        }
    }

    class RemoveRuleAction
        extends AbstractAction
    {
        public RemoveRuleAction() {
            putValue(SMALL_ICON, AgentIcons.AGENT_DELETE);
            putValue(SHORT_DESCRIPTION, msg("remove"));
        }

        public void actionPerformed(ActionEvent e) {
            removeRule();
            recomputeCurrentRules();
            refreshTable();
        }
    }

    class RunRuleAction
        extends AbstractAction
    {
        public RunRuleAction() {
            putValue(SMALL_ICON, AgentIcons.AGENT_RUN);
            putValue(SHORT_DESCRIPTION, msg("run"));
        }

        public void actionPerformed(ActionEvent e) {
            runRules();
        }
    }

    class ImportAction
        extends AbstractAction
    {
        public ImportAction() {
            putValue(SMALL_ICON, AgentIcons.UPLOAD);
            putValue(SHORT_DESCRIPTION, msg("import"));
        }

        public void actionPerformed(ActionEvent e) {
            importRule();
            recomputeCurrentRules();
            refreshTable();
        }
    }

    class ExportAction
        extends AbstractAction
    {
        public ExportAction() {
            putValue(SMALL_ICON, AgentIcons.DOWNLOAD);
            putValue(SHORT_DESCRIPTION, msg("export"));
        }

        public void actionPerformed(ActionEvent e) {
            exportRule();
        }
    }

    /**
	 * Recompute current rules.
	 */
	void recomputeCurrentRules() {
		designContext.recomputeCurrentRules();
	}

	/**
	 * Run the selected Rule(s), currently only runs one Rule.
	 */
	protected void runRules() {

		List<Rule> rules = getCurrentRules();
		// List<Rule> selectedRules = new ArrayList<Rule>();

		// For each row in the rule table...
		for (int idx : rulesTable.getSelectedRows()) {

			Rule selectedRule = rules.get(idx);

			designContext.getPreviewController().runRule(selectedRule);
			break;
		}

	}

	/**
	 * Export the selected Rule, providing an XML file for it.
	 */
	protected void exportRule() {

		List<Rule> rules = getCurrentRules();

		// For each row in the rule table...
		for (int idx : rulesTable.getSelectedRows()) {

			Rule selectedRule = rules.get(idx);
			File tmpRuleFile = agentSystem.tempRuleFile(selectedRule);
			
			/*
			String xmlRule = agentSystem.ruleAsXML(selectedRule);

			StringResource resource = new StringResource(xmlRule, ".xml",
					"text/xml", ExternalizeManager.REQUEST);
			*/
			String name = selectedRule.getName();
			name = name.replace(' ', '_');

			FileResource resource = new FileResource(tmpRuleFile);
			resource.setExternalizerFlags(resource.getExternalizerFlags() | ExternalizeManager.REQUEST);			
			
			Map headers = new HashMap();
			headers.put("Content-Disposition", "attachment; filename="
					+ name+".osbl-agent.xml");
			resource.setHeaders(headers.entrySet());

			final ScriptListener listener = new JavaScriptListener(null, null,
					"location.href='" + resource.getURL() + "'");
			SessionManager.getSession().getScriptManager().addScriptListener(
					listener);

			// Stop after first selected Rule, in case there are more than one...
			break;
		}
	}

	/**
	 * Allows the user to import a Rule from an XML file.
	 */
	protected void importRule() {

		final SFileChooser fileInputChooser = new SFileChooser();

		fileInputChooser.setFileNameFilter("text/xml");

		SOptionPane.showInputDialog(this,
				msg("xmlImportMessage"),
				msg("xmlImportTitle"), fileInputChooser, new ActionListener() {

					public void actionPerformed(ActionEvent e) {

						// User uploaded a file...
						if (e.getActionCommand().equals(SOptionPane.OK_ACTION)) {


							Rule newRule = null;
							try {
								newRule = agentSystem
										.XMLToRule(fileInputChooser
												.getSelectedFile());
							} catch (IOException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}

							if (newRule != null) {
								// Make our current user the owner for the freshly imported
								// Rule, since exported rules retain their creatorUser in its 
								// fields.
								newRule.setCreatorUser(DesignContext.getCurrentUser());
								
								newRule.setPublic(false);
								
								agentSystem.addRule(newRule);
								
								recomputeCurrentRules();
								
								refreshTable();
							}

						}

					}
				});
	}

	/**
	 * Gets the current Rules.
	 * 
	 * @return the current Rules
	 */
	protected List<Rule> getCurrentRules() {
		return designContext.getCurrentRules();
	}

	/** Used to name the initially unnamed Rules. */
	private int untitledNr = 0;

	/**
	 * Adds a new Rule to the system.
	 */
	protected void addRule() {

		// Create a Rule and assign a default name to it.
		Rule newRule = new Rule("untitled" + untitledNr++);

		newRule.setCreatorUser(DesignContext.getCurrentUser());

		showEditorWindow();

		ruleEditorPanel.setRule(newRule);
	}

	/**
	 * Refresh the table that shows the Rules according to the filters.
	 */
	void refreshTable() {
		
		((SidePanelTableModel)rulesTable.getModel()).refresh();
		
	}

	/**
	 * Removes the selected Rule from the system.
	 */
	protected void removeRule() {

		List<Rule> rules = getCurrentRules();

		// Look for selected Rules in the table's rows...
		for (int idx : rulesTable.getSelectedRows()) {

			Rule selectedRule = rules.get(idx);

			// ... but only delete the Rules that this user owns
			if (selectedRule.getCreatorUser().equals(
					DesignContext.getCurrentUser()))
				agentSystem.removeRule(selectedRule);
		}

	}

	/**
	 * Sets the RuleEditor up.
	 */
	void setupEditor() {

		// This is to communicate with the editor panel
		ruleEditorPanel = designContext.getRuleEditorPanel();

		// The editor panel also needs to know the current user
		// (to only let the user edit his/her own Rules)
		ruleEditorPanel.setCurrentUser(DesignContext.getCurrentUser());
		// The editor panel also needs to communicate with us
		ruleEditorPanel.setAgentSidebarPanel(this);

		ruleEditorWindow = new Window();
		ruleEditorWindow.setContentPane(ruleEditorPanel);

		// Buttons like "Apply", "Revert", "Close"
		List<SComponent> buttons = new ArrayList<SComponent>();

		buttons.add(ruleEditorPanel.closeButton);
		buttons.add(ruleEditorPanel.revertButton);
		buttons.add(ruleEditorPanel.applyButton);

		ruleEditorWindow.setControls(buttons);
		
		ruleEditorWindow.setTitle(msg("editorWindowTitle"));

		ruleEditorPanel.closeButton.addActionListener(new ActionListener() {
			// On "close", show whatever was underneath
			public void actionPerformed(ActionEvent e) {

				if (ruleEditorPanel.isDirty()) {
					// Show a confirmation, informing the user that by closing
					// the
					// window he will lose all changes made...
					SOptionPane
							.showYesNoDialog(
									ruleEditorPanel,
									msg("closeEditorMessage"),
									msg("closeEditorTitle"), new ActionListener() {

										public void actionPerformed(
												ActionEvent e) {

											if (e.getActionCommand().equals(
													SOptionPane.NO_ACTION)) {
											} else {
												Client.getInstance().popWindow(
														ruleEditorWindow);
											}
										}
									});
					// Rule is not dirty, just close the window...
				} else {
					Client.getInstance().popWindow(ruleEditorWindow);
				}
			}
		});

		ruleEditorPanel.revertButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ruleEditorPanel.revertChanges();
			}
		});

		ruleEditorPanel.applyButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				ruleEditorPanel.applyChanges();
				Rule editedRule = ruleEditorPanel.getRule();

				// Add rule to the system if it was a new Rule.
				if (agentSystem.getRules().contains(editedRule) == false)
					agentSystem.addRule(editedRule);
				else
					agentSystem.updateRules();

				// Force refresh to eventually
				// reflect a name-change.
				recomputeCurrentRules();
				refreshTable();
			}
		});
	}

	/**
	 * Shows the editor window.
	 */
	void showEditorWindow() {

		if (ruleEditorWindow == null)
			setupEditor();

		Client.getInstance().pushWindow(designContext.getParentWindow(), ruleEditorWindow);
	}

	/**
	 * A custom EditableTableCellRenderer, the class BooleanEditableCellRenderer presents
	 * boolean values in a table-cell as a checkbox. It reflects the corresponding boolean
	 * value and updates the model when the checkbox changes.
	 */
	class BooleanEditableCellRenderer extends SCheckBox implements
			EditableTableCellRenderer {

		/** The label. */
		SLabel label = new SLabel();

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.wingx.table.EditableTableCellRenderer#getValue()
		 */
		public Object getValue() {
			return isSelected() ? Boolean.TRUE : Boolean.FALSE;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.wings.table.STableCellRenderer#getTableCellRendererComponent(org.wings.STable,
		 *      java.lang.Object, boolean, int, int)
		 */
		public SComponent getTableCellRendererComponent(STable table,
				Object value, boolean isSelected, int row, int column) {

			// Remember that we extend SCheckBox, so we ARE a SCheckBox

			// If "value" is a Boolean (which the TableModel returns in case the
			// Rule
			// is an user-editable Rule, this is, an "own" Rule)..
			if (value instanceof Boolean) {
				// ... check/uncheck according to the boolean value of "value"
				this.setSelected(value.equals(Boolean.TRUE));
				// don't show any text... it's our own Rule, we know we created
				// it
				this.setText("");
				// let the user check/uncheck de SCheckBox, and...
				this.setEnabled(true);
				// return ourselves
				return this;
				// That "value" is not a Boolean means that the Rule the
				// TableModel is
				// processing is not our own but a foreign public Rule...
			} else if (value instanceof String) {
				// Make the checkbox selected (checked).
				// TODO: Is this necessary? We are returning a SLabel after
				// all...
				// Maybe it's necessary, because of how the mechanism of
				// renderers work...
				this.setSelected(true);

				// Instead of returning ourselves, we have a SLabel in which
				// we'll
				// set the user-name of the user that the current Rule belong
				// and
				// will just return that. We save some space, because the
				// check-box
				// would be non-editable anyway (a user can only make his/her
				// Rules
				// public/non-public)
				label.setText((String) value);

				// Note that because of how the renderers mechanism work, the
				// right
				// solution is to instantiate ONE SLabel, set it each time
				// according
				// to the parameters and return it, and NOT creating one
				// instance
				// each time.

				// Return the SLabel with the users-id instead of the check-box.
				return label;
			}

			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.wingx.table.EditableTableCellRenderer#getLowLevelEventListener(org.wings.STable,
		 *      int, int)
		 */
		public LowLevelEventListener getLowLevelEventListener(STable table,
				int row, int column) {

			return this;
		}
	}

}
