package org.osbl.agent.gui;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.concern.Controller;
import org.concern.ControllerException;
import org.concern.ControllerLookup;
import org.concern.UnknownSubjectException;
import org.concern.Work;
import org.concern.model.Activity;
import org.concern.model.AsynchronousActivity;
import org.concern.model.Process;
import org.osbl.agent.logic.AgentSystem;
import org.osbl.agent.logic.RuntimeContext;
import org.osbl.agent.logic.TaskRuntimeContext;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;
import org.osbl.client.wings.concern.TaskEditor;
import org.osbl.client.wings.concern.TaskForm;
import org.osbl.client.wings.concern.TaskList;
import org.osbl.client.wings.form.NestedObjectForm;
import org.osbl.client.wings.shell.Client;
import org.wings.SComboBox;
import org.wings.SComponent;
import org.wings.SConstants;
import org.wings.SDimension;
import org.wings.SLabel;
import org.wings.SListCellRenderer;
import org.wings.SPanel;


/**
 * The Class TaskDesignContext.
 * 
 * @author Sebastian Nozzi.
 */
public class TaskDesignContext extends SubjectDesignContext {

	/** The activity combo. */
	SComboBox activityComboFilter;

	/** The activity combo specifier. */
	SComboBox activityComboSpecifier;
	
	/** The old process combo index. */
	int oldProcessComboIndex;
	
	/**
	 * Instantiates a new task design context.
	 * 
	 * @param taskEditor the task editor
	 */
	public TaskDesignContext(TaskEditor taskEditor) {
		super(taskEditor);

		// We need THE AgentSystem instance for accessing the Rules repository
		agentSystem = AgentSystem.getSingletonInstance();

		ruleEditorPanel = new RuleEditorPanel(this);
		sidePanel = new SidePanel(this);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getAgentSidePanel()
	 */
	public SidePanel getAgentSidePanel() {
		return sidePanel;
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.DesignContext#getRuleEditorPanel()
	 */
	public RuleEditorPanel getRuleEditorPanel() {
		return ruleEditorPanel;
	}

	/**
	 * Update activities.
	 * 
	 * @param processList the process list
	 * @param activityCombo the activity combo
	 * @param addAny the add any
	 */
	protected void updateActivities(List<Process> processList, SComboBox activityCombo, boolean addAny) {

		// Remove all activities already there.
		activityCombo.removeAllItems();

		if(addAny){
			// First item is reserved for the wildcard "all"
			activityCombo.addItem(msg("all"));
		}

		// For each selected processes (maybe one, maybe many)
		for (Process process : processList) {

			// Get the activities and add them to the activity combo
			for (Object act : process.getActivities()) {
				
				// Only add AsynchronousActivities (those that require
				// user intervention and for which we are offering Agents)
				if(act instanceof AsynchronousActivity)
					activityCombo.addItem(act);
			}
		}
	}
	
	/**
	 * Adds the process activity combos.
	 * 
	 * @param processCombo the process combo
	 * @param activityCombo the activity combo
	 * @param target the target
	 */
	protected void addProcessActivityCombos(SComboBox processCombo, SComboBox activityCombo, SPanel target){

		GridBagConstraints constraints = new GridBagConstraints();

		super.addProcessCombo(processCombo, target);

		// Add the label for the activity combo
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 1;
		target.add(new SLabel(msg("activity")+":"), constraints);

		// Add the activity combo
		constraints.gridx = 1;
		constraints.gridy = 1;
		constraints.gridwidth = constraints.REMAINDER;
		target.add(activityCombo, constraints);
		activityCombo.setHorizontalAlignment(SConstants.LEFT_ALIGN);
		
		activityCombo.setRenderer(new SListCellRenderer(){

			SLabel label = new SLabel();
			
			public SComponent getListCellRendererComponent(SComponent list, java.lang.Object value, boolean isSelected, int index) {
				
				String result = value.toString();
				
				// For the special case of "<all>", value is a String and not an Activity
				if(value instanceof Activity){
					
					result = ((Activity)value).getName();
					
					// Since an Activity doesn't know its parent Process, we have to look it up
					for(Process process : agentSystem.getProcessList()){
						if(process.getActivities().contains(value)){
							// Once found, we can construct the key to ask for a translation
							// of the Activity
							result = Client.getInstance().getResourceProvider().getMessage("process."+process.getName()+"."+((Activity)value).getName());
							break;
						}
					}
				}
				
				label.setText(result);
				
				return label;
			}
		});	
		
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#recomputeCurrentRules()
	 */
	public void recomputeCurrentRules() {

		currentRules.clear();
		
		// If a specific activity is selected (index 0 is for "all")...
		if (activityComboFilter.getSelectedIndex() != 0) {

			// We'll want to find the parentWindow Process for the Activity
			// here
			Process parentProcess = null;
			// Get the selected activity (which IS a valid Activity, since
			// idx!=0)
			Activity act = (Activity) activityComboFilter.getSelectedItem();

			// If the processCombo is in "all" (index 0)
			if (processComboFilter.getSelectedIndex() == 0) {
				// Look for the parentWindow Process to which the Activity
				// belongs
				for (Process process : agentSystem.getProcessList()) {
					if (process.getActivities().contains(act)) {
						// Parent found!
						parentProcess = process;
						break;
					}
				}
				// Otherwise finding the parentWindow process is
				// straightforward...
			} else {
				// ... get it from the combo box!
				parentProcess = getSelectedProcessList(processComboFilter,true).get(0);
			}

			// At this point we have one Process/Activity pair, so...

			// ...re-compute rules for our Process/Activity pair.
			recomputeCurrentRules(parentProcess, act);
			// Activity combo was on "all"...
		} else {
			// Get list of selected Process (may be one or many) and
			// iterate over them/it
			for (Process proc : getSelectedProcessList(processComboFilter,true)) {
				// For each Process get its Activities...
				for (Object act : proc.getActivities()) {
					// Re-compute rules for each Process/Activity pair.
					recomputeCurrentRules(proc, (Activity) act);
				}
			}
		}
	}
	
	/**
	 * Recompute current rules.
	 * 
	 * @param proc the proc
	 * @param act the act
	 */
	private void recomputeCurrentRules(Process proc, Activity act) {

		List<Rule> rulesToAdd = new ArrayList<Rule>();
		
		// Now filter out non-corresponding Rules...
		for (Rule aRule : agentSystem.getRules()) {
			
			if (belongsToDesignContext(aRule)){
				String processName = (String) aRule.getMetaInformation().get(
						RuleContext.PROCESS_NAME_KEY);
				String activityName = (String) aRule.getMetaInformation().get(RuleContext.ACTIVITY_NAME_KEY);

				if(aRule.isPublic()==false && aRule.getCreatorUser().equals(DesignContext.getCurrentUser())==false){
					continue;
				}
				
				if (processName.equals(proc.getName()) && activityName.equals(act.getName())){
					rulesToAdd.add(aRule);
				}
			}
		}
		
		currentRules.addAll(rulesToAdd);
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#addSideFilters(org.osbl.agent.gui.SidePanel)
	 */
	public void addSideFilters(final SidePanel targetPanel) {

		// This two combos will let the user specify the process/activity
		// pair.
		processComboFilter = new SComboBox();
		activityComboFilter = new SComboBox();
		
		activityComboFilter.setPreferredSize(new SDimension("140px",SDimension.AUTO));
		processComboFilter.setPreferredSize(new SDimension("140px",SDimension.AUTO));

		addProcessActivityCombos(processComboFilter, activityComboFilter, targetPanel);
		
		populateProcessCombo(processComboFilter,true);
		
		// Force the activities combo to update
		updateActivities(agentSystem.getProcessList(), activityComboFilter, true);

		// Selecting another Process causes Activities and Rules to be
		// updated.
		processComboFilter.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				updateActivities(getSelectedProcessList(processComboFilter,true), activityComboFilter, true);
				recomputeCurrentRules();
				targetPanel.refreshTable();
			}
		});

		// Selecting another Activity just causes Rules to be udated.
		activityComboFilter.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				recomputeCurrentRules();
				targetPanel.refreshTable();
			}
		});
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#addSpecifierComponents(org.osbl.agent.gui.RuleEditorPanel)
	 */
	public void addSpecifierComponents(RuleEditorPanel targetPanel) {
		// This two combos will let the user specify the process/activity
		// pair.
		processComboSpecifier = new SComboBox();
		activityComboSpecifier = new SComboBox();
		
		
		addProcessActivityCombos(processComboSpecifier, activityComboSpecifier, targetPanel.contentPane);

		populateProcessCombo(processComboSpecifier,false);
		
		// Force the activities combo to update
		updateActivities(getSelectedProcessList(processComboSpecifier,false), activityComboSpecifier, false);
		
		updateBeanMeta();
		
		oldProcessComboIndex = processComboSpecifier.getSelectedIndex();

		processComboSpecifier.addActionListener(new ProcessChangeListener(){
			protected void processChanged() {
				// Force the activities combo to update
				updateActivities(getSelectedProcessList(processComboSpecifier,false), activityComboSpecifier, false);
			}
		});
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#enableSpecifierComponents(boolean)
	 */
	public void enableSpecifierComponents(boolean enabled){
		super.enableSpecifierComponents(enabled);
		activityComboSpecifier.setEnabled(enabled);
	}

	/**
	 * Gets the specified activity.
	 * 
	 * @return the specified activity
	 */
	public Activity getSpecifiedActivity() {
		return (Activity)activityComboSpecifier.getSelectedItem();
	}

		
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#populateRule(org.osbl.agent.model.Rule)
	 */
	public void populateRule(Rule editedRule) {

		Activity activity = getSpecifiedActivity();
		
		editedRule.getMetaInformation().put(RuleContext.ACTIVITY_NAME_KEY, activity.getName());
		
		super.populateRule(editedRule);
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#changeSpecifiers(org.osbl.agent.model.Rule)
	 */
	public void changeSpecifiers(Rule rule) {

		super.changeSpecifiers(rule);
		
		String ruleActivityImpl = (String)rule.getMetaInformation().get(RuleContext.ACTIVITY_NAME_KEY);

		setConfirmProcessChange(false);

		// It CAN be NULL for new Rules.
		if(ruleActivityImpl!=null){
			for(int i=0;i<activityComboSpecifier.getItemCount();i++){
				Activity comboActivity = (Activity)activityComboSpecifier.getItemAt(i);
				if(ruleActivityImpl.equals(comboActivity.getName()))
					activityComboSpecifier.setSelectedIndex(i);
			}
		}

		setConfirmProcessChange(true);
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#getRuntimeObjectClass()
	 */
	public Class getRuntimeObjectClass() {
		return Work.class;
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#getRuntimeContext()
	 */
	public RuntimeContext getRuntimeContext() {
		
		return new TaskRuntimeContext();
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.SubjectDesignContext#getPreviewControllerInstance()
	 */
	public PreviewController getPreviewControllerInstance() {

		PreviewController result = new PreviewController(this){
			
			TaskForm taskForm = new TaskForm();
			
			public NestedObjectForm getObjectForm() {
				return taskForm;
			}

			public Object getListObject(int rowIdx) {
				Object result = ((TaskList)designContext.getObjectEditor().getList()).getRow(rowIdx);
				return result;
			}

			public void populateForm(Object targetObject) {
				taskForm.setTask((Work)targetObject);
				taskForm.getEnvironment().setActive(true);
				taskForm.setObject(targetObject);
			}

			protected void postUpdateActions(Object targetObject) {
				
				Work task = (Work)targetObject; 
		        Controller controller = getController(targetObject);
		    	
		    	Map<String,String> annotations = new HashMap<String,String>();
	            annotations.put("user", DesignContext.getCurrentUser());
	            
	            try {
					controller.complete(task.getSubjectId(), task.getActivity(), annotations);
		            controller.announceSubject(task.getSubjectId());
				} catch (UnknownSubjectException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (ControllerException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			protected Controller getController(Object targetObject) {
				Work task = (Work)targetObject; 
		        return ControllerLookup.getInstance().getController(task.getProcess());
			}

			public Object getSubjectInstance(Object targetObject) {
				return ((Work)targetObject).getSubject();
			}
			
		};
		
		return result;
	}
	
	
}
