package org.osbl.agent.gui.condition;

import java.util.ArrayList;
import java.util.List;

import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.osbl.agent.gui.ConditionController;
import org.osbl.agent.gui.OperationController;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.action.SetCurrentTimeAction;
import org.osbl.agent.model.condition.BinaryOperator;
import org.osbl.agent.model.condition.Operator;
import org.osbl.agent.model.condition.PropertyCondition;
import org.osbl.client.wings.shell.Client;
import org.wings.SComboBox;
import org.wings.SComponent;



/**
 * The class PropertyConditionController models the PropertyCondition.
 * 
 * It serves as a superclass for ConditionControllers that want to offer the comparison
 * of a property against a value.
 * 
 * It presents the user with an operator combo-box. Anything beyond that element is 
 * provided by the subclasses. Subclasses can also choose not to include the operator
 * combo at all (as in {@link SetCurrentTimeAction}), although this is rare.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class PropertyConditionController extends ConditionController {

	/** The operator combo. */
	private SComboBox operatorCombo;

	/** The property meta. */
	protected PropertyMeta propertyMeta;

	/**
	 * Instantiates a new PropertyConditionController.
	 */
	public PropertyConditionController() {

		// Combo for the Operators that apply to the selected property
		// and the value entered by the user in the inputField.
		operatorCombo = new SComboBox();

		// Get the Operators from getOperators (overriden by subclasses)...
		for (Operator relationalOperator : getOperators()) {
			// ...and add each Operator to the operatorCombo.
			operatorCombo.addItem(relationalOperator);
		}

	}

	/**
	 * Gets the property meta.
	 * 
	 * @return the property meta
	 */
	public PropertyMeta getPropertyMeta() {
		return propertyMeta;
	}


	/**
	 * Sets the property meta.
	 * 
	 * @param propertyMeta the new property meta
	 */
	public void setPropertyMeta(PropertyMeta propertyMeta) {
		this.propertyMeta = propertyMeta;
	}

	/**
	 * Gets the value to compare to.
	 * 
	 * @return the value to compare to
	 */
	public abstract Object getValueToCompareTo();

	/**
	 * Sets the value to compare to.
	 * 
	 * @param condition the new value to compare to
	 */
	public abstract void setValueToCompareTo(PropertyCondition condition);
	
	/**
	 * Sets the operator.
	 * 
	 * @param operatorToBeSelected the new operator
	 */
	public void setOperator(Operator operatorToBeSelected){
		// iterate over all operators in the combo-box...
		for (int i = 0; i < operatorCombo.getItemCount(); i++) {
			Operator controllerOp = (Operator) operatorCombo.getItemAt(i);

			// ...until we find the one that matches...
			if (controllerOp.equals(operatorToBeSelected))
				// ...and make it the selected item in the combo-box.
				operatorCombo.setSelectedIndex(i);
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.RelationalConditionController#canBeReplacedBy(agent.controller.OperationController)
	 */
	public boolean canBeReplacedBy(OperationController candidateController) {
		
		// There are many PropertyConditions in the UI, one for each property...
		
		// So, a replacement is found only if the class matches ...
		if (candidateController instanceof PropertyConditionController) {
			PropertyConditionController otherControlller = (PropertyConditionController) candidateController;
			// and they represent the same property...
			return ((PropertyCondition) otherControlller.getCondition())
					.getPropertyMetaName().equals(propertyMeta.getName());
		// Otherwise no match is found.
		} else
			return false;
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.ConditionController#populateCondition(org.osbl.agent.model.Condition)
	 */
	protected void populateCondition(Condition condition) {
		
		// Set the operation in the condition to whatever is selected in the UI
		((PropertyCondition) condition).setOperator((Operator) operatorCombo
				.getSelectedItem());
		
		((PropertyCondition) condition).setBeanMetaClassName(propertyMeta.getBeanMeta().getName());

		// Set the propertyMeta to the one set in this class.
		((PropertyCondition) condition).setPropertyMetaName(propertyMeta.getName());

		// Set the valueToCompareTo to whatever the user entered or selected in the UI.
		// (subclasses of this controller let the user enter/choose this information in
		// different ways).
		((PropertyCondition) condition).setValueToCompareTo(getValueToCompareTo());
	}	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.RelationalConditionController#getOperators()
	 */
	/**
	 * Gets the operators.
	 * 
	 * @return the operators
	 */
	public List<Operator> getOperators() {
		
		List<Operator> result = new ArrayList<Operator>();

		// Since we are dealing with fields in a generic way we can
		// only define operators we can be sure apply to all fields...
		
		result.add(new BinaryOperator(msg("operatorIs"),"=="));
		result.add(new BinaryOperator(msg("operatorIsNot"),"!="));
		
		// Subclasses of this class will add more specific Operators
		// on their own, depending of the type they represent.

		return result;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.RelationalConditionController#setCondition(org.osbl.agent.model.condition.Condition)
	 */
	public void setCondition(Condition condition) {

		// Get the operator that is set in the PropertyCondition given...
		Operator conditionOp = ((PropertyCondition) condition).getOperator();
		
		setOperator(conditionOp);

		// Attemp to find and set the propertyMeta according to the data
		// stored in the Condition.
		
		String className = ((PropertyCondition) condition).getBeanMetaClassName();
		String propertyName = ((PropertyCondition) condition).getPropertyMetaName();

		try {
			BeanMeta beanMeta = Client.getInstance().getBeanMetaProvider().getBeanMeta(Class.forName(className));
			setPropertyMeta(beanMeta.getProperty(propertyName));
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// Let subclasses modify the input/choice component to reflect the comparison
		// value stored in the condition.
		setValueToCompareTo((PropertyCondition) condition);
	}
	
	/* (non-Javadoc)
	 * @see org.osbl.agent.gui.OperationController#populateComponentList(java.util.List)
	 */
	protected void populateComponentList(List<SComponent> componentList) {
		componentList.add(operatorCombo);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		
		return "  " + propertyMeta.getLabel() +"   ("+propertyMeta.getName()+")";
	}
	
}
