package org.osbl.agent.gui.condition;

import java.util.List;

import org.concern.ControllerLookup;
import org.concern.Controller;
import org.concern.model.Process;
import org.osbl.agent.gui.ConditionController;
import org.osbl.agent.gui.OperationController;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.condition.BinaryOperator;
import org.osbl.agent.model.condition.Operator;
import org.osbl.agent.model.condition.SubjectCondition;
import org.osbl.client.wings.shell.Client;
import org.wings.SComboBox;
import org.wings.SComponent;
import org.wings.SLabel;
import org.wings.SListCellRenderer;


/**
 * The class SubjectConditionController models the SubjectCondition.
 * 
 * It presents the user with a combo-box with all available con:cern Conditions and the
 * possibility (also through a combo-box) to choose whether the choosen condition should
 * be met or not.
 * 
 * @author Sebastian Nozzi.
 */
public class SubjectConditionController extends ConditionController {

	/** The con:cern process. */
	private Process process;

	/** The conditions combo. */
	private SComboBox conditionsCombo;

	/** The operator combo. */
	private SComboBox operatorCombo;

	/**
	 * Instantiates a new SubjectConditionController.
	 */
	public SubjectConditionController() {
		this(null);
	}

	/**
	 * Instantiates a new subject condition controller.
	 * 
	 * @param process the process
	 */
	public SubjectConditionController(Process process) {
		this(process, null);
	}

	/**
	 * Instantiates a new subject condition controller.
	 * 
	 * @param process the con:cern process
	 * @param condition the con:cern condition
	 */
	public SubjectConditionController(Process process,
			org.concern.model.Condition condition) {

		// Create a combo-box for choosing the Operator
		operatorCombo = new SComboBox();

		// There is not much that can happen to a condition...
		// ... either the condition is met...
		operatorCombo.addItem(new BinaryOperator(msg("operatorIs"), "=="));
		// ... or it's not.
		operatorCombo.addItem(new BinaryOperator(msg("operatorIsNot"), "!="));

		// Here come all (concern)conditions of the Process
		conditionsCombo = new SComboBox();

		this.process = process;

		// Populate the combo-box with all conditions if the Process
		// is not null... (it can be null if this Controller was instantiated
		// when restoring an existing Rule, in which case setCondition will be
		// called later anyway).
		if (process != null) {
			populateConditionsCombo(process, null);
		}

	}

	/**
	 * Populate conditions combo.
	 * 
	 * @param process the process
	 * @param selectedConditionName the selected condition
	 */
	protected void populateConditionsCombo(Process process,
			String selectedConditionName) {

		// Iterate over all (concern) conditions in the Process...
		for (Object condObj : process.getConditions()) {

			// ... add each (concern) condition into the conditionsCombo...
			conditionsCombo.addItem(condObj);

			// ... and if the condition matches the selectedCondition (which we
			// get as a parameter) make it the selected item of the combo.
			if (((org.concern.model.Condition) condObj).getName().equals(
					selectedConditionName)) {
				conditionsCombo.setSelectedItem(condObj);
			}
		}
		
		final Process processForRenderer = process;

		conditionsCombo.setRenderer(new SListCellRenderer() {

			SLabel label = new SLabel();

			public SComponent getListCellRendererComponent(SComponent list,
					java.lang.Object value, boolean isSelected, int index) {

				if(value!=null && processForRenderer!=null){
					
					label.setText(Client.getInstance().getResourceProvider()
							.getMessage(
									"process."
											+ processForRenderer.getName()
											+ "."
											+ ((org.concern.model.Condition) value)
													.getName()));
				}else{
					label.setText("");
				}

				return label;
			}

		});		
		

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#canBeReplacedBy(agent.controller.OperationController)
	 */
	public boolean canBeReplacedBy(OperationController candidateController) {

		// Since there is only one SubjectConditionController it's enough if the
		// classes match to be replaced.
		return (candidateController instanceof SubjectConditionController);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see agent.controller.OperationController#getComponents()
	 */
	protected void populateComponentList(List<SComponent> componentList) {

		// The operator-combo (see Operators defined by this class).
		componentList.add(operatorCombo);
		// All conditions of the Process
		componentList.add(conditionsCombo);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.condition.ConditionController#setCondition(org.osbl.agent.model.condition.Condition)
	 */
	public void setCondition(Condition condition) {

		// Update our operatorCombo to reflect the operator set in the
		// SubjectCondition
		if (((SubjectCondition) condition).getOperator().equals(
				new BinaryOperator("=="))) {
			operatorCombo.setSelectedIndex(0);
		} else
			operatorCombo.setSelectedIndex(1);

		String processName = ((SubjectCondition) condition).getProcessName();

		Controller controller = ControllerLookup.getInstance().getController(
				processName);
		process = controller.getProcess();

		// Remove all elements in the conditionsCombo, since...
		conditionsCombo.removeAllItems();

		// ... we'll be populating the combo anyway.
		populateConditionsCombo(process, ((SubjectCondition) condition)
				.getConcernConditionName());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// For our string version return the name of the Process

		return Client.getInstance().getResourceProvider().getMessage(
				"process." + process.getName());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osbl.agent.gui.ConditionController#populateCondition(org.osbl.agent.model.Condition)
	 */
	protected void populateCondition(Condition condition) {

		// In order to evaluate correctly, the SubjectCondition needs some
		// information...

		// ... a Process (not really, see explanation below)
		((SubjectCondition) condition).setProcessName(process.getName());
		// ... a org.concern.model.Condition (which is not the same as a
		// org.osbl.agent.model.Condition)
		((SubjectCondition) condition)
				.setConcernConditionName(((org.concern.model.Condition) conditionsCombo
						.getSelectedItem()).getName());
		// ... and an Operator (which is one of those defined by this class, see
		// above)
		((SubjectCondition) condition).setOperator((Operator) operatorCombo
				.getSelectedItem());

		/*
		 * The Process is not really needed for the evaluation, but is needed
		 * for the inverse process: when a SubjectConditionController is
		 * instantiated from a SubjectCondition.
		 * 
		 * The information about the Process is needed in order to later
		 * retrieve the concernConditions and show them in the combo-box. This
		 * is necessary because a org.concern.model.Condition doesn't know about
		 * the Process it belongs to.
		 * 
		 */
	}

}
