package org.osbl.agent.logic;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.concern.Controller;
import org.concern.ControllerLookup;
import org.concern.model.Process;
import org.osbl.agent.model.Rule;


/**
 * The AgentSystem class serves as the central repository for Rules. There is only one AgentSystem
 * instance per OSBL installation, using the singleton pattern. It is responsible for storing the Rules and
 * offering basic operations for them.
 * 
 * The Rules in memory are kept synchronized with a persistent storage. Currently the persistence
 * mechanism is accomplished by serializing to an XML file and reading it back again.
 * The same XML framework is used for importing and exporting individual Rules.
 * 
 * The AgentSystem object provides basic operations for adding, retrieving and deleting Rules. Being a
 * singleton object potentially accessible by many processes at the same time, those operations are all
 * thread-safe (“synchronized”).
 * 
 * Rules are stored in an non-hierarchical indiscriminate manner. The AgentSystem class knows nothing
 * about Contexts. It is up to the callee to implement the corresponding recognition and filtering
 * algorithms.
 * 
 * @author Sebastian Nozzi.
 */
public class AgentSystem {

	/** The activity user rules. */
	private List<Rule> rules;

	/** The process list. */
	private ArrayList<Process> processList;
	
	/** The singleton instance. */
	private static AgentSystem singletonInstance = null;

	/**
	 * Instantiates a new agent system.
	 */
	private AgentSystem() {

		rules = new ArrayList<Rule>();
		
		// xstream = new XStream(new DomDriver());
		
		populateProcessList();
		
		loadRules();

	}

	/**
	 * Gets the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static AgentSystem getSingletonInstance() {

		// Create the singleton instance
		if(singletonInstance==null)
			singletonInstance = new AgentSystem();
		
		return singletonInstance;
	}
	
	
	/**
	 * Returns all Rules in the system.
	 * 
	 * @return all the Rules.
	 */
	public synchronized List<Rule> getRules(){
		return rules;
	}

	/**
	 * Gets the Rules that a given user has created.
	 * 
	 * @param user the user-id.
	 * 
	 * @return the Rules for that user-id.
	 */
	public synchronized List<Rule> getRulesForUser(String user) {

		return getRulesForUser(user,false);
	}

	/**
	 * Gets the all published Rules from other users but the one specified.
	 * 
	 * @param notThisUser the user that is to be avoided.
	 * 
	 * @return the public Rules for all other users but the one specified.
	 */
	public synchronized List<Rule> getPublicRulesOfOthers(String notThisUser) {

		List<Rule> allOthers = getRulesForUser(notThisUser,true);
		List<Rule> publicOthers = new ArrayList<Rule>();
		
		for(Rule aRule : allOthers){
			if(aRule.isPublic())
				publicOthers.add(aRule);
		}
		
		return publicOthers;
	}

	/**
	 * Either gets the Rules for one user, or all the Rules except for that user.
	 * 
	 * @param userId the user-id.
	 * @param avoidUser whether to avoid that user or not.
	 * 
	 * @return the Rules collected.
	 */
	protected synchronized List<Rule> getRulesForUser(String userId, boolean avoidUser) {

		List<Rule> result = new ArrayList<Rule>();
		
		for(Rule aRule : rules){
			if(aRule.getCreatorUser().equals(userId)==(!avoidUser))
				result.add(aRule);
		}
		
		return result;
	}

	/**
	 * Adds a Rule into the system.
	 * 
	 * @param newRule the new Rule.
	 */
	public synchronized void addRule(Rule newRule) {

		rules.add(newRule);
		
		saveRules();
	}
	
	/**
	 * In case Rules are indirectly modified (outside this class, which is very common)
	 * and the client wants to make sure that those changes are persistent and reflected
	 * across the system, it should call this method.
	 */
	public synchronized void updateRules(){
		saveRules();
	}

	/**
	 * Removes a Rule from the system.
	 * 
	 * @param ruleToDelete the rule to delete.
	 */
	public synchronized void removeRule(Rule ruleToDelete) {

		/*boolean result = */ 
		rules.remove(ruleToDelete);
		
		saveRules();
	}


	/**
	 * Save the Rules to a persistent store.
	 */
	protected void saveRules() {

		FileOutputStream fileOut;

		try {
			fileOut = new FileOutputStream("rules.xml");

			XMLEncoder xmlEncoder = new XMLEncoder(fileOut);
			
			xmlEncoder.writeObject(rules);
			
			xmlEncoder.close();

			fileOut.close();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	/**
	 * Provides a temporary file with an exported Rule.
	 * 
	 * @param rule the Rule for which the file is to be generated.
	 * 
	 * @return a temporary file with an exported Rule.
	 */
	public File tempRuleFile(Rule rule){
		
		File tempFile = null;
		
		try {
			tempFile = File.createTempFile("temporary-osbl-agent", ".xml");
			tempFile.deleteOnExit();
			
			FileOutputStream fileOutStream = new FileOutputStream(tempFile);
			
			XMLEncoder xmlEncoder = new XMLEncoder(fileOutStream);
			
			xmlEncoder.writeObject(rule);
			
			xmlEncoder.close();
			
			fileOutStream.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return tempFile;
	}
	
	/**
	 * Provides an XML-serialized version of a Rule.
	 * 
	 * @param rule the Rule to be serialized.
	 * 
	 * @return the resulting XML String.
	 */
	public String ruleAsXML(Rule rule){
		
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

		XMLEncoder xmlEncoder = new XMLEncoder(outputStream);
        
		xmlEncoder.writeObject(rule);
		
		xmlEncoder.close();		
		
		try {
			return new String(outputStream.toByteArray(),"UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			return "";
		}
	}
	
	/**
	 * From a given XML document it reconstructs a Rule object.
	 * 
	 * @param ruleXMLFile the XML file containing a Rule.
	 * 
	 * @return the generated Rule.
	 */
	public Rule XMLToRule(File ruleXMLFile){
		
		try{

			XMLDecoder xmlDecoder = new XMLDecoder(new BufferedInputStream(
				new FileInputStream(ruleXMLFile)));

			Rule newRule = (Rule) xmlDecoder.readObject();
			
			xmlDecoder.close();
	
			return newRule;
			
		}catch(ClassCastException e){
			return null;
		} catch (FileNotFoundException e) {
			return null;
		}
	}
	

	/**
	 * Loads all Rules from the persistent store.
	 */
	protected void loadRules() {

		FileInputStream fileIn;

		try {
			fileIn = new FileInputStream("rules.xml");
			
			XMLDecoder xmlDecoder = new XMLDecoder(fileIn);
			
			rules = (List<Rule>) xmlDecoder.readObject();

			xmlDecoder.close();
			
			fileIn.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Populates the (con:cern) Process list.
	 */
	protected void populateProcessList() {

		// This will store all available Processes
		processList = new ArrayList<Process>();

		// Get the singleton ControllerLookup singletonInstance.
		ControllerLookup controllerLookup = ControllerLookup.getInstance();

		// With all controller names...
		for (String controllerName : controllerLookup.getControllerNames()) {
			// ...get the real controller...
			Controller controller = controllerLookup
					.getController(controllerName);

			// ...and from it the Process, and add it to our list.
			processList.add(controller.getProcess());
		}
	}

	/**
	 * Convenience method to get the list of running (con:cern) Processes.
	 * Used by the DesignContexts that interact with con:cern.
	 * 
	 * @return the list of Processes.
	 */
	public ArrayList<Process> getProcessList() {
		return processList;
	}
	
	
}
