package org.osbl.agent.logic;

import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;


/**
 * RuntimeContext subclasses are used at Rule run-time to provide specific context-dependent
 * functionality in this area. An instance of this class is responsible for two things:
 * <ul>
 * <li>Populating a RuleContext with all the relevant information needed for Rule run-time.</li>
 * <li>Deciding if a given Rule is compatible with an target object candidate.</li>
 * </ul>
 * This latter aspect is useful in case a Rule wants to be applied to a list of objects whose type is not
 * known in advance. It would guarantee that the Rule is only run on objects for which it makes sense.
 * In some cases, the target object passed to populate the RuleContext might be a proxy object. The
 * context-specific implementation should then retrieve the real target object corresponding to the proxy,
 * and populate the RuleContext with this new object instead.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class RuntimeContext {

	/**
	 * Tests the applicability of a given Rule can be tested on a target (or proxy) object.
	 * 
	 * @param rule the Rule
	 * @param targetOrProxy the target (or proxy) object the Rule will potentially be run on.
	 * 
	 * @return true, if successful
	 */
	public abstract boolean ruleIsApplicable(Rule rule, Object targetOrProxy) ;

	/**
	 * Populates ruleContext with at least the target object.
	 * 
	 * In case the targetOrProxy object is not the final target, this method is expected
	 * to retrieve it.
	 * 
	 * Any other information that is populated into the ruleContext is context-dependent.
	 * 
	 * @param ruleContext the RuleContext to be populated.
	 * @param targetOrProxy the target (or proxy) object.
	 */
	public abstract void populateRuleContext(RuleContext ruleContext, Object targetOrProxy);

}
