package org.osbl.agent.model;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;


/**
 * A rule consists principally of two key components: Conditions and Actions.
 * 
 * Conditions define under which circumstances the Actions should be executed.
 * 
 * This class serves as an coherent unit of Conditions and Actions and allows
 * other operations such as giving the Rule a name and specify wether other
 * users should be able to view and use the Rule or not.
 * 
 * @see Condition, Action
 * @author Sebastian Nozzi.
 */
public class Rule {

	/** The name of the Rule. */
	private String name;

	/** The meta information. */
	private Hashtable<String, Object> metaInformation;

	/** Flag that indicates the public status of the Rule. */
	private boolean isPublic;

	/** The creator user-id. */
	private String creatorUser;

	/** The List of Conditions. */
	private List<Condition> conditions;

	/** The List of Actions. */
	private List<Action> actions;

	/**
	 * Instantiates a new Rule with the given name.
	 * 
	 * @param name Name of the new Rule.
	 */
	public Rule(String name) {
		this();
		this.name = name;
	}

	/**
	 * Instantiates a new Rule (with an empty name).
	 */
	public Rule() {
		name = "";
		conditions = new ArrayList<Condition>();
		actions = new ArrayList<Action>();
		metaInformation = new Hashtable<String, Object>();
	}

	/**
	 * Gets the meta-information repository.
	 * 
	 * @return the meta-data repository
	 */
	public Hashtable<String, Object> getMetaInformation() {
		return metaInformation;
	}

	/**
	 * Assigns a new meta-information repository.
	 * 
	 * @param metaInformation the meta-data repository
	 */
	public void setMetaInformation(Hashtable<String, Object> metaInformation) {
		this.metaInformation = metaInformation;
	}

	/**
	 * Gets the list of Actions.
	 * 
	 * To add/remove Actions to a Rule use this method to get the List and
	 * opperate directly on it.
	 * 
	 * @return the list of Actions.
	 */
	public List<Action> getActions() {
		return actions;
	}

	/**
	 * Sets a new Action list. The new list is used thus replacing all existing
	 * Actions.
	 * 
	 * @param actions the list of Actions.
	 */
	public void setActions(List<Action> actions) {
		this.actions = actions;
	}

	/**
	 * Gets the list of Conditions.
	 * 
	 * To add/remove Conditions to a rule use this method to get the List and
	 * opperate directly on it.
	 * 
	 * @return the list of Conditions.
	 */
	public List<Condition> getConditions() {
		return conditions;
	}

	/**
	 * Sets a new Condition list. The new list is used thus replacing all
	 * existing Conditions.
	 * 
	 * @param conditions the list of Conditions
	 */
	public void setConditions(List<Condition> conditions) {
		this.conditions = conditions;
	}

	/**
	 * Gets the name of the Rule.
	 * 
	 * @return the name of the Rule.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name of the Rule.
	 * 
	 * @param name the new name.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Checks if is the Rule is marked as public.
	 * 
	 * Public Rules are usable (but not editable) by other users in the
	 * AgentSystem.
	 * 
	 * @return true, if the Rule is public.
	 */
	public boolean isPublic() {
		return isPublic;
	}

	/**
	 * Marks the Rule as public.
	 * 
	 * Public Rules are usable (but not editable) by other users in the
	 * AgentSystem.
	 * 
	 * @param isPublic true if the Rule is to be published for all.
	 */
	public void setPublic(boolean isPublic) {
		this.isPublic = isPublic;
	}

	/**
	 * Gets the creator user-id.
	 * 
	 * The user-id is used for security purposes (for example an user can only
	 * edit/delete his/her own Rules in the system).
	 * 
	 * @return the creator user-id
	 */
	public String getCreatorUser() {
		return creatorUser;
	}

	/**
	 * Sets the creator user-id.
	 * 
	 * The user-id is used for security purposes (for example an user can only
	 * edit/delete his/her own Rules in the system).
	 * 
	 * @param user the new creator user-id.
	 */
	public void setCreatorUser(String user) {
		this.creatorUser = user;
	}

	/**
	 * Evaluate Conditions from the internal List of Conditions.
	 * 
	 * The evaluation stops as soon as one Condition evaluates to false.
	 * 
	 * @param context the RuleContext used to perform the evaluation. It should have
	 * been populated with at least the target instance.
	 * 
	 * @return true, if all Conditions evaluated to true.
	 */
	public boolean evaluateConditions(RuleContext context) {

		int i = 1;

		// Lazily evaluate conditions, this is, stop as soon as one
		// evaluates to "false"
		for (Condition condition : conditions) {

			// As soon as one evaluates to "false" the whole evaluation is
			// "false"
			if (condition.evaluate(context) == false) {
				System.out.println("Condition evaluated to FALSE, exiting.");
				return false;
			}
			System.out.println("Condition evaluated to TRUE.");
			i++;
		}

		// At this point all conditions evaluated to "true"
		return true;
	}

	/**
	 * Execute all Actions of the internal list of Actions.
	 * 
	 * @param context the RuleContext used to perform the evaluation. It should have
	 * been populated with at least the target instance.
	 */
	public void executeActions(RuleContext context) {

		// Execute all actions, one after the other
		for (Action action : actions) {
			System.out.println("Executing Action.");
			action.execute(context);
		}
	}

	/**
	 * If the Rule has a name, then it is used as String version, otherwise the
	 * default implementation is used.
	 * 
	 * @return an user readable version, using the Rule's name, if available.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// If the name of the condition was set, return it...
		if (name != null && name.equals("") == false)
			return getName();
		// ...otherwise use the standard implementation
		else
			return super.toString();
	}

	/**
	 * Provides a better implementation than the default, comparing the fields and
	 * then performing "equals" on the Conditions and Actions.
	 * 
	 * The included Conditions and Actions should also override "equals" and provide
	 * a better implementation.
	 * 
	 * @param obj the obj
	 * 
	 * @return true, if equals
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {

		if (obj instanceof Rule == false)
			return false;

		Rule other = (Rule) obj;

		if (this == other)
			return true;

		if (!name.equals(other.name))
			return false;

		if (isPublic != other.isPublic)
			return false;

		if (!creatorUser.equals(other.creatorUser))
			return false;

		// Note that (Array)List has an "equals" implementation according
		// to our needs, however, in order for it to work we need to provide
		// suitable implementations for the Actions and Conditions ourselves.

		if (!actions.equals(other.actions))
			return false;

		if (!conditions.equals(other.conditions))
			return false;

		// The implementation of "equals" of the Hashtable ought to be
		// good enough as we only store Strings in it.

		// By the way, this comparison will catch changes in fields that
		// are populated by the DesingContexts... for example an
		// Activity change, where applicable (for Tasks, for example).

		if (!metaInformation.equals(other.metaInformation))
			return false;

		// If everything else fails... test succeeded, assume they are equal.

		return true;
	}

	// TODO: Nobody seems to be using this method!
	/**
	 * Runs the Rule for the given RuleContext.
	 * 
	 * It does the same as calling "evaluateConditions" and if successfull calling
	 * "executeActions".
	 * 
	 * @param rtc the rtc
	 */
	public void run(RuleContext rtc) {

		boolean resultOfEvaluation = evaluateConditions(rtc);

		if (resultOfEvaluation)
			executeActions(rtc);
	}

	/**
	 * Convenience static method to properly compare two properties of an object,
	 * taking into account that any of them could be "null".
	 * 
	 * It avoids potential NullPointerExceptions that can be caused by comparing properties
	 * using "equals".
	 * 
	 * @param thisProperty one property
	 * @param otherProperty the other property
	 * 
	 * @return true, if successful
	 */
	public static boolean propertyEquals(Object thisProperty, Object otherProperty){
		
		return (thisProperty == null) 
        ? otherProperty == null 
        : thisProperty.equals(otherProperty);
	}
}
