package org.osbl.agent.model.action;

import org.conform.BeanMeta;
import org.conform.PropertyMeta;
import org.osbl.agent.model.RuleContext;


/**
 * This Action sets a Date (or supported class) field to the current time.
 * 
 * @author Sebastian Nozzi.
 */
public class SetCurrentTimeAction extends SetValueAction {

	/** The token that will point to the current time. Users can enter it in the UI. */
	static public final String predefinedNow = "now";
	
	/**
	 * Checks if a given class is a supported time class.
	 * Examples could be java.sql.Timestamp, java.util.Date, etc.
	 * 
	 * @param timeClass the class to analyse
	 * 
	 * @return true, if is a supported time class
	 */
	public static boolean isSupportedTimeClass(Class timeClass){
		// Returns true if the class is one of the time related classes
		// supported by this SetCurrentTimeAction.
		return (java.sql.Timestamp.class.isAssignableFrom(timeClass)
				|| java.sql.Date.class.isAssignableFrom(timeClass)
				|| java.sql.Time.class.isAssignableFrom(timeClass)
				|| java.util.Date.class.isAssignableFrom(timeClass) );
	}

	/* (non-Javadoc)
	 * @see org.osbl.agent.model.action.SetPropertyAction#execute(org.osbl.agent.model.RuleContext)
	 */
	public void execute(RuleContext context) {

		// Here we'll have an instance of whichever class applies, 
		// set to our current time.
		Object currentDateTime = null;

		// Using the propertyMeta's type, see what class we need to instantiate
		// and set this to the current time...

		Class type = Object.class;
		
		Object beanMeta = context.get(context.BEAN_META);
		if(beanMeta instanceof BeanMeta){
			PropertyMeta propertyMeta = ((BeanMeta)beanMeta).getProperty(getPropertyMetaName());
			type = propertyMeta.getType();
		}
		
		if (type.isAssignableFrom(java.util.Date.class))
			currentDateTime = new java.util.Date();
		else if (type.isAssignableFrom(java.sql.Time.class))
			currentDateTime = new java.sql.Time(new java.util.Date().getTime());
		else if (type.isAssignableFrom(java.sql.Timestamp.class))
			currentDateTime = new java.sql.Timestamp(new java.util.Date().getTime());
		else if (type.isAssignableFrom(java.sql.Date.class))
			currentDateTime = new java.sql.Date(new java.util.Date().getTime());

		context.put(predefinedNow, currentDateTime);
		
		// ...and execute the action using the evaluation context		
		super.execute(context);
	}
	
}
