package org.osbl.agent.model.condition;

import org.osbl.agent.model.Condition;
import org.osbl.agent.model.RuleContext;

import bsh.EvalError;
import bsh.Interpreter;


/**
 * This Condition class executes a beanshell script and evaluates to <code>true</code> if
 * the script's last expression is either <code>true</code> (primitive or wrapper class)
 * or, for any other type, not null.
 * 
 * At evaluation time, a variable <code>subject</code> is bound to the subjectInstance of
 * the runtime context, so that beanshell scripts can refer to this value.
 * 
 * @see http://www.beanshell.org/
 * @author Sebastian Nozzi.
 */
public class BeanShellCondition implements Condition {

	/** The beanshell script source-code. */
	private String script;

	/**
	 * Instantiates a new beanshell condition with an initial script source-code.
	 * 
	 * @param script the script
	 */
	public BeanShellCondition(String script) {
		this.script = script;
	}

	/**
	 * Instantiates a new beanshell condition.
	 */
	public BeanShellCondition() {
	}

	/**
	 * Gets the script's source-code.
	 * 
	 * @return the script's souce-code.
	 */
	public String getScript() {
		return script;
	}

	/**
	 * Sets the script source-code.
	 * 
	 * @param script the new script's souce-code.
	 */
	public void setScript(String script) {
		this.script = script;
	}

	/**
	 * *
	 * Evaluates the beanshell script and returns <code>true</code> if the last
	 * expression of the script evaluates to <code>true<code> or, in case its
	 * type is neither boolean or Boolean, the last expression is not null.
	 * 
	 * In case the script is empty, not set or has errors, it returns false.
	 * 
	 * @param context the context
	 * 
	 * @return true, if evaluate
	 * 
	 * @see org.osbl.agent.model.condition.Condition#evaluate(org.osbl.agent.model.RuleContext)
	 */
	public boolean evaluate(RuleContext context) {
		
		// Only evaluate if there is a script to execute
		if (script != null && script.equals("") == false) {
			
			// Instantiate a BeanShell interpreter.
			Interpreter bsh = new Interpreter();

			try {
				// Make "subject" a predefined variable in the interpreter's context.
				// "subject" will refer to our subjectInstance...
				bsh.set("subject", context.getTargetObject());
				
				// Finally, evaluate script.
				Object result = bsh.eval(script);

				if(result != null){
				
					// If the script evaluated to a primitive "boolean" (not Boolean), Beanshell
					// will return the result as a bsh.Primitive. By calling getValue() on it
					// we get the corresponging Java wrapper class (and maybe Boolean).
					// The test for "Boolean" is done below anyway...
					if(result instanceof bsh.Primitive)
						result = ((bsh.Primitive)result).getValue();
				
					// If the result is a valid Boolean...
					if (result instanceof Boolean)
						// ...return its booleanValue.
						return ((Boolean) result).booleanValue();
				}

				// If we got here the result was either "null" or not a boolean.
				// We'll evaluate to "false" only if the evaluation is explicitly "false" 
				// or "null"...
				return result != null;
				
			} catch (EvalError e) {
				e.printStackTrace();
			}
		}
		
		// If there was no script to evaluate, return our default behaviour.
		return false;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof BeanShellCondition == false)
			return false;
		
		BeanShellCondition other = (BeanShellCondition) obj;
		
		return (script.equals(other.script));
	}	
}
