package org.osbl.agent.model.condition;

import ognl.Ognl;
import ognl.OgnlException;

import org.osbl.agent.model.Condition;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;


/**
 * This class evaluates an OGNL expression, where the root object is the subjectInstance
 * of the runtime context.
 * 
 * @see http://www.ognl.org/
 * @author Sebastian Nozzi.
 */
public class OgnlCondition implements Condition {

	/** The OGNL expression. */
	private String expression;

	/**
	 * Instantiates a new OgnlCondition with the given expression.
	 * 
	 * @param expression the expression
	 */
	public OgnlCondition(String expression) {
		this.expression = expression;
	}

	/**
	 * Instantiates a new OgnlCondition.
	 */
	public OgnlCondition() {
	}

	/**
	 * Gets the OGNL expression.
	 * 
	 * @return the expression
	 */
	public String getExpression() {
		return expression;
	}

	/**
	 * Sets the OGNL expression.
	 * 
	 * @param expression the new expression
	 */
	public void setExpression(String expression) {
		this.expression = expression;
	}

	/**
	 * Evaluates the specified OGNL expression, using the subjectInstance of the
	 * runtime context as the root object.
	 * 
	 * @param context the context
	 * 
	 * @return true, if evaluate
	 * 
	 * @see org.osbl.agent.model.condition.Condition#evaluate(org.osbl.agent.model.RuleContext)
	 */
	public boolean evaluate(RuleContext context) {
		// If there is an expression to evaluate...
		if (expression != null && expression.equals("") == false) {
			try {
				// ...evaluate it via OGNL, using the subject instance of the
				// evaluation context as a root object (which is used when the 
				// expression references fields or methods) and return a boolean 
				// as a result
				return ((Boolean) Ognl.getValue(expression, context, context.getTargetObject())).booleanValue();
			} catch (OgnlException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return false;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof OgnlCondition == false)
			return false;
		
		OgnlCondition other = (OgnlCondition) obj;
		
		return Rule.propertyEquals(expression, other.expression);
	}	

}
