package org.osbl.agent.model.condition;

import org.osbl.agent.model.RuleContext;

import ognl.Ognl;
import ognl.OgnlException;


/**
 * This class represents an operator which can be used in Conditions to evaluate the
 * operands specified in them.
 * 
 * It is defined abstract to allow different kinds of operators.
 * 
 * At the time of evaluation the concrete class of the operator has to provide a valid
 * Java expression by overriding {@link #getExpression(String, String)}. For constructing
 * such expression, the implementation can (and generally will) use whatever is stored
 * in the operatorCode.
 * 
 * For presentation purposes (for example when adding it to a ComboBox) it is possible
 * to assign a descriptive user friendly label to the operator.
 * 
 * @author Sebastian Nozzi.
 */
public abstract class Operator {

	/** Some piece of code that aids the operator class to construct its evaluating expression. */
	private String operatorCode;

	/** A descriptive label for the operator. */
	private String label;

	/** Wether the result of the evaluation should be negated or not. */
	private boolean negated;

	/**
	 * Instantiates a new operator.
	 */
	public Operator() {
	}

	/**
	 * Instantiates a new operator with a given String, serving both for code and for label.
	 * 
	 * @param operatorString the operator string.
	 */
	public Operator(String operatorString) {
		this(operatorString, operatorString);
	}

	/**
	 * Instantiates a new operator, specifying label and code.
	 * 
	 * @param label the label.
	 * @param operatorCode the operator code.
	 */
	public Operator(String label, String operatorCode) {
		setOperatorCode(operatorCode);
		setLabel(label);
	}

	/**
	 * Gets the a descriptive label of the operator.
	 * 
	 * @return the label
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * Sets a descriptive label for the operator.
	 * 
	 * @param label the new label.
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/**
	 * Gets the operator code.
	 * 
	 * @return the operator code.
	 */
	public String getOperatorCode() {
		return operatorCode;
	}

	/**
	 * Sets the operator code, which aids the operator class to construct its evaluating expression.
	 * 
	 * @param operatorCode the new operator code.
	 */
	public void setOperatorCode(String operatorCode) {
		this.operatorCode = operatorCode;
	}

	/**
	 * Checks if the result of the evaluation will be negated.
	 * 
	 * @return true, if result will be negated.
	 */
	public boolean isNegated() {
		return negated;
	}

	/**
	 * Sets wether the result of the evaluation via this operator will be negated or not.
	 * 
	 * @param negated true, if it should be negated, false if not.
	 */
	public void setNegated(boolean negated) {
		this.negated = negated;
	}

	/**
	 * Two operators are equal if their classes match and their internal code matches (which
	 * is used in constructing the evaluating expression and therefor is a vital and
	 * differentiating part of an operator).
	 * 
	 * If the classes don't match, the default implementation is used.
	 * 
	 * @param obj the obj
	 * 
	 * @return true, if equals
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		// Two Operators are equal if...
		if (getClass().equals(obj.getClass())) {
			// ... the class is strictly the same and the operatorCode is the
			// same (which is used for constructing the evaluation expression.
			return ((Operator) obj).getOperatorCode() == this.operatorCode;
		} else
			return false;
	}

	/**
	 * Uses the specified label to return a friendly String version.
	 * 
	 * @return the string
	 * 
	 * @see #setLabel(String)
	 * @see #getLabel()
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getLabel();
	}

	/**
	 * Perform the defined evaluation in this operator. The passed parameters left and rightExpression
	 * are also implementation dependant. They would be typically the left and right operands.
	 * 
	 * A "root object" also has to be specified, for which all references to methods and/or
	 * fields are relative to.
	 * 
	 * @param leftExpression the left expression, or operand
	 * @param rightExpression the right expression, or operand
	 * @param context the context
	 * 
	 * @return the boolean result if the evaluation was successful, false in case of error.
	 */
	public boolean evaluate(String leftExpression, String rightExpression, RuleContext context ) {
		
		Object rootObject = context.getTargetObject();

		try {
			// Get the expression that will be evaluated (provided by
			// subclasses)
			String expression = getExpression(leftExpression, rightExpression);

			// If it should be negated, compare against "false"
			if (isNegated())
				expression = "(" + expression + ")==false";

			// Use OGNL to evaluate the expression.
			// Our rootObject is the object to which the expression makes
			// reference, for example, when refering to fields or methods.
			return ((Boolean) Ognl.getValue(expression, context, rootObject))
					.booleanValue();

		} catch (OgnlException e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Subclasses override and implement this method, in which they should return a valid
	 * Java (actually OGNL) expression that is then evaluated.
	 * 
	 * The implementation uses the given left and right parts and most certainly also
	 * the stored operatorCode (that defines more or less the nature of the operation).
	 * 
	 * @param left the left part of the expression.
	 * @param right the right part of the expression.
	 * 
	 * @return the expression as suitable for the concrete implemented operator.
	 */
	protected abstract String getExpression(String left, String right);

}
