package org.osbl.agent.model.condition;

import org.osbl.agent.model.Condition;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;


/**
 * This evaluates a condition related to one of the fields, or properties,
 * of the subject instance (passed in the {@link org.osbl.agent.model.RuleContext runtime-context}).
 * 
 * Thus, the specified propertyMeta MUST be a valid property of the subject class.
 * 
 * Users of this condition specify the property via a {@link org.conform.PropertyMeta propertyMeta},
 * specify the value to compare to, and provide an suitable {@link Operator operator} for the
 * evaluation of this two operands.
 * 
 * @author Sebastian Nozzi.
 */
public class PropertyCondition implements Condition {

	/** The operator. */
	private Operator operator;

	/** The value to compare to. */
	private Object valueToCompareTo;

	/** The bean meta class name. */
	private String beanMetaClassName;
	
	/** The property meta name. */
	private String propertyMetaName;

	/**
	 * Gets the operator.
	 * 
	 * @return the operator
	 */
	public Operator getOperator() {
		return operator;
	}

	/**
	 * Sets the operator.
	 * 
	 * @param operator the new operator
	 */
	public void setOperator(Operator operator) {
		this.operator = operator;
	}

	/**
	 * Gets the bean meta class name.
	 * 
	 * @return the bean meta class name
	 */
	public String getBeanMetaClassName() {
		return beanMetaClassName;
	}

	/**
	 * Sets the bean meta class name.
	 * 
	 * @param beanMetaClassName the new bean meta class name
	 */
	public void setBeanMetaClassName(String beanMetaClassName) {
		this.beanMetaClassName = beanMetaClassName;
	}

	/**
	 * Gets the property meta name.
	 * 
	 * @return the property meta name
	 */
	public String getPropertyMetaName() {
		return propertyMetaName;
	}

	/**
	 * Sets the property meta name.
	 * 
	 * @param propertyMetaName the new property meta name
	 */
	public void setPropertyMetaName(String propertyMetaName) {
		this.propertyMetaName = propertyMetaName;
	}

	/**
	 * Gets the value to compare to in the evaluation (the right operand).
	 * 
	 * @return the value to compare to
	 */
	public Object getValueToCompareTo() {
		return valueToCompareTo;
	}

	/**
	 * Sets the value to compare to in the evaluation (the right operand).
	 * 
	 * @param valueToCompareTo the new value to compare to
	 */
	public void setValueToCompareTo(Object valueToCompareTo) {
		this.valueToCompareTo = valueToCompareTo;
	}

	/**
	 * Uses the specified operator to perform an evaluation using the specified
	 * propertyMeta and the valueToCompareTo as operands, returning the result
	 * of such evaluation.
	 * 
	 * If the valueToCompareTo is not set or null, returns false without evaluation.
	 * 
	 * @param context the context
	 * 
	 * @return true, if evaluate
	 * 
	 * @see org.osbl.agent.model.condition.Condition#evaluate(org.osbl.agent.model.RuleContext)
	 */
	public boolean evaluate(RuleContext context) {

		Object value = getValueToCompareTo();

		// Only compare if there is something to compare against
		if (value != null) {

			// Let the operator do the evaluation...
			// The name of the propertyMeta is a property of our subjectInstance.
			return operator.evaluate(propertyMetaName, (String) value, context);

		}

		return false;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof PropertyCondition == false)
			return false;
		
		PropertyCondition other = (PropertyCondition) obj;
		
		if(! Rule.propertyEquals(operator, other.operator))
			return false;
		
		if(! Rule.propertyEquals(beanMetaClassName, other.beanMetaClassName))
			return false;

		if(! Rule.propertyEquals(propertyMetaName, other.propertyMetaName))
			return false;
		
		if(! Rule.propertyEquals(valueToCompareTo, other.valueToCompareTo))
			return false;
			
		return true;
	}		
	
}
