package org.osbl.agent.model.condition;

import org.concern.Controller;
import org.concern.ControllerException;
import org.concern.ControllerLookup;
import org.concern.UnknownSubjectException;
import org.osbl.agent.model.Condition;
import org.osbl.agent.model.Rule;
import org.osbl.agent.model.RuleContext;


/**
 * This condition class allows the evaulation of the
 * {@link org.concern.model.Condition Conditions} defined for the Process of the
 * subject instance.
 * 
 * @author Sebastian Nozzi.
 */
public class SubjectCondition implements Condition {

	/** The con:cern condition name. */
	private String concernConditionName;

	/** The process name. */
	private String processName;
	
	/** The operator, which should be one of those defined by this class. */
	private Operator operator;

	/**
	 * Instantiates a new subject condition.
	 */
	public SubjectCondition() {

	}

	/**
	 * Gets the con:cern condition name.
	 * 
	 * @return the concern condition name
	 */
	public String getConcernConditionName() {
		return concernConditionName;
	}

	/**
	 * Sets the con:cern condition name.
	 * 
	 * @param concernConditionName the new concern condition name
	 */
	public void setConcernConditionName(String concernConditionName) {
		this.concernConditionName = concernConditionName;
	}	

	/**
	 * Gets the con:cern process name.
	 * 
	 * @return the process name
	 */
	public String getProcessName() {
		return processName;
	}

	/**
	 * Sets the con:cern process name.
	 * 
	 * @param processName the new process name
	 */
	public void setProcessName(String processName) {
		this.processName = processName;
	}

	/**
	 * Gets the operator.
	 * 
	 * @return the operator
	 */
	public Operator getOperator() {
		return operator;
	}

	/**
	 * Sets the operator, which should be one of those defined by this class.
	 * 
	 * @param operator the new operator
	 */
	public void setOperator(Operator operator) {
		this.operator = operator;
	}

	/**
	 * Communicates with the corresponding concern controller and evaluates the
	 * specified concern condition, returning the result.
	 * 
	 * @param context the context
	 * 
	 * @return true, if evaluate
	 * 
	 * @see org.osbl.agent.model.condition.Condition#evaluate(org.osbl.agent.model.RuleContext)
	 */
	public boolean evaluate(RuleContext context) {

		boolean concernResult = false;

		String processName = (String) context.get(RuleContext.PROCESS_NAME_KEY);
		Controller controller = ControllerLookup.getInstance().getController(processName);

		try {
			Boolean result = controller.matchCondition((String) context
					.get(context.SUBJECT_ID), concernConditionName);
			concernResult = result.booleanValue();
		} catch (UnknownSubjectException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ControllerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// Dummy context in which the subjectInstance is a boolean, used
		// for the evaluation of the operator, which will use the subjectInstance
		// as its root-object (for OGNL evaluation).		
		RuleContext dummyContext = new RuleContext();
		dummyContext.setTargetObject(new Boolean(concernResult));
		
		return operator.evaluate("booleanValue()", "true", dummyContext);

	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj){
		
		if(this==obj)
			return true;
		
		if(obj instanceof SubjectCondition == false)
			return false;
		
		SubjectCondition other = (SubjectCondition) obj;
		
		if(! Rule.propertyEquals(operator, other.operator))
			return false;
		
		if( ! Rule.propertyEquals(concernConditionName, other.concernConditionName))
			return false;
		
		return true;
	}


}
