package org.osbl.client.action;

import java.util.EventListener;

/**
 * An ObjectAction is an Action, that can be performed on certain Objects.
 * It is quite similar to Swing Actions, except that the actionPerformed method provides the objects, for which the
 * action was caused.
 * Typical uses are the ObjectDetail, that will call the action for the currently shown object and the ObjectList, that
 * will call the action for selected objects.
 */
public interface ObjectAction
    extends EventListener
{
    /**
     *
     * @param event
     */
    void actionPerformed(ObjectActionEvent event);

    public static final String DEFAULT = "Default";
    /**
     * The key used for storing the <code>String</code> name
     * for the action, used for a menu or button.
     */
    public static final String NAME = "Name";
    /**
     * The key used for storing a short <code>String</code>
     * description for the action, used for tooltip text.
     */
    public static final String SHORT_DESCRIPTION = "ShortDescription";
    /**
     * The key used for storing a longer <code>String</code>
     * description for the action, could be used for context-sensitive help.
     */
    public static final String LONG_DESCRIPTION = "LongDescription";
    /**
     * The key used for storing a small <code>Icon</code>, such
     * as <code>ImageIcon</code>, for the action, used for toolbar buttons.
     */
    public static final String SMALL_ICON = "SmallIcon";
    /**
     * The key used to determine the command <code>String</code> for the
     * <code>ActionEvent</code> that will be created when an
     * <code>Action</code> is going to be notified as the result of
     * residing in a <code>Keymap</code> associated with a
     * <code>JComponent</code>.
     */
    public static final String ACTION_COMMAND_KEY = "ActionCommandKey";
    /**
     * The key used for storing a <code>KeyStroke</code> to be used as the
     * accelerator for the action.
     *
     * @since 1.3
     */
    public static final String ACCELERATOR_KEY="AcceleratorKey";
    /**
     * The key used for storing a <code>KeyEvent</code> to be used as
     * the mnemonic for the action.
     *
     * @since 1.3
     */
    public static final String MNEMONIC_KEY="MnemonicKey";

    /**
     * Gets one of this object's properties using the associated key.
     * @see #putValue
     */
    public Object getValue(String key);

    /**
     * Sets one of this object's properties using the associated key.
     *
     * @param key    a <code>String</code> containing the key
     * @param value  an <code>Object</code> value
     */
    public void putValue(String key, Object value);

    /**
     * Sets the enabled state of the <code>Action</code>.  When enabled,
     * any component associated with this object is active and
     * able to fire this object's <code>actionPerformed</code> method.
     * If the value has changed, a <code>PropertyChangeEvent</code> is sent
     * to listeners.
     *
     * @param  b true to enable this <code>Action</code>, false to disable it
     */
    public void setEnabled(boolean b);
    /**
     * Returns the enabled state of the <code>Action</code>. When enabled,
     * any component associated with this object is active and
     * able to fire this object's <code>actionPerformed</code> method.
     *
     * @return true if this <code>Action</code> is enabled
     */
    public boolean isEnabled();
}
