package org.osbl.client.feeder;

import java.io.Serializable;
import java.util.*;

/**
 * Allows for substitution of variables in a String.
 * In addition, it's possible, to use a ResourceBundle for translation of
 * the substituted values. This addresses the special case, when
 * substitutions are arbitrary Objects, aren't human readable or have to
 * be localized. For example Locales, MessageIDs, ErrorIds, etc.
 */
class StringTemplate
    implements Serializable
{
    private String template = null;
    private Map map = null;
    private String defaultValue = "";
    private boolean upper = false;
    private ResourceBundle bundle = null;

    public StringTemplate() {
    }

    public StringTemplate(String template) {
        this.template = template;
    }

    public StringTemplate(String template, Map map) {
        this.template = template;
        this.map = map;
    }

    public void setTemplate(String template) {
        this.template = template;
    }

    public String getTemplate() {
        return template;
    }

    public void setMap(Map map) {
        this.map = map;
    }

    public Map getMap() {
        return map;
    }

    public void setResourceBundle(ResourceBundle bundle) {
        this.bundle = bundle;
    }

    public ResourceBundle getResourceBundle() {
        return bundle;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setUpperCase(boolean upper) {
        this.upper = upper;
    }

    public boolean isUpperCase() {
        return upper;
    }

    public String toString() {
        int pos = 0;
        int end = 0;
        StringBuffer result = new StringBuffer();
        boolean has_brace;

        if (template == null)
            return "";

        if (map == null)
            return template;

        while ((pos = template.indexOf('$', pos)) >= 0) {
            if (template.charAt(pos + 1) == '$') {
                pos++;
                continue;
            }

            // text between last variable and here
            result.append(template.substring(end, pos));

            has_brace = (template.charAt(pos + 1) == '{');
            if (has_brace) {
                end = template.indexOf('}', pos + 1);
                handleBrace(result, template.substring(pos + 2, end));
                pos = ++end;
            }
            else {
                end = pos + 1;
                if (template.charAt(end) == '#')
                    end++;
                while (end < template.length() && isVariableChar(template.charAt(end)))
                    end++;
                handleVariable(result, template.substring(pos + 1, end));
                pos = end;
            }
        }

        result.append(template.substring(end, template.length()));
        return result.toString();
    }

    private void handleVariable(StringBuffer buffer, String variable) {
        boolean translate = false;
        if (variable.charAt(0) == '#') {
            translate = true;
            variable = variable.substring(1);
        }

        String value = getValue(variable);
        if (value != null)
            buffer.append((translate) ? translate(variable, value) : value);
        else if (defaultValue != null)
            buffer.append(defaultValue);
        else
            buffer.append(variable);
    }

    private void handleBrace(StringBuffer buffer, String contents) {
        List vars = new LinkedList();
        String delimiter = "";

        boolean translate = false;
        if (contents.charAt(0) == '#') {
            translate = true;
            contents = contents.substring(1);
        }

        StringTokenizer tokenizer = new StringTokenizer(contents, ",");
        while (tokenizer.hasMoreTokens()) {
            String token = tokenizer.nextToken();
            if (token.charAt(0) == '"') {
                while (!(token.charAt(token.length() - 1) == '"') || token.length() == 1)
                    token = token + "," + tokenizer.nextToken();

                delimiter = token.substring(1, token.length() - 1);
            }
            else {
                vars.add(token);
            }
        }

        if (vars.size() == 1) {
            String variable = (String)vars.get(0);
            String value = getValue(variable);
            if (value != null) {
                buffer.append(delimiter)
                    .append((translate) ? translate(variable, value) : value);
            }
        }
        else {
            boolean first = true;
            Iterator it = vars.iterator();
            while (it.hasNext()) {
                String variable;
                String value = getValue(variable = (String)it.next());
                if (value != null) {
                    if (!first)
                        buffer.append(delimiter);
                    buffer.append((translate) ? translate(variable, value) : value);
                    first = false;
                }
            }
        }
    }

    private String getValue(String variable) {
        if (upper)
            variable = variable.toUpperCase();
        Object value = map.get(variable);
        if (value != null)
            return value.toString();
        else
            return null;
    }

    public String translate(String name, String value) {
        String text;
        try {
            //logger.finest("translating: " + name + "." + value);
            text = bundle.getString(name + "." + value);
        }
        catch (MissingResourceException e) {
            text = value;
        }
        return text;
    }

    /**
     * gibt zur�ck, ob es sich bei dem Zeichen um ein Zeichen, wie es in
     * einer Variable vorkommen kann, handelt.
     *
     * @param  x ein Zeichen, ein Zeichen ..
     * @result <code>true</code>, wenn Zeichen in Variable vorkommen kann, sonst <code>false</code>
     */
    private boolean isVariableChar(char x) {
        return ((x >= 'A' && x <= 'Z') ||
            (x >= 'a' && x <= 'z') ||
            (x >= '0' && x <= '9') ||
            (x == '.' || x == '_'));
    }
}
