package org.osbl.client.wings.form;

import org.osbl.client.action.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.*;
import org.osbl.client.ClientServiceProvider;
import org.osbl.*;
import org.osbl.plugin.*;
import org.hibernate.ObjectNotFoundException;
import org.apache.commons.logging.LogFactory;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;


/**
 * A generic base class, that implements basic ObjectEditor functionality.
 * It coordinates ObjectList and ObjectForm.
 */
public abstract class GenericObjectEditor
    implements ObjectEditor, ObjectContextAware
{
    private static org.apache.commons.logging.Log LOG = LogFactory.getLog(GenericObjectEditor.class);

    protected Environment environment = new DelegateEnvironment(true);
    protected List<Tool> tools = new ArrayList<Tool>();

    protected ObjectList objectList;
    protected ObjectForm objectForm;
    private boolean editing;
    private Object object;

    protected AddAction addAction = new AddAction(base());
    protected EditAction editAction = new EditAction(base());
    protected PreviousAction previousAction = new PreviousAction(base());
    protected NextAction nextAction = new NextAction(base());
    protected ListAction listAction = new ListAction(base());
    protected SaveAction saveAction = new SaveAction(base());
    protected DeleteObjectAction deleteObjectAction = new DeleteObjectAction(base());
    protected GenericObjectForm.DelegateObjectAction formDeleteAction = new GenericObjectForm.DelegateObjectAction(deleteObjectAction);
    protected GenericObjectList.DelegateObjectAction listDeleteAction = new GenericObjectList.DelegateObjectAction(deleteObjectAction);
    protected ObjectContext context;

    public GenericObjectEditor() {
        initializeTools();
        initializeEditor();
        environment.setActive(true);
    }

    protected void initializeTools() {
        PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("ClientPluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint("org.osbl.client.wings.form.editorTools", EditorTool.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);

        for (Extension extension : list) {
            final Class<EditorTool> toolClass = (Class<EditorTool>)extension.getImplementation();
            try {
                EditorTool tool = toolClass.newInstance();
                tool.setEditor(this);
                tools.add(tool);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    public ObjectContext getContext() {
        if (context == null) {
            context = new ObjectContext();
        }
        return context;
    }

    public void setContext(ObjectContext context) {
        this.context = context;
    }

    public Environment getEnvironment() {
        return environment;
    }

    public void showList() {
        editing = false;

        getList();

        if (objectForm != null) {
            objectForm.getEnvironment().setActive(false);
            objectForm.setObject(null);
        }
        objectList.getEnvironment().setActive(true);

        objectList.refresh();
    }

    public void showForm() {
        editing = true;

        getForm();

        if (objectList != null)
            objectList.getEnvironment().setActive(false);
        objectForm.getEnvironment().setActive(true);

        objectForm.setObject(object);
    }

    /**
     * Initializations like installing global object actions go here.
     * Override this method for your own initializations.
     */
    protected void initializeEditor() {
    }

    public ObjectForm getForm() {
        if (objectForm == null) {
            objectForm = initializeForm();
            if (objectForm instanceof ObjectContextAware)
                ((ObjectContextAware)objectForm).setContext(getContext());
            formControls(objectForm.getEnvironment());
            formTools(objectForm.getEnvironment());
            objectForm.getEnvironment().setDelegate(environment);
            formDeleteAction.setForm(objectForm);
        }
        return objectForm;
    }

    protected void formControls(Environment environment) {
        environment.addControl(new XButton(listAction));
        environment.addControl(new XButton(previousAction));
        environment.addControl(new XButton(nextAction));
        environment.addControl(new XButton(saveAction));
        environment.addControl(new XButton(formDeleteAction));
    }

    protected void formTools(Environment environment) {
        for (Tool tool : tools)
            environment.addTool(tool);
    }

    /**
     * Create and initialize the conform component.
     * @return the conform component
     */
    protected abstract ObjectForm initializeForm();

    public ObjectList getList() {
        if (objectList == null) {
            objectList = initializeList();
            if (objectList instanceof ObjectContextAware)
                ((ObjectContextAware)objectList).setContext(getContext());
            listControls(objectList.getEnvironment());
            listTools(objectList.getEnvironment());
            objectList.getEnvironment().setDelegate(environment);
            listDeleteAction.setList(objectList);

            objectList.setLinkAction(linkPropertyName(), new AbstractObjectAction() {
                public void actionPerformed(ObjectActionEvent event) {
                    Object object = event.getTargets()[0];
                    try {
                        revertObject(object);
                        objectList.setCurrent(object);
                        setObject(object);

                        if (editAction.isEnabled())
                            editAction.actionPerformed(new ActionEvent(GenericObjectEditor.this, 0, "edit"));
                    }
                    catch (Exception e) {
                        LOG.warn(e);
                        String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                        String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.doesNotExist", name);
                        XOptionPane.showMessageDialog(null, message, name, XOptionPane.ERROR_MESSAGE, new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                objectList.refresh();
                            }
                        });
                    }
                }
            });
        }
        return objectList;
    }

    protected String linkPropertyName() {
        return "key";
    }

    protected void listControls(Environment environment) {
        environment.addControl(new XButton(addAction));
        environment.addControl(new XButton(listDeleteAction));
    }

    protected void listTools(Environment environment) {
        for (Tool tool : tools)
            environment.addTool(tool);
    }

    /**
     * Create and initialize the list component.
     * @return the list component
     */
    protected abstract ObjectList initializeList();

    public Object getObject() {
        return object;
    }

    public void setObject(Object object) {
        this.object = object;
    }

    public void setSelectionMode(int selectionMode) {
        getList().setSelectionMode(selectionMode);
    }

    public void setSelectedObject(Object object) {
        getList().setSelectedObject(object);
    }

    public Object getSelectedObject() {
        return getList().getSelectedObject();
    }

    public List getSelectedObjects() {
        return getList().getSelectedObjects();
    }

    public void setSelectedObjects(List objects) {
        getList().setSelectedObjects(objects);
    }

    public abstract Class getType();

    public abstract void saveObject(Object object);
    public abstract void deleteObject(Object object);
    public abstract void revertObject(Object object);

    /**
     * Provide the ChooseAction and append the object list actions if any.
     * @return a list of actions, displayed along with the list.
     */
    protected List<Action> getListActions() {
        return null;
    }

    /**
     * Provide the ListAction, PreviousAction, NextAction and SaveAction and append the object conform actions if any.
     * @return a list of actions, displayed along with the conform.
     */
    protected List<Action> getFormActions() {
        return null;
    }

    protected String base() {
        return getType().getName();
    }

    protected String unsavedChangesMessage(String name) {
        String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.unsavedChanges", name);
        if (objectForm instanceof GenericObjectForm) {
            GenericObjectForm genericObjectForm = (GenericObjectForm)objectForm;
            String changes = genericObjectForm.getChangeStatus().composeMessageDetail();
            message = message + "\n\n" + changes;
        }
        return message;
    }

    protected class ListAction extends AbstractAction {
        public ListAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.list");
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = unsavedChangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            showList();
                    }
                });
            }
            else
                showList();
        }

        public void setEnabled(boolean newValue) {
        }
    }

    protected class NextAction extends AbstractAction {
        public NextAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.next");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(objectForm.getType().getName());
                String message = unsavedChangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            next();
                    }
                });
            }
            else
                next();
        }

        private void next() {
            objectList.next();
            setObject(objectList.getCurrent());
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
            if (formDeleteAction != null)
                formDeleteAction.setEnabled(true);
        }
    }

    protected class PreviousAction extends AbstractAction {
        public PreviousAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.previous");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = unsavedChangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            previous();
                    }
                });
            }
            else
                previous();
        }

        private void previous() {
            objectList.previous();
            setObject(objectList.getCurrent());
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
            if (formDeleteAction != null)
                formDeleteAction.setEnabled(true);
        }
    }

    protected class AddAction extends AbstractAction {
        public AddAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.add");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent e) {
            setObject(null);
            showForm();
            objectList.setSelectedObject(null);
            previousAction.setEnabled(false);
            nextAction.setEnabled(false);
            if (formDeleteAction != null)
                formDeleteAction.setEnabled(false);
        }
    }

    protected class EditAction extends AbstractAction {
        public EditAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.edit");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent e) {
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
            if (formDeleteAction != null)
                formDeleteAction.setEnabled(true);
        }
    }

    protected class SaveAction extends AbstractAction {
        public SaveAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.save");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent event) {
            if (objectForm.hasValidationIssues())
                return;

            try {
                saveObject(objectForm.getObject());
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.saved", name);
                objectForm.setObject(objectForm.getObject());
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));
            }
            catch (Exception e) {
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_ERROR, e.getMessage()));
                e.printStackTrace();
            }

            getList().refresh();
            objectList.setSelectedObject(objectForm.getObject());
        }
    }

    protected class DeleteObjectAction extends AbstractObjectAction {
        public DeleteObjectAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.delete");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ObjectActionEvent event) {
            try {
                final Object[] objects = event.getTargets();
                final ReferenceCheckers referenceCheckers = (ReferenceCheckers)ServiceProvider.getInstance().getService("ReferenceCheckers");
                List references = referenceCheckers.referenceChecks(getType(), Arrays.asList(objects));

                if (references == null || references.size() == 0) {
                    String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                    String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.reallyDelete", name, objects.length);

                    XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            if (XOptionPane.YES_ACTION == e.getActionCommand()) {
                                for (Object object : objects)
                                    deleteObject(object);

                                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.deleted", name, objects.length);
                                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));

                                if (editing)
                                    showList();
                                else
                                    getList().refresh();
                            }
                        }
                    });
                }
                else {
                    String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                    String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.reallyDeletePartially", name, objects.length);

                    XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            if (XOptionPane.YES_ACTION == e.getActionCommand()) {
                                int i = 0;
                                for (Object object : objects) {
                                    List references = referenceCheckers.referenceChecks(getType(), Collections.singleton(object));
                                    if (references == null || references.size() == 0) {
                                        deleteObject(object);
                                        i++;
                                    }
                                }

                                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.partiallyDeleted", name, i, objects.length);
                                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));

                                if (editing)
                                    showList();
                                else
                                    getList().refresh();
                            }
                        }
                    });
                }
            }
            catch (Exception e) {
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_ERROR, e.getMessage()));
                e.printStackTrace();
            }
        }
    }
}
