package org.osbl.client.wings.form;

import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import org.conform.*;
import org.conform.format.Format;
import org.conform.format.NoFormat;
import org.conform.wings.devel.MetaViewer;
import org.conform.wings.*;
import org.osbl.client.action.ObjectActionEvent;
import org.wings.*;

import org.osbl.client.action.ObjectAction;
import org.osbl.client.wings.form.layouter.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.*;
import org.osbl.client.ClientServiceProvider;
import org.osbl.plugin.*;

/**
 * An abstract implementation of generic ObjectForm functionality.
 * It assumes, that the UI is built from one or more BeanMetas following a descriptive layout specification.
 * Databinding is accomplished using BeanDatas.
 */
public abstract class GenericObjectForm
    implements ObjectForm, ObjectContextAware, VisibilityListener
{
    private static final NoFormat NO_FORMAT = new NoFormat();

    protected DelegateEnvironment environment = new DelegateEnvironment();

    protected int viewMode = EDIT;
    protected boolean documentLook;
    protected Object object;
    protected SortedMap<String, BeanMeta> metas = new TreeMap<String, BeanMeta>(PATH_COMPARATOR);
    protected SortedMap<String, BeanData> datas = new TreeMap<String, BeanData>(PATH_COMPARATOR);
    protected SComponent component;

    private Map<ObjectAction, DelegateObjectAction> actionDelegates = new HashMap<ObjectAction, DelegateObjectAction>();
    private ValidationStatus validationStatus = new ValidationStatus(environment);
    protected ChangeStatus changeStatus = new ChangeStatus(environment);
    private Map<Integer, Modifier> variations = new HashMap<Integer, Modifier>();
    private ComponentProvider componentProvider;

    private Map<SComponent,BeanData> dataByComponent = new HashMap<SComponent,BeanData>();
    private Set<BeanData> initializedDatas = new HashSet<BeanData>();
    protected Layouter layouter;
    protected ObjectContext context;

    protected GenericObjectForm() {
        addViewModeVariation(VIEW, READONLY_VARIATION);
    }

    public Environment getEnvironment() {
        getComponent();
        return environment;
    }

    public void setViewMode(int viewMode) {
        if (component != null)
            throw new IllegalStateException("Must be called before the conform's component has been built!");

        this.viewMode = viewMode;
    }

    public void addViewModeVariation(int viewMode, Modifier modifier) {
        variations.put(viewMode, modifier);
    }

    public void setDocumentLook(boolean documentLook) {
        this.documentLook = documentLook;
    }

    /**
     * Lazily construct the details component according to the provided Layouter and BeanMeta.
     * @return the component showing the object
     */
    public SComponent getComponent() {
        if (component == null) {
            layouter = initLayout(createLayoutInstruction());
            layouter.addVisibilityListener(this);
            component = layouter.build(getComponentProvider());
            environment.setContentPane(component);
            installTools();
            initializeForm();

            if (object != null)
                visibilityChanged(layouter.getVisibles());
        }
        return component;
    }

    protected void installTools() {
        PluginManager pluginManager = (PluginManager)ClientServiceProvider.getInstance().getService("ClientPluginManager");
        final ExtensionPoint extensionPoint = new ExtensionPoint("org.osbl.client.wings.form.formTools", FormTool.class);
        final List<Extension> list = pluginManager.getExtensions(extensionPoint);

        for (Extension extension : list) {
            final Class<FormTool> toolClass = (Class<FormTool>)extension.getImplementation();
            try {
                FormTool tool = toolClass.newInstance();
                tool.setForm(this);
                environment.addTool(tool);
            }
            catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * Perform initialization tasks.
     * Override this method, if you want to perform initialization tasks on some UI elements after construction. This
     * is the place, where PropertyChangeListeners can be registered on BeanDatas.
     */
    protected void initializeForm() {
    }

    protected ComponentProvider getComponentProvider() {
        if (componentProvider == null)
            componentProvider = new AbstractComponentProvider() {
                protected BeanMeta getBeanMeta(String path) {
                    return GenericObjectForm.this.getBeanMeta(path);
                }

                protected BeanData getBeanData(String path) {
                    return GenericObjectForm.this.getBeanData(path);
                }

                protected SComponent createCustomComponent(String key) {
                    return GenericObjectForm.this.createCustomComponent(key);
                }

                protected SLabel createCustomLabel(String key) {
                    return GenericObjectForm.this.createCustomLabel(key);
                }

                protected SComponent createComponent(String bean, String property) {
                    SComponent component = super.createComponent(bean, property);
                    dataByComponent.put(component, getBeanData(bean));
                    return component;
                }

                protected SComponent createReadOnlyComponent(String bean, String property) {
                    SComponent component = super.createReadOnlyComponent(bean, property);
                    dataByComponent.put(component, getBeanData(bean));
                    return component;
                }
            };
        return componentProvider;
    }

    public void visibilityChanged(VisibilityEvent event) {
        List<SComponent> visibleComponents = event.getVisibles();
        visibilityChanged(visibleComponents);
        focus();
    }

    protected void visibilityChanged(List<SComponent> visibleComponents) {
        Map<SComponent,BeanData> visibleDataByComponent = new HashMap<SComponent,BeanData>(dataByComponent);
        visibleDataByComponent.keySet().retainAll(visibleComponents);
        Collection<BeanData> visibleDatas = new HashSet<BeanData>(visibleDataByComponent.values());

        for (BeanData data : visibleDatas)
            initData(data);
    }

    public void setApplicability(Instruction instruction, boolean applicable) {
        layouter.setApplicability(instruction, applicable);
    }

    /**
     * Retrieve or lazily create a BeanMeta for the respective object.
     * A details implementation might display complex object graphs. Then the path for the root of the object graph is
     * named ROOT and child objects are addressed using dot notation.
     * @param path addresses an object in the object graph.
     * @return a BeanMeta for the respective object / subobject
     */
    public final BeanMeta getBeanMeta(String path) {
        if (path == null)
            throw new IllegalArgumentException("Use ComponentProvider.ROOT instead of null!");

        BeanMeta meta = metas.get(path);
        if (meta == null) {
            meta = createBeanMeta(path);
            Modifier modifier = variations.get(viewMode);
            if (modifier != null)
                modifier.modify(meta);
            metas.put(path, meta);
            environment.addDevelopmentAction(new ViewerAction(meta));
        }
        return meta;
    }

    /**
     * Retrieve or lazily create a BeanData for the respective object.
     * A details implementation might display complex object graphs. Then the path for the root of the object graph is
     * named ROOT and child objects are addressed using dot notation.
     * @param path addresses an object in the object graph.
     * @return a BeanMeta for the respective object / subobject
     */
    public final BeanData getBeanData(String path) {
        if (path == null)
            throw new IllegalArgumentException("Use ROOT instead of null!");

        if (!ComponentProvider.ROOT.equals(path)) {
            int pos = path.lastIndexOf('.');
            getBeanData(pos > -1 ? path.substring(0, pos) : ComponentProvider.ROOT);
        }

        BeanData data = datas.get(path);
        if (data == null) {
            data = createBeanData(path);
            datas.put(path, data);
            addForValidation(data);
            addForChanges(data);
        }

        return data;
    }

    private void addForChanges(BeanData data) {
        data.addPropertyChangeListener(changeStatus);
    }

    private void addForValidation(Data data) {
        data.addValidationListener(validationStatus);

        if (data instanceof BeanData) {
            BeanData beanData = (BeanData)data;
            DeferredValidationListener deferredValidationListener = new DeferredValidationListener(beanData);
            beanData.addPropertyChangeListener(deferredValidationListener);
        }
    }

    /**
     * Are there any unresolved validations issues pending?
     * @return true, if there are pending issues, false otherwise
     */
    public boolean hasValidationIssues() {
        activateValidation();
        return validationStatus.getIssues().size() != 0;
    }

    /**
     * Activate validation.
     * Validation issues will be published.
     */
    protected void activateValidation() {
        for (Data data: datas.values()) {
            if (data instanceof BeanData && ((BeanData)data).getBeanMeta().isApplicable())
                data.setIssuePublishingActive(true);
        }
    }

    /**
     * Activate validation.
     * Validation issues will not be published.
     */
    protected void deactivateValidation() {
        for (Data data: datas.values()) {
            data.setIssuePublishingActive(false);
        }
    }

    public ValidationStatus getValidationStatus() {
        return validationStatus;
    }

    public ChangeStatus getChangeStatus() {
        return changeStatus;
    }

    public boolean hasChanges() {
        return changeStatus.hasChanges();
    }

    public void clearChanges() {
        changeStatus.clearChanges();
    }

    /**
     * Retrieve the subobject, addressed by the specified path.
     * A details implementation might display complex object graphs. Then the path for the root of the object graph is
     * named ROOT and child objects are addressed using dot notation.
     * @param path addresses an object in the object graph.
     * @return the object / subobject of the object graph
     */
    protected abstract Object getObject(String path);

    /**
     * Set the subobject, addressed by the specified path.
     * A details implementation might display complex object graphs. Then the path for the root of the object graph is
     * named ROOT and child objects are addressed using dot notation.
     * @param path addresses an object in the object graph.
     * @param object the object / subobject of the object graph
     */
    protected abstract void setObject(String path, Object object);

    public Object getObject() {
        return object;
    }

    public void setObject(Object object) {
        deactivateValidation();
        clearDatas();

        this.object = object;

        BeanData data = datas.get(ComponentProvider.ROOT);
        if (object == null) {
            this.object = data.createInstance();
            data.setValue(this.object);
            data.initialize();
        }
        else {
            data.setValue(this.object);
        }

        if (layouter != null)
            visibilityChanged(layouter.getVisibles());

        focus();

        environment.setTitle(getBeanMeta(ComponentProvider.ROOT).getType().getName() + ".editor.title", titleArguments());

        clearChanges();
    }

    protected void focus() {
        for (SComponent component : layouter.getFocusableComponents()) {
            if (isWritable(component) && component.isRecursivelyVisible()) {
                component.requestFocus();
                break;
            }
        }
    }

    private boolean isWritable(SComponent component) {
        if (component instanceof STextComponent) {
            STextComponent textComponent = (STextComponent)component;
            return textComponent.isEditable();
        }
        return component.isEnabled();
    }

    protected String titleArguments() {
        Format format = getBeanMeta(ComponentProvider.ROOT).getFormat();
        if (format == null)
            format = NO_FORMAT;
        return format.format(getObject());
    }

    public Class getType() {
        return getBeanMeta(ComponentProvider.ROOT).getType();
    }

    protected void clearDatas() {
        for (Map.Entry<String, BeanData> entry : datas.entrySet()) {
            BeanData data = entry.getValue();
            data.setValue(null);
        }
        initializedDatas.clear();
    }

    public void initData(BeanData data) {
        if (initializedDatas.contains(data))
            return;

        initializedDatas.add(data);

        changeStatus.setActive(false);

        String path = null;
        for (Map.Entry<String, BeanData> entry : datas.entrySet()) {
            if (entry.getValue() == data) {
                path = entry.getKey();
                break;
            }
        }
        assert path != null;

        Object object = getObject(path);
        if (object == null) {
            object = data.createInstance();
            data.setValue(object);
            data.initialize();
            setObject(path, object);
        }
        else {
            try {
                data.setValue(object);
            }
            catch (ValidationException e) {
                System.err.println("object " + object + " has validation issues on initialization (" + data.getMeta().getType() + ")");
                e.printStackTrace(System.err);
            }
        }

        changeStatus.setActive(true);
    }

    /**
     * Construct a descriptive layout specification.
     * @return the layout
     * @param instruction layout instructions
     */
    protected Layouter initLayout(Instruction instruction) {
        if (instruction instanceof Page) {
            return documentLook ? new DocumentLayouter((Page)instruction) : new FormLayouter((Page)instruction);
        }
        else if (instruction instanceof PageSet) {
            return documentLook ? new DocumentLayouter((PageSet)instruction) : new FormLayouter((PageSet)instruction);
        }
        throw new IllegalArgumentException("Page or PageSet expected");
    }

    protected abstract Instruction createLayoutInstruction();

    protected abstract BeanMeta createBeanMeta(String path);

    protected BeanData createBeanData(String path) {
        return new DefaultBeanData(getBeanMeta(path));
    }

    protected SComponent createCustomComponent(String key) {
        throw new UnsupportedOperationException("No component for '" + key + "'. Maybe you should override this method?");
    }

    protected SLabel createCustomLabel(String key) {
        return new SLabel(Client.getInstance().getResourceProvider().getMessage(key));
    }

    public void addObjectAction(ObjectAction objectAction) {
        DelegateObjectAction action = new DelegateObjectAction(objectAction);
        actionDelegates.put(objectAction, action);
        environment.addControl(new XButton(action));
    }

    public void removeObjectAction(ObjectAction objectAction) {
        throw new UnsupportedOperationException("Remove is not implemented.");
    }

    public ObjectAction[] getObjectActions() {
        Set<ObjectAction> objectActions = actionDelegates.keySet();
        return objectActions.toArray(new ObjectAction[objectActions.size()]);
    }

    protected void updateLabelAndEditor(String beanName, String propertyName) {
        updateLabel(beanName, propertyName);
        updateEditorComponent(beanName, propertyName);
    }

    protected void updateEditorComponent(String beanName, String propertyName) {
        BeanMeta beanMeta = getBeanMeta(beanName);
        PropertyMeta propertyMeta = beanMeta.getProperty(propertyName);
        SComponent component = getComponentProvider().getEditorComponent(beanName, propertyName);
        ComponentFactory.INSTANCE.configureComponent(propertyMeta, component, false);
    }

    protected void updateLabel(String beanName, String propertyName) {
        BeanMeta beanMeta = getBeanMeta(beanName);
        PropertyMeta propertyMeta = beanMeta.getProperty(propertyName);
        SLabel component = getComponentProvider().getLabel(beanName, propertyName);
        ComponentFactory.INSTANCE.configureLabel(propertyMeta, component, false);
    }

    public ObjectContext getContext() {
        if (context == null) {
            System.out.println("new context: " + context);
            context = new ObjectContext();
        }
        return context;
    }

    public void setContext(ObjectContext context) {
        this.context = context;
    }

    protected static class DelegateObjectAction
        extends AbstractAction
    {
        ObjectForm form;
        ObjectAction objectAction;

        public DelegateObjectAction(ObjectAction objectAction) {
            this.objectAction = objectAction;
        }

        public void setForm(ObjectForm form) {
            this.form = form;
        }

        public void actionPerformed(ActionEvent e) {
            ObjectActionEvent objectEvent = new ObjectActionEvent(form, form.getObject());
            objectAction.actionPerformed(objectEvent);
        }

        public Object getValue(String key) {
            return objectAction.getValue(key);
        }

        public void putValue(String key, Object value) {
            objectAction.putValue(key, value);
        }

        public void setEnabled(boolean b) {
            objectAction.setEnabled(b);
        }

        public boolean isEnabled() {
            return objectAction.isEnabled();
        }
    }

    private static final Comparator<? super String> PATH_COMPARATOR = new Comparator<String>() {
        public int compare(String o1, String o2) {
            if (o1 == null)
                return -1;
            if (o2 == null)
                return 1;
            return o1.compareTo(o2);
        }
    };

    private static final Modifier READONLY_VARIATION = new Modifier() {
        public void modify(BeanMeta beanMeta) throws ModifierException {
            for (int i = 0; i < beanMeta.getProperties().length; i++) {
                PropertyMeta propertyMeta = beanMeta.getProperties()[i];
                propertyMeta.setWritable(false);
            }
        }
    };

    class ViewerAction
        extends AbstractAction
    {
        BeanMeta beanMeta;

        public ViewerAction(BeanMeta beanMeta) {
            this.beanMeta = beanMeta;
            putValue(Action.NAME, "Meta: " + beanMeta.getType().getSimpleName());
        }

        public void actionPerformed(ActionEvent e) {
            XOptionPane.showInputDialog(component, beanMeta.getName(), "Meta", new MetaViewer(beanMeta), new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                }
            });
        }
    }
}
