package org.osbl.client.wings.form;

import org.conform.BeanMeta;
import org.osbl.agent.gui.DesignContext;
import org.osbl.agent.gui.ObjectDesignContext;
import org.osbl.client.action.ObjectActionEvent;
import org.osbl.client.action.ObjectAction;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.AbstractTool;
import org.osbl.client.wings.shell.Environment;
import org.wings.*;
import org.wings.border.SLineBorder;
import org.wings.table.*;
import org.wingx.XTable;
import org.wingx.XScrollPane;
import org.wingx.table.XTableClickListener;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import java.util.*;
import java.util.List;
import java.awt.event.ActionEvent;
import java.awt.*;

/**
 * A generic base class, that implements basic ObjectList functionality. It uses
 * an XTable width a column model constructed from con:conform bean metas and an
 * ObjectTableModel. Furthermore it implements TitleProvider, ButtonsProvider
 * and ToolsProvider.
 */
public abstract class GenericObjectList extends XTable implements ObjectList, ObjectContextAware
{

    protected DelegateEnvironment environment = new DelegateEnvironment();

    protected BeanMeta beanMeta;

    protected ObjectTableModel model;

    private Map<ObjectAction, DelegateObjectAction> actionDelegates = new HashMap<ObjectAction, DelegateObjectAction>();

    private XScrollPane component;
    protected ObjectContext context;
    protected Map<String, Object> filters = new HashMap<String, Object>();

    protected GenericObjectList() {
        super(null);
        beanMeta = createBeanMeta();
        model = createModel(beanMeta);

        setName(beanMeta.getType().getSimpleName());
        setModel(model);
        setColumnModel(model.getColumnModel());
        setSelectionMode(SListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        setRowSelectionRenderer(new SDefaultTableRowSelectionRenderer() {
            public SComponent getTableCellRendererComponent(STable table, Object value, boolean selected, int row, int col) {
                SComponent component = super.getTableCellRendererComponent(table, value, selected, row, col);
                if (row == getCurrentIndex())
                    setIcon(XIcons.ROW_CURRENT);
                else if (selected)
                    setIcon(getSelectionMode() != SListSelectionModel.SINGLE_SELECTION ? XIcons.ROW_CHECKED : XIcons.ROW_SELECTED);
                else
                    setIcon(XIcons.ROW_PLAIN);

                return component;
            }
        });
    }

    public ObjectContext getContext() {
        if (context == null) {
            System.out.println("new context: " + context);
            context = new ObjectContext();
        }
        return context;
    }

    public void setContext(ObjectContext context) {
        this.context = context;
    }

    public Environment getEnvironment() {
        getComponent();
        return environment;
    }

    /**
     * Create the bean meta, that shall be used for the column model and
     * probably the model, too.
     *
     * @return the bean meta
     */
    protected abstract BeanMeta createBeanMeta();

    /**
     * Create the table model.
     *
     * @param beanMeta
     *            the bean meta
     * @return the table model
     */
    protected abstract ObjectTableModel createModel(BeanMeta beanMeta);

    /**
     * Refresh the model. This can be an expensive operation. Call it only, if
     * you expect, that the data has changed.
     */
    public void refresh() {
        model.refresh();
    }

    Object current;
    int currentIndex = -1;


    public Object getCurrent() {
        return current;
    }

    public void setCurrent(Object current) {
        if (this.current != current) {
            this.current = current;
            currentIndex = -1;
            reload();
        }
    }

    protected int getCurrentIndex() {
        if (currentIndex == -1 && current != null) {
            Object id = idOf(current);
            if (id != null) {
                int rowCount = model.getRowCount();
                for (int i = 0; i < rowCount; i++) {
                    if (id.equals(idOf(getRow(i)))) {
                        currentIndex = i;
                        break;
                    }
                }
            }
        }
        return currentIndex;
    }

    public boolean hasNext() {
        int index = getCurrentIndex();
        return index != -1 && index < getRowCount() - 1;
    }

    public void next() {
        int index = getCurrentIndex();
        if (index < getRowCount() - 1) {
            index ++;
            setCurrent(getRow(index));
            currentIndex = index;
        }
    }

    public boolean hasPrevious() {
        return getCurrentIndex() > 0;
    }

    public void previous() {
        int index = getCurrentIndex();
        if (index > 0) {
            index --;
            setCurrent(getRow(index));
            currentIndex = index;
        }
    }

    public Object getSelectedObject() {
        int selectedRow = getSelectedRow();
        return selectedRow > -1 ? getRow(selectedRow) : null;
    }

    // TODO: inefficient
    public void setSelectedObjects(List objects) {
        clearSelection();

        for (Object object : objects) {
            Object id = idOf(object);
            if (id == null) {
                setSelectedRow(-1);
                continue;
            }

            int rowCount = model.getRowCount();
            for (int i = 0; i < rowCount; i++) {
                if (id.equals(idOf(getRow(i)))) {
                    getSelectionModel().addSelectionInterval(i, i);
                    continue;
                }
            }
        }
    }

    protected Object idOf(Object object) {
        try {
            return object.getClass().getMethod("getId").invoke(object);
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public List getSelectedObjects() {
        int[] selectedRows = getSelectedRows();
        List selectedObjects = new ArrayList(selectedRows.length);
        for (int selectedRow : selectedRows) {
            selectedObjects.add(getRow(selectedRow));
        }
        return selectedObjects;
    }

    /**
     * Obtain the rowId from the ObjectTableModel and load the respective
     * object.
     *
     * @param index
     *            the row index
     * @return the object in row <code>index</code>
     */
    public abstract Object getRow(int index); // was protected

    public ListSelectionListener[] getSelectionListeners() {
        return (ListSelectionListener[]) getListeners(ListSelectionListener.class);
    }

    public void setSelectedObject(Object object) {
        if (object == null) {
            clearSelection();
            return;
        }

        Object id = idOf(object);
        if (id == null) {
            setSelectedRow(-1);
            return;
        }

        int rowCount = model.getRowCount();
        for (int i = 0; i < rowCount; i++) {
            if (id.equals(idOf(getRow(i)))) {
                setSelectedRow(i);
                return;
            }
        }
    }

    public void setLinkAction(String name, final ObjectAction linkAction) {
        for (int i = 0; i < columnModel.getColumnCount(); i++) {
            STableColumn column = columnModel.getColumn(i);
            if (name.equals(column.getIdentifier())) {
                addClickListener(i, new XTableClickListener() {
                    public void clickOccured(int row, int column) {
                        Object object = getRow(row);
                        ObjectActionEvent event = new ObjectActionEvent(GenericObjectList.this, object);
                        linkAction.actionPerformed(event);
                    }
                });
                break;
            }
        }
    }

    public SComponent getComponent() {
        if (component == null) {
            component = new XScrollPane(this);
            component.setBorder(new SLineBorder(new Color(153,153,153), 1));
            environment.setContentPane(component);

            initializeList();
            environment.setTitle(beanMeta.getType().getName() + ".editor.title");
        }
        return component;
    }

    public Class getType() {
        return beanMeta.getType();
    }

    protected void initializeList() {
    }

    public void addObjectAction(ObjectAction objectAction) {
        DelegateObjectAction action = new DelegateObjectAction(objectAction);
        actionDelegates.put(objectAction, action);
        environment.addControl(new XButton(action));
    }

    public void removeObjectAction(ObjectAction objectAction) {
        throw new UnsupportedOperationException("Remove is not implemented.");
    }

    public ObjectAction[] getObjectActions() {
        Set<ObjectAction> objectActions = actionDelegates.keySet();
        return objectActions.toArray(new ObjectAction[objectActions.size()]);
    }

    public void setFilter(String property, Object value) {
        filters.put(property, value);
    }

    public Object getFilter(String property) {
        return filters.get(property);
    }

    public void clearFilters() {
        filters.clear();
    }

    public static class DelegateObjectAction extends AbstractAction {
        ObjectAction objectAction;
        private ObjectList list;

        public DelegateObjectAction(ObjectAction objectAction) {
            this.objectAction = objectAction;
        }

        public void setList(ObjectList list) {
            this.list = list;
        }

        public void actionPerformed(ActionEvent event) {
            ObjectActionEvent objectEvent = new ObjectActionEvent(list, list.getSelectedObjects().toArray());
            objectAction.actionPerformed(objectEvent);
        }

        public Object getValue(String key) {
            return objectAction.getValue(key);
        }

        public void putValue(String key, Object value) {
            objectAction.putValue(key, value);
        }

        public void setEnabled(boolean b) {
            objectAction.setEnabled(b);
        }

        public boolean isEnabled() {
            return objectAction.isEnabled();
        }
    }
}
