package org.osbl.client.wings.form;

import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.form.ValidationStatus;
import org.osbl.persistence.model.TreeEntity;
import org.osbl.persistence.model.TreeNodeEntity;
import org.osbl.persistence.RevertCommand;
import org.conform.BeanMeta;
import org.conform.format.NoFormat;
import org.conform.format.Format;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.*;

public abstract class GenericObjectTreeNode<O>
    implements MutableTreeNode, Comparable, ChangeListener
{
    private GenericObjectTreeModel model;
    protected O object;
    protected GenericObjectTreeNode parent;
    protected List<GenericObjectTreeNode> children;
    Format format;
    private Map clientProperties;
    private boolean trans;

    public GenericObjectTreeModel getModel() {
        if (model == null)
            model = getParent().getModel();
        return model;
    }

    public void setModel(GenericObjectTreeModel model) {
        this.model = model;
    }

    public O getObject() {
        return object;
    }

    public void setObject(O object) {
        if (object == null)
            throw new IllegalArgumentException("null not allowed");

        this.object = object;
    }

    public void setUserObject(Object userObject) {
        setObject((O)userObject);
    }

    protected void maybeLoadChildren() {
        if (children != null)
            return;
        if (trans)
            children = Collections.EMPTY_LIST;
        else
            loadChildren();
    }

    protected abstract void loadChildren();

    public boolean isTransient() {
        return trans;
    }

    public void setTransient(boolean trans) {
        if (!trans && this.trans)
            children = null;
        this.trans = trans;
    }

    public void removeFromParent() {
        MutableTreeNode parent = getParent();
        if (parent != null)
            parent.remove(this);
        model = null;
    }

    public void remove(int index) {
        maybeLoadChildren();
        MutableTreeNode child = children.remove(index);
        child.setParent(null);

        getModel().nodesWereRemoved(GenericObjectTreeNode.this, new int[] { index }, new Object[] { child });
    }

    public void remove(MutableTreeNode child) {
        maybeLoadChildren();
        int index = children.indexOf(child);
        children.remove(child);
        child.setParent(null);

        getModel().nodesWereRemoved(GenericObjectTreeNode.this, new int[] { index }, new Object[] { child });
    }

    public void setParent(MutableTreeNode newParent) {
        parent = (GenericObjectTreeNode)newParent;
    }

    public GenericObjectTreeNode getParent() {
        return parent;
    }

    public void insert(MutableTreeNode newChild, int index) {
        maybeLoadChildren();
        MutableTreeNode oldParent = (MutableTreeNode)newChild.getParent();

        if (oldParent != null)
            oldParent.remove(newChild);

        newChild.setParent(this);

        if (index == -1)
            index = children.size();

        children.add(index, (GenericObjectTreeNode)newChild);
        sortChildren();
        index = children.indexOf(newChild);

        getModel().nodesWereInserted(GenericObjectTreeNode.this, new int[] { index });
    }

    protected void sortChildren() {
        Collections.sort(children);
    }

    public int getChildCount() {
        maybeLoadChildren();
        return children.size();
    }

    public abstract boolean getAllowsChildren();

    public abstract boolean isLeaf();

    public Enumeration<GenericObjectTreeNode> children() {
        maybeLoadChildren();
        return Collections.enumeration(children);
    }

    public TreeNode getChildAt(int childIndex) {
        maybeLoadChildren();
        return children.get(childIndex);
    }

    public int getIndex(TreeNode node) {
        maybeLoadChildren();
        return children.indexOf(node);
    }

    public int compareTo(Object o) {
        GenericObjectTreeNode node = (GenericObjectTreeNode)o;
        if (isLeaf() && !node.isLeaf())
            return 1;
        else if (!isLeaf() && node.isLeaf())
            return -1;

        if (object instanceof Comparable && node.object instanceof Comparable)
            return ((Comparable)object).compareTo(node.object);
        else
            return toString().compareTo(node.toString());
    }

    public String toString() {
        if (object == null) {
            String type = Client.getInstance().getResourceProvider().getMessage(getObjectType().getName());
            return Client.getInstance().getResourceProvider().getMessage("businessobject.newOfType", type);
        }
        else
            return format(object);
    }

    protected String format(Object object) {
        if (format == null) {
            BeanMeta beanMeta = Client.getInstance().getBeanMetaProvider().getBeanMeta(getObjectType());
            format = beanMeta.getFormat();
            if (format == null)
                format = new NoFormat();
        }
        return format.format(object);
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        GenericObjectTreeNode that = (GenericObjectTreeNode)o;

        if (object != null ? !object.equals(that.object) : that.object != null) return false;

        return true;
    }

    public int hashCode() {
        return (object != null ? object.hashCode() : 0);
    }

    /**
     * @return a small HashMap
     * @see #putClientProperty
     * @see #getClientProperty
     */
    private Map getClientProperties() {
        if (clientProperties == null) {
            clientProperties = new HashMap(2);
        }
        return clientProperties;
    }

    /**
     * Returns the value of the property with the specified key.  Only
     * properties added with <code>putClientProperty</code> will return
     * a non-null value.
     *
     * @return the value of this property or null
     * @see #putClientProperty
     */
    public final Object getClientProperty(Object key) {
        if (clientProperties == null) {
            return null;
        } else {
            return getClientProperties().get(key);
        }
    }

    /**
     * Add an arbitrary key/value "client property" to this component.
     * <p/>
     * The <code>get/putClientProperty<code> methods provide access to
     * a small per-instance hashtable. Callers can use get/putClientProperty
     * to annotate components that were created by another module, e.g. a
     * layout manager might store per child constraints this way.  For example:
     * <pre>
     * componentA.putClientProperty("to the left of", componentB);
     * </pre>
     * <p/>
     * If value is null this method will remove the property.
     * Changes to client properties are reported with PropertyChange
     * events.  The name of the property (for the sake of PropertyChange
     * events) is <code>key.toString()</code>.
     * <p/>
     * The clientProperty dictionary is not intended to support large
     * scale extensions to SComponent nor should be it considered an
     * alternative to subclassing when designing a new component.
     *
     * @see #getClientProperty
     */
    public final void putClientProperty(Object key, Object value) {
        if (value != null) {
            getClientProperties().put(key, value);
        } else {
            getClientProperties().remove(key);
        }
        getModel().nodeChanged(this);
    }

    public abstract Class getObjectType();

    public abstract Class[] getChildTypes();

    public void stateChanged(ChangeEvent e) {
        if (e.getSource() instanceof ValidationStatus) {
            ValidationStatus validationStatus = (ValidationStatus)e.getSource();
            putClientProperty("issues", new HashSet(validationStatus.getIssues()));
        }
        getModel().nodeChanged(this);
    }

    public void connect() {
        if (parent.getObject() instanceof TreeEntity) {
            getModel().getTreeLogic().connectNodeToTree((TreeEntity)parent.getObject(), (TreeNodeEntity)getObject());
        }
        else if (parent.getObject() instanceof TreeNodeEntity) {
            getModel().getTreeLogic().connectNodeToNode((TreeNodeEntity)parent.getObject(), (TreeNodeEntity)getObject());
        }
    }

    public void buildContext(ObjectContext context) {
        GenericObjectTreeNode parentTreeNode = getParent();
        if (parentTreeNode != null)
            parentTreeNode.buildContext(context);

        context.putObject("node", this);
        installContext(context);
    }

    protected void installContext(ObjectContext context) {
    }

    public abstract O loadObject();

    public static final class BreadthFirstEnumeration implements Enumeration<TreeNode>
    {
        protected GenericObjectTreeNode.BreadthFirstEnumeration.Queue queue;

        public BreadthFirstEnumeration(TreeNode rootNode) {
            super();
            Vector v = new Vector(1);
            v.addElement(rootNode);
            queue = new GenericObjectTreeNode.BreadthFirstEnumeration.Queue();
            queue.enqueue(v.elements());
        }

        public boolean hasMoreElements() {
            return (!queue.isEmpty() &&
                ((Enumeration)queue.firstObject()).hasMoreElements());
        }

        public TreeNode nextElement() {
            Enumeration	enumer = (Enumeration)queue.firstObject();
            TreeNode	node = (TreeNode)enumer.nextElement();
            Enumeration	children = node.children();

            if (!enumer.hasMoreElements()) {
                queue.dequeue();
            }
            if (children.hasMoreElements()) {
                queue.enqueue(children);
            }
            return node;
        }

        // A simple queue with a linked list data structure.
        final class Queue {
            GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode head;	// null if empty
            GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode tail;

            final class QNode {
                public Object	object;
                public GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode next;	// null if end
                public QNode(Object object, GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode next) {
                    this.object = object;
                    this.next = next;
                }
            }

            public void enqueue(Object anObject) {
                if (head == null) {
                    head = tail = new GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode(anObject, null);
                } else {
                    tail.next = new GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode(anObject, null);
                    tail = tail.next;
                }
            }

            public Object dequeue() {
                if (head == null) {
                    throw new NoSuchElementException("No more elements");
                }

                Object retval = head.object;
                GenericObjectTreeNode.BreadthFirstEnumeration.Queue.QNode oldHead = head;
                head = head.next;
                if (head == null) {
                    tail = null;
                } else {
                    oldHead.next = null;
                }
                return retval;
            }

            public Object firstObject() {
                if (head == null) {
                    throw new NoSuchElementException("No more elements");
                }

                return head.object;
            }

            public boolean isEmpty() {
                return head == null;
            }
        }
    }
}
