package org.osbl.client.wings.form;

import org.osbl.client.action.*;
import org.osbl.client.wings.*;
import org.osbl.client.wings.shell.*;

import javax.swing.*;
import java.util.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A generic base class, that implements basic ObjectViewer functionality.
 * It encapsulates an ObjectForm.
 */
public abstract class GenericObjectViewer
    implements ObjectViewer
{
    protected Environment environment = new DelegateEnvironment(true);

    protected ObjectList objectList;
    protected ObjectForm objectForm;
    private boolean editing;
    private Object object;

    protected ShowAction showAction = new ShowAction(base());
    protected PreviousAction previousAction = new PreviousAction(base());
    protected NextAction nextAction = new NextAction(base());
    protected ListAction listAction = new ListAction(base());
    protected SelectObjectAction selectObjectAction = new SelectObjectAction(base());
    protected DeselectObjectAction deselectObjectAction = new DeselectObjectAction(base());
    protected GenericObjectForm.DelegateObjectAction formSelectAction = new GenericObjectForm.DelegateObjectAction(selectObjectAction);
    protected GenericObjectList.DelegateObjectAction listSelectAction = new GenericObjectList.DelegateObjectAction(selectObjectAction);
    protected GenericObjectForm.DelegateObjectAction formDeselectAction = new GenericObjectForm.DelegateObjectAction(deselectObjectAction);
    protected GenericObjectList.DelegateObjectAction listDeselectAction = new GenericObjectList.DelegateObjectAction(deselectObjectAction);

    private List<ObjectAction> objectActions = new LinkedList<ObjectAction>();
    private Window window;

    public GenericObjectViewer() {
        initializeViewer();
        environment.setActive(true);
    }

    public Environment getEnvironment() {
        return environment;
    }

    public void showList() {
        editing = false;

        getList();

        if (objectForm != null)
            objectForm.getEnvironment().setActive(false);
        objectList.getEnvironment().setActive(true);

        objectList.refresh();
    }

    public void showForm() {
        editing = true;

        getForm();

        if (objectList != null)
            objectList.getEnvironment().setActive(false);
        objectForm.getEnvironment().setActive(true);

        objectForm.setObject(object);
    }

    /**
     * Initializations like installing global object actions go here.
     * Override this method for your own initializations.
     */
    protected void initializeViewer() {
    }

    protected ObjectForm getForm() {
        if (objectForm == null) {
            objectForm = initializeForm();
            objectForm.setViewMode(ObjectForm.VIEW);
            formControls(objectForm.getEnvironment());
            objectForm.getEnvironment().setDelegate(environment);
            formSelectAction.setForm(objectForm);
            formDeselectAction.setForm(objectForm);
        }
        return objectForm;
    }

    protected void formControls(Environment environment) {
        environment.addControl(new XButton(listAction));
        environment.addControl(new XButton(previousAction));
        environment.addControl(new XButton(nextAction));
    }

    /**
     * Create and initialize the conform component.
     * @return the conform component
     */
    protected abstract ObjectForm initializeForm();

    protected ObjectList getList() {
        if (objectList == null) {
            objectList = initializeList();
            listControls(objectList.getEnvironment());
            objectList.getEnvironment().setDelegate(environment);
            listSelectAction.setList(objectList);
            listDeselectAction.setList(objectList);

            objectList.setLinkAction("key", new AbstractObjectAction() {
                public void actionPerformed(ObjectActionEvent e) {
                    Object object = e.getTargets()[0];
                    objectList.setCurrent(object);
                    setObject(object);

                    if (showAction.isEnabled())
                        showAction.actionPerformed(new ActionEvent(GenericObjectViewer.this, 0, "edit"));
                }
            });

            /*
            this.objectList.setSelectionMode(STable.SINGLE_SELECTION);
            this.objectList.addSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    if (e.getValueIsAdjusting())
                        return;

                    Object selectedObject = objectList.getSelectedObject();
                    if (selectedObject == null) {
                        objectList.setSelectedObject(getObject());
                        return;
                    }
                    setObject(selectedObject);
                    if (showAction.isEnabled())
                        showAction.actionPerformed(new ActionEvent(GenericObjectViewer.this, 0, "edit"));

                    previousAction.setEnabled(objectList.hasPrevious());
                    nextAction.setEnabled(objectList.hasNext());
                }
            });
            */
        }
        return objectList;
    }

    protected void listControls(Environment environment) {
    }

    /**
     * Create and initialize the list component.
     * @return the list component
     */
    protected abstract ObjectList initializeList();

    public Object getObject() {
        return object;
    }

    public void setObject(Object object) {
        this.object = object;
    }

    public void setSelectionMode(int selectionMode) {
        getList().setSelectionMode(selectionMode);
    }

    public void setSelectedObject(Object object) {
        getList().setSelectedObject(object);
    }

    public Object getSelectedObject() {
        return getList().getSelectedObject();
    }

    public List getSelectedObjects() {
        return getList().getSelectedObjects();
    }

    public void setSelectedObjects(List objects) {
        getList().setSelectedObjects(objects);
    }

    public abstract Class getType();

    /**
     * Provide the ChooseAction and append the object list actions if any.
     * @return a list of actions, displayed along with the list.
     */
    protected List<Action> getListActions() {
        return null;
    }

    /**
     * Provide the ListAction, PreviousAction, NextAction and SaveAction and append the object conform actions if any.
     * @return a list of actions, displayed along with the conform.
     */
    protected List<Action> getFormActions() {
        return null;
    }

    protected String base() {
        return getType().getName();
    }

    private String unsavedchangesMessage(String name) {
        String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.unsavedChanges", name);
        if (objectForm instanceof GenericObjectForm) {
            GenericObjectForm genericObjectForm = (GenericObjectForm)objectForm;
            String changes = genericObjectForm.changeStatus.composeMessageDetail();
            message = message + "\n\n" + changes;
        }
        return message;
    }

    protected class ListAction extends AbstractAction {
        public ListAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.list");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[r]");
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = unsavedchangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            showList();
                    }
                });
            }
            else
                showList();
        }
    }

    protected class NextAction extends AbstractAction {
        public NextAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.next");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[r]");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(objectForm.getType().getName());
                String message = unsavedchangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            next();
                    }
                });
            }
            else
                next();
        }

        private void next() {
            objectList.next();
            setObject(objectList.getCurrent());
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
        }
    }

    protected class PreviousAction extends AbstractAction {
        public PreviousAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.previous");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[r]");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            if (objectForm.hasChanges()) {
                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = unsavedchangesMessage(name);
                XOptionPane.showConfirmDialog(null, message, name, new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        if (XOptionPane.YES_ACTION == e.getActionCommand())
                            previous();
                    }
                });
            }
            else
                previous();
        }

        private void previous() {
            objectList.previous();
            setObject(objectList.getCurrent());
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
        }
    }

    protected class AddAction extends AbstractAction {
        public AddAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.add");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[w]");
        }

        public void actionPerformed(ActionEvent e) {
            setObject(null);
            showForm();
            objectList.setSelectedObject(null);
            previousAction.setEnabled(false);
            nextAction.setEnabled(false);
        }
    }

    protected class ShowAction extends AbstractAction {
        public ShowAction(String base) {
            putValue(Action.ACTION_COMMAND_KEY, base + ".buttons.show");
            putValue(AuthorizedAction.PERMISSION_KEY, base + "[r]");
        }

        public void actionPerformed(ActionEvent e) {
            showForm();
            previousAction.setEnabled(objectList.hasPrevious());
            nextAction.setEnabled(objectList.hasNext());
        }
    }

    protected class SelectObjectAction
        extends AbstractObjectAction
    {
        public SelectObjectAction(String base) {
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(base + ".buttons.select"));
            putValue(Action.ACTION_COMMAND_KEY, Client.getInstance().getResourceProvider().getMessage(base + ".buttons.list"));
        }

        public void actionPerformed(ObjectActionEvent event) {
            try {
                List list = objectList.getSelectedObjects();
                list.addAll(Arrays.asList(event.getTargets()));
                objectList.setSelectedObjects(list);

                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.selected", name, list.size());
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));
            }
            catch (Exception e) {
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_ERROR, e.getMessage()));
                e.printStackTrace();
            }

            if (editing)
                showList();
            else
                objectList.refresh();
        }
    }

    protected class DeselectObjectAction
        extends AbstractObjectAction
    {
        public DeselectObjectAction(String base) {
            putValue(Action.NAME, Client.getInstance().getResourceProvider().getMessage(base + ".buttons.deselect"));
            putValue(Action.ACTION_COMMAND_KEY, Client.getInstance().getResourceProvider().getMessage(base + ".buttons.list"));
        }

        public void actionPerformed(ObjectActionEvent event) {
            try {
                List list = objectList.getSelectedObjects();
                list.removeAll(Arrays.asList(event.getTargets()));
                objectList.setSelectedObjects(list);

                String name = Client.getInstance().getResourceProvider().getMessage(getType().getName());
                String message = Client.getInstance().getResourceProvider().getMessage("businessobject.messages.selected", name, list.size());
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_INFO, message));
            }
            catch (Exception e) {
                getEnvironment().setStatusMessage(new Client.Message(Client.Message.TYPE_ERROR, e.getMessage()));
                e.printStackTrace();
            }

            if (editing)
                showList();
            else
                objectList.refresh();
        }
    }
}
