package org.osbl.client.wings.form;

import org.wings.SComponent;
import org.osbl.client.action.ObjectAction;
import org.osbl.client.wings.shell.Environment;

import javax.swing.*;
import java.util.List;

/**
 * An ObjectList displays a list of objects in list manner or table manner.
 * It supports selection, navigation (link) and allows a user to perform actions on selected objects.
 * @author hengels
 */
public interface ObjectList
{
    public static final int SINGLE_SELECTION = ListSelectionModel.SINGLE_SELECTION;
    public static final int MULTIPLE_SELECTION = ListSelectionModel.MULTIPLE_INTERVAL_SELECTION;

    /**
     * The environment provided by the editor.
     */
    Environment getEnvironment();

    /**
     * Return the SComponent, that shows the list of objects.
     * Consecutive calls to this method must always return the same component instance. The implementor might choose to
     * create the component lazily on the first call.
     *
     * @return the component showing the list
     */
    SComponent getComponent();

    Class getType();

    /**
     * The content of the list is refreshed.
     * Call this method, if you expect, that the list's content has changed.
     * This method might perform expensive operations.
     */
    void refresh();

    /**
     * Return the current object.
     * @return the current object
     */
    Object getCurrent();

    /**
     * Set the currently selected object
     * @param object the current object
     */
    void setCurrent(Object object);

    /**
     * Tell wether there is at least one object behind the current object in the list.
     * @return true, if the current object is not the last object, false otherwise
     */
    boolean hasNext();

    /**
     * Set cursor to the the next object in the list.
     */
    void next();

    /**
     * Tell wether there is at least one object before the current object in the list.
     * @return true, if the current object is not the first object, false otherwise
     */
    boolean hasPrevious();

    /**
     * Set cursor to the previous object in the list.
     */
    void previous();

    /**
     * Set the selection mode.
     * ObjectLists support single selection and multi selection. Consult the wingS- or Swing- documentation for further
     * information on selection models.
     *
     * @param selectionMode either STable.SINGLE_SELECTION or STable.MULTIPLE_SELECTION
     */
    void setSelectionMode(int selectionMode);

    /**
     * Set the selected object.
     * SelectionListeners will be informed of this change!
     * @param object the object to be selected
     */
    void setSelectedObject(Object object);

    /**
     * Retrieve the currently selected object.
     * @return the selected object or null, if nothing is selected. If there are multiple selected objects, return the
     * first one.
     */
    Object getSelectedObject();

    /**
     * Set the selected object.
     * SelectionListeners will be informed of this change!
     * @param objects the objects to be selected
     */
    void setSelectedObjects(List objects);

    /**
     * Retrieve a list of the currently selected objects.
     * @return a list of selected objects.
     */
    List getSelectedObjects();

    /**
     * Expose an ObjectAction visually as a link.
     * ObjectList implementations should support a link notion. That is, one column (typically the key column) is
     * rendered in link style (ie. bold and blue). If a user clicks on a link, the ObjectAction will be called on the
     * object in the respective row.
     * @param name the column name (typically property name)
     * @param linkAction the action to called on click
     */
    void setLinkAction(String name, ObjectAction linkAction);
}
