package org.osbl.client.wings.form.editor;

import org.conform.wings.Editor;
import org.conform.wings.editor.ComponentInvalidator;
import org.conform.wings.editor.FormattedListCellRenderer;
import org.conform.*;
import org.conform.format.Format;
import org.wings.*;
import org.osbl.client.wings.XButton;
import org.osbl.client.wings.shell.*;
import org.osbl.client.wings.form.*;
import org.osbl.client.action.AbstractObjectAction;
import org.osbl.client.action.ObjectActionEvent;

import javax.swing.*;
import javax.swing.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.*;
import java.util.List;
import java.awt.*;
import java.awt.event.ActionEvent;

public class ManyToManyAssociationEditor
    implements Editor
{
    public SComponent createComponent(PropertyMeta propertyMeta) {
        Panel panel = new Panel(propertyMeta);
        configureComponent(propertyMeta, panel, false);
        return panel;
    }

    public void configureComponent(PropertyMeta propertyMeta, SComponent component, boolean erroneous) {
        Panel panel = (Panel)component;
        panel.setEnabled(propertyMeta.isWritable());
        panel.setVisible(propertyMeta.isReadable());
    }

    public void setPropertyData(SComponent component, PropertyData propertyData) {
        BeanData beanData = propertyData.getBeanData();
        if (beanData != null)
            beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), new ComponentInvalidator(component));

        if (component instanceof Panel) {
            Panel panel = (Panel)component;
            panel.setPropertyData(propertyData);

            if (beanData != null)
                beanData.addPropertyChangeListener(propertyData.getPropertyMeta().getName(), panel);
        }
    }

    public PropertyData getPropertyData(SComponent component) {
        if (component instanceof Panel) {
            Panel panel = (Panel)component;
            return panel.getPropertyData();
        }

        return null;
    }

    protected static class Panel
        extends SContainer
        implements PropertyChangeListener
    {
        protected SList included;
        protected SList excluded;
        protected SButton include;
        protected SButton exclude;
        protected SButton manage;
        protected PropertyData propertyData;
        private List<? extends EditorAction> actions;
        private IncludeExcludeModel model;

        public Panel(PropertyMeta propertyMeta) {
            super(new SGridBagLayout());

            include = new XButton(new Panel.IncludeAction());
            exclude = new XButton(new Panel.ExcludeAction());
            manage = new XButton(new Panel.ManageAction());

            included = new SList();
            included.setPreferredSize(SDimension.FULLWIDTH);
            excluded = new SList();
            excluded.setPreferredSize(SDimension.FULLWIDTH);

            Format format = propertyMeta.getFormat();
            if (format != null) {
                included.setCellRenderer(new FormattedListCellRenderer(format));
                excluded.setCellRenderer(new FormattedListCellRenderer(format));
            }
            actions = (List<? extends EditorAction>)propertyMeta.getAttribute(Editor.ACTIONS);
            if (actions == null)
                actions = createActions();

            for (EditorAction action : actions)
                action.setComponent(this);

            initialize();
        }

        protected List<? extends EditorAction> createActions() {
            return Arrays.asList(new IncludeAction(), new ExcludeAction(), new ManageAction());
        }

        protected void initialize() {
            GridBagConstraints constraints = new GridBagConstraints();
            constraints.gridheight = actions.size() + 1;
            constraints.weightx = .4;
            constraints.gridy = 0;
            add(included, constraints);
            constraints.gridx = 2;
            add(excluded, constraints);

            constraints.gridheight = 1;
            constraints.gridx = 1;
            constraints.weightx = .2;
            constraints.insets = new Insets(0, 6, 6, 6);

            for (Action action : actions) {
                add(new XButton(action), constraints);
                constraints.gridy ++;
            }
            add(new SLabel(), constraints);
        }

        public PropertyData getPropertyData() {
            return propertyData;
        }

        public void setPropertyData(PropertyData propertyData) {
            this.propertyData = propertyData;
            for (EditorAction action : actions)
                action.setPropertyData(propertyData);

            IncludeExcludeModel model = new IncludeExcludeModel(propertyData);
            model.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    BeanData beanData = Panel.this.propertyData.getBeanData();
                    List value = getModel().getIncludedModel().getList();
                    Panel.this.propertyData.setValue(value);
                    PropertyChangeListener[] propertyChangeListeners = beanData.getPropertyChangeListeners(Panel.this.propertyData.getPropertyMeta().getName());
                    for (PropertyChangeListener propertyChangeListener : propertyChangeListeners) {
                        propertyChangeListener.propertyChange(new PropertyChangeEvent(beanData, Panel.this.propertyData.getPropertyMeta().getName(), value, value));
                    }
                }
            });
            setModel(model);

            DomainProvider domainProvider = propertyData.getPropertyMeta().getDomainProvider();
            if (domainProvider instanceof DynamicDomainProvider) {
                ((DynamicDomainProvider)domainProvider).addChangeListener(new ChangeListener() {
                    public void stateChanged(ChangeEvent e) {
                        propertyChange(null);
                    }
                });
            }
        }

        public void setEnabled(boolean enabled) {
            included.setEnabled(enabled);
            excluded.setEnabled(enabled);
            for (Action action : actions) {
                action.setEnabled(enabled);
            }
        }

        public void propertyChange(PropertyChangeEvent evt) {
            getModel().refresh();
        }

        public void setModel(IncludeExcludeModel model) {
            this.model = model;
            included.setModel(model.getIncludedModel());
            excluded.setModel(model.getExcludedModel());
        }

        public IncludeExcludeModel getModel() {
            return model;
        }

        protected class IncludeAction
            extends AbstractEditorAction
        {
            public IncludeAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.include");
            }

            public void actionPerformed(ActionEvent e) {
                Object[] objects = ((Panel)getComponent()).excluded.getSelectedValues();
                for (int i = 0; i < objects.length; i++) {
                    Object object = objects[i];
                    ((Panel)getComponent()).getModel().include(object);
                }
                excluded.clearSelection();
            }
        }

        protected class ExcludeAction
            extends AbstractEditorAction
        {
            public ExcludeAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.exclude");
            }

            public void actionPerformed(ActionEvent e) {
                Object[] objects = ((Panel)getComponent()).included.getSelectedValues();
                for (int i = 0; i < objects.length; i++) {
                    Object object = objects[i];
                    ((Panel)getComponent()).getModel().exclude(object);
                }
                included.clearSelection();
            }
        }

        static protected class ManageAction
            extends AbstractEditorAction
        {
            private EditorWindow window;

            public ManageAction() {
                putValue(Action.ACTION_COMMAND_KEY, ".buttons.manage");
            }

            public void actionPerformed(ActionEvent e) {
                EditorWindow window = getEditorWindow();
                ObjectEditor editor = window.getEditor();
                editor.setObject(null);
                editor.showList();
                org.osbl.client.wings.shell.Window parent = org.osbl.client.wings.shell.Window.currentWindow(((Panel)getComponent()).included);
                Client.getInstance().pushWindow(parent, window);
            }

            private EditorWindow getEditorWindow() {
                if (window == null) {
                    ObjectEditor editor = getObjectEditor();
                    window = new EditorWindow(editor);
                    window.addPopAction(new PopAction());
                }
                return window;
            }

            private ObjectEditor getObjectEditor() {
                ObjectEditor objectEditor = (ObjectEditor)getPropertyData().getPropertyMeta().getAttribute("objectEditor");
                if (objectEditor == null)
                    objectEditor = Editors.get(getPropertyData().getPropertyMeta().getType());
                return objectEditor;
            }

            protected class PopAction
                extends AbstractObjectAction
            {
                public PopAction() {
                    putValue(Action.ACTION_COMMAND_KEY, ".buttons.back");
                }

                public void actionPerformed(ObjectActionEvent event) {
                    DomainProvider domainProvider = propertyData.getPropertyMeta().getDomainProvider();
                    if (domainProvider instanceof DynamicDomainProvider) {
                        DynamicDomainProvider dynamicDomainProvider = (DynamicDomainProvider)domainProvider;
                        dynamicDomainProvider.refresh();
                    }
                    //((Panel)getComponent()).getModel().refresh();
                }

                public void setEnabled(boolean enabled) {
                }
            }
        }
    }

    public static class IncludeExcludeModel
    {
        private PropertyData propertyData;
        private ListListModel excludedModel = new ListListModel();
        private ListListModel includedModel = new ListListModel();
        private ChangeListener changeListener;


        public IncludeExcludeModel(PropertyData propertyData) {
            this.propertyData = propertyData;
            refresh();
        }

        public void refresh() {
            List value = (List)propertyData.getValue();
            if (value == null)
                return;

            DomainProvider domainProvider = propertyData.getPropertyMeta().getDomainProvider();

            List included = new ArrayList(value);
            List available = domainProvider.getDomain();
            List excluded = new ArrayList(available);
            excluded.removeAll(included);

            includedModel.setList(included);
            excludedModel.setList(excluded);
        }

        public ListListModel getExcludedModel() {
            return excludedModel;
        }

        public ListListModel getIncludedModel() {
            return includedModel;
        }

        public void include(Object object) {
            if (excludedModel.remove(object))
                includedModel.add(object);

            if (changeListener != null)
                changeListener.stateChanged(new ChangeEvent(this));
        }

        public void exclude(Object object) {
            if (includedModel.remove(object))
                excludedModel.add(object);

            if (changeListener != null)
                changeListener.stateChanged(new ChangeEvent(this));
        }

        public void addChangeListener(ChangeListener changeListener) {
            this.changeListener = changeListener;
        }
    }
}
