package org.osbl.client.wings.shell;

import java.awt.event.ActionEvent;
import javax.swing.JFileChooser;
import javax.swing.JProgressBar;
import org.apache.batik.bridge.*;
import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
import org.apache.batik.dom.svg.SVGDocumentFactory;
import org.apache.batik.gvt.GraphicsNode;
import org.apache.batik.gvt.renderer.StaticRenderer;
import org.apache.batik.util.XMLResourceDescriptor;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGDocument;
import org.wings.*;
import org.wings.session.BrowserType;
import org.wings.session.SessionManager;
import org.wings.util.SessionLocal;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;

public class SVGButtonRenderer
{
    private static final transient Log log = LogFactory.getLog( SVGButtonRenderer.class );

    SessionLocal iconCaches = new SessionLocal() {
        protected Object initialValue() {
            return Collections.synchronizedMap(new HashMap<BufferedImage, SIcon>());
        }
    };
    protected static final Map<String, BufferedImage> renderedImageCache = Collections.synchronizedMap(new HashMap<String, BufferedImage>());

    /**
     * The returned width of the textwidths somehow does not fit the real life. This correcture factor can be used to adjust.
     */
    protected static final double TEXTWIDTH_CORRECTURE_FACTOR = 1.0;

    private static SVGButtonRenderer instance = null;

    private SVGButtonRenderer()
    {
    }

    /**
     * This method provides dynamical rendered buttons ....
     *
     * @param buttonText        The text which should appear inside the button
     * @param buttonTemplateUri The template file for the button
     */
    public SIcon retrieveRenderedButtonIcon( String buttonText, String buttonTemplateUri, Dimension imageSize )
    {
        BufferedImage bufferedImage = retrieveRenderedButtonImage(buttonText, buttonTemplateUri, imageSize);
        Map<BufferedImage, SIcon> renderedIconCache = (Map<BufferedImage, SIcon>)iconCaches.get();
        SIcon icon = renderedIconCache.get(bufferedImage);
        if (icon == null) {
            icon = new SImageIcon(bufferedImage, isIE() ? "image/gif" : "image/png");
            renderedIconCache.put(bufferedImage, icon);
        }
        return icon;
    }

    private boolean isIE() {
        return BrowserType.IE.equals(SessionManager.getSession().getUserAgent().getBrowserType());
    }

    /**
     * This method provides dynamical rendered buttons ....
     *
     * @param buttonText        The text which should appear inside the button
     * @param buttonTemplateUri The template file for the button
     */
    public BufferedImage retrieveRenderedButtonImage( String buttonText, String buttonTemplateUri, Dimension imageSize )
    {
        if (buttonText == null) {
            buttonText = "";
        }

        // Cached version available?!
        String cacheKey = buttonTemplateUri + buttonText;
        BufferedImage cachedVersion = renderedImageCache.get(cacheKey);
        if ( cachedVersion != null && imageSize.width == cachedVersion.getWidth() && imageSize.height == cachedVersion.getHeight())
        {
            log.debug( "Returning CACHED version of ButtonImage for button '" + buttonText + "' with template '"
                    + buttonTemplateUri + "'" );
            return cachedVersion;
        }

        if (imageSize == null) {
            imageSize = calculateButtonImageSize(buttonText);
        }

        return renderAndCacheNewButton( buttonText, buttonTemplateUri, imageSize );
    }

    private synchronized BufferedImage renderAndCacheNewButton( String buttonText, String buttonTemplateURI, Dimension imageSize )
    {
        if (buttonText == null) {
            buttonText = "";
        }
        log.debug( "Rendering unknown image button for button '" + buttonText + "'" );
        String cacheLookupKey = buttonTemplateURI.concat( buttonText );

        // Recheck if cached version is available, perhaps other thread already generated this button?
        BufferedImage cachedVersion = renderedImageCache.get(cacheLookupKey);
        if ( cachedVersion != null && imageSize.width == cachedVersion.getWidth() && imageSize.height == cachedVersion.getHeight())
        {
            log.warn( "Concurrent thread seems to have already rendered button'" + buttonText
                    + "'. Returning existing instance." );
            return ( BufferedImage ) renderedImageCache.get( cacheLookupKey );
        }

        // Render button
        long renderingTime = System.currentTimeMillis();
        BufferedImage img = renderButton( buttonText, buttonTemplateURI, imageSize );
        renderingTime = System.currentTimeMillis() - renderingTime;
        log.debug( "Finished rendering button in " + renderingTime + " ms. Caching and returning result." );
        renderedImageCache.put(cacheLookupKey, img);
        return img;
    }

    /**
     * This method takes the given button text, inserts it into a SVG template document retrieved via the classpath, renders
     * the resulting SVG image into an image and returns this Image
     *
     * @param buttonText The text which should appear inside the button.
     */
    private BufferedImage renderButton( String buttonText, String buttonTemplateURI, Dimension buttonImageSize )
    {
        GVTBuilder builder = new GVTBuilder();
        UserAgent userAgent = new Log4jUserAgentAdapter();
        DocumentLoader loader = new DocumentLoader( userAgent );
        BridgeContext ctx = new BridgeContext( userAgent, loader );
        StaticRenderer renderer = new StaticRenderer();
        GraphicsNode gvtRoot = null;
        BufferedImage image = null;

        try
        {
            // create according svg document
            Document svgDoc = createButtonDocument( buttonText, buttonImageSize, buttonTemplateURI );
            log.debug( "Building GVT Tree" );
            gvtRoot = builder.build( ctx, svgDoc );
            log.debug( "Populating GVT tree rood node to renderer [" + gvtRoot + "]" );
            renderer.setTree( gvtRoot );

            // Allocate according image
            BufferedImage display =
                    new BufferedImage( ( int ) buttonImageSize.getWidth(), ( int ) buttonImageSize.getHeight(),
                            BufferedImage.TYPE_INT_BGR );

            // Set Transform (?!)
            Element elt = ( ( SVGDocument ) svgDoc ).getRootElement();
            renderer.setTransform( ViewBox.getViewTransform( null, elt, display.getWidth(), display.getHeight() ) );

            // Cacluculate offscreen Image
            renderer.updateOffScreen( display.getWidth(), display.getHeight() );

            //Repaint whole image
            Rectangle r = new Rectangle( 0, 0, display.getWidth(), display.getHeight() );
            renderer.repaint( r );

            // Return resulting image
            image = renderer.getOffScreen();
        }
        catch ( Exception e )
        {
            log.fatal( "Uncaught Exception", e );
        }
        return image;
    }

    public static SVGButtonRenderer getInstance()
    {
        if ( instance == null )
        {
            synchronized ( SVGButtonRenderer.class )
            {
                if ( instance == null )
                    instance = new SVGButtonRenderer();
            }
        }
        return instance;
    }

    /**
     * Calls the method which creates a String with the SVG XML Document and parses it into an XML SVGDocument.
     */
    private SVGDocument createButtonDocument( String buttonText, Dimension buttonImageSize, String buttonTemplateURI )
    {
        log.debug( "Creating SVG template document" );
        try
        {
            String templateDocument = prepareTemplateDocument( buttonText, buttonImageSize, buttonTemplateURI );

            log.debug( "Parsing resulting Document String to an SVGDocument" );
            ByteArrayInputStream istream = new ByteArrayInputStream( templateDocument.getBytes("UTF-8") );
            SVGDocumentFactory svgDocumentFactory =
                    new SAXSVGDocumentFactory( XMLResourceDescriptor.getXMLParserClassName() );
            return svgDocumentFactory.createSVGDocument( null, istream );
        }
        catch ( Exception e )
        {
            log.fatal( "Uncaught Exception", e );
        }
        return null;
    }

    /**
     * Calculates the estimated size for the button immage.
     */
    public Dimension calculateButtonImageSize( String buttonText )
    {
        if (buttonText == null) {
            buttonText = "";
        }
        log.debug( "Estimating image size for button with label '" + buttonText + "'" );
        Font font = new Font( "sans-serif", Font.BOLD, 10 );
        // XXX - Hack
        // This was previously new Label(), and this threw a HeadLessException.
        // Heavyweight components DON'T concern in headless environments
        // It's still pretty stupid to use JLabel, but we need a quick fix
        FontMetrics fm = new JLabel().getFontMetrics( font );

        Dimension maxSize = new Dimension( Integer.MAX_VALUE, Integer.MAX_VALUE );
        Dimension minSize = new Dimension( 0, 0 );

        //Calculate the approximate space for the text
        int textWidth = fm.stringWidth( buttonText );
        int height = 20;
        int horizontalBorder = 21; // BSC: orig "20" but due to wrong font width with short texts (i.e. Ok) expanded.
        int width = ( ( int ) ( textWidth * TEXTWIDTH_CORRECTURE_FACTOR ) ) + horizontalBorder; // Apply lenght calculation correction factor.

        // Adjust size to min/max requirements
        if ( width > maxSize.width )
            width = maxSize.width;
        else if ( width < minSize.width )
            width = minSize.width;
        if ( height > maxSize.height )
            height = maxSize.height;
        else if ( height < minSize.height )
            height = minSize.height;

        return new Dimension( width, height );
    }

    /**
     * The platform-specific implementation of how to retrieve the actual button template document. This implementation
     * differs for servlet/wings/swing applications.
     */
    private InputStream getTemplateDocument( String templateFileURI )
    {
        InputStream templateDocument = Thread.currentThread().getContextClassLoader().getResourceAsStream( templateFileURI );
        if ( templateDocument == null )
            log.warn( "Failed to retrieve template document +'" + templateFileURI + "'. Check URI/implementation!" );
        return templateDocument;
    }

    /**
     * Reads the template document from the given URI and resolves the included tags.
     */
    private String prepareTemplateDocument( String templateText, Dimension buttonSize, String templateFileURI )
    {
        InputStream templateFileStream = null;
        InputStreamReader streamReader = null;
        BufferedReader r = null;
        try
        {
            templateFileStream = getTemplateDocument( templateFileURI );
            streamReader = new InputStreamReader( templateFileStream, "ISO-8859-1" );
            r = new BufferedReader( streamReader );
            StringBuffer templateDocumentString = new StringBuffer();
            String line = "";

            log.debug( "Reading template file " + templateFileURI );
            while ( line != null )
            {
                templateDocumentString.append( line );
                line = r.readLine();
            }

            //templateDocumentString.append("\n");
            log.debug( "Replacing tokens." );
            String template =
                    replaceTokens( templateDocumentString.toString(), templateText, ( int ) buttonSize.getWidth(),
                            ( int ) buttonSize.getHeight() );

            log.debug( "Returning resulting document " + template );
            return template;
        }
        catch ( Exception ex )
        {
            log.fatal( "Uncatched Exception", ex );
        }
        finally
        {
            try
            {
                if ( r != null )
                    r.close();
                if ( streamReader != null )
                    streamReader.close();
                if ( templateFileStream != null )
                    templateFileStream.close();
            } catch ( IOException e )
            {
                log.error( "Unexpected error", e );
            }
        }
        return null;
    }

    /**
     * Recursive method to replace all occurrences of template snippets like "@width@" ...
     */
    private String replaceTokens( String template, String buttonText, int width, int height )
    {
        // replace the text element
        if ( template.indexOf( "@text@" ) >= 0 )
            template =
                    template.substring( 0, template.indexOf( "@text@" ) ) + buttonText
                            + template.substring( template.indexOf( "@text@" ) + 6 );

        // find parse and replace the width element
        if ( template.indexOf( "@width" ) >= 0 )
        {
            template = replaceSizeToken( template, "@width", width );
            return replaceTokens( template, buttonText, width, height );
        } else if ( template.indexOf( "@height" ) >= 0 )
        {
            template = replaceSizeToken( template, "@height", height );
            return replaceTokens( template, buttonText, width, height );
        } else
            return template;
    }

    /**
     * Special methode to replace the token with pixel information.
     * Such a token can look like this: @width-5@ This means we have to insert the width of the button minus 5 pixels.
     * The optional number within the token can be "+" or "-"
     */
    private String replaceSizeToken( String template, String tokenName, int value )
    {
        int index = template.indexOf( tokenName );
        if ( index < 0 )
            return template;

        String element = template.substring( index, template.indexOf( "@", index + 1 ) );

        // parse if value has to be adjusted.. (example: @width-4@)
        int diff = 0;
        if ( element.indexOf( "-" ) >= 0 )
            diff = Integer.parseInt( element.substring( tokenName.length() ) );
        else if ( element.indexOf( "+" ) >= 0 )
            diff = Integer.parseInt( element.substring( tokenName.length() + 1 ) );

        //replace the element:
        template = template.substring( 0, index ) + ( value + diff ) + template.substring( index + element.length() + 1 );

        return template;
    }

    /**
     * Pipes all apache-bastik message output for the user agent to log4j
     */
    static class Log4jUserAgentAdapter extends UserAgentAdapter
    {
        public void displayMessage( String message )
        {
            log.info( message );
        }
    }
    
    public static void main(String[] args) {
        SwingUtilities.invokeLater(
            new Runnable() {
                public void run() {
                    startApp();
                }
            }
        );        
    }
    
    public static void startApp() {
        final JFrame frWindow = new JFrame();
        frWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final JPanel paMain = new JPanel();
        final JProgressBar progressBar = new JProgressBar();
        final SVGButtonRenderer brImageCreator = new SVGButtonRenderer();
        progressBar.setVisible(false);
        paMain.setLayout(new BorderLayout());
        
        JButton btLoadFile = new JButton(
            new AbstractAction("Load SVG file") {
                public void actionPerformed(ActionEvent e) {
                    JFileChooser fcOpenSVGFile = new JFileChooser("/home/hengels/jdevel/osb3/osbuild/openshop/modules/core/src/web/WEB-INF/classes/svg/");
                    fcOpenSVGFile.setDialogTitle("Load SVG File");
                    fcOpenSVGFile.setMultiSelectionEnabled(false);
                    fcOpenSVGFile.addChoosableFileFilter( 
                        new FileFilter() {
                            public boolean accept(File pathname) {
                                return pathname.isFile() && (pathname.getAbsolutePath().endsWith(".svg") || pathname.getAbsolutePath().endsWith(".SVG"));
                            }

                            public String getDescription() {
                                return "SVG Files";
                            }
                        }
                    );
                    if( fcOpenSVGFile.showOpenDialog(frWindow) == JFileChooser.APPROVE_OPTION ) {
                        final File fSelectedFile = fcOpenSVGFile.getSelectedFile();
                        //String filePath = selectedFile.getAbsolutePath();
                        //String normalizedPath = filePath.replace('\\', '/');
                        //String fileURI = "file://" + normalizedPath;
                        progressBar.setVisible(true);
                        progressBar.setIndeterminate(true);
                        //thread
                        Thread t = new Thread(
                            new Runnable() {
                                public void run() {
                                    //Dimension dim = brImageCreator.calculateButtonImageSize("Aufgaben");
                                    Dimension dim = new Dimension(20, 100);
                                    final BufferedImage biImageButton = brImageCreator.retrieveRenderedButtonImage("Aufgaben", fSelectedFile.getName(), dim);
                                    SwingUtilities.invokeLater(
                                        new Runnable() {
                                            public void run() {
                                                Component c = ((BorderLayout) paMain.getLayout()).getLayoutComponent(BorderLayout.CENTER);
                                                if( c != null ) {
                                                    paMain.remove(c);
                                                }
                                                paMain.add(
                                                    new JButton(
                                                    new ImageIcon(biImageButton)
                                                    ),
                                                    BorderLayout.CENTER
                                                    );
                                                //paMain.setBackground(Color.YELLOW);
                                                paMain.validate();

                                                //thread end
                                                progressBar.setVisible(false);
                                            }
                                        }
                                    );
                                }
                            }
                        );
                        t.start();
                    }
                }
            }
        );
        JButton btClean = new JButton(
            new AbstractAction("Clean") {
                public void actionPerformed(ActionEvent e) {
                    Component c = ((BorderLayout) paMain.getLayout()).getLayoutComponent(BorderLayout.CENTER);
                    if( c != null ) {
                        paMain.remove(c);
                    }
                    paMain.validate();
                }
            }
        );
        JPanel paButtons = new JPanel();
        paButtons.setLayout(new FlowLayout());
        paButtons.add(btLoadFile);
        paButtons.add(btClean);
        paMain.add(paButtons, BorderLayout.NORTH);
        paMain.add(progressBar, BorderLayout.SOUTH);
        frWindow.setContentPane(paMain);
        frWindow.setSize(new Dimension(640,480));
        //window.pack();
        frWindow.setVisible(true);
    }
}
