package org.osbl.client.wings.shell.smart;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.wings.*;
import org.wings.resource.DefaultURLResource;
import org.wings.header.Link;
import org.wings.session.BrowserType;
import org.osbl.client.wings.shell.smart.ProcessClientStyles;
import org.osbl.client.wings.shell.SVGButtonRenderer;
import org.osbl.client.wings.shell.Templates;
import org.osbl.client.wings.*;

/**
 * The <code>SideBarPane</code> could contain components and each
 * component you add with {@link this#addTab(String, org.wings.SComponent)}
 * will get a tab button to set the component visible or not. You
 * are able to pin the sidebar if you set the pinnable as true.
 *
 * @author leon
 * @author rro
 */
public class SideBarPane extends SPanel
{
    private final static String SVG_LEFT = "svg/tabbed_pane_left.svg";
    private final static String SVG_LEFT_SELECTED = "svg/tabbed_pane_left_selected.svg";
    private final static String SVG_LEFT_DISABLED = "svg/tabbed_pane_left_disabled.svg";
    private final static String SVG_RIGHT = "svg/tabbed_pane_right.svg";
    private final static String SVG_RIGHT_SELECTED = "svg/tabbed_pane_right_selected.svg";
    private final static String SVG_RIGHT_DISABLED = "svg/tabbed_pane_right_disabled.svg";

    // Styles for sidebar on the left.
    protected Link leftVisibleStyles = new Link("stylesheet", null, "text/css", null, new DefaultURLResource(isIE() ? "../templates/navigations-msie.css" : "../templates/navigations.css"));

    // Styles for sidebar on the right.
    protected Link rightVisibleStyles = new Link("stylesheet", null, "text/css", null, new DefaultURLResource(isIE() ? "../templates/tools-msie.css" : "../templates/tools.css"));

    // Whether the sidebar is pinnable or not.
    private boolean pinnable;

    // Whether the sidebar is pinned or not.
    private boolean pinned = true;

    // The tab area contains the tab icons.
    private SPanel tabArea;

    // The pin button on the tab area.
    private SButton pinButton;

    // The placement of the sidebar.
    private int placement;

    // SComponent to SButton mapping - component -> tab button.
    private Map<SComponent, SButton> component2Button = new HashMap<SComponent, SButton>();

    // The current activated button.
    private SButton activeButton;

    // The current activated component.
    private SComponent activeComponent;
    private Map<String, SImageIcon> icons = new HashMap<String, SImageIcon>();
    private boolean shaded = true;

    /**
     * A new sidebar pane that is not pinnable.
     *
     * @param placement The placement on the frame.
     */
    public SideBarPane(int placement) {
        this(placement, false);
    }

    /**
     * A new sidebar pane.
     *
     * @param placement The placement on the frame.
     * @param pinnable  Whether the sidebar should be pinnable
     *                  or not.
     */
    public SideBarPane(int placement, boolean pinnable) {
        this.placement = placement;
        this.pinnable = pinnable;

        try {
            setLayout(new STemplateLayout(Templates.load(placement == SConstants.LEFT ? "/sidebar_left.html" : "/sidebar_right.html")));
        }
        catch (IOException ex) {
            throw new RuntimeException(ex);
        }

        setPreferredSize(SDimension.FULLAREA);

        tabArea = new SPanel(new SFlowDownLayout());
        tabArea.add(createPinButton());
        add(tabArea, "tabs");
    }

    /**
     * Let the code generator delegate write the component's code
     * to the device. The code generator is the actual 'plaf'.
     *
     * @param device the Device to write into
     * @throws IOException Thrown if the connection to the client gets broken,
     *                     for example when the user stops loading
    public void write(Device device) throws IOException {
        Utils.renderContainer(device, this);
    }
    */

    /**
     * Returns whether the sidebar is allows to be pinned
     * or not.
     *
     * @return Whether the sidebar is allows to be pinned
     *         or not.
     */
    public boolean isPinnable() {
        return pinnable;
    }

    /**
     * Sets whether the sidebar should be pinnable or not.
     *
     * @param pinnable Whether the sidebar should be pinnable
     *                 or not.
     */
    public void setPinnable(boolean pinnable) {
        this.pinnable = pinnable;
    }

    /**
     * Returns whether the sidebar is pinned or unpinned.
     *
     * @return Whether the sidebar is pinned or unpinned.
     */
    public boolean isPinned() {
        return pinned;
    }

    /**
     * Shows the sidebar.
     */
    public void show() {
        // empty
    }

    /**
     * Hides the sidebar.
     */
    public void hide() {
        if (activeButton != null) {
            activeButton.setIcon(getButtonIcon(activeButton.getToolTipText(), true, false));
            activeButton = null;
        }

        if (activeComponent != null) {
            hideTab(activeComponent);
        }
    }

    /**
     * Adds a component to the sidebar. The title will be used for
     * tab title creation.
     *
     * @param title The title of the tab.
     * @param c     The component that should be shown in a tab.
     */
    public void addTab(final String title, SComponent c) {
        c.setVisible(false);
        final SButton button = new SButton() {
            public void requestFocus() {}
        };
        button.setToolTipText(title);
        button.setStyle(ProcessClientStyles.SIDE_BAR_BUTTON);
        button.setIcon(getButtonIcon(title, true, false));
        button.setDisabledIcon(getButtonIcon(title, false, false));
        button.putClientProperty("component", c);

        component2Button.put(c, button);

        button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                if (activeButton != null && activeButton != button) {
                    activeButton.setIcon(getButtonIcon(activeButton.getToolTipText(), true, false));
                }

                // Close side bar pane if selected and open button was pressed.
                if (activeButton != null && activeButton == button) {
                    hide();
                }
                // Open side bar pane if selected or display another c.
                else {
                    activeButton = button;
                    activeButton.setIcon(getButtonIcon(title, true, true));
                    showTab((SComponent) activeButton.getClientProperty("component"));
                }
            }
        });

        tabArea.add(button);
    }

    /**
     * Removes the component from the sidebar.
     *
     * @param c The component that should be removed from
     *          sidebar.
     */
    public void removeTab(SComponent c) {
        SButton button = component2Button.remove(c);
        tabArea.remove(button);

        if (button == activeButton) {
            if (activeComponent != null) {
                remove(activeComponent);
            }
            activeComponent = null;
            activeButton = null;
            makeVisible(false);
        }
    }

    public boolean getEnabledFor(SComponent component) {
        SButton button = component2Button.get(component);
        return button.isEnabled();
    }

    public void setEnabledFor(SComponent component, boolean enabled) {
        SButton button = component2Button.get(component);
        button.setEnabled(enabled);
        if (button == activeButton)
            hide();
    }

    /**
     * Set the visibility.
     *
     * @param visible wether this component will show or not
     */
    public void setVisible(boolean visible) {

        if (visible == isVisible()) return;

        super.setVisible(visible);

        if (activeComponent != null) {
            activeComponent.setVisible(visible);
        }
    }

    /**
     * Show the component on the sidebar.
     *
     * @param c The component that will be shown.
     */
    protected void showTab(SComponent c) {
        if (c != null) {
            if (activeComponent != null) {
                activeComponent.setVisible(false);
                remove(activeComponent);
                activeButton.setIcon(getButtonIcon(activeButton.getToolTipText(), true, false));
            }

            c.setVisible(true);
            activeComponent = c;
            add(activeComponent, "content");
            activeButton = component2Button.get(c);
            if (activeButton != null) {
                activeButton.setIcon(getButtonIcon(activeButton.getToolTipText(), true, true));
            }
            makeVisible(true);
        }
    }

    /**
     * Hide the component on the sidebar.
     *
     * @param c The component that will be hidden.
     */
    protected void hideTab(SComponent c) {
        if (c != null) {
            makeVisible(false);
            remove(c);
            activeComponent = null;
            c.setVisible(false);
        }
    }

    /**
     * Creates the pin button.
     */
    private SButton createPinButton() {
        pinButton = new SButton(XIcons.PIN_PINNED);
        pinButton.setStyle(ProcessClientStyles.SIDE_BAR_BUTTON);
        pinButton.setVisible(false);
        pinButton.addActionListener(new ActionListener() {

            /**
             * Invoked when an action occurs.
             */
            public void actionPerformed(ActionEvent e) {

                SIcon icon;
                if (pinned) {
                    icon = XIcons.PIN_UNPINNED;
                }
                else {
                    icon = XIcons.PIN_PINNED;
                }
                pinButton.setIcon(icon);

                // Change pin state.
                pinned = !pinned;
            }
        });

        return pinButton;
    }

    /**
     * Makes the sidebar visible or unvisible on current parent
     * frame. It decides whether the sidebar is on the left or
     * on the right.
     *
     * @param visible Whether the sidebar should be visible or
     *                not.
     */
    private void makeVisible(boolean visible) {
        if (shaded == !visible)
            return;

        shaded = !visible;

        // Show pin button if the sidebar is visible and the sidebar
        // allows a pinned state.
        pinButton.setVisible(visible && pinnable);

        SFrame parentFrame = getParentFrame();
        if (placement == SConstants.LEFT) {
            if (visible) parentFrame.addHeader(leftVisibleStyles);
            else parentFrame.removeHeader(leftVisibleStyles);
        }
        else {
            if (visible) parentFrame.addHeader(rightVisibleStyles);
            else parentFrame.removeHeader(rightVisibleStyles);
        }
        if (isIE())
            parentFrame.reload();
    }

    /**
     * Returns whether the browser type equals internet explorer
     * or not.
     *
     * @return Whether the browser type equals internet explorer
     *         or not.
     */
    private boolean isIE() {
        return BrowserType.IE.equals(getSession().getUserAgent().getBrowserType());
    }

    private SImageIcon getButtonIcon(String text, boolean enabled, boolean selected) {
        String key;

        if (!enabled)
            key = text + "_disabled";
        else
            key = text + (selected ? "_selected" : "_deselected");

        SImageIcon icon = icons.get(key);
        if (icon == null) {
            SVGButtonRenderer renderer = SVGButtonRenderer.getInstance();
            Dimension d = renderer.calculateButtonImageSize(text);
            Dimension newDim = new Dimension(d.height, d.width + 10);
            String res;
            if (placement == SConstants.LEFT) {
                if (!enabled)
                    res = SVG_LEFT_DISABLED;
                else
                    res = selected ? SVG_LEFT_SELECTED : SVG_LEFT;
            }
            else {
                if (!enabled)
                    res = SVG_RIGHT_DISABLED;
                else
                    res = selected ? SVG_RIGHT_SELECTED : SVG_RIGHT;
            }
            BufferedImage img = renderer.retrieveRenderedButtonImage(text, res, newDim);
            icon = new SImageIcon(img, isIE() ? "image/gif" : "image/png");
            icons.put(key, icon);
        }
        return icon;
    }
}
