package org.osbl.spring;

import org.osbl.ServiceProvider;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.beans.factory.*;
import org.springframework.beans.*;

import java.util.*;
import java.beans.PropertyDescriptor;

public class SpringServiceProvider
    extends ServiceProvider
    implements BeanFactoryAware
{
    private static Map<Class<? extends Object>,BeanHelper> BEAN_HELPER_CACHE = new HashMap<Class<? extends Object>, BeanHelper>();

    DefaultListableBeanFactory beanFactory;

    public SpringServiceProvider() {
        INSTANCE = this;
    }

    public List<String> getServiceNames() {
        return Arrays.asList(getBeanFactory().getBeanDefinitionNames());
    }

    public Object getService(String name) {
        return getBeanFactory().getBean(name);
    }

    public <T> T getService(Class<T> type) {
        ListableBeanFactory beanFactory = getBeanFactory();
        do {
            String[] names = beanFactory.getBeanNamesForType(type);
            switch (names.length) {
                case 0:
                    if (beanFactory instanceof HierarchicalBeanFactory) {
                        HierarchicalBeanFactory hierarchicalBeanFactory = (HierarchicalBeanFactory)beanFactory;
                        beanFactory = (ListableBeanFactory)hierarchicalBeanFactory.getParentBeanFactory();
                        continue;
                    }
                    else
                        beanFactory = null;
                case 1:
                    return (T) getService(names[0]);
                default:
                    throw new IllegalArgumentException("More than one bean of type '" + type + "' found in beanFactory");
            }
        } while (beanFactory != null);

        return null;
    }

    public void autowireServices(Object bean) {
        Class<? extends Object> type = bean.getClass();
        SpringServiceProvider.BeanHelper helper = getBeanHelper(type);
        BeanWrapper bw = helper.getBeanWrapper();
        // prevent multiple thread access
        synchronized (bw) {
            bw.setWrappedInstance(bean);
            populateBean(type.getName(), helper.getBeanDefinition(), bw);
        }
    }

    private SpringServiceProvider.BeanHelper getBeanHelper(Class<? extends Object> type) {
        SpringServiceProvider.BeanHelper helper = BEAN_HELPER_CACHE.get(type);
        if (helper == null) {
            helper = new SpringServiceProvider.BeanHelper(new BeanWrapperImpl(), new RootBeanDefinition(type, false));
            BEAN_HELPER_CACHE.put(type, helper);
        }
        return helper;
    }

    private void populateBean(String beanName, RootBeanDefinition bd, BeanWrapper bw) {
        PropertyValues pvs = bd.getPropertyValues();
        MutablePropertyValues mpvs = new MutablePropertyValues(pvs);
        String[] propertyNames = getPropertyNames(bd, bw);
        for (int i = 0; i < propertyNames.length; i++) {
            String propertyName = propertyNames[i];
            Class<?> requiredType = bw.getPropertyDescriptor(propertyName).getPropertyType();
            Object bean = getService(requiredType);
            if (bean != null) {
                PropertyValue propertyValue = new PropertyValue(propertyName, bean);
                mpvs.addPropertyValue(propertyValue);
            }
        }
        bw.setPropertyValues(mpvs);
    }

    private String[] getPropertyNames(RootBeanDefinition mergedBeanDefinition, BeanWrapper bw) {
        Set<String> result = new TreeSet<String>();
        PropertyValues pvs = mergedBeanDefinition.getPropertyValues();
        PropertyDescriptor[] pds = bw.getPropertyDescriptors();
        for (int i = 0; i < pds.length; i++) {
            if (pds[i].getWriteMethod() != null &&
                    !pvs.contains(pds[i].getName()) &&
                    !BeanUtils.isSimpleProperty(pds[i].getPropertyType())) {
                result.add(pds[i].getName());
            }
        }
        return result.toArray(new String[result.size()]);
    }

    public void setBeanFactory(BeanFactory beanFactory) throws BeansException {
        this.beanFactory = (DefaultListableBeanFactory)beanFactory;
    }

    private static class BeanHelper {
        private BeanWrapper beanWrapper;
        private RootBeanDefinition beanDefinition;

        public BeanHelper(BeanWrapper beanWrapper, RootBeanDefinition beanDefinition) {
            this.beanWrapper = beanWrapper;
            this.beanDefinition = beanDefinition;
        }

        public BeanWrapper getBeanWrapper() {
            return beanWrapper;
        }

        public RootBeanDefinition getBeanDefinition() {
            return beanDefinition;
        }
    }

    private DefaultListableBeanFactory getBeanFactory() {
        return beanFactory;
    }
}
