package org.osbl.riskmanagement.gui.report;

import org.wings.*;
import org.wings.session.SessionManager;
import org.wings.io.Device;
import org.wings.plaf.css.*;
import org.jfree.chart.*;
import org.jfree.chart.labels.XYToolTipGenerator;
import org.jfree.chart.urls.XYURLGenerator;
import org.jfree.chart.entity.StandardEntityCollection;
import org.jfree.chart.annotations.XYBoxAnnotation;
import org.jfree.chart.renderer.xy.*;
import org.jfree.chart.axis.*;
import org.jfree.chart.plot.*;
import org.jfree.data.xy.*;
import org.jfree.ui.Layer;
import org.osbl.persistence.*;
import org.osbl.ServiceProvider;
import org.osbl.client.action.ActionProvider;
import org.osbl.client.wings.shell.Client;
import org.osbl.client.wings.action.ShowEditorAction;
import org.osbl.riskmanagement.model.Risk;
import org.osbl.riskmanagement.model.Criticality;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.*;
import java.util.List;
import java.math.BigDecimal;
import java.io.IOException;


public class RiskChart
    extends SComponent
    implements LowLevelEventListener
{
    private String imageMap;
    private SIcon icon;

    public RiskChart() {
        setCG(new AbstractComponentCG() {
            public void writeInternal(Device device, SComponent component) throws IOException {
                writeTablePrefix(device, component);
                writeImageMap(device, (RiskChart)component);
                writeIconWithMap(device, component.getName() + "_map", icon, Utils.isMSIE(component));
                writeTableSuffix(device, component);
            }

            public void writeIconWithMap(Device device, String name, SIcon icon, boolean isMSIE) throws IOException {
                device.print("<img class=\"nopad\"");
                if (isMSIE && icon.getURL().toString().endsWith(".png") && icon.getIconWidth() > 0 && icon.getIconHeight() > 0) {
                    Utils.optAttribute(device, "style", "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + icon.getURL() + "', sizingMethod='scale')");
                    Utils.optAttribute(device, "src", AbstractLabelCG.TRANSPARENT_ICON.getURL());
                }
                else
                    Utils.optAttribute(device, "src", icon.getURL());

                Utils.optAttribute(device, "width", icon.getIconWidth());
                Utils.optAttribute(device, "height", icon.getIconHeight());
                Utils.optAttribute(device, "usemap", name);
                device.print(" alt=\"");
                device.print(icon.getIconTitle());
                device.print("\"/>");
            }

            private void writeImageMap(Device device, RiskChart riskChart) throws IOException {
                device.print(riskChart.getImageMap());
            }
        });
        setBackground(Color.WHITE);
        setPreferredSize(new SDimension(600, 400));

        Persistence persistence = (Persistence)ServiceProvider.getInstance().getService("RiskManagementPersistence");
        SimpleQueryCommand listCommand = (SimpleQueryCommand)persistence.createCommand("list");
        listCommand.setType(Risk.class);
        listCommand.addFilter("obsolete", Operator.NULL, null);
        listCommand.addFilter("criticality", Operator.NOT_EQUAL, Criticality.UNKNOWN);

        List<Risk> list = listCommand.execute();

        JFreeChart chart = createRiskMap(list);

        SDimension dimension = getPreferredSize();
        ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());
        BufferedImage bufferedImage = chart.createBufferedImage(dimension.getWidthInt(), dimension.getHeightInt(), info);

        this.imageMap = ChartUtilities.getImageMap(getName() + "_map", info);
        this.icon = new SImageIcon(bufferedImage);
    }

    private JFreeChart createRiskMap(final List<Risk> risks) {
        NumberAxis xAxis = new NumberAxis("Amount");
        xAxis.setRange(0, 160);
        xAxis.setTickUnit(new NumberTickUnit(40));
        NumberAxis yAxis = new NumberAxis("Probability");
        yAxis.setRange(0, 100);
        yAxis.setTickUnit(new NumberTickUnit(25));

        XYSeries series = new XYSeries("Risks", false);
        for (Risk risk : risks) {
            BigDecimal amount = risk.getAmount();
            BigDecimal probability = risk.getProbability();
            BigDecimal coverage = risk.getCoverage();
            BigDecimal multiplied = amount.multiply(probability);
            BigDecimal percentage = multiplied.divide(coverage, 2);
            if (percentage.intValue() > 160)
                percentage = new BigDecimal(160);
            series.add(percentage, probability);
        }
        XYDataset dataset = new XYSeriesCollection(series);

        Stroke dashed = XYPlot.DEFAULT_GRIDLINE_STROKE;
        Paint gray = XYPlot.DEFAULT_GRIDLINE_PAINT;
        Paint green = ChartColor.VERY_LIGHT_GREEN;
        Paint red = ChartColor.VERY_LIGHT_RED;

        StandardXYItemRenderer renderer = new StandardXYItemRenderer(
            StandardXYItemRenderer.SHAPES,
            new XYToolTipGenerator() {
                public String generateToolTip(XYDataset dataset, int series, int item) {
                    Number x = dataset.getX(series, item);
                    Number y = dataset.getY(series, item);
                    Risk risk = risks.get(item);
                    BigDecimal coverage = risk.getCoverage();
                    BigDecimal amount = risk.getAmount();
                    BigDecimal probability = risk.getProbability();
                    BigDecimal multiplied = amount.multiply(probability).divide(new BigDecimal(100));
                    return risk.getKey() + " " + multiplied + " / " + coverage;
                }
            },
            new XYURLGenerator() {
                public String generateURL(XYDataset xyDataset, int series, int item) {
                    Risk risk = risks.get(item);
                    return "javascript:wingS.request.sendEvent(null,true,true,'" + getName() + "', '" + risk.getId() + "');";
                }
            });
        /*
        XYBubbleRenderer renderer = new XYBubbleRenderer();
        renderer.setToolTipGenerator(new XYToolTipGenerator() {
            public String generateToolTip(XYDataset dataset, int series, int item) {
                Risk risk = risks.get(item);
                BigDecimal coverage = risk.getCoverage();
                BigDecimal amount = risk.getAmount();
                BigDecimal probability = risk.getProbability();
                BigDecimal multiplied = amount.multiply(probability).divide(new BigDecimal(100));
                return risk.getKey() + " " + multiplied + " / " + coverage;
            }
        });
        renderer.setURLGenerator(new XYURLGenerator() {
            public String generateURL(XYDataset xyDataset, int series, int item) {
                Risk risk = risks.get(item);
                return "javascript:wingS.request.sendEvent(null,true,true,'" + getName() + "', '" + risk.getId() + "');";
            }
        });
        */
        renderer.setShapesFilled(Boolean.TRUE);
        renderer.setPaint(Color.BLUE);
        renderer.addAnnotation(new XYBoxAnnotation(0, 0, 40, 25, dashed, gray, green), Layer.BACKGROUND);
        renderer.addAnnotation(new XYBoxAnnotation(0, 25, 40, 50, dashed, gray, green), Layer.BACKGROUND);
        renderer.addAnnotation(new XYBoxAnnotation(40, 0, 80, 25, dashed, gray, green), Layer.BACKGROUND);
        renderer.addAnnotation(new XYBoxAnnotation(120, 75, 160, 100, dashed, gray, red), Layer.BACKGROUND);
        renderer.addAnnotation(new XYBoxAnnotation(120, 50, 160, 75, dashed, gray, red), Layer.BACKGROUND);
        renderer.addAnnotation(new XYBoxAnnotation(80, 75, 120, 100, dashed, gray, red), Layer.BACKGROUND);

        XYPlot plot = new XYPlot(dataset, xAxis, yAxis, renderer);

        JFreeChart chart = new JFreeChart(plot);
        chart.setBackgroundPaint(getBackground());
        return chart;
    }

    /*
    public static void main(String[] args) {

        XYSeries series = new XYSeries("Test Data");
        series.add(10, 10);
        series.add(10, 90);
        series.add(90, 90);
        series.add(90, 10);
        series.add(50, 50);
        XYDataset data = new XYSeriesCollection(series);

        ChartRenderingInfo info = new ChartRenderingInfo(new StandardEntityCollection());

        JFreeChart chart = createRiskMap(
            "Propability", "Amount",
            null);

        BufferedImage bufferedImage = chart.createBufferedImage(400, 400, info);
        String imageMap = ChartUtilities.getImageMap("test", info);
        System.out.println("imageMap = " + imageMap);

        ChartPanel chartPanel = new ChartPanel(chart);
        JFrame frame = new JFrame();
        frame.setContentPane(chartPanel);
        frame.setSize(new Dimension(400, 400));
        frame.show();
    }
    */

    public String getImageMap() {
        return imageMap;
    }

    String navigate;

    public void processLowLevelEvent(String name, String[] values) {
        System.out.println("name = " + name + " " + Arrays.asList(values));
        navigate = values[0];
        SForm.addArmedComponent(this);
    }

    public void fireIntermediateEvents() {
    }

    public void fireFinalEvents() {
        super.fireFinalEvents();
        if (navigate != null) {
            final ActionProvider sessionActionProvider = Client.getInstance().getActionProvider();
            ShowEditorAction action = (ShowEditorAction)sessionActionProvider.getAction("org.concern.actions.subjects");
            if (action != null && action.isEnabled()) {
                Map query = new HashMap();
                query.put("process", "RiskManagement");
                query.put("subject", "" + navigate);
                query.put("locale", "" + SessionManager.getSession().getLocale().toString());
                action.navigate(query);
            }
            navigate = null;
        }
    }

    public boolean isEpochCheckEnabled() {
        return false;
    }
}
